###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package DBAccess::DBIObj;

use strict;

use DBI;
use Data::Dumper;

use ErrorHandling;
use MetadotConfig;
use DBAccess::DBObj;
use Time::HiRes qw( gettimeofday tv_interval time );
my $TIMESTAMP;

use vars qw(@ISA);
@ISA=qw(DBAccess::DBObj);

# Caches statement handles for complex queries
my $STH_CACHE = {};


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = $class->SUPER::new;

    bless ($self, $class);
    # aju -- turned this off
    $self->{debug_on} = (MetadotConfig->get_param_value('sql_logfile'));
    # aju -- changed this from using the c: drive but really
    # it needs to be set from the params.
    $self->{logdirectory} = "/tmp";
    return $self;
}



#return or create dbh
sub get_dbh {
    my $self = shift;

  die "abstract method.  Do not instantiate";
}

sub close_dbh {
    my $self = shift;
    die "abstract method.  Do not instantiate";
}

sub _init_dbh {
    my $self = shift;
    my $dbh = shift;
    $dbh->{FetchHashKeyName} = 'NAME_lc';
}

# ========================================
# a set of DBI methods that should be implementable by all DBs that
# support this driver.
# The 'locking' methods were moved to the mysql subclass, since at least in their
# current incarnation they are mysql-specific.

# aju 01-25-02: simply check whether or not a given table exists in the db,
# by using the 'tables()' method.  Don't do an actual query on the
# table content.
sub existsTable {
    my ($self, $tablename) = @_;

    my $dbh = $self->get_dbh();
    # get list of all tables in the db
    my @tablelist = $dbh->tables();
    my $found = 0;
    foreach my $tname (@tablelist) {
        # fist case is for Mysql (table names are returned in the form TABLE_NAME)
        # second case is for Oracle (table names are returned in the form DATABASE_NAME.TABLE_NAME)
        ## SPECIAL THANKS TO Jeff Brown FOR FINDING THAT SOME VERSIONS OF
        ## MYSQL RETURN BACKTICKS AROUND TABLENAME
        if ($tname =~ /^\`?$tablename\`?$/i || $tname =~ /^(.*)\.$tablename$/i) {
            $found = 1;
            last;
        }
    }
    return $found;
}

sub sqlSimple  {
    my($self, $sql)=@_;

    if ($self->{debug_on}) {
        $self->sql_log($sql);
        $self->timer_start();
    }

    my $dbh = $self->get_dbh();

    if(!$dbh->do($sql)) {
        my $err=$dbh->errstr;
        my $errormsg = "sqlSimple: SQL statement:<br /><br />$sql<br /><br />\nError: <br /><br />$err<br /><br />\nat $0";
        ErrorHandling->printError($errormsg);
        exit(-1);
    }
    if ($self->{debug_on}) {
        $self->timer_log();
    }
}

#aju 01-2-02
sub sqlSimpleSelect {
    my ($self, $sql) = @_;

    if ($self->{debug_on}) {
        $self->sql_log($sql);
        $self->timer_start();
    }

    my $dbh = $self->get_dbh();
    my $sth = $dbh->prepare($sql);
    if($sth->execute()) {
        if ($self->{debug_on}) {
            $self->timer_log();
        }
        return $sth;
    } else {
        $sth->finish();
        my $err = $dbh->errstr;
        my $errormsg = "sqlSimpleSelect: SQL statement:<br /><br />$sql<br /><br />\nError: $err<br /><br />\nat $0";
        ErrorHandling->printError($errormsg);

        return undef;
        die "could not execute: $sql\n";
    }
}

sub sqlSelect {
    my ($self, $select, $from, $where, $other)=@_;

    my $dbh = $self->get_dbh();

    my $sql = "SELECT $select ";
    $sql .= "FROM $from " if $from;
    $sql .= "WHERE $where " if $where;
    $sql .= "$other" if $other;

    if ($self->{debug_on}) {
        $self->sql_log($sql);
        $self->timer_start();
    }

    my ($c) = $dbh->prepare($sql) or die "Sql has gone to hell\n";
    unless ($c->execute()) {
        my $err = $dbh->errstr;
        my $errormsg = "sqlSelect: SQL statement:<br /><br />$sql<br /><br />\nError: $err<br /><br />\nat $0";

        use Carp qw(confess);
        confess "$errormsg\n";
    }
    my (@r) = $c->fetchrow();
    $c->finish();

    if ($self->{debug_on}) {
        $self->timer_log();
    }

    return @r;

}

sub sqlSelectMany  {
    my($self, $select, $from, $where, $other)=@_;

    my $dbh = $self->get_dbh();

    my $sql="SELECT $select ";
    $sql.="FROM $from " if $from;
    $sql.="WHERE $where " if $where;
    $sql.="$other" if $other;

    if ($self->{debug_on}) {
        $self->sql_log($sql);
        $self->timer_start();
    }

    my $sth = $dbh->prepare($sql);
    if($sth->execute()) {
        if ($self->{debug_on}) {
            $self->timer_log();
        }
        return $sth;
    } else {
        $sth->finish();
        my $err = $dbh->errstr;
        my $errormsg = "sqlSelectMany: SQL statement:<br /><br />$sql<br /><br />\nError: $err<br /><br />\nat $0";
        ErrorHandling->printError($errormsg);

        return undef;
        die "could not execute: $sql\n";
    }
}


# select the count of the set of rows with multiple distinct fields
# (different databases have different syntax for this)
sub sqlSelectCountDistinct {
    my($self, $select, $from, $where, $other)=@_;

    die("Abstract method");
}


sub sqlSimplePrepared  {
    my($self, $sql, @values)=@_;
    # try to get a cached sth, otherwise prepare a sth and cache it
    my $err = '';
    my $sth = $self->_retrieve_sth($sql);
    if (!$sth) {
        my $dbh = $self->get_dbh();
        $sth = $dbh->prepare($sql) if $dbh;
        $self->_cache_sth($sql,$sth);
        # report an error
        unless ($sth) {
            my $err .= "Could not obtain a statement handle for '$sql'\n";
            $err .= "dbh object = $dbh\n";
            if ($dbh) {
                $err .= "Error message: ".$dbh->errstr;
            }
        }
    }
    # now process the statement
    if ($sth) {
        unless ($sth->execute(@values)) {
            $sth->finish();
            my $dbh = $self->get_dbh();
            $err .= "Could not execute query for '$sql'\n";
            $err .= "Error message: ".$dbh->errstr;
        }
    }

    if ($err) {
        ErrorHandling->printError($err);
    }

    return $sth;
}

sub _cache_sth {
    my $self = shift;
    my $key = shift;
    my $sth = shift;

    my $timestamp = time();
    $STH_CACHE->{$key}->{timestamp} = $timestamp;
    $STH_CACHE->{$key}->{sth} = $sth;

    #print STDERR "DBIObj STORED sth for key='$key\', timestamp=$timestamp, object='$sth'\n";
}

sub _retrieve_sth {
    my $self = shift;
    my $key = shift;
    my $max_age = shift || 28800; # default = 8 hrs

    my $result;

    if($STH_CACHE->{$key}) {
        if (time() - $STH_CACHE->{$key}->{timestamp} > $max_age) {
            my $sth = $STH_CACHE->{$key}->{sth};
            $sth->finish() if (defined $sth);
            delete $STH_CACHE->{$key};
            #print STDERR "DBIObj EXPIRED sth for key '$key\', age=",(time()-$STH_CACHE),"\n";
        } else {
            $result = $STH_CACHE->{$key}->{sth};
            #print STDERR "DBIObj RETRIEVED sth for key '$key\', got '$result'\n";
        }
    } else {
        #print STDERR "DBIObj NOT FOUND sth for key '$key\'\n";
    }
    return $result;
}

# return true if the table has a sequenced primary key
sub is_sequence_needed {
    my $self = shift;
    my $table = shift;
    die "Abstract method\n";
}

# Return the primary key field(s) for a table. Currently is only
# necessary for sequenced primary keys
sub get_primary_key_field {
    my $self = shift;
    my $table = shift;
    die "Abstract method\n";
}

# get the next primary key in the sequence
sub get_next_primary_key {
    my $self = shift;
    my $table = shift;
    die "Abstract method\n";
}

# get the primary key from the last insert. this method is not uniformly supported by DBD drivers
sub get_insert_id {
    my $self = shift;
    return undef;
}

sub sqlInsert {
    my($self, $table, %data)=@_;

    my $dbh = $self->get_dbh();

    $self->validate_data(
        table       => $table,
        parameters  => \%data,
    );


    my ($primary_key_field, $primary_key_value);
    my $sequence_is_needed = $self->is_sequence_needed($table);
    if ($sequence_is_needed) {
        # this is crazy, but sometimes this code doesn't work if this comment is removed. (Creating a supergizmo in oracle)
        $primary_key_field = $self->get_primary_key_field($table);
        $primary_key_value = $self->get_next_primary_key($table);
        $data{$primary_key_field} = $primary_key_value;
    }

    my $columns = join(', ', keys %data);
    my $placeholders = join(', ', (('?') x scalar(keys(%data))));
    my $sql = 'insert into ' . $table . ' ('. $columns . ') values (' . $placeholders . ')';

    if ($self->{debug_on}) {
        $self->sql_log($sql."\n".Dumper(\%data));
        $self->timer_start();
    }

    my $sth = $dbh->prepare($sql);
    my $i = 1;

    foreach my $key (keys %data) {
        $sth->bind_param($i++, $data{$key});
    }

    #first get the sequence
    my $result = 0;
    if ($sequence_is_needed) {
        $sth->execute() || die $dbh->errstr . "\n$sql";
        $result = $primary_key_value;
    } else {
        $result = $sth->execute() || die $dbh->errstr . "\n$sql";
        if (defined($self->get_insert_id())) {
            $result = $self->get_insert_id();
        }
    }


    #print STDERR "got primary key '$primary_key'\n";
    if ($self->{debug_on}) {
        $self->timer_log();
    }
    return $result;

}

sub sqlInsertReplace {
    # CKIM 2001-12-20: Issues a sqlDelete followed by a sqlInsert.
    # Must define a %params hash with the following:
    # - table     => name of the table
    # - data      => { hash containing the data elements }
    # - keyfields => [ list of key fields ]

    my ($self, %params) = @_;
    my $table = $params{table};
    my %data = %{$params{data}};
    my @keyfields = @{$params{keyfields}};

    #print STDERR "CKIM DEBUG ================= sqlInsertReplace table=$table\n";

    my $delete_condition;
    for (my $i; $keyfields[$i]; $i++) {
        my $key = $keyfields[$i];
        $delete_condition .= "$key='$data{$key}'";
        if ($keyfields[$i+1]) {
            $delete_condition .= " AND ";
        }
    }
    my $result = $self->sqlDelete($table, $delete_condition);
    my $result2 = $self->sqlInsert($table, %data);
    #print STDERR "sqlDelete table=$table condition=\"$delete_condition\"\nresult=$result\nsqlInsert result=$result2\n";
}


#Issues a sqlUpdate if possible, otherwise issues a sqlInsert
sub sqlInsertUpdate {
    my($self, $table, $where, %data)=@_;


    my ($exists) = $self->sqlSelect('count(*)', $table, $where);

    if ($exists) {
        return $self->sqlUpdate($table, $where, %data);
    } else  {
        return $self->sqlInsert($table, %data);
    }
}

sub sqlDelete {
    my($self, $fromtable, $condition)=@_;
    my $sql;

    my $dbh = $self->get_dbh();

    if($condition){
    $sql = "DELETE from $fromtable WHERE $condition";
    }else{
    $sql = "DELETE from $fromtable";
    }

    if ($self->{debug_on}) {
        $self->sql_log($sql);
        $self->timer_start();
    }

    if(!$dbh->do($sql)) {
        my $err = $dbh->errstr;
        my $errormsg = "sqlDelete: SQL statement:<br />$sql<br />\nError: $err<br />\nat $0";
        ErrorHandling->printError($errormsg);
        exit(-1);
    }
    if ($self->{debug_on}) {
        $self->timer_log();
    }
}

sub sqlUpdate {


    my($self, $table,$where ,%data)=@_;

    $self->validate_data(
        table       => $table,
        parameters  => \%data,
    );


    my $dbh = $self->get_dbh();

    my $sql="UPDATE $table SET";
    foreach (keys %data) {

    if (/^-/) {
        s/^-//;
        $sql .= "\n  $_ = $data{-$_} " . ",";
    } else {

        $sql .= "\n  $_ = " . $dbh->quote($data{$_}).",";
    }

    }

    chop($sql);

    $sql .= "\nWHERE $where\n";


    if ($self->{debug_on}) {
        $self->sql_log($sql);
        $self->timer_start();
    }

    if(!$dbh->do($sql)) {
        my $err = $dbh->errstr;
        my $errormsg = "sqlUpdate: SQL statement:<br /><br />\n$sql<br /><br />\nError: $err\n<br /><br />\nat $0";
        ErrorHandling->printError($errormsg);
        exit(-1);
    }
    if ($self->{debug_on}) {
        $self->timer_log();
    }
    return;
}

sub quote {
    my $self = shift;
    my $string = shift;

    my $dbh = $self->get_dbh();

    return $dbh->quote($string);
}

sub xsqlSimple {
    my $self = shift;
    my ($sql)=@_;

    return (0, "No sql query to execute!") if (!$sql);

    my $dbh = $self->get_dbh();

    if ($self->{debug_on}) {
        $self->sql_log($sql);
        $self->timer_start();
    }

    my ($c)=$dbh->prepare($sql) or die "Sql has gone to hell\n";
    if(not ($c->execute())) {
        my $err=$dbh->errstr;
        my $errormsg = "###Error : sqlSelect: SQL statement:$sql\n\nError: $err\nat $0";
        return (-1, $errormsg);
    }
    my @r;
    @r=$c->fetchrow() if uc($sql) =~ m/^\s*SELECT/;
    $c->finish();
    if ($self->{debug_on}) {
        $self->timer_log();
    }
    return (0, @r);
}

sub sqlSaveAppObject {
    my ($self, $object) = @_;
    my $pk_name = $object->get_pkey_name();
    my $pk_value = $object->get_pkey_value();
    my $result = 1;
    # save object itself first
    if (defined($pk_value) && $pk_value ne '') {
        $result &&= $self->_sqlUpdateAppObject($object);
    } else {
        $result &&= $self->_sqlInsertAppObject($object);
    }
    return $result if !$result;
    # now try to save it's children
    $pk_value = $object->get_pkey_value();
    foreach my $field (keys %{$object}) {
#        eval {
            my $value = $object->get_value($field);
            # if value is reference type - try to set value of
            # relation field and then save value
            if (ref $value) {
                $value->set_value($pk_name, $pk_value);
                $value->save();
            }
#        };
    }
    return $result;
}

sub sqlDeleteAppObject {
    my ($self, $object) = @_;
    my $pk_name = $object->get_pkey_name();
    my $pk_value = $object->get_pkey_value();
    return undef unless (defined($pk_value) && $pk_value ne '');

    # first try to delete it's children
    foreach my $field (keys %{$object}) {
        eval {
            my $value = $object->get_value($field);
            if (ref $value) { # if value is reference type - try to save it
                $value->set_value($pk_name, $pk_value);
                $value->delete();
            }
        };
    }

    my $dbh = $self->get_dbh();
    my $table_name = $object->get_table_name();
    my $pkey_name =  $object->get_pkey_name();

    my $sql="DELETE FROM $table_name WHERE $pkey_name = '$pk_value'\n";
    if(!$dbh->do($sql)) {
        my $err = $dbh->errstr;
        my $errormsg = "sqlDeleteObject: SQL statement:<br /><br />$sql<br /><br />\nError: $err<br /><br />\nat $0";
        ErrorHandling->printError($errormsg);
        exit(0);
    } else {
        return $pk_value;
    }
}

sub _sqlUpdateAppObject {
    my ($self, $object) = @_;

    my $dbh = $self->get_dbh();
    my $table = $object->get_table_name();
    my $pkey_value =  $object->get_pkey_value();
    my $pkey_name =  $object->get_pkey_name();
    my $fields = $object->get_fields();
    my $where = "$pkey_name = ";
    if ($fields->{$pkey_name}->{'type'} eq 'number') {
        $where .= "$pkey_value";
    } else {
        $where .= "".$dbh->quote($pkey_value);
    }

    my $sql = "UPDATE $table SET";
    foreach my $field_name (keys %{$fields}) {
        next if (($field_name eq $pkey_name) || !defined($object->get_value($field_name)));
        if ($fields->{$field_name}->{'type'} eq 'number') {
            $sql .= "\n  $field_name = " . $object->get_value($field_name) .",";
        } else {
            $sql .= "\n  $field_name = " . $dbh->quote($object->get_value($field_name)) .",";
        }
    }
    chop($sql);
    $sql .= "\nWHERE $where\n";

    #      print STDERR "in sqlUpdate, doing sql: $sql\n";
    if(!$dbh->do($sql)) {
        my $err = $dbh->errstr;
        my $errormsg = "_sqlUpdateObject: SQL statement:<br /><br />\n$sql<br /><br />\nError: $err\n<br /><br />\nat $0";
        ErrorHandling->printError($errormsg);
        exit(-1);
    }
    return $object->get_pkey_value();
}

sub _sqlInsertAppObject {
    my ($self, $object) = @_;

    my $dbh = $self->get_dbh();
    my $table_name = $object->get_table_name();
    my $pkey_name =  $object->get_pkey_name();
    my $fields = $object->get_fields();

    my ($names, $values);

    foreach my $field_name (keys %{$fields}) {
        next if (($field_name eq $pkey_name) || !defined($object->get_value($field_name)));
        if ($fields->{$field_name}->{'type'} eq 'number') {
            $values .="\n  ".$object->get_value($field_name).",";
        } else {
            $values .= "\n  " . $dbh->quote($object->get_value($field_name)) . ",";
        }
        $names .= "$field_name,";
    }
    chop($names);
    chop($values);

    my $sql="INSERT INTO $table_name ($names) VALUES($values)\n";

    #      print STDERR "got sql insert query: $sql\n";
    if(!$dbh->do($sql)) {
        my $err=$dbh->errstr;
        my $errormsg = "_sqlInsertObject: SQL statement:<br /><br />$sql<br /><br />\nError: $err<br /><br />\nat $0";
        ErrorHandling->printError($errormsg);
        exit(0);
    } else {
        return $object->set_pkey_value($dbh->{'mysql_insertid'});
    }
}

sub sql_log {
    my ($self, @messages) = @_;

    my $loghandle = $self->get_log_handle();
    if ($loghandle) {
        my $logtext = ('-'x70)."\n";
        $logtext .= join('', @messages)."\n";
            my $i = 0;
            if ($self->{debug_caller_info}) {
                while ((my @caller_info) = caller($i++) ) {
                    $logtext .= "caller information: ";
                    foreach my $value (@caller_info) {
                        if ($value) {
                            $logtext .= $value . " ";
                        }
                    }
                    $logtext .= "\n";
                }
            }
            print $loghandle $logtext;
        close ($loghandle);
    }
}


sub timer_start {
    my $self = shift;
    $TIMESTAMP = [gettimeofday];
}

sub timer_log {
    my $self = shift;
    my $fh = $self->get_log_handle();
    if ($fh) {
        print $fh "-- Timestamp: ".localtime(time())."; Elapsed time: ", tv_interval ( $TIMESTAMP ), "\n";
        close ($fh);
    }
}

sub get_log_handle {
    my $self = shift;

    my $logfile = MetadotConfig->get_param_value('sql_logfile');
    if ($logfile) {
        if (open (SQL_LOG, ">>$logfile")) {
            return \*SQL_LOG;
        } else {
            print STDERR "Could not open sql log file '$logfile'\n";
            return undef;
        }
    }
}

# given a data hash, check the schema to see if any values are illegal.
# make the necessary corrections. This exists because Oracle barfs if you
# try to set a field with a value that is too large

# return a status code of 1 if there is an error,
# followed by a list of error messages
sub validate_data {
    return undef;
}


# returns the function or keyword that gets the current time
sub db_now_function {
    my $class = shift;
    die ("Abstract method");
}


# returns an expression that will calculate the number of seconds between two datetime
sub db_timediff_function {
    my $class = shift;
    my $time1 = shift;
    my $time2 = shift;

    die ("Abstract method");
}


1;


__END__

=head1 NAME

DBAccess::DBiObj - "Abstract" class to support databases that have a
DBI driver.

=head1 SYNOPSIS

    use DBAccess::DBIObj;

    $dbobj->get_dbh();
    $dbobj->sqlSimple();
    $dbobj->sqlSelect();
    $dbobj->sqlSelectMany();
    $dbobj->sqlInsert();
    $dbobj->sqlDelete();
    $dbobj->sqlUpdate();
    $dbobj->quote();
    $dbobj->xsqlSimple();

=head1 DESCRIPTION

The DBIObj class is "abstract" and is not intended to be instantiated
directly.  It implements a set of methods that help to provide an SQL
query interface. Some general db access functionality can be supported
at the DBI level and is implemented in this class.  Other
functionality, such as the creation of a database handle, or the way
locking functionality is implemented, can differ
across specific databases and is required to be overridden by
subclasses for specific DBI-enabled databases, such as MySQL or
Oracle.

Abstract method; should be implemented by subclass:

    $dbobj->get_dbh();

Executes a valid SQL statement:

    $dbobj->sqlSimple($sqlstmt);

Execute an sql statement by constructing it from the given 'select',
'from', and 'where' clauses as separate strings.  The $other string is
appended on the end. Returns a list comprising the first row of the results.

    @r = $dbobj->sqlSelect($select, $from, $where, $other);

Execute an sql statement by constructing it from the given 'select',
'from', and 'where' clauses as separate strings.  The $other string is
appended on the end. Returns a statement handle that can be used to
obtain the query result data via methods like $sth->fetchrow().

    $sth = $dbobj->sqlSelectMany($select, $from, $where, $other);

Implements an sql insert statement.  $table is the table name, %data
must contain name/value pairs, where the names correspond to field
names, and the corresponding values are the data to be inserted.

    $dbobj->sqlInsert($table, %data);

Implements an sql delete statement constructed as follows:
DELETE from $fromtable WHERE $condition.

    $dbobj->sqlDelete($fromtable, $condition);

Implements an sql update statement. $table is the table to update.
$where contains the 'where' clause.  The %data hash is used to
generate the 'set' clause.  If the first character of a key in the
hash is a "-", then the value is not quoted in the 'set' clause.

    $dbobj->sqlUpdate($table, $where, %data);

Calls $dbh->quote($string) on the given string, where $dbh is a
database handle.

    $dbobj->quote($string);

Executes the given $sqlstmnt. Returns status and the first row
of results if the query was a 'select'.  If the query was not a
'select', the list is empty.  Returned status is 0 if ok, -1 if
error.  If -1, second arg returned is an error message rather than a
result list.

    ($status, @r) = $dbobj->xsqlSimple($sqlstmnt);

=cut
