###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package DBAccess::Oracle;
use strict;
use Data::Dumper;

use MetadotConfig;
use DBAccess::DBIObj;
use DBAccess::OracleSequence;
use DBD::Oracle qw(:ora_types);

use vars qw(@ISA);
@ISA=qw(DBAccess::DBIObj);



sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    # If we have to set the ORACLE_HOME environment variable, do it here
    my $oracle_home = MetadotConfig->get_param_value('oracle_home');
    if ($oracle_home) {
        #print STDERR "Setting ORACLE_HOME to '$oracle_home'\n";
        $ENV{ORACLE_HOME} = $oracle_home;
    }


    my $self = $class->SUPER::new;
    $self->{server_name} =  shift;
    $self->{db_name} =  shift;
    $self->{db_user} =  shift;
    $self->{db_pass} =  shift;
    $self->{db_host} = shift;
    $self->{db_sid} = shift;
    bless ($self, $class);
    #$self->instantiate_sequence_objects();

    ## hash will store all tables that have a primary key and/or sequence
    $self->set_tables_hash();

    return $self;
}

sub get_next_primary_key {
    my $self = shift;
    my $table = shift;

    return $self->{tables}->{$table}->{seq}->nextval();
}

sub is_sequence_needed {
    my $self = shift;
    my $table = shift;
    return ($self->{tables}->{$table}->{seq});
}

sub get_primary_key_field {
    my $self = shift;
    my $table = shift;
    return $self->{tables}->{$table}->{pk};
}

sub set_tables_hash {
    my $self = shift;
    my $extra_tables_ref = shift;


    ### Creates structure of tables hash that stores tables
    ### that contain a primary key and/or a sequence. Table
    ### key 'seq' will be assigned an actual sequence object
    ### if equal to 1; otherwise, assigned undef.
    my $tables =
        {
         users             => {
                                  pk  => 'userid',
                                  seq => 1,
                              },
         subscription      => {
                                  pk  => 'sid',
                                  seq => 1,
                              },
         sessions          => {
                                  pk  => 'sessionid',
                                  seq => 0,
                              },
         poll_vote         => {
                                  pk  => 'poll_vote_id',
                                  seq => 1,
                              },
         poll_response     => {
                                  pk  => 'poll_response_id',
                                  seq => 1,
                              },
         permissions       => {
                                  pk  => 'id',
                                  seq => 0,
                              },
         params            => {
                                  pk  => 'params_id',
                                  seq => 1,
                              },
         mypage            => {
                                  pk  => 'mpid',
                                  seq => 1,
                              },
         message           => {
                                  pk  => 'mid',
                                  seq => 1,
                              },
         mypage            => {
                                  pk  => 'mpid',
                                  seq => 1,
                              },
         instance          => {
                                  pk  => 'iid',
                                  seq => 1,
                              },
         grp               => {
                                  pk  => 'grp_id',
                                  seq => 1,
                              },
         channelitem       => {
                                  pk  => 'ciid',
                                  seq => 1,
                              },
         channel_category  => {
                                  pk  => 'ccid',
                                  seq => 1,
                              },
         channel           => {
                                  pk  => 'cid',
                                  seq => 1,
                              },
         addressbook       => {
                                  pk  => 'ciid',
                                  seq => 0,
                              },
         timestamps        => {
                                  pk  => 'timestamp_id',
                                  seq => 1,
                              },
         answer            => {
                                  pk  => 'aid',
                                  seq => 1,
                              },
         customer            => {
                                  pk  => 'customer_id',
                                  seq => 1,
                              },
         event_log            => {
                                  pk  => 'unique_id',
                                  seq => 1,
                              },
         extended_user        => {
                                  pk  => 'userid',
                                  seq => 0,
                              },
         global_param   => {
                                  pk  => 'global_param_name',
                                  seq => 1,
                              },
         profile            => {
                                  pk  => 'profile_id',
                                  seq => 1,
                              },
         profile_global_param            => {
                                  pk  => 'profile_global_param_id',
                                  seq => 1,
                              },
         questions            => {
                                  pk  => 'qid',
                                  seq => 1,
                              },
         server            => {
                                  pk  => 'server_id',
                                  seq => 1,
                              },
         server_creator            => {
                                  pk  => 'server_creator_id',
                                  seq => 1,
                              },
         server_global_param       => {
                                  pk  => 'server_global_param_id',
                                  seq => 1,
                              },
         server_status            => {
                                  pk  => 'server_status_id',
                                  seq => 1,
                              },
         strategy            => {
                                  pk  => 'strategy_id',
                                  seq => 1,
                              },
         timestamps            => {
                                  pk  => 'timestamp_id',
                                  seq => 1,
                              },
         theme                 => {
                                  pk  => 'theme',
                                  seq => 1,
                              },
        };

        if (defined($extra_tables_ref) && scalar keys %$extra_tables_ref) {
            my %all_tables = (%$tables, %$extra_tables_ref);
            $self->{tables} = \%all_tables;
        } else {
            $self->{tables} = $tables;
        }

        ## initialize sequence object for hash
        $self->instantiate_sequence_objects();
}

sub instantiate_sequence_objects {
    my $self = shift;
    my $dbh = $self->get_dbh();
    #note:  grpmembers, lists, gizmotoolitem, permissions, and gizmoitemparam do not have sequences, since they are bridge tables
    foreach my $table (keys(%{$self->{tables}})) {
        unless($self->{tables}->{$table}->{seq}) {
            $self->{tables}->{$table}->{seq} = undef;
            next;
        }
        my $table_sequence_name = uc($table . '_seq');
        #my $sequence =  new DBAccess::OracleSequence($dbh, "$table_sequence_name");
        $self->{tables}->{$table}->{seq} = new DBAccess::OracleSequence($dbh, "$table_sequence_name");
    }

}


#return or create dbh
#this must be here so that we can properly set the date and readlength format
# if we call the super to get a dbh, and then apply the date and readlength format,
# we will end up setting those values each time even for cached database handles
sub get_dbh {
    my $self = shift;

    my $server_name = $self->{server_name};
    my $dbh = $self->{DBH_CACHE}->{$server_name}->{DBH};
    # if the handle for this server is already in the hash, return it
    if (defined $dbh && $dbh->ping() && !$dbh->state()) {
        return $dbh;
    }

    my $connect_string;
    if ($self->{db_sid}) {
        $connect_string = $self->{db_name} . ':host=' . $self->{db_host} . ';sid=' . $self->{db_sid};
    } else {
        $connect_string = "$self->{db_name}:";
    }
    $dbh = DBI->connect($connect_string, $self->{db_user}, $self->{db_pass});

    if(!$dbh){
        my $msg = "Unable to connect to SQL Server: <br />\n";
        $msg .= $connect_string . '; db_user: ' . $self->{db_user} . '; db_password: (not displayed)';
        if($ENV{SCRIPT_NAME}){
            ErrorHandling->printError($msg);
        } else {
            print "$msg\n";
        }
    } else {
        #print STDERR "Oracle dbh initialized as $dbh\n";
        $dbh->{LongReadLen} = '16000000';
        my $oracle_date_initialization_sql = "alter session set NLS_DATE_FORMAT = 'YYYY-MM-DD hh24:mi:ss'";
        $dbh->do($oracle_date_initialization_sql);

         #print STDERR "storing new dbh for server $server_name\n";
        # put the fresh handle in the cache
        $self->{DBH_CACHE}->{$server_name}->{DBH} = $dbh;

        $self->_init_dbh($dbh);
    }
    return $dbh;
}

sub close_dbh {
    my $self = shift;
    my $dbh = $self->get_dbh(@_);
    my $server_name = $self->{server_name};
    my $result;

    if (defined $dbh) {
        delete $self->{DBH_CACHE}->{$server_name}->{DBH};
        $result = $dbh->disconnect();
    }
    return $result;
}


# select the count of the set of rows with multiple distinct fields
sub sqlSelectCountDistinct {
    my($self, $select, $from, $where, $other)=@_;

    my $dbh = $self->get_dbh();

    my $sql="SELECT DISTINCT $select ";
    $sql.="FROM $from " if $from;
    $sql.="WHERE $where " if $where;
    $sql.="$other" if $other;

    $sql = "SELECT COUNT(*) FROM ($sql)";

    if ($self->{debug_on}) {
        $self->sql_log($sql);
        $self->timer_start();
    }

    my $sth = $dbh->prepare($sql);
    if($sth->execute()) {
        if ($self->{debug_on}) {
            $self->timer_log();
        }
        my ($r) = $sth->fetchrow();
        $sth->finish();

        return $r;
    } else {
        $sth->finish();
        my $err = $dbh->errstr;
        my $errormsg = "sqlSelectCountDistinct: SQL statement:<br /><br />$sql<br /><br />\nError: $err<br /><br />\nat $0";
        ErrorHandling->printError($errormsg);

        return undef;
        die "could not execute: $sql\n";
    }
}


sub sqlInsert {
    my($self, $table, %data)=@_;

    $self->validate_data(
        table       => $table,
        parameters  => \%data,
    );


    use Data::Dumper;
    $self->sql_log("INSERT into $table, data is: ", Dumper(\%data), "\n");

    #special case for instance and permissions, since instance has more than one clob, and permissions has a blob
    if ($table eq 'instance') {
        # WEBEPU not using clobs
        #my $iid = $self->_insert_into_instance(%data);
        #return $iid;
    } elsif ($table eq 'permissions') {
        $self->_insert_into_permissions(%data);
        return;
    } elsif ($table eq 'message') {
        my $mid = $self->_insert_into_message(%data);
        return $mid;
    } elsif ($table eq 'channel') {
        my $cid = $self->_insert_into_channel(%data);
        return $cid;
    } elsif ($table eq 'channelitem') {
        my $cid = $self->_insert_into_channelitem(%data);
        return $cid;
    }

    return $self->SUPER::sqlInsert($table, %data);
}

sub _insert_into_permissions {
    my $self = shift;
    my %data = @_;
    my $dbh = $self->get_dbh();

    my $sth = $dbh->prepare("INSERT INTO permissions (id, isa, permissions) VALUES (?,?,?)");

    $sth->bind_param(1, $data{id});
    $sth->bind_param(2, $data{isa});
    $sth->bind_param(3, $data{permissions}, {ora_type => ORA_BLOB, ora_field => 'permissions' });
    $sth->execute() || die $dbh->errstr;
}

sub _insert_into_channelitem {
    my $self = shift;
    my %data = @_;
    my $dbh = $self->get_dbh();

    my $ciid = $self->get_next_primary_key('channelitem');

    my $sql = "INSERT INTO channelitem (ciid, cid, newsversion, description, url, creationdate, urgent, name, mid) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)";
    #print STDERR "_insert_into_channelitem (ciid=$ciid) will execute:\n$sql\n";

    my $sth = $dbh->prepare($sql);
    $sth->bind_param(1, $ciid);
    $sth->bind_param(2, $data{cid});
    $sth->bind_param(3, $data{newsversion});
    $sth->bind_param(4, $data{description}, {ora_type => ORA_CLOB, ora_field => 'description'});
    $sth->bind_param(5, $data{url});
    $sth->bind_param(6, $data{creationdate});
    $sth->bind_param(7, $data{urgent});
    $sth->bind_param(8, $data{name});
    $sth->bind_param(9, $data{mid});
    $sth->execute() || die $dbh->errstr;

    return $ciid;
}

sub _insert_into_message {
    my $self = shift;
    my %data = @_;
    my $dbh = $self->get_dbh();

    my $mid = $self->get_next_primary_key('message');

    my $sql = "INSERT INTO message (mid, iid, parentmid, userid, name, description, creationdate, updatedate, updatedby, status, priority, email, authorname, authoremail, recipient_uid, isa) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
    #print STDERR "_insert_into_message (mid=$mid) will execute:\n$sql\n\n with data\n",Dumper \%data;

    my $sth = $dbh->prepare($sql);
    $sth->bind_param(1, $mid);
    $sth->bind_param(2, $data{iid});
    $sth->bind_param(3, $data{parentmid});
    $sth->bind_param(4, $data{userid});
    $sth->bind_param(5, $data{name});
    $sth->bind_param(6, $data{description});
    $sth->bind_param(7, $data{creationdate});
    $sth->bind_param(8, $data{updatedate});
    $sth->bind_param(9, $data{updatedby});
    $sth->bind_param(10, $data{status});
    $sth->bind_param(11, $data{priority});
    $sth->bind_param(12, $data{email});
    $sth->bind_param(13, $data{authorname});
    $sth->bind_param(14, $data{authoremail});
    $sth->bind_param(15, $data{recipient_uid});
    $sth->bind_param(16, $data{isa});
    $sth->execute() || die $dbh->errstr;

    return $mid;
}

sub _insert_into_instance {
    my $self = shift;
    my %data = @_;
    my $dbh = $self->get_dbh();

    my $sql = "insert into instance (iid, parentiid, userid, isa, name, description, cool, showchilds";
    $sql .= ", displaylook, url, keywords, instanceorder, showfrom, creationdate";
    $sql .= ", updatedate, nbitems, nbreads, updatedby, hiddencomment, lastpost";
    $sql .= ", file1, createdby, longdescription, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10";
    $sql .= ", c1, c2, c3, c4, c5, d1, d2, d3, d4, d5, i1, i2, i3, i4, i5, file1_size, location_id, short_name, showend)";
    $sql .= " values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

#    $self->sql_log($sql);
    my $iid = $self->get_next_primary_key('instance');

#    $self->sql_log(">>>>".$data{name}.$data{location_id});
    my $sth = $dbh->prepare($sql);

        $sth->bind_param(1, $iid);
        $sth->bind_param(2, $data{parentiid});
        $sth->bind_param(3, $data{userid});
        $sth->bind_param(4, $data{isa});
        $sth->bind_param(5, $data{name});
        $sth->bind_param(6, $data{description}, {ora_type => ORA_CLOB, ora_field => 'description' });
        $sth->bind_param(7, $data{cool});
        $sth->bind_param(8, $data{showchilds});
        $sth->bind_param(9, $data{displaylook});
        $sth->bind_param(10, $data{url});
        $sth->bind_param(11, $data{keywords});
        $sth->bind_param(12, $data{instanceorder});
        $sth->bind_param(13, $data{showfrom});
        $sth->bind_param(14, $data{creationdate});
        $sth->bind_param(15, $data{updatedate});
        $sth->bind_param(16, $data{nbitems});
        $sth->bind_param(17, $data{nbreads});
        $sth->bind_param(18, $data{updatedby});
        $sth->bind_param(19, $data{hiddencomment});
        $sth->bind_param(20, $data{lastpost});
        $sth->bind_param(21, $data{file1});
        $sth->bind_param(22, $data{createdby});
        $sth->bind_param(23, $data{longdescription});
        $sth->bind_param(24, $data{t1});
        $sth->bind_param(25, $data{t2});
        $sth->bind_param(26, $data{t3});
        $sth->bind_param(27, $data{t4});
        $sth->bind_param(28, $data{t5});
        $sth->bind_param(29, $data{t6});
        $sth->bind_param(30, $data{t7});
        $sth->bind_param(31, $data{t8});
        $sth->bind_param(32, $data{t9});
        $sth->bind_param(33, $data{t10});
        $sth->bind_param(34, $data{c1});
        $sth->bind_param(35, $data{c2});
        $sth->bind_param(36, $data{c3});
        $sth->bind_param(37, $data{c4});
        $sth->bind_param(38, $data{c5});
        $sth->bind_param(39, $data{d1});
        $sth->bind_param(40, $data{d2});
        $sth->bind_param(41, $data{d3});
        $sth->bind_param(42, $data{d4});
        $sth->bind_param(43, $data{d5});
        $sth->bind_param(44, $data{i1});
        $sth->bind_param(45, $data{i2});
        $sth->bind_param(46, $data{i3});
        $sth->bind_param(47, $data{i4});
        $sth->bind_param(48, $data{i5});
        $sth->bind_param(49, $data{file1_size});
        $sth->bind_param(50, $data{location_id});
        $sth->bind_param(51, $data{short_name});
        $sth->bind_param(52, $data{showend});
    $sth->execute() || die $dbh->errstr . "\n$sql";
    return $iid;

}

sub _insert_into_channel{
    my $self = shift;
    my %data = @_;
    my $dbh = $self->get_dbh();

    my $sql = "insert into channel (cid, isa, name, description, url, " .
              "fetchevery, iid, lastfetched, httpstatus, nbitems, version, " .
              "updatedate, creationdate, homeurl, lastprocessed, html, ccid) " .
              " values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

    my $cid = $self->get_next_primary_key('channel');


    my $sth = $dbh->prepare($sql);

        $sth->bind_param(1, $cid);
        $sth->bind_param(2, $data{isa});
        $sth->bind_param(3, $data{name});
        $sth->bind_param(4, $data{description}, {ora_type => ORA_CLOB, ora_field => 'description' });
        $sth->bind_param(5, $data{url});
        $sth->bind_param(6, $data{fetchevery});
        $sth->bind_param(7, $data{iid});
        $sth->bind_param(8, $data{lastfetched});
        $sth->bind_param(9, $data{httpstatus});
        $sth->bind_param(10, $data{nbitems});
        $sth->bind_param(11, $data{version});
        $sth->bind_param(12, $data{updatedate});
        $sth->bind_param(13, $data{creationdate});
        $sth->bind_param(14, $data{homeurl});
        $sth->bind_param(15, $data{lastprocessed});
        $sth->bind_param(16, $data{html});
        $sth->bind_param(17, $data{ccid});

    $sth->execute() || die $dbh->errstr . "\n$sql";
    return $cid;

}

sub sqlUpdate {
    my($self, $table, $where, %data)=@_;

    # Oracle uses bound parameters, so we should just discard the ^- construct
    # that is used elsewhere, to indicate something is a literal
    foreach my $key (keys(%data)) {
        if ($key =~ /^-/) {
            my $value = $data{$key};
            delete $data{$key};
            $key =~ s/^-//;
            $data{$key} = $value;
        }
    }


    $self->validate_data(
        table       => $table,
        parameters  => \%data,
    );


    if ($table eq 'instance') {
        # WEBEPU not using clobs
        #$self->_update_into_instance($where, %data);
        #return;
    } elsif ($table eq 'permissions') {
        $self->_update_into_permissions($where, %data);
        return;
    } elsif ($table eq 'channel') {
        $self->_update_into_channel($where, %data);
        return;
    } elsif ($table eq 'channelitem') {
    $self->_update_into_channelitem($where, %data);
    return;
    }



    my $dbh = $self->get_dbh();

    my $things_to_update = join (' = ?, ', keys %data);
    my $sql = 'update ' . $table . ' set ' . $things_to_update . ' = ? where ' . $where;

    #print STDERR "sqlUpdate will execute:\n$sql\n\n";
    # print STDERR "with data:\n", Dumper(\%data);

    my $sth = $dbh->prepare($sql);

    my $i = 1;
#    $self->sql_log($sql);
    foreach my $key (keys %data) {
        $sth->bind_param($i++, $data{$key});
    }
    $sth->execute();
    return;
}

sub _update_into_permissions {
    my ($self, $where, %data) = @_;
    my $dbh = $self->get_dbh();
    my $things_to_update = join (' = ?, ', keys %data);
    my $sql = 'update permissions set ' . $things_to_update . ' = ? where ' . $where;
    my $i = 1;
    my $sth = $dbh->prepare($sql);

    foreach my $key (keys %data) {
        if ($key eq 'permissions') {
        $sth->bind_param($i++, $data{$key}, {ora_type => ORA_BLOB, ora_field => 'permissions' });
        } else {
            $sth->bind_param($i++, $data{$key});
        }
    }
    $sth->execute();
}

sub _update_into_instance {
    my ($self, $where, %data) = @_;
    my $dbh = $self->get_dbh();
    my $things_to_update = join (' = ?, ', keys %data);
    my $sql = 'update instance set ' . $things_to_update . ' = ? where ' . $where;
    my $sth = $dbh->prepare($sql);
    my $need_to_add_clob = 1;
    my $i = 1;
    foreach my $key (keys %data) {
        #print "key is $key\n";
        if ($need_to_add_clob &&
                (
                   ($key eq 'description') ||
                   ($key eq 'longdescription')
                 )
        ) {  #end of conditional on the if, start of block to be executed if the if is true.
            $sth->bind_param($i++, $data{$key}, {ora_type => ORA_CLOB, ora_field => $key });
            $need_to_add_clob = 0;
        } else {
            $sth->bind_param($i++, $data{$key});
        }
    }
    $sth->execute();
}

sub _update_into_channelitem {
    my ($self, $where, %data) = @_;
    my $dbh = $self->get_dbh();
    my $things_to_update = join (' = ?, ', keys %data);
    my $sql = "update channelitem set " . $things_to_update . " = ? where " . $where;
    my $sth = $dbh->prepare($sql);
    my $need_to_add_clob = 1;
    my $i = 1;
    foreach my $key (keys %data) {
    if ($need_to_add_clob &&
        ($key eq 'description') ) {
        $sth->bind_param($i++, $data{$key}, {ora_type => ORA_CLOB, ora_field => $key });
        $need_to_add_clob = 0;
    }
    else {
            $sth->bind_param($i++, $data{$key});
        }
    }
    $sth->execute();
}


sub _update_into_channel{
    my ($self, $where, %data) = @_;
    my $dbh = $self->get_dbh();
    my $things_to_update = join (' = ?, ', keys %data);
    my $sql = "update channel set " . $things_to_update . " = ? where " . $where;
    my $sth = $dbh->prepare($sql);
    my $need_to_add_clob = 1;
    my $i = 1;
    foreach my $key (keys %data) {
        #print "key is $key\n";
        if ($need_to_add_clob &&
                (
                   ($key eq 'description') ||
                   ($key eq 'html')
                 )
        ) {  #end of conditional on the if, start of block to be executed if the if is true.
            $sth->bind_param($i++, $data{$key}, {ora_type => ORA_CLOB, ora_field => $key });
            $need_to_add_clob = 0;
        } else {
            $sth->bind_param($i++, $data{$key});
        }
    }
    $sth->execute();
}


sub get_table_id {
        my $self = shift;
        my $table_name = shift;

        my $table_sequence_name = uc($table_name."_seq");
        my $id = $self->get_next_primary_key($table_name);

        return $id;
}


sub db_now_function {
    return 'sysdate';
}


# returns an expression that will calculate the number of seconds between two datetime
sub db_timediff_function {
    my $class = shift;
    my $time1 = shift;
    my $time2 = shift;

    return "($time1-$time2)*86400";
}



### All the table / sequence / key field stuff will eventually be replaced by a DBSchema object.
### Please add any new schema-related functionality to this schema

use DBAccess::DBSchema;

# 'schemae' is the plural of 'schema'. We may need to access multiple schemae
my %SCHEMAE = ();

sub get_schema {
    my ($self, %args) = @_;

    unless (%SCHEMAE) {
        $self->initialize_schemae();
    }

    if ($args{name}) {
        return $SCHEMAE{$args{name}};
    } else {
        return $SCHEMAE{metadot};
    }
}


# currently, the schema is only used to define maxmimum widths of certain fields
sub initialize_schemae {
    my $self = shift;
    my $mdot = DBAccess::DBSchema->new( name=>'metadot');
    $SCHEMAE{metadot} = $mdot;

    #print STDERR "initialize_schema returns $mdot\n";
    return $mdot;
}


sub existsTable {
    my ($self, $table) = @_;

    # get the table info via DBI
    my $schema  = uc($self->{db_user});
    my $type    = 'TABLE';

    # capitalize the table name for oracle
    if ($table) {
        $table = uc($table);
    }

    my $sth = $self->get_dbh()->table_info({
        TABLE_SCHEM => $schema,
        TABLE_TYPE  => $type,
        TABLE_NAME  => $table,
    });

    my $found;
    if ($sth) {
        $found = 1;
        $sth->finish();
    }

    return ($found);
}


# probes a table in the database, returns a Table object complete with the
# associated Column objects. In case there is more than one match, it
# returns multiple tables
sub probe_table {
    my $self = shift;
    my $table = shift;
    my @results;

    # get the table info via DBI
    my $schema  = uc($self->{db_user});
    my $type    = 'TABLE';

    # capitalize the table name for oracle
    if ($table) {
        $table = uc($table);
    }

    my $sth = $self->get_dbh()->table_info({
        TABLE_SCHEM => $schema,
        TABLE_TYPE  => $type,
        TABLE_NAME  => $table,
    });

    if ($sth) {
        while (my $table_hash = $sth->fetchrow_hashref()) {
            my $table_name = $table_hash->{table_name};
            #print STDERR "Got TABLE '$table_name'\n";
            my $table_obj = DBAccess::DBSchema::Table->new( \%$table_hash );

            my $column_sth = $self->get_dbh()->column_info( $schema, $schema, $table_name, '%');
            if ($column_sth) {
                while (my $col_hash = $column_sth->fetchrow_hashref()) {
                    #print STDERR "Got COLUMN $col_hash->{column_name}\n";
                    my $col_obj = DBAccess::DBSchema::Column->new( \%$col_hash );
                    $table_obj->set_column($col_obj);
                    #print STDERR "Added column $col_obj to table $table_obj\n";
                }
                $column_sth->finish();
            }

            #print STDERR "Probed table object:\n",Dumper $table_obj;
            push @results, $table_obj;
        }
        $sth->finish();
    }
    #print STDERR "probe_table '$table' returns\n",Dumper \@results;
    return @results;
}


# returns
sub validate_data {
    my ($self, %args) = @_;
    my $table_name  = $args{table};
    my $parameters  = $args{parameters};
    my $schema      = $self->get_schema();

    my $status = 0;
    my @errors;

    if ($schema && $table_name && ref($parameters) && ref($parameters) ne 'ARRAY') {
        #print STDERR "\n\nValidating data for table '$table_name' against schema $schema\n============================\n";
        #print STDERR Dumper $schema;

        my @keys = keys(%$parameters);
        my $table = $schema->get_table(uc($table_name));

        unless ($table) {
            # in case the table is not found, try to probe for it in the db
            #print STDERR "Table was not stored in schema. Checking against database.\n";
            my @tables = $self->probe_table($table_name);
            foreach $table (@tables) {
                $schema->set_table($table);
            }

            # and try again to retrieve the table
            $table = $schema->get_table(uc($table_name));
            #print STDERR "Retrieved table:\n",Dumper $table;
        }

        if ($table) {

            foreach my $key (@keys) {
                my $column = $table->get_column(uc($key));
                if ($column) {
                    #print STDERR "Found column $key ($column) in schema (table $table)\n";

                    # check for excessive length
                    my $size = $column->get_size();
                    my $type_name = $column->get_type_name();

                    my $is_text = ($type_name eq 'VARCHAR2' || $type_name eq 'CLOB');
                    #print STDERR "Checking if '$type_name' is text: $is_text\n";
                    #print STDERR "size='$size'; length='".length($parameters->{$key})."; disable trim?: ".MetadotConfig->get_param_value('disable_oracle_text_trim')."\n";
                    if ($is_text && defined($size) && length($parameters->{$key}) > $size && !MetadotConfig->get_param_value('disable_oracle_text_trim')) {
                        $status = 1;
                        push @errors, "$table_name.$key ($type_name) exceeded maximum length of $size. Some data was truncated.\n";
                        $parameters->{$key} = substr($parameters->{$key}, 0, $size);
                    }


                    # check for invalid null
                    my $nullable = $column->is_nullable();
                    #print STDERR "Is column nullable? '$nullable'. value is '".$parameters->{$key}."'\n";
                    if (defined($nullable) && !$nullable && !$parameters->{$key}) {
                        $status = 1;
                        # try to fix numbers by setting to 0, text to empty string
                        if (DBAccess::DBSchema::Type->is_numeric($column->get_type_name)) {
                            if ($parameters->{$key} != 0) {
                                $parameters->{$key} = 0;
                                push @errors, "$key was null; Substituted with the number 0";
                            }
                        } elsif (DBAccess::DBSchema::Type->is_text($column->get_type_name)) {
                            $parameters->{$key} = $self->quote('');
                            push @errors, "$key was empty. Substituted with empty quotes.";
                        } else {
                            push @errors, "$key was null or empty; could not correct";
                        }
                    }
                }
            }
        }
    }

    if (@errors) {
        print STDERR "Validation completed with ",scalar(@errors)," errors.\n";
        print STDERR join("\n", @errors),"\n";
    }
    return ($status, @errors);
}



# begins a transaction
sub begin_work {
    my $self = shift;
    my $dbh = $self->get_dbh();
    $dbh->begin_work();
}

# commits a transaction
sub commit {
    my $self = shift;
    my $dbh = $self->get_dbh();
    $dbh->commit();
}


# rolls back a transaction
sub rollback {
    my $self = shift;
    my $dbh = $self->get_dbh();
    $dbh->rollback();
}


# lock a resource
sub sqlLock {
    my ($self, %args) = @_;

    my $table = $args{table};
    my $mode  = uc($args{mode}) || 'EXCLUSIVE';
    my $result;

    if ($table) {
        my $dbh = $self->get_dbh();
        my $sql = "LOCK TABLE $table IN $mode MODE";
        $dbh->do($sql);
    } else {
        print STDERR "Lock requested, but no table specified.\n";
    }

    return $result;
}


# unlock a resource
sub sqlUnlock {
    print STDERR "sqlUnlock not supported in Oracle; do a commit instead.\n";
}



1;
