###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
################################################################################
# STOCKQUOTES v1.1 - MetaDot Portal Server Gizmo Tool                          #
#                                                                              #
# created by: Pierre Renaldo                                                   #
#                                                                              #
# This tool allows a user to monitor the status of his/her prefered stocks.    #
# A user can also be alertred by email when a quote reach a define threshold.  #
# It fetches the data from the Yahoo Stocks server.                            #
#                                                                              #
################################################################################

package StockQuotes;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $CGI);

use Carp;
use GizmoTool;
use Channel;
use Utility;
use Sendmail;

use vars qw(@ISA);
@ISA=qw(GizmoTool);

################################################################
####                     INFORMATION                       #####
################################################################

my $NAME          = "Stock Quotes";
my $ABSTRACT      = "This tool allows a user to monitor the status of his/her prefered stocks. Users can also configure this tool to be alerted by email when one of the quote they are monitoring reaches a define threshold.";
my $VERSION       = "1.1";
my $LICENSE       = "GPL";
my $AUTHOR        = "Pierre Renaldo" ;
my $DOWNLOAD      = "http://www.metadot.com";
my $SERVER        = "http://quote.yahoo.com";
my $FETCH_URL     = "http://quote.yahoo.com/d/quotes.cgi";
my $REFRESH_EVERY = 20 ; #minutes
my $DESCRIPTION   = "Stock Quotes from Yahoo";

my %QUOTE_LABELS = (
  "symbol" => 0,
  "price"  => 1,
  "date"   => 2,
  "time"   => 3,
  "change" => 4,
  "close"  => 5,
  "high"   => 6,
  "low"    => 7,
  "volume" => 8
);

my %DISPLAYS = (
  "novice" => "Novice",
  "trader" => "Trader"
);
my %ALERTS = (
  "none" => "None",
  "email" => "Email Me"
);

my %INDICATORS = (
  "" => "",
  "price" => "Price",
  "change" => "Change"
);

my %POSITIONS = (
  "" => "",
  "above" => "above",
  "below" => "below"
);

################################################################
####                   ACCESS CONTROL                      #####
################################################################

my $AC =
  {
   BUNDLES => {
               MOD => { DESC => 'Edit',
                        OPS => [qw(add edit add_item add_listitem save_item save_listitem)],
                        DEFAULT_LEVEL => 2,
                        MIN_LEVEL => 2,
                      },
               DEL => { DESC => 'Delete and Cut',
                        OPS => [qw(delete_item delete_gizmotool delete_gizmotool_ok delete)],
                        DEFAULT_LEVEL => 8,
                        MIN_LEVEL => 8,
                        PARENT_DEP => {cut => 'MOD'},
                      },
               DISP => { DESC => 'View',
                         OPS => [qw(preview info show)],
                         DEFAULT_LEVEL => 0,
                         MIN_LEVEL => 0,
                         DEFAULTS_TO => 'on',
                       },
              },
   DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
  };

sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  return $AC;
}

################################################################
####                     INITIALIZE                        #####
################################################################

sub initialize {
  my $proto = shift;

  my $channel = Channel->new();

  # package name (mandatory)
  $channel->{is_a} = __PACKAGE__;
  # the on_schedule function will be called by the daemon every... (mandatory)
  $channel->{fetch_every} = $REFRESH_EVERY*60; # in seconds
  # name of the Gizmo Tool (mandatory)
  $channel->{name} = $NAME;
  # small description of the Gizmo Tool (mandatory)
  $channel->{description} = $DESCRIPTION;
  # server url where you fetch information
  $channel->{url} = $FETCH_URL;
  # home page of the server
  $channel->{home_url} = $SERVER;
  # tool bottom html
  $channel->{html} = "";

  $channel->add();
}

################################################################
####                     CONSTRUCTOR                       #####
################################################################

sub new {
  my $proto = shift;
  my $cid   = shift;
  my $uid   = shift;

  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new($cid,$uid);

  $self->{is_a} = __PACKAGE__;

  $self->{default}->{symbol}    = "SLB";
  $self->{default}->{display}   = "novice";  # novice | expert | trader
  $self->{default}->{alert}     = "none";    # none | email
  $self->{default}->{indicator} = "";    # price | change
  $self->{default}->{position}  = "";    # none | email
  $self->{default}->{threshold} = "";    # price | change


  bless ($self, $class);
  return $self;
}

################################################################
####                      REFRESH ITEM                     #####
################################################################

sub refresh_item {
  my $self = shift;
  my $params = shift;
  my $old_quote = shift;

  # build the url to fetch the quote
  my $symbol = $params->{symbol};
  ## Thanks to David Fancher for this fix
  my $url = $FETCH_URL . "?s=$symbol&f=sl1d1t1c1ohgv&e=.csv";

  # fetch and clean the new quote
  my $response = Utility->fetch_url($url);

  my $status = $response->code;
  return if ($status != 200);

  my $quote = $response->content;
  $quote =~ s/\"//g;
  chomp $quote;

  # check if an alert has to be triggered
  my $alert = $params->{alert} || "";
  unless ($alert eq "none") {
    my $indicator = $params->{indicator};
    my $position  = $params->{position} || "";
    my $threshold = $params->{threshold};

    my @old_quotation = split(/,/, $old_quote);
    my @new_quotation = split(/,/, $quote);

    my $new_alert = 0;
    my $old_alert = 1;
    if ($position eq "below") {
      $new_alert = $new_quotation[$QUOTE_LABELS{$indicator}] <= $threshold ;
      $old_alert = $old_quotation[$QUOTE_LABELS{$indicator}] <= $threshold ;
    } elsif ($position eq "above") {
      $new_alert = $new_quotation[$QUOTE_LABELS{$indicator}] >= $threshold ;
      $old_alert = $old_quotation[$QUOTE_LABELS{$indicator}] >= $threshold ;
    }
    $self->_warn_users($params,@new_quotation) if ($new_alert && !$old_alert);
  }

  return("Quote for $symbol", $quote);
}

sub _warn_users {
  my $self = shift;
  my $params = shift;
  my @quote = @_;

  my $symbol    = $params->{symbol};
  my $indicator = $params->{indicator};
  my $position  = $params->{position};
  my $threshold = $params->{threshold};

  my $pos = ($position eq "above") ? ">" : "<";

  my $ciid = $self->_exists_item_param($params);
  my $subject = "[M.] StockQuotes - $symbol: $indicator $pos $threshold";
  my $basename = $params->{'basename'};
  my $message ="
This is an alert from StockQuotes
>>> $symbol $indicator is $position $threshold.

There is the quote of $quote[$QUOTE_LABELS{symbol}] on $quote[$QUOTE_LABELS{date}] at $quote[$QUOTE_LABELS{time}]
Price:  $quote[$QUOTE_LABELS{price}]
Change: $quote[$QUOTE_LABELS{change}]
Low:    $quote[$QUOTE_LABELS{low}]
High:   $quote[$QUOTE_LABELS{high}]
Close:  $quote[$QUOTE_LABELS{close}]
Volume: $quote[$QUOTE_LABELS{volume}]

You are receiving this email because you are monitoring this server with MetaDot - WebMonitor.
Go to http://$basename/userchannel.pl and click EDIT in the WebMonitor frame to change your preferences.
";

    my ($users) = DBAccess->sqlSelectMany("email","users, mypage, gizmotoolitem",
                  "mypage.mpid=gizmotoolitem.mpid and users.userid=mypage.userid and ciid=$ciid");
  while (my ($email) = $users->fetchrow ) {
    my $sm = Sendmail->new($email, "", "", $subject, $message, "yes");
    $sm->sendMail();
  }
}

################################################################
####                      DISPLAY                          #####
################################################################

sub display_item {
  my $self = shift;
  my $ciid = shift;

  my $content = $self->{items}->{$ciid}->{content};
  my $display = $self->{items}->{$ciid}->{params}->{display};

  my @quote = split(/,/, $content);

  my $q_symbol = $quote[$QUOTE_LABELS{symbol}];
  my $q_price = $quote[$QUOTE_LABELS{price}];
  my $q_change = $quote[$QUOTE_LABELS{change}];
  my $q_low = $quote[$QUOTE_LABELS{low}];
  my $q_high = $quote[$QUOTE_LABELS{high}];
  my $q_close = $quote[$QUOTE_LABELS{close}];
  my $q_volume = $quote[$QUOTE_LABELS{volume}];
  my $q_date = $quote[$QUOTE_LABELS{date}];
  my $q_time = $quote[$QUOTE_LABELS{time}];

  my $color = ($q_change<0) ? "red" : "green" ;
  my $table = "<th align='left'><a href='http://finance.yahoo.com/q?d=t&amp;s=$q_symbol'>$q_symbol</a></th>\n";
  $table   .= "<td><b>$q_price</b></td>\n";
  $table   .= "<td><font color=$color>$q_change</font></td>\n";
  if ($display eq "trader") {
    $table .= "<td>lo: $q_low</td>\n";
    $table .= "<td>hi: $q_high</td>\n";
    $table .= "<td>close: $q_close</td>\n";
    $table .= "<td>vol: $q_volume</td>\n";
    $table .= "<td>$q_date $q_time</td>\n";
  }

  return "<tr>\n$table\n</tr>\n";
}

sub format_list_of_items {
  my $self = shift ;

  return "<center><table>".join("\n",@_)."</table></center>";
}

sub new_item_params_form {
    my $self = shift;
    my $cid = $self->cid();
    my $isa = $self->is_a();

    my $display = $self->{default}->{display};
    my $alert   = $self->{default}->{alert};

    my $q = new CGI;
    my $display_popup   = $q->popup_menu(-name    => 'display',
                     -default => $display,
                     -values  => [sort(keys(%DISPLAYS))],
                     -labels  => \%DISPLAYS);

    my $alert_popup     = $q->popup_menu(-name    => 'alert',
                     -default => $alert,
                     -values  => [sort(keys(%ALERTS))],
                     -labels  => \%ALERTS);

    my $indicator_popup = $q->popup_menu(-name    => 'indicator',
                     -default => "",
                     -values  => [sort(keys(%INDICATORS))],
                     -labels  => \%INDICATORS);

    my $position_popup  = $q->popup_menu(-name    => 'position',
                     -default => "",
                     -values  => [sort(keys(%POSITIONS))],
                     -labels  => \%POSITIONS);

    my $form = "
<p />
<table border=\"0\" cellspacing = 5 cellpadding = 5>
<tr><td>
<table border=\"0\">
<!-- New Param Form -->
<FORM action='userchannel.pl' method='post'>

<tr><th align='right'>Symbol:</th>
<td><input type=text name='symbol'></td></tr>

<tr><th align='right'>Display:</th>
<td>$display_popup</td></tr>

<tr><th align='right'>Alert:</th>
<td>$alert_popup</td></tr>

<tr><th align='right'>When:</th>
<td>$indicator_popup $position_popup <input type=text size='8' name='threshold'></td></tr>

</table></td>
<td valign='middle' align='center'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='add_item'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Add'>
</FORM>
<!-- End New Param Form -->
</td></tr>
</table>
";

    return $form;
}

sub edit_item_params_form {
    my $self = shift;
    my $ciid = shift;
    my $cid = $self->cid();
    my $isa = $self->is_a();

    my $symbol    = $self->{items}->{$ciid}->{params}->{symbol};
    my $display   = $self->{items}->{$ciid}->{params}->{display};
    my $alert     = $self->{items}->{$ciid}->{params}->{alert} || "none";
    my $indicator = $self->{items}->{$ciid}->{params}->{indicator};
    my $position  = $self->{items}->{$ciid}->{params}->{position};
    my $threshold = $self->{items}->{$ciid}->{params}->{threshold};

    my $q = new CGI;
    my $display_popup   = $q->popup_menu(-name => 'display',
                     -default => $display,
                     -values => [sort(keys(%DISPLAYS))],
                     -labels => \%DISPLAYS);

    my $alert_popup     = $q->popup_menu(-name => 'alert',
                     -default => $alert,
                     -values => [sort(keys(%ALERTS))],
                     -labels => \%ALERTS);

    my $indicator_popup = $q->popup_menu(-name => 'indicator',
                     -default => $indicator,
                     -values => [sort(keys(%INDICATORS))],
                     -labels => \%INDICATORS);

    my $position_popup  = $q->popup_menu(-name => 'position',
                     -default => $position,
                     -values => [sort(keys(%POSITIONS))],
                     -labels => \%POSITIONS);


    my $forms = "
<p />
<table border=\"0\" cellspacing = 5 cellpadding = 5>
<tr><td>
<table border=\"0\">
<!-- Edit Form -->
<FORM action='userchannel.pl' method='post'>

<tr><th align='right'>Symbol:</th>
<td><input type=text name='symbol' value='$symbol'></td></tr>

<tr><th align='right'>Display:</th>
<td>$display_popup</td></tr>

<tr><th align='right'>Alert:</th>
<td>$alert_popup</td></tr>

<tr><th align='right'>When:</th>
<td>$indicator_popup $position_popup <input type=text size='8' name='threshold' value='$threshold'></td></tr>

</table></td>
<td valign='middle' align='center'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='save_item'>
<INPUT type='hidden' name='ciid' value='$ciid'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Save'>
</FORM>
<!-- End Edit Form -->

<!-- Delete Button/Form -->
<FORM action='userchannel.pl' method='post'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='delete_item'>
<INPUT type='hidden' name='ciid' value='$ciid'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Delete'>
</FORM>
<!-- End Delete Button/Form -->
</td></tr>
</table>
";
    return $forms;
}

sub _load_form_params{
    my $self = shift;
    my %params;

    $params{symbol} = uc($FORM{symbol});
    $params{display} = $FORM{display};
    $params{alert} = $FORM{alert};
    if ($FORM{alert} eq "none") {
      $params{indicator} = "";
      $params{position}  = "";
      $params{threshold} = "";
    } else {
      $params{indicator} = $FORM{indicator};
      $params{position}  = $FORM{position};
      $params{threshold} = ($FORM{threshold} =~ m/^[+-]?\d+\.?\d*$/) ? $FORM{threshold} : 0;
    }

    return \%params;
}

sub _show_info {

return "
<H1>$NAME</H1>
<p />
<table border=\"1\">
<tr>
<th align='right' valign='top'>Abstract:</th>
<td align='left'>$ABSTRACT</td>
</tr>
<tr>
<th align='right' valign='top'>Version:</th>
<td align='left'>$VERSION</td>
</tr>
<tr>
<th align='right' valign='top'>Author:</th>
<td align='left'>$AUTHOR</td>
</tr>
<tr>
<th align='right' valign='top'>License:</th>
<td align='left'>$LICENSE</td>
</tr>
<tr>
<th align='right' valign='top'>Download:</th>
<td align='left'><a href='$DOWNLOAD'>$DOWNLOAD</a></td>
</tr>
<tr>
<th align='right' valign='top'>Server:</th>
<td align='left'><a href='$SERVER'>$SERVER</a></td>
</tr>
<tr>
<th align='right' valign='top'>Refresh every:</th>
<td align='left'>$REFRESH_EVERY minutes</td>
</tr>
</table>
<p />
<br />
<center><a href='userchannel.pl'>Back to MyPage</a></center>
<br />
";
}

1;
