###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::AccessBroker::Default;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE);
use MetadotConfig;

use constant _SITE_MGR_ => 1000;
use constant _STRATEGY_ => 900;

# this is a singleton;
my $self = {};


# ALL WILL BE FORBIDDEN, EXCEPT THOSE IN THIS LIST
# ALL OPERATIONS SHOULD BE LISTED HERE

my $ALLOWED_OPS = {
           server_setup => '',
           admin_tasks => _SITE_MGR_,
           advanced_styles => _SITE_MGR_,
           register_users => _STRATEGY_,
           manage_registrations => \&user_can_manage_registrations,
          };

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    bless ($self, $class);

    return $self;
}

sub is_allowed_to_do {
    my $self = shift;
    my $op = shift || die "must provide operation name";
    my $user = shift || $USER;

    my $result = 0;

    # admin is allowed to do anything
    $result = ( $user->belongs_to_grp('ADMIN', $user->uid()));

    # check if the op is permitted
    my $permission = $ALLOWED_OPS->{$op};
    if (defined $permission) {
        # some ops are permitted for SITE_MGR
        unless ($result) {
            if($permission == _SITE_MGR_) {
                $result = ($user->belongs_to_grp('SITE_MGR', $user->uid()));
            }
        }

        # some ops are specified via a reference to a method. This method is
        # responsible for returning the result. The method will receive
        # $op and $user as params
        unless ($result) {
            if (ref($permission) && ref($permission) eq 'CODE') {
#                print STDERR "is_allowed_to_do checks $permission($self, $op, $user)\n";
                $result = &{$permission}($self, $op, $user);
            }
        }

        # some ops are specified as STRATEGY classes. Specify the strategy class
        # in metadot.conf as [op]_strategy. If no strategy class is defined, then
        # result is false.
        # Strategy classes should themselves be subclasses of AccessBroker
        unless ($result) {
            if ($permission == _STRATEGY_) {
                my $strategy_class = MetadotConfig->get_param_value("$op".'_strategy');
                $DISPLAY->debug_msg(4, "AccessBroker->is_allowed_to_do($op) tries to use strategy $strategy_class");

                if ($strategy_class) {
                    no strict 'subs';
                    eval("require($strategy_class)");
                    if ($@) {
                        $DISPLAY->debug_msg(3, "AccessBroker->is_allowed_to_do($op) failed to evaluate strategy $strategy_class");
                    } else {
                        $result = $strategy_class->new()->is_allowed_to_do($op, $user);
                        $DISPLAY->debug_msg(4, "AccessBroker->is_allowed_to_do($op) gets result $result from strategy class $strategy_class");
                    }
                }
            }
        }
    }

#     print STDERR "AccessBroker->is_allowed_to_do($op) yields '$result'\n";
    $DISPLAY->debug_msg(5,"AccessBroker\->is_allowed_to_do($op, ".$user->uid().") yields '$result'");
    return $result;
}



sub user_can_manage_registrations {
    my ($class, $op, $user) = @_;

    if( !defined($user) ){
        $DISPLAY->debug_msg(5,"USER IS NOT DEFINED");
        $user = $USER;
    }

    $DISPLAY->debug_msg(4, "checking if user can manage registrations");

    my $result = (
      $class->is_allowed_to_do('admin_tasks',$user)
# this is unsafe because group managers can register users into their own
# groups, but that does not mean they should have blanket permission to
# manage registrations
#      || $class->is_allowed_to_do('register_users',$user)
      ) || 0;

#    print STDERR "user_can_manage_registrations starts with $result\n";
    # allow the user class to determine this
    unless($result) {
        $result = $user->is_allowed_to_do('accept_registration',$user);
#        print STDERR "checked user permissions and got $result\n";
    }

    $DISPLAY->debug_msg(4, "user can manage registrations? [$result]");
    return $result;
}


1;
