###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::BaseGizmoContainer;


use Metadot::AbstractGizmoContainer;
use Metadot qw($DISPLAY $PARAMS $SESSION);
use MetadotObject;
use Metadot::RequestCache::GizmoListRequestCache;
use Metadot::GizmoBase;
use DBAccess;
use Utility;

use vars qw(@ISA);
@ISA = qw(Metadot::AbstractGizmoContainer);



# this gets a list of the container's children.
sub get_children {
    my $self = shift;
    my $args = shift;

    $DISPLAY->debug_msg(4, "$self\->get_children begins");

    my @child_list = $self->build_child_list($args);
    $DISPLAY->debug_msg(4, "$self\->get_children has child list @child_list");

    # first determine which ones I'm allowed to display
    my @allowed_list = ();
    foreach my $gizmo (@child_list) {
        $DISPLAY->debug_msg(5, "$self\->get_children checking $gizmo");
        $DISPLAY->debug_msg(5, "$self\->get_children checking if $gizmo can 'is_allowed_to_do: ".$gizmo->can('is_allowed_to_do'));
        if ($gizmo->can('is_allowed_to_do') && $gizmo->is_allowed_to_do('show', $USER) ) {
            push @allowed_list, $gizmo;
            $DISPLAY->debug_msg(5, "$self\->get_children ACCEPTED $gizmo");
        }
    }
    return \@allowed_list;
}


sub get_navigation_children {
    my $self = shift;
    my $args = shift;
    my $child_list = $self->build_child_list({type=>'Metadot::Navigable'});
    my @result = ();


    CHILD: foreach my $child (@$child_list) {
        # filter based on hidden
        unless ($args->{show_hidden}) {
            if ($child && $child->is_hidden() && !$SESSION->admin_mode()) {
                next CHILD;
            }
        }

        # possibly filter by user permissions
        if ($args->{user}) {
            my $op = $args->{op} || 'show';
            unless ($child && $child->is_allowed_to_do($op, $args->{user})) {
                next CHILD;
            }
        }

        # passed all the filters
        push @result, $child;
    }

    return \@result;
}



# Event listener for when a child is deleted
sub on_child_deletion {
    my $self       = shift;
    my $child_list = shift;
    # do nothing
}


# returns true if the instance is a container of the supplied instance
sub is_container_of {
    my $self = shift;
    my $child = shift;
    my $result = 0;

    if (defined $child) {
        my $self_id = $self->id();
        my $parent = $child->get_parent();
        my %visited = ();
        while ($parent && !$result && !$visited{$parent->id()}) {
            my $parent_id = $parent->id();
            $visited{$parent_id} = 1;
            if ($parent_id == $self_id) {
                $result = 1;
            } else {
                $parent = $parent->get_parent();
            }
        }
    }

    return $result;
}



# Returns a list of child types accepted by this gizmo. Values are computed
# by taking the difference between allowed_child_types and prohibited_child_types
sub child_types {
    my $self   = shift;
    my $result = Utility->list_subtract(
        $self->allowed_child_types(),
        $self->prohibited_child_types()
        );

    return $result;
}


# returns a list of allowed child types. null or empty list means all child types are allowed
sub allowed_child_types {
    my $self = shift;

    return undef;
}



# returns a list of prohibited child types. null or empty list means NO child types are prohibited
sub prohibited_child_types {
    my $self = shift;

    return undef;
}



# returns boolean based on whether a child type is allowed by the container
sub is_child_type_allowed {
    my $self = shift;
    my $child_type = shift || die("Must specify a child type");

    my $result;

    # check to see if it's prohibited first
    if (Utility->array_contains($child_type, $self->prohibited_child_types())) {
        $result = 0;
    } else {
        # Check the list of allowed children. If none is defined, then all children
        # are permitted.
        my $allowed_types = $self->allowed_child_types();
        if (defined $allowed_types && ref($allowed_types)) {
            $result = Utility->array_contains($child_type, $allowed_types);
        } else {
            # empty list - all are permitted
            $result = 1;
        }
    }

    $DISPLAY->debug_msg(5, "is_child_type_allowed($child_type) returns '$result'");
    return $result;
}




# for the new Gizmo3000, each child defers to its parent hierarchy
# for controls.
sub get_edit_controls_for_child {
    my $self    = shift;
    my $child   = shift;

    my $args    = shift || {};
    my $requested_ops = $args->{ops};

    if (defined $requested_ops) {
        $DISPLAY->debug_msg(5, "Requested ops: ".join(' ', @$requested_ops));
    }

    my $visited;  # prevents looping
    if (defined $args->{visited}) {
        $visited = $args->{visited};
    } else {
        $args->{visited} = $visited = {};
    }

    (defined $child && $child->isa('Metadot::Gizmo3000'))
        || die("Not a valid child $child");

    my @controls = ();

    # I only add these controls to my own children, and only when I'm
    # in admin mode
    if ($child->parent_id() == $self->id() && $SESSION->admin_mode()) {
        my $edit_control_class = 'Metadot::Gizmo3000::IconEditControl';
        my $images_location = $PARAMS->{imageslocation};

        foreach my $operation (@{ &BASIC_CHILD_OPS }) {
            my $op = $operation->{op};

            # checking if this op is requested
            if (!defined $requested_ops || Utility->array_contains($op, $requested_ops)) {
                $DISPLAY->debug_msg(5, "$self checking if user is allowed to '$op'");
                if ($self->is_allowed_to_do($op)) {
                    $DISPLAY->debug_msg(5, "$self says that user is allowed to '$op'");
                    my $control = $edit_control_class->new(
                            operation   => $op,
                            actor_iid   => $self->id(),
                            target_iid  => $child->id(),
                            icon_url    => $images_location.'/obj/'.$operation->{icon},
                            description => $operation->{description},
                            arguments   => $operation->{arguments},
                        );
                    push @controls, $control;
                    $DISPLAY->debug_msg(5, "$self added control $control for $op and child $child");
                }
            }
        }
    }


    # TODO: make this work

#     # check to see if my parent exists. if so, get their controls also.
#     my $parent_id = $self->parent_id();
#     if (defined($parent_id) && !$visited->{$parent_id}++) {
#         push @controls,
#              $self->get_parent()->get_edit_controls_for_child( $child, $args);
#     }


    return @controls;
}




# a class method to get the children of a gizmo
sub build_child_list {
    my $self   = shift;
    my $params = shift || {};

    my $class = ref($self) || die('Instance method');

    # Try to determine the parent object from args
    my $id = $params->{parent_iid};
    $id = $params->{parent_id} unless( defined $id );

    # or try to get from self, if this was called in instance mode
    $id = $self->id() unless( defined $id );

    $id = undef unless Utility->is_valid_integer($id); # secu_200401

    $DISPLAY->debug_msg(5, "$self\->build_child_list for id '$id'");

    # if there is no id, then we return null
    if (!(defined($id)) || ($id eq '')) {
        return undef;
    }


    my @child_list;

    # check cache
    my $container_obj = Metadot::GizmoBase->restore($id);
    my $cached_results = Metadot::RequestCache::GizmoListRequestCache->check_cache($container_obj, $params);
    if (defined $cached_results) {
        @child_list = @{$cached_results};
        $DISPLAY->debug_msg(5, 'build_child_list retrieved results from cache');
    } else {

        # determine the types of children we wish to include in the list.
        # an empty or null list returns default types

        #type is replaced by types, but we retain backward compatibility
        my $type = $params->{type};
        my @types = ();
        @types = @{$params->{types}} if $params->{types};
        push(@types, $type) if defined($type);



        # determine the types of children we wish to exclude from the list
        my @excluded;
        @excluded = @{$params->{excluded}} if $params->{excluded};


        # determine if any SQL where conditions should be added to the query
        #usage note: where_conditions should NOT be prepended with 'AND';
        #            this will be done locally
        my $additional_where_conditions = $params->{where_conditions};
        my $additional_statements       = $params->{additional_statements} || 'order by instanceorder';


        # build the where conditions
        my @where_conditions = ();

        # starting with the parentiid
        push @where_conditions,"parentiid = $id";

        # add the condition for types

        #CKIM 2002-04-10: added special handling for 'Category' to get its equivalents
        if (Utility->array_contains('Category', \@types) || Utility->array_contains('Metadot::Navigable', \@types)) {
            @types = Utility->uniquify_array(@types, Gizmo->list_navigation_gizmos());
        }

        if (@types) {
            if (scalar(@types) > 1) {
                push @where_conditions, 'isa IN ('.Utility->convert_list_to_sql_list(@types).')';
            } else {
                push @where_conditions, 'isa = '.DBAccess->quote($types[0]);
            }
        } else {
            if (@excluded) {
                if (scalar(@excluded > 1)) {
                push @where_conditions, 'NOT(isa IN ('.Utility->convert_list_to_sql_list(@excluded).'))';
                } else {
                    push @where_conditions, 'isa != '.DBAccess->quote($types[0]);
                }
            }
        }

        if ($additional_where_conditions) {
            push @where_conditions, $additional_where_conditions;
        }

        $where_clause = join(' AND ', @where_conditions);

        $DISPLAY->debug_msg(5, "$class\->build_child_list using where: $where_clause $additional_statements");

        my $child_sth = DBAccess->sqlSelectMany("iid, isa", 'instance' , $where_clause, $additional_statements);

        @child_list = ();
        while(my ($child_id, $child_class)=$child_sth->fetchrow()) {
            $DISPLAY->debug_msg(5, "trying to restore $child_class ($child_id)");
            my $child_object;
            eval {
                $child_object = $child_class->restore($child_id);
                push @child_list, $child_object;
                $DISPLAY->debug_msg(5, "got child: $child_object");
            };
        }
        $child_sth->finish();

        Metadot::RequestCache::GizmoListRequestCache->update_cache($container_obj, $params, \@child_list);
    }



    if (wantarray) {
        return @child_list;
    } else {
        return \@child_list;
    }
}


use constant BASIC_CHILD_OPS => [
    {
      op            => 'cut_child',
      icon          => 'cut.gif',
      description   => 'Cut me',
    },
    {
      op            => 'move_child_up_down',
      icon          => 'uparrow.gif',
      description   => 'Move me up',
      arguments     => {
          delta => -1,
          }
    },
    {
      op            => 'move_child_up_down',
      icon          => 'downarrow.gif',
      description   => 'Move me down',
      arguments     => {
          delta => 1,
          }
    },
];








sub delete_children {
    my $self = shift;
    my $id = $self->id;

    # delete Gizmo childs
    my ($childs)=DBAccess->sqlSelectMany("iid, isa", "instance", "parentiid=$id");
    $DISPLAY->debug_msg(3, "$self\->delete_children, my iid=$id");

    while(my ($giid, $gisa)=$childs->fetchrow()) {
        $DISPLAY->debug_msg(4, "about to delete $gisa $giid");

        # the library may not have been used. Unfortunately, there are cases where
        # a Gizmo, like Question, may not successfully "require"
        eval {
            require "Gizmo/$gisa.pm";
            $DISPLAY->debug_msg(5, "Successfully required Gizmo/$gisa.pm");
        };
        # try to instantiate and delete the instance
        eval {
            my $gizmo = $gisa->restore($giid);
            $gizmo->delete("YES");
            $DISPLAY->debug_msg(5, "Successfully deleted $gizmo");
        };
        if ($@) {
            $DISPLAY->print_click_back("Error", "There was a problem deleting $gisa ($giid)");
        }
    }
    $childs->finish();
}





# This method handles what to do after you have finished creating a child.
sub show_create_child_success {
    my $self  = shift;
    my $child = shift;

    my $isa      = $child->is_a();
    my $parentid = $self->id();

    $DISPLAY->two_choices(
        "<Big>New " . $child->get_gizmo_name()
        . " added!</big><br /><br />What do you want to do?",

        "confirm",

        "Add another " . $child->get_gizmo_name(),
        "index.pl?child_type=$isa&isa="
        .($FORM{isa} || $self->is_a())
        ."&iid=$parentid&op=add_child&app=",

        "Enough for now!",
        "index.pl?id=$parentid",
        );
    exit(0);
}


# moves a child up or down in the order
sub move_child_up_down {
    my $self = shift;
    my $child = shift || die('Must specify the child to move');
    my $delta = shift;

    # make sure delta is valid
    $delta = undef unless Utility->is_valid_integer($delta); # secu_200401
    return unless ($delta);


    # make sure child is mine
    ($child->parent_id() == $self->id())
        || die('This is not my child! My iid is '.$self->id()
               .' and child\'s parent is '.$child->parent_id());


    my $parent_id  = $self->id();
    my $child_isa  = $child->is_a();
    my $child_id   = $child->id();
    my $order      = $child->instance_order();

    # determine the first, last child. we filter the instances to get our children
    my $filter = "parentiid=$parent_id";
    $filter = $self->_adjust_child_reordering_filter($child, $filter);

    my ($last_one, $first_one) = DBAccess->sqlSelect("max(instanceorder), min(instanceorder)", "instance",  $filter);
        my $new_order = $order + $delta;

    if ($new_order > $last_one) {
        #already at bottom, will wrap to top
        $new_order = $first_one - 1;
    } elsif ($new_order < $first_one) {
        #already at top, will wrap to bottom
        $new_order = $last_one + 1;
    }


    my $where = "parentiid=$parent_id and instanceorder=$new_order";

    # move another one out of the way if necessary
    $DISPLAY->debug_msg(4, "$self\->move_child_up_down about to update where '$where'");
    $self->get_object_accessor()->update_where( class_field=>'is_a', where=>$where, updates=>{instance_order=>$order});

    # change the child's position
        $child->instance_order($new_order);
        $child->save();

    $self->reorder_children(); #make sure everything is well ordered.
    return $child->instance_order();
}



# when reordering children, allow child classes to adjust the filter
sub _adjust_child_reordering_filter {
    my $self   = shift;
    my $child  = shift;
    my $filter = shift;

    return $filter;
}



sub reorder_children {
    my $self = shift;
    my $args = shift;

    # if this is called as a class method, parent_obj must be included in the args
    unless (ref $self) {
        $self = $args->{parent_obj};
    }

    unless (ref $self) {
        return undef;
    }

    my $id=$self->id();
    my $isa = $self->is_a();

    $DISPLAY->debug_msg(5, __PACKAGE__.": $self\->reorder_children");

    my $i=0;
    my ($c) = DBAccess->sqlSelectMany("iid", "instance", "parentiid=$id", "order by instanceorder");
    while(my ($sonid) = $c->fetchrow()) {
        $DISPLAY->debug_msg(5, __PACKAGE__.": $self reordering child $sonid");
        $self->get_object_accessor()->update_where( where=>"iid=$sonid", updates=>{instance_order=>$i});
        $i++;
    }
    $c->finish();

    $DISPLAY->debug_msg(5, __PACKAGE__.": $self\->reorder_children FINISHED");
}

sub move_into_position {
        my $self     = shift;
        my $child    = shift || die("Must specify the child to move into position");
        my $position = shift;
        my $alternate = 1;

        ref($self) || die("instance method");
        ($child->parent_id() == $self->id()) || die("This is not my child!");

        #print STDERR "\nIN container retrieve where parentid[".$self->id()."] and instanceOrder[".$position."]\n\n";

        my @obstacles = $self->get_object_accessor()->retrieve_where( where=>"ParentIID=".$self->id()." AND InstanceOrder>=$position");
        #print STDERR "\nIN THE CONTAINER FUNC will move [$#obstacles] obstacles\n";
        foreach my $obstacle (@obstacles) {
            $obstacle->instance_order( $obstacle->instance_order() + 1);
            $obstacle->save();
            #print STDERR "\nmoving[". $obstacle->id() . "]\n";
        }

        $child->instance_order($position);
}





# returns the vertical position of the top child
sub top_child_position {
  my $self = shift;

  my $iid = $self->id();
  my ($result) = DBAccess->sqlSelect("MIN(instanceorder)", "instance",  "parentiid=$iid");

  return $result || 0;
}



# returns the vertical position of the bottom child
sub bottom_child_position {
    my $self = shift;

    my $iid = $self->id();
    my ($result) = DBAccess->sqlSelect("MAX(instanceorder)", "instance",  "parentiid=$iid");

    return $result || 0;
}


# returns the suggested vertical position of the next child
sub next_vertical_position {
    my $self = shift;
    return $self->bottom_child_position() + 1;
}











1;
