###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License=> or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful=> but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not=> write to the Free Software
# Foundation=> Inc.=> 59 Temple Place=> Suite 330=> Boston=> MA  02111-1307  USA
#
# For more information=> please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::ChannelProcessor;

use strict;
use vars qw(@ISA);

use Digest::MD5;
use Fcntl ':flock';     #import flock + constants
use XML::RSS;

use Channel;
use DBAccess;
use Metadot qw($PARAMS $CGI);
use Metadot::PortalFactory;
use MetadotConfig;
use MetadotObject;
use Sendmail;
use Subscription;
use Utility;

=head1 NAME

Metadot::ChannelProcessor

=head1 VERSION

Version $Revision: 1.1.2.35 $

=head1 SYNOPSIS

my $cp = Metadot::ChannelProcessor->get_instance(
    timestamp => $db_now
);

$cp->do_channels($channel_id);
$cp->do_subscriptions(frequency => Subscription->FREQUENCY_ALL());

=head1 DESCRIPTION

ChannelProcessor is responsible processing channels and subscriptions. This
class is a work in progress, as part of an effort to refactor metadotd.pl into
something more manageable. The main benefit of this class for now is that it has
an API which can be unit tested. Apart from that, it is basically code that has
been extracted from metadotd.pl and minimally refactored to allow it to operate
in an OO fashion.

=head1 METHODS

=cut






@ISA = qw(MetadotObject);


use constant MO_INSTANCE_PROPERTIES => {
      debug_level   => 0,
      timestamp     => undef,       # the timestamp used for updating
                                    # subscriptions and channels

      fetch_flag    => undef,       # lets the processor know whether this
                                    # server fetches email replies

      channel_dir   => 'channels',       # the location where channel files are kept

      rss_cache     =>  undef,

      sendmail_class    => undef,   # class to be used when sending mail.

      number_processed  => 0,       # the number of items processed in last
                                    # command

      silent          => undef,           # if true, no actual message will be sent. however,
                                          # database updates will occur as if the message
                                          # was sent

      error           => undef,     # the current error condition

      no_process_external  => undef,       # the -n argument, tells the processor
                                           # not to process external channels

      fetched_urls      => undef,   # track the urls we have visited
      unavailable_urls  => undef,
};


use constant HORIZONTAL_RULE  => '-' x 72;
use constant BASE_CHANNEL_DIR => 'channels';
use constant CACHE_ITEM_MAX   => 10;

sub initialize {
    my $self = shift;

    $self->timestamp(Utility->db_now()) unless defined($self->timestamp());
    $self->rss_cache({}) unless defined($self->rss_cache());
    $self->fetched_urls({}) unless defined($self->fetched_urls());
    $self->unavailable_urls({}) unless defined($self->unavailable_urls());
}


=item C<get_instance>

A factory method to get a default instance.

my $cp = Metadot::ChannelProcessor->get_instance();

=cut
# factory method
sub get_instance {
    my $class = shift;
    my @args  = @_;

    my $type = MetadotConfig->get_param_value('channel_processor_type');
    my $result;

    if (defined $type) {
        my $file = Utility->package_to_filename($type);
        eval {
            require $file;
            $result = $type->new(@args);
        };
        if ($@) {
            print STDERR "Error instantiating $class of type '$type'\n";
        }
    }

    $result ||= $class->new(@args);

    return $result;
}


=item C<do_channels>

Creates channelitem entries in the database. For each channel, it will determine
the channel's type, extract all the content from the channel, and create
channelitem records.

Optionally, you can specify a cid of the channel to process. If no cid is
specified, all channels for the current vserver will be processed.

The return value is the number of channels processed

  $cp->do_channels();
  $cp->do_channels($cid);

=cut

sub do_channels {
    my ($self, @channelids) = @_;

    $self->debug_msg(3, "---------- do_channels(@channelids)");

    my @gizmo_tools_array = GizmoTool->lists_tools();
    my %gizmo_tools;
    for (@gizmo_tools_array) {
        $gizmo_tools{$_}++;
    }

    my ($httpstatus, $OK);
    my $where;


    if (@channelids && defined $channelids[0]) {
        if (scalar(@channelids) == 1) {
            $where .= "cid=$channelids[0]";
        } else {
            $where .= "cid IN (".join(',', @channelids).")";
        }
    } else {
        $where = '1=1';
    }

    $self->debug_msg(4, "do_channels selects channels WHERE $where");

    # gets a list of all channels to process
    my ($c) = DBAccess->sqlSelectMany( "cid, name, url, iid, isa, version, lastfetched, httpstatus", "channel", "$where", "ORDER BY cid DESC" );

    my $s="";

    $self->reset();

    # Do all channels
    while(my ($cid, $name,$url, $iid, $isa, $newsversion, $lastfetched, $httpstatus)=$c->fetchrow()) {

        $self->debug_msg(5, "do_channels has cid $cid");

        next unless $isa;

        my $html_cache = "";
        $newsversion++;     # we do the next version here!

        #CKIM: save the new version number.
        DBAccess->sqlUpdate("channel", "cid=$cid", version => $newsversion);


        $self->debug_msg(3, "=> Channel $name; cid=$cid; isa=$isa; version=$newsversion");
        $OK=0;

        if ( defined($isa) && ($isa eq Channel->CHANNEL_TYPE_EXTERNAL())) {

            # process the external channel ONLY if the most recent http
            # status is successful
            # AND ONLY if the -noprocess flag is NOT turned ON
            if ( !$self->no_process_external() && defined($httpstatus) && ($httpstatus == 200) ) {

                $html_cache = $self->do_external_channel($newsversion, $cid, $url);
                $self->debug_msg(4, "Processed external channel, processed ".$self->number_processed());

                if ($self->number_processed() > 0) {
                    $OK=1;
                } else {
                    # nothing in it!

                    # CKIM: Since $OK does not equal one, later on in this method the
                    #       code will avoid deleting oldnews. This seems like an
                    #       awkward way to do it, but who am I to argue?

                }

            }
        } else {
            $OK=1;

            my $basename = $PARAMS->{basename};

            if ( defined $isa && ($isa eq Channel->CHANNEL_TYPE_INTERNAL())) {
                # internal: do all msg of this channel
                # the html is now generated in my page
                # the iid is stored in the url field

                # We take the news from these internal channel
                # and we store them into the channelitem table
                # replace either update or insert automatically

                #### This part is good for category news item
                #### may need to do another query for other stuff

                #### Modified this to use the Gizmo API rather than DBAccess

                #my ($cisa, $isa) = DBAccess->sqlSelect("parentiid, isa", "instance", "iid=$iid");
                #$self->debug_msg(5, "ChannelID $cid is pointing to a ".($cisa || '').": ".($iid || ''));
                #my ($d)=DBAccess->sqlSelectMany("iid, name, url, description", "instance", "parentiid=$iid and isa='Newsitem' and (showfrom IS NULL OR showfrom <= '".$self->timestamp()."')", "order by showfrom desc, iid desc");

                my $channel_parent = Metadot::PortalFactory->get_gizmo_object($iid);
                if (defined $channel_parent) {
                    $self->debug_msg(4, "Checking for channelitem gizmos under $channel_parent ".($channel_parent->name() || '')."; id=".$channel_parent->id());
                    my $channelitem_gizmos = $self->get_channelitem_gizmos(
                        channel_parent  => $channel_parent,
                        lastfetched     => $lastfetched,
                    );

                    # order my channelitem gizmos by show_from || creation_date
                    my @sorted = sort({($a->show_from() || $a->creation_date()) cmp ($b->show_from() || $b->creation_date())} @$channelitem_gizmos);

                    #########################################################

                    # add the channelitems to the database
                    foreach my $channelitem_gizmo (@sorted) {
                        my $ciid            = $channelitem_gizmo->id();
                        my $cname           = $channelitem_gizmo->name();
                        my $cdescription    = $channelitem_gizmo->description();
                        my $curl            = $channelitem_gizmo->url();
                        my $creationdate    = $channelitem_gizmo->creation_date();

                        if ( $cdescription =~ s/\A<!-- form type is text -->//) {
                            # we need to call this to initialize global vars
                            $cdescription = GizmoBuilder->untranslate_white_space( $cdescription );
                            $cdescription = $CGI->unescapeHTML( $cdescription );
                        }

                        $self->debug_msg(4, "item $cname: ciid=$ciid");
                        $curl ||="$basename/index.pl?id=$ciid&isa=Newsitem&op=show";

                        #########################################################

                        $self->debug_msg(5, "Adding channel item $cid, $cdescription, with version $newsversion");
                        $self->add_channel_item(
                            cid             => $cid,
                            name            => $cname,
                            url             => $curl,
                            description     => $cdescription,
                            version         => $newsversion,
                            iid             => $ciid,
                            creationdate    => $creationdate,
                        );
                    }
                }

                $html_cache = $self->generate_channel_html($cid);

            } elsif ($isa eq Channel->CHANNEL_TYPE_DISCUSSION()) {
                $self->debug_msg(5, "ChannelID $cid is pointing to a $isa: $iid\n");

                my $where = "iid=$iid AND status='OK'";
                if (defined $lastfetched) {
                    $where .= " AND creationdate >= ".DBAccess->quote($lastfetched);
                }

                $self->debug_msg(5, "Getting discussion messages WHERE $where");
                my ($d)=DBAccess->sqlSelectMany("mid, name, description, creationdate", "message", $where, "ORDER BY creationdate, mid");
                my $murl;

                ### If no messages are returned, get the last 5 messages
                my $limit = 0;
                my $limit_to = 5;
                if( $d->rows == 0 ){ 
                    # Limit to only last 5 messages
                    $limit = 1;
                    $self->debug_msg(5,"NO ROWS RETURNED FOR CID[$cid] IID[$iid]");
                    $where = "iid = $iid AND status = 'OK'";
                    ($d) = DBAccess->sqlSelectMany("mid, name, description, creationdate",
                                                    'message',
                                                    $where,
                                                    "ORDER BY creationdate, mid"
                                                  ); 
                }

                while (my ($mmid, $mname, $mdescription, $mdate)=$d->fetchrow()) {

                    $self->debug_msg(5, $self->number_processed().": item $mname: mid=$mmid\n");
                    $murl = "$basename/index.pl?mid=$mmid&isa=Message&op=show";

                    #################################################################

                    $self->add_channel_item(
                        cid             => $cid,
                        name            => $mname,
                        url             => $murl,
                        description     => $mdescription,
                        version         => $newsversion,
                        mid             => $mmid,
                        iid             => $iid,
                        creationdate    => $mdate,
                    );

                    if( $limit ){
                        $limit_to--;
                        if( $limit_to == 0 ){
                            last;
                        }
                    }
                }
                $d->finish();

                $html_cache = $self->generate_channel_html($cid);
            }

            $self->debug_msg(4, "      found ".$self->number_processed()." item(s).\n");

        }

        ######## FINISHED processing the channel items


        ### here we log the success
        ### save nb items, inc version and update lastfetched
        if ($OK == 1) {
            DBAccess->sqlUpdate("channel", "cid=$cid",
               nbitems => $self->number_processed(),
               html    => $html_cache,
               lastfetched => $self->timestamp(),
            );


            ### mark old items for future deletion
            ### (to be able to delete them when too old for subscriptions.)

            ### However, we don't want to mark gizmo tool items as oldnews
            unless (defined $isa && $gizmo_tools{$isa}) {
                $self->debug_msg(4, "marking channelitems as old news: cid='$cid', newsversion != '$newsversion'");

                my $ciid_cursor;
                unless ($isa eq Channel->CHANNEL_TYPE_EXTERNAL()) {
                    # Even if a channelitem is old, make sure that we have something to display in the channel. This doesn't apply to
                    # external channels, because we want it to always be based on the RSS channel
                    my ($current_count) = DBAccess->sqlSelect('count(*)', 'channelitem', "cid=$cid AND newsversion = $newsversion");
                    my $ciid_sth        = DBAccess->sqlSelectMany('ciid', 'channelitem', "cid=$cid AND newsversion != $newsversion", 'ORDER BY ciid DESC');
                    my $i = $current_count;
                    my $min_items = CACHE_ITEM_MAX();

                    while ($i++ < $min_items && (($ciid_cursor) = $ciid_sth->fetchrow())) {
                        $self->debug_msg(4, "ciid $ciid_cursor will not be marked for deletion: current article count is now $i");
                    }
                    $ciid_sth->finish();

                }

                my $update_where = "cid = $cid AND newsversion != $newsversion";

                if (defined $ciid_cursor) {
                    $update_where .= " AND ciid<$ciid_cursor";
                }

                $update_where .= ' AND (urgent IS NULL or urgent !='.DBAccess->quote('oldnews').')';


                $self->debug_msg(4, "Flagging oldnews WHERE $update_where");
                DBAccess->sqlUpdate("channelitem", $update_where, (urgent => 'oldnews'));
                $self->debug_msg(5, "Finished flagging oldnews for channel $cid");
            } else {
                $self->debug_msg(5,  "matched on a gizmo tool $isa for channel $cid, won't set to oldnews");
            }
        }
    }

    $c->finish();
    $self->debug_msg(3, "Channel update completed.");

    return $self->number_processed();
}



=item C<generate_channel_html>

Generates the HTML for a channel, to be used in the MyPage element

  $cp->generate_channel_html(
    cid         => $cid,
    max_items   => $max_items,
  );

  $cp->generate_channel_html(
    cid         => $cid,
  );

=cut
sub generate_channel_html {
    my $self = shift;
    my $cid  = shift;
    my $max_items = shift || CACHE_ITEM_MAX();
    my $result = '';

    if (defined $cid) {
        $self->debug_msg(4, "Creating html for channel $cid");

        my $channelitem_sth = DBAccess->sqlSelectMany('url, name', 'channelitem', "cid=$cid", "ORDER BY ciid DESC");
        my $item_count = 0;
        while (($item_count++ < $max_items) && (my($url, $name) = $channelitem_sth->fetchrow())) {
            $self->debug_msg(5, "Creating html for channelitem $name $url");

            # due to an error in earlier versions of ChannelProcessor, it's possible that the
            # link includes the URI schema twice, as in http://http://something.com

            $self->debug_msg(5,"BEFORE PATTERN MATCHING URL[$url]");
            if( $url =~ /http:\/\/http/ ){
                $url =~ s/http:\/\/http:\/\//http:\/\//;
            }
            $self->debug_msg(5,"URL[$url]");
            $result .= "<li><a target='_NEWSITEM' href='$url'><font size=\"-3\">$name</font></a><br />\n";
        }
        $channelitem_sth->finish();
    }

    return $result;
}




=item C<get_channelitem_gizmos>

Returns all of the gizmo children that generate channelitems. Typically this
is called on a Category, and returns Newsitems. Currently, the Newsitem class
is hard-coded here. We can easily refactor that as an interface at a later time.

  $cp->get_channelitem_gizmos(
    channel_parent  => $gizmo_that_defines_the_channel,
    max_items       => $max_items,
  );

=cut

sub get_channelitem_gizmos {
    my $self = shift;
    my %args = @_;

    my $channel_parent = $args{channel_parent};
    my $max_items      = $args{max_items} || 0;
    my $visited        = $args{visited} || {};
    my $results        = $args{results} || [];

    my $num_results    = scalar @$results;

    my @children;
    # get the immediate descendants
    if (defined $channel_parent && UNIVERSAL::isa($channel_parent, 'Metadot::BaseGizmoContainer')) {
        $self->debug_msg(4, "get_channelitem_gizmos, parent is ".$channel_parent->id()."; max_items=$max_items; result count: $num_results");

        @children = $channel_parent->build_child_list({
            where_conditions        => "(showfrom IS NULL OR showfrom <= '".$self->timestamp()."') AND (showend IS NULL OR showend >= '".$self->timestamp()."')",
            additional_statements   => "ORDER BY showfrom, creationdate",
        });

        foreach my $child (@children) {
            last if($max_items && $num_results++ > $max_items);
            if (UNIVERSAL::isa($child, 'Newsitem')) {
                push @$results, $child;
                $self->debug_msg(4, "got channelitem_gizmo $child ".($child->name()||'').'; id='.$child->id());
            }
        }
    }


    # recurse if needed
    foreach my $result (@children) {
        last if ($max_items && scalar @$results > $max_items);
        if (UNIVERSAL::isa($result, 'Metadot::BaseGizmoContainer') && !UNIVERSAL::isa($result, 'Metadot::Navigable') && !$visited->{$result->id()}++) {

            $self->debug_msg(5, "RECURSING into $result ".$result->id());

            $self->get_channelitem_gizmos(
                channel_parent  => $result,
                max_items       => $max_items,
                visited         => $visited,
                results         => $results,
            );
        }
    }

    return $results;
}







=item C<add_channel_item>

Creates a single channelitem entry in the database. If the channelitem has
already existed in the past, the existing ones will be updated as 'oldnews' for
tracking purposes.

  $cp->add_channel_item(
    cid  => 532,
    name => 'My news article',
    url  => 'some url',
    description => 'some description',
    version     => 400,  # the channel's current version
    mid         => 10,   # the mid, if any, of the news item
    iid         => 100,  # the iid, if any, of the news item
  );

=cut

sub add_channel_item {
    my ($self, %args) = @_;

    my $cid             = $args{cid};
    my $name            = $args{name};
    my $url             = $args{url};
    my $description     = $args{description};
    my $version         = $args{version};
    my $mid             = $args{mid};
    my $iid             = $args{iid};
    my $creationdate    = $args{creationdate};

    (defined $cid) || die('Must specify the cid');

    unless (defined $version) {
        ($version) = DBAccess->sqlSelect('version', 'channel', 'cid='.$cid);
    }


    $self->debug_msg(5, "in add_channel_item with mid ".(defined $mid ? $mid : 'undef')." and version ".(defined $version ? $version : 'undef'));

#check if this is a new item
#in order to do this we check if there is already the same
#item with the same name and description from the previous
#version of the channel
#if we find a previous item then we get the creation date and use it
#to save it again in the newer version. This avoids having the notification
#system send the same channelitem many time.

    my $xtitle = DBAccess->quote($name || '');

    # these are the conditions that define a duplicate channelitem
    my @duplicate_conditions =
    (
     "cid = $cid",
     "newsversion != $version",
     "name = $xtitle",
    );
    if (defined $mid) {
        push @duplicate_conditions, "mid=$mid";
    }
    my $duplicate_where = join(' AND ', @duplicate_conditions);

    my ($firstseen) = DBAccess->sqlSelect("creationdate", "channelitem", $duplicate_where);
    $self->debug_msg(5, "Checking for duplicate channelitems based on $duplicate_where; got ".(defined $firstseen ? $firstseen : '[undef]'));




# build the new record. If the channelitem existed in the past, we update the
# old record. If the channelitem has an MID, then we include that in the record.

    my %channelitem_data =
    (
        cid             => $cid,
        name            => $name,
        url             => $url,
        description     => $description,
        newsversion     => $version,
    );

    $channelitem_data{mid} = $mid if (defined $mid);
    $channelitem_data{iid} = $iid if (defined $iid);

    if ($firstseen) {
        # update the existing record
        DBAccess->sqlUpdate('channelitem', $duplicate_where, %channelitem_data);
    } else {
        # insert the new record
        $channelitem_data{creationdate} = $creationdate || $self->timestamp();
        DBAccess->sqlInsert('channelitem', %channelitem_data);
    }


    # update the channel's updatedate
    DBAccess->sqlUpdate ("channel", "cid=$cid", (updatedate => $self->timestamp()));

    $self->increment_number_processed();
}











=item C<do_external_channel>

Reads a stored channel .xml file and parses it, caches it, and stores it in the db. This
method assumes that channel .xml was created, as with metadotd.pl. Eventually,
this class will get the channel content directly from the source.

  $cp->do_external_channel([news version], [cid], [channel url]);

=cut

sub do_external_channel  {
    my ($self, $newsversion, $cid, $url)=@_;

    return if ($self->no_process_external());


    my $html_cache="";

    my $httpstatus=0;
    my $channeldir = $self->channel_dir();

    $self->debug_msg(3, "in do_external_channel, cid=$cid, newsversion:$newsversion\n");

    my $content="";

    my $url_digest = Digest::MD5::md5_hex( $url );
    $self->debug_msg(4, "Reading from file $channeldir/$url_digest.xml\n");

    unless (-r "$channeldir/$url_digest.xml"){
        warn "Cannot read $channeldir/$url_digest.xml\n";
        return ('');
    }

    $self->debug_msg(4, "About to parse $channeldir/$url_digest.xml\n");

    eval{           ### TRY
        my $rss_instance = $self->rss_cache()->{$url_digest};

        unless (defined $rss_instance)
        {
            $rss_instance = XML::RSS->new();
            $rss_instance->parsefile("$channeldir/$url_digest.xml");
            $self->rss_cache()->{$url_digest} = $rss_instance;
        }


        $html_cache = $self->cache_html_channel(
            cid         => $cid,
            newsversion => $newsversion,
            rss         => $rss_instance,
        );

        $self->debug_msg(4, "Finished parsing");
    };

    ### CATCH
    if ($@) {
        my ($name, $url)=DBAccess->sqlSelect ("name, url", "channel", "cid=$cid");
        warn "\n\n###WARNING: '$name' Channel file contains incorrect RSS content cid=$cid:\n\n$@\n\nThis is not an error but a warning explaining that Metadot Portal Server fetched a file that is not a correct RSS file. If this error persists please contact the creator of this channel file or the channel webmaster, i.e. the guy running the website at $url\n\n\n";
    }

    $self->debug_msg(4, "do_external_channel: end  cid=$cid\n");
    return $html_cache;

}














=item C<cache_html_channel>

Reads a stored channel .xml file and parses it, caches it, and stores it in the db. This
method assumes that channel .xml was created, as with metadotd.pl. Eventually,
this class will get the channel content directly from the source.

  $cp->cache_html_channel(
    cid         => $cid,            # the cid to update
    newsversion => $newsversion,    # the channel's newsversion
    rss         => $rss_instance    # an XML::RSS instance to process
    updatedb    => 1,               # flags whether to save this to the database (defaults true)
  );

=cut

sub cache_html_channel {
    my ($self, %args) = @_;

    my $cid         = $args{cid};
    my $newsversion = $args{newsversion};
    my $rss         = $args{rss};

    my $updatedb    = $args{updatedb};
    $updatedb       = 1 unless (defined $updatedb);

    use Unicode::String;

    my $i=0;

    $self->debug_msg(4, "### chacheHTMLChannel($cid)");
    return 0 if (!$cid);
    my $updatedate = "";
    ($updatedate) = DBAccess->sqlSelect("updatedate", "channel", "cid=$cid") if($updatedb);

    # cache channel image
    my $image = "";
    my $image_url = Unicode::String::utf8( $rss->{'image'}->{'url'} )->latin1;
    if ($image_url) {
      my $image_link = Unicode::String::utf8( $rss->{'image'}->{'link'} || "" )->latin1;
      my $image_title = Unicode::String::utf8( $rss->{'image'}->{'title'} || "")->latin1;
      my $width = Unicode::String::utf8( $rss->{'image'}->{'width'} || "" )->latin1;
      my $height = Unicode::String::utf8( $rss->{'image'}->{'height'} || "" )->latin1;
      $width = "width='$width'" if ($width);
      $height = "height='$height'" if ($height);
      $image = "<img src='$image_url' alt='$image_title' border='0' $width  $height>";
      $image = "<a target='_NEWSITEM' href='$image_link'><font size=\"-3\">$image</font></a>" if ($image_link);
    }

    # cache html channel news + save into channelitem table (for subscription)
    my $html_content = ($image) ? "<center>$image</center><br />\n" : "";

    foreach my $item (@{$rss->{'items'}}) {

        next unless defined($item->{'title'}) && defined($item->{'link'});

        my $title = Unicode::String::utf8( $item->{'title'} )->latin1;
        my $url = Unicode::String::utf8( $item->{'link'} )->latin1;

        my $description;
        if (defined ($item->{'description'})) {
            $description = Unicode::String::utf8( $item->{'description'} )->latin1
        }

        # add item to the html cache
        $html_content .= "<li><a target='_NEWSITEM' href='$url'><font size=\"-3\">$title</font></a><br />\n";

        # save the item in the database
        if ($updatedb) {
            $self->add_channel_item(
                cid         => $cid,
                name        => $title,
                url         => $url,
                description => $description,
                version     => $newsversion,
            );
        }

        $i++;
    }

    # if there's a textinput element
    my $input_title = Unicode::String::utf8($rss->{'textinput'}->{'title'})->latin1;
    if ($input_title) {
      my $input_link = Unicode::String::utf8( $rss->{'textinput'}->{'link'} )->latin1;
      my $input_description = Unicode::String::utf8($rss->{'textinput'}->{'description'})->latin1;
      my $input_name = Unicode::String::utf8($rss->{'textinput'}->{'name'})->latin1;
      $html_content .= "<p /><form method='get' action='$input_link'>\n";
      $html_content .= "$input_description<br />\n";
      $html_content .= "<input type='text' name='$input_name'><br />";
      $html_content .= "<input type='submit' value='$input_title'></form>";
    }

    # if there's a copyright element


    # XXX [claudio] we keep the ShareM version way
    #    my $copyright = Unicode::String::utf8($rss->{'channel'}->{'copyright'})->latin1;
    my $copyright = undef;
    if ($copyright) {
    $html_content .= "<p><sub>$copyright</sub></p>";
      }

    my $channel_title = Unicode::String::utf8($rss->{'channel'}->{'title'})->latin1;
    $channel_title =~ s/^\s*//;

    DBAccess->sqlUpdate("channel", "cid=$cid",
          (
           name        => $channel_title,
           homeurl     => Unicode::String::utf8( $rss->{'channel'}->{'link'} )->latin1,
           description => Unicode::String::utf8($rss->{'channel'}->{'description'})->latin1,
           nbitems     => $i
          ))if($updatedb);
    $self->debug_msg(4, "found $i items");

    return $html_content;

}


=item C<fetch_channels>

Fetches one or more channel and places it in temporary storage

$cp->fetch_channels($channelid);

=cut
sub fetch_channels {
  my ($self, @channelids) = @_;

  return 0 if ($self->no_process_external());

  $self->reset();

  my $whereclause = 'isa='.DBAccess->quote(Channel->CHANNEL_TYPE_EXTERNAL());

  if (@channelids && defined $channelids[0]) {
      if (scalar(@channelids) == 1) {
          $whereclause .= " AND cid=$channelids[0]";
      } else {
          $whereclause .= " AND cid IN (".join(',', @channelids).")";
      }
  }

  $self->debug_msg(3,"fetch_all_channels(filter=$whereclause)");

  my ($c)=DBAccess->sqlSelectMany("cid, url", "channel", $whereclause, "ORDER BY cid DESC");
  while(my ($cid, $url)=$c->fetchrow()) {
    $self->fetch_one_channel($cid, $url);
  }
  $c->finish();

  return $self->number_processed();
}





=item C<fetch_one_channel>

Fetches a channel and places it in temporary storage

$cp->fetch_one_channel($channelid);
$cp->fetch_one_channel($channelid, $url);

=cut

sub fetch_one_channel {
    my ($self, $cid, $url)=@_;
    my $response;
    my $httpstatus=0;
    my $msg = "";

    $self->debug_msg(3, "fetch_one_channel($cid, $url)\n");

    my $result = 0;

    if(defined $cid) {
        # the url is optional, grab it from the database if necessary
        unless (defined $url) {
            ($url) = DBAccess->sqlSelect('url', 'channel', "cid=$cid");
        }

        if (defined $url) {
            unless( $self->fetched_urls()->{$url} || $self->unavailable_urls()->{$url} ) {

                $self->debug_msg(4, "About to fetch '$url'");

                $response = Utility->fetch_url($url, 10);
                $httpstatus = $response->code;

                if ($httpstatus != 200){
                    $self->debug_msg(4, "httpstatus=$httpstatus: ".HTTP_RETURN_CODES()->{$httpstatus});

                    #should log the problem
                    $msg =  "### error retrieving cid=$cid, $url; http status=$httpstatus: ".HTTP_RETURN_CODES()->{$httpstatus};
                    #logMessage ("$msg");
                    warn ($msg);

                    $self->unavailable_urls()->{$url} = 1;
                } else {
                    ### Store the xml into a file
                    my $channeldir = $self->channel_dir();

                    if (! -d $channeldir ) {
                      mkdir($channeldir, 0755) || warn ("cannot create directory: $channeldir: $?");
                    }


                    my $url_digest = Digest::MD5::md5_hex( $url );

                    $self->debug_msg(4, "in creating file, using $url_digest for $url");

                    open FILE, ">$channeldir/$url_digest.xml" || warn "Cannot write $channeldir/$url_digest.xml\n";
                    flock (FILE, LOCK_EX);
                    seek(FILE, 0, 2);
                    print FILE $response->content;
                    flock (FILE, LOCK_UN);
                    close FILE;

                    $self->fetched_urls()->{$url} = 1;
                }
            }

            #### We finished one channel
            #### here we log the attempt
            DBAccess->sqlUpdate("channel", "cid=$cid",
                lastfetched   => $self->timestamp(),
                httpstatus    => $httpstatus,
                lastprocessed => $self->timestamp(),
            );

            $self->increment_number_processed();
            $result = 1;
        }
    }

    return $result;
}








=item C<do_subscriptions>

Notifies users of messages based on their subscriptions. Returns the number
of notifications which were sent.

  $cp->do_subscriptions(
    frequency   => Subscription->FREQUENCY_ALL(),   # The subscription frequency to process.
                                                    # Default is ALL

    userid      => $userid_filter,                  # The userid of the user to be notified.
                                                    # By default, all subscribers are notified.

    channelid   => $channel_filter,                 # The channel to process. By default,
                                                    # all channels are processed.
  );

The return value is the number successfully processed
If there was an error, it will appear in $cp->error().

=cut

sub do_subscriptions {
    my ($self, %args) = @_;

    my $frequency   = $args{frequency};
    my $userid      = $args{userid};
    my $channelid   = $args{channelid};

    $frequency = Subscription->FREQUENCY_ALL() unless ( defined $frequency && ($frequency eq Subscription->FREQUENCY_DAILY() || $frequency eq Subscription->FREQUENCY_WEEKLY() ));

    $self->debug_msg(3, "------------ at do_subscriptions");
    if ($userid) {
        $self->debug_msg(3, "have userid $userid");
    }

    $self->debug_msg(4, "Processing '$frequency' subscriptions...\n");
    $self->reset();

    my $basename    = $PARAMS->{basename};

    # select channels that have subscriptions with specific frequency
    # CKIM: this was ignoring the -id parameter
    my $channel_where = "channel.cid=subscription.cid and frequency='$frequency'";
    if (defined $channelid) {
        $self->debug_msg(3, "Processing channel $channelid");
        $channel_where = "channel.cid=$channelid AND ".$channel_where;
    }

    $self->debug_msg(4, "Filtering channels based on $channel_where");

    # BQ: Added channel.isa to the GROUP BY for Oracle compatability
    my ($channel_sth) = DBAccess->sqlSelectMany(
          "distinct channel.cid, channel.name, channel.iid, MIN(subscription.processdate) AS lasttime, channel.isa",
          "subscription, channel",
          "$channel_where GROUP BY channel.cid, channel.name, channel.iid, channel.isa"
    );

    while( my ($cid, $channelname, $iid, $lasttime, $channel_isa ) = $channel_sth->fetchrow()) {

        # flag to indicate whether the message should be in html
        my $message_is_html;
        my $line_break = '';

        # For each channel we build a message gathering all new channelitems
        $self->debug_msg(4,
            "in do_subscriptions: Processing channel cid=$cid; name=".(defined $channelname ? $channelname : 'undef')
            ."; lasttime=".(defined $lasttime ? $lasttime : 'undef')
            ."; iid=".(defined $iid ? $iid : 'undef'));

        my $message = "";
        my $topics = "";

        # create the filter for channelitems
        my $channelitem_where = "cid=$cid";

        if (defined $lasttime) {
            $channelitem_where .= " AND creationdate >= '$lasttime'";
        }

        # set a cap in case of severe backlogs
        my $max_age;
        if ($frequency eq Subscription->FREQUENCY_WEEKLY()) {
            $max_age = 86400 * 14;
        } else {
            $max_age = 86400 * 7;
        }
        my $max_age_dbtime = Utility->db_time_format(time - $max_age);

        $channelitem_where .= " AND creationdate >= '$max_age_dbtime'";

        $self->debug_msg(4,
            "---doing db query to find items added after ".(defined $lasttime ? $lasttime : 'undef')
            .": " . "SELECT ciid, name, description, url, creationdate, mid FROM channelitem WHERE $channelitem_where");

        my ($channelitem_sth)= DBAccess->sqlSelectMany(
            "ciid, name, description, url, creationdate, mid",
            "channelitem",
            $channelitem_where,
            "ORDER BY ciid ASC");  # must be ordered this way so that last_ciid is always accurate

        my $count = 1;
        my $authorname = '';
        my $authoremail = '';
        my $date = '';
        my ($isa) = DBAccess->sqlSelect("isa", "instance", "iid=$iid") if($iid);

        while( my ($ciid, $title, $description, $url, $creationdate, $mid, $newsiid )=$channelitem_sth->fetchrow()) {

            # For each channelitem we add the information to the message
            $self->debug_msg(4,
                "in do_subscriptions, Processing new channelitem ciid=".(defined $ciid ? $ciid : 'undef')
                ."; name=".(defined $title ? $title : 'undef')
                ."; creationdate=".(defined $creationdate ? $creationdate : 'undef')
                ."; mid=".(defined $mid ? $mid : 'undef'));

            $title        = "" if (!$title);
            $description  = "" if (!$description);
            $creationdate = "" if (!$creationdate);
            $url          = "" if (!$url);

            $url = "http://$url" if ($url !~ /^http/);

            my $attachment_text = '';

            if ($isa && ($isa eq Channel->CHANNEL_TYPE_DISCUSSION())) {
                my $where = "iid=$iid";
                if (defined($mid) && $mid > 0) {
                    $where .= " AND mid=$mid";
                }

                # flag this as html
                $message_is_html = Discussion->message_content_type_is_html();

                # get the messages
                ($mid, $authorname, $date, $authoremail) = DBAccess->sqlSelect("mid, authorname, creationdate, authoremail", "message", $where);

                $self->debug_msg(4, "This is a discussion message; mid=$mid");

                ### NZ get attachment for message
                if (defined $mid) {
                    $self->debug_msg(4, "Checking for attachment...");
                    my ($attch_id,$attch_name) = DBAccess->sqlSelect("id, file_name","uploads","id = $mid");

                    if( $attch_id ){ ### construct href for attachment
                        $self->debug_msg(4, "An attachment was found.");
                        $attachment_text .= "This message contains an attachment: ";
                        my $attachment_href = "$basename/index.pl?isa=Message&op=download_file&field_name=message_attachment&discussion_id=$iid&mid=$mid";

                        # If the message is HTML, wrap in <a> element and escape the href
                        if ($message_is_html) {
                            $attachment_text .= qq~<a href="~.Utility->escapeHREF($attachment_href).qq~">Click here to download</a> ($attch_name)~;
                            $attachment_text = "<p>\n$attachment_text\n</p>\n";
                        } else {
                            $attachment_text .= $attachment_href." ($attch_name)\n\n";
                        }
                        $self->debug_msg(5, "Attachment text is:\n$attachment_text\n");
                    }

                }
            }

            if ($frequency ne 'all') {

                ### GENERATE A DIGEST MESSAGE ###

                my $message_link = $url;

                # internal messages are in html
                $message_is_html ||= ($channel_isa && ($channel_isa eq Channel->CHANNEL_TYPE_INTERNAL()));

                if ($message_is_html) {
                    $line_break   = '<br />';
                    if ($message_link) {
                        $message_link = qq~<a href="~.Utility->escapeHREF($message_link).qq~">$message_link</a>~;
                    }
                } else {
                    $line_break = "\n";
                }


                # add the current channelitem to message
                $topics .= $count . ") $title$line_break";
                $message .= "\n\nMessage ".$count++. ": $title$line_break";
                $message .= "Author:  $authorname$line_break";
                if ($date) {
                    $message .= "Date:    $date$line_break";
                }
                $message .= HORIZONTAL_RULE."$line_break";
                $message .= $description . "$line_break$line_break" if $description;
                $message .= $attachment_text;
                $message .= "To view this message, click here:$line_break    $message_link$line_break" if $message_link;
                $message .= "$line_break";

                if ($isa eq Channel->CHANNEL_TYPE_DISCUSSION()) {
                    my $cgi_title = $CGI->escape($title);
                    my $server_name = MetadotConfig->get_server_name() || $ENV{'SERVER_NAME'} || '';
                    my $message_id2 = "+" . $server_name . "+" . $iid . "+" . $mid;

                    # see if the discussion has an email alias.
                    my $sender_email;
                    if ($PARAMS->{allow_discussion_email_alias}) {
                        my $discussion = $isa->restore($iid);
                        $sender_email = $discussion->get_email_alias({full=>1});
                    }
                    unless ($sender_email) {
                        $sender_email = $self->determine_mail_sender_address();
                    }

                    # had to fix the quotes
                    my $email = "\"+$server_name+$iid+$mid\" <$sender_email>";
                    my $email_escape = $CGI->escape($email);
                    
                    my $reply_message;
                    if ($message_is_html) {
                        $reply_message = qq~<a href="mailto:$email_escape">Click here</a> to reply to this message.~;
                    } else {
                        #CKIM: the line break above the "If the link above..." line is INTENTIONAL, please nobody
                        #      remove it.

                        $reply_message = "To reply to this message, click here:${line_break}mailto:$email_escape?subject=$cgi_title${line_break}${line_break}";
                        $reply_message .= "If the link above is not clickable, please send an email to the following address:${line_break}  $email";
                    }

                    $message .= $line_break . $reply_message . $line_break . $line_break . $line_break;
                }
            } else {

                ### GENERATE AN ASAP MESSAGE ###

                my $uctitle = "";   # aju 09-9-01: why was this needed??
                if ($description ne '') {
                    $description .= "\n";
                }

                #CKIM: cleaned up some ugly code, and restored message headers
                if ($isa && ($isa eq Channel->CHANNEL_TYPE_DISCUSSION())) {
                    $message = "\n$description";
                    $message .= "\n$attachment_text" if ($attachment_text);

                } elsif ($channel_isa && ($channel_isa eq Channel->CHANNEL_TYPE_INTERNAL())) {
                    # these are formatted in HTML
                    $message = <<EOM
<h1>$uctitle</h1>
$description
<hr/><a href="$url">$url</a>
Created on: $creationdate<br />
EOM
;
                } else {
                    $message = " $uctitle\n$description\n$url\n$creationdate\n\n";
                }

                $self->debug_msg(4, "Sending to users $title");
                my $result = $self->send_to_users(
                    iid             => $iid,
                    newsiid         => $newsiid,
                    ciid            => $ciid,
                    cid             => $cid,
                    channelname     => $channelname,
                    frequency       => $frequency,
                    message         => $message,
                    processdate     => $creationdate,
                    userid          => $userid,
                    title           => $title,
                    mid             => $mid,
                    authorname      => $authorname,
                    date            => $date,
                    authoremail     => $authoremail,
                );
                $self->debug_msg(4, "Message was sent with return code '$result'");
                return $self->number_processed() if ($self->error());
            }
        }

        ### finished all channelitems
        $channelitem_sth->finish();

        if ($frequency ne 'all' && $message ne '') {
            $message = $line_break . $topics . $line_break . $message;
            my $error = $self->send_to_users(
                iid             => $iid,
                cid             => $cid,
                channelname     => $channelname,
                frequency       => $frequency,
                message         => $message,
                userid          => $userid,
            );
            return $self->number_processed() if ($self->error());
        }

        # update timestamp last subscription proccessing for all user subscriptions of that frequency
        $self->debug_msg(3, "End of processing channel $cid, recording processing timestamp\n");

        # newsitems handle their own
        DBAccess->sqlUpdate("subscription", "cid=$cid and frequency='$frequency'", (processdate => $self->timestamp()));

    }
    $channel_sth->finish();

    $self->debug_msg(3, "Sent ".$self->number_processed()." notification(s)");

    $self->debug_msg(5, "End of do_subscriptions");

    return $self->number_processed();
}










=item C<send_to_users>

Looks up all subscriptions for a given channel. Filters the subscriptions by
processdate and frequency. Finds all users who should be notified by the
subscription (individual or group of users). Sends the message by the medium
of their choice. Marks the subscription as processed according to the current
timestamp.


    my $count = $cp->send_to_users(
        frequency       => $frequency,      # the frequency
        message         => $message,        # the message to send
        date            => $date,           # the date to be displayed on the message
        processdate     => $creationdate,   # the processdate of the news. If the
                                            # subscription was processed since then,
                                            # the news will not be sent, since it is
                                            # most likely a repeat.

        title           => $title,          # the title of the message
        authorname      => $authorname,     # the name of the author
        authoremail     => $authoremail,    # the email of the author

        iid             => $iid,            # the iid, if any of the instance that generated the news
        newsiid         => $newsiid,        # the newsiid ??
        ciid            => $ciid,           # the channelitem id
        cid             => $cid,            # the channel id
        channelname     => $channelname,    # the channel name
        userid          => $userid,         # the userid to filter on (optional). Note, if
                                            # a userid is specified, this will not catch
                                            # the user's group subscriptions, because this
                                            # would prevent the rest of the group from later
                                            # being notified.
        mid             => $mid,
    );

=cut

sub send_to_users {
    my ($self, %args) = @_;

    my $iid             = $args{iid};
    my $newsiid         = $args{newsiid};
    my $ciid            = $args{ciid};
    my $cid             = $args{cid};
    my $channelname     = $args{channelname};
    my $frequency       = $args{frequency};
    my $message         = $args{message};
    my $processdate     = $args{processdate};
    my $userid          = $args{userid};
    my $title           = $args{title};
    my $mid             = $args{mid};
    my $authorname      = $args{authorname};
    my $date            = $args{date};
    my $authoremail     = $args{authoremail};

    if($message ne "") {
        # [vadim] get server_name
        my $server_name = MetadotConfig->get_server_name() || $ENV{'SERVER_NAME'} || "";

        my $isa = '';
        my $newsisa = '';

        my $object;

        my $user;
        my $attch_base = '';
        $self->debug_msg(3,
            "in send_to_users with:\n iid=".(defined $iid ? $iid : 'undef')
            ."; isa=".(defined $isa ? $isa : 'undef')
            ."; ciid=".(defined $cid ? $cid : 'undef')
            ."; processdate=".(defined $processdate ? $processdate : 'undef')
            ."; uid=".(defined $userid ? $userid : 'undef')
            ."; ciid=".(defined $ciid ? $ciid : 'undef')
            .";\n title=".($title || '')
            ."\n ");

        my $sender_email = $self->determine_mail_sender_address();

        my $show_discussion_channel_in_subject = $PARAMS->{show_discussion_channel_in_subject};

        # determine the instance class
        ($isa) = DBAccess->sqlSelect("isa", "instance", "iid=$iid") if($iid);
        ($newsisa) = DBAccess->sqlSelect("isa", "instance", "iid=$newsiid") if($newsiid);
            $self->debug_msg(4,
                "isa(".(defined $iid ? $iid : 'undef').")='"
                .(defined $isa ? $isa : 'undef')
                ."'; newsisa(".(defined $newsiid ? $newsiid : 'undef')
                .")='".(defined $newsisa ? $newsisa : 'undef')."'");

        # get the object
        if (defined($newsisa) && $newsisa ne '') {
            require "Gizmo/$newsisa.pm";
            $object = $newsisa->new($newsiid);
        } elsif (defined($isa) && $isa ne '') {
            require "Gizmo/$isa.pm";
            $object = $isa->new($iid);
        }

        # see if the object has an email alias.
        if (defined($object) && $object->can('get_email_alias') && $PARAMS->{allow_discussion_email_alias}) {
            my $alias = $object->get_email_alias({full=>1});
            if ($alias) {
                $sender_email = $alias;
            }
        }


        # get the basename for the link-back
        my $basename = $PARAMS->{basename};

        # CKIM: now that we have group subscriptions, we will collect them into a list of hashes.

        # store the opt-outs
        my $opt_outs = undef;

        # get the subscriptions
        my $where = "frequency='$frequency' AND cid=$cid AND opt_out IS NULL";

        if (defined $ciid) {
            $where .= " AND (last_ciid IS NULL OR last_ciid<$ciid)";
        }

        if ($userid) {
            $where .= " AND userid=$userid";
        }

        $self->debug_msg(5, "Checking for subscriptions WHERE $where");
        my $c = DBAccess->sqlSelectMany("sid, userid, grp_id, medium, optional, last_ciid, processdate", "subscription", $where);

        # allows us to track which users have already received emails
        my %visited_users = ();

        my $HORIZONTAL_RULE = HORIZONTAL_RULE();

        while( my ($sid, $uid, $grp_id, $medium, $optional, $last_ciid, $s_processdate)=$c->fetchrow()) {
            $self->debug_msg(5,
                "Got subscription $sid, last_ciid=".(defined $last_ciid ? $last_ciid : 'undef')
                ."; processdate=".(defined $s_processdate ? $s_processdate : 'undef'));

            # in case last_ciid is null but processdate is not, then we need to
            # check the processdate to make sure this channelitem was not already
            # sent with this subscription
            if (!defined($last_ciid) && defined($s_processdate) && defined($processdate)) {
                if ($s_processdate gt $processdate) {
                    $self->debug_msg(5, "Skipped this subscription because '$s_processdate' is later than '$processdate'");
                    next;
                }
            }

            my @uids = ();
            if (defined $uid) {
                # individual suscription
                push @uids, $uid;
                $self->debug_msg(5, "Got individual subscriber $uid");
            }

            if (defined $grp_id) {
                # group subscription. I get all the userids from the groups, then subtract all the opt-outs
                my @group_uids = Group->get_member_uids($grp_id, 'recurse');
                $self->debug_msg(5, "Got group uids @group_uids");

                # check for opt-outs
                if ($optional) {
                    $self->debug_msg(5, 'This group subscription is optional. Will check for opt-outers.');

                    # see if we already have the opt-outs for this channel
                    unless (defined $opt_outs) {
                        # get all the opt-outs for this channel
                        $opt_outs = {};
                        my $opt_out_sth = DBAccess->sqlSelectMany('userid', 'subscription', "cid=$cid AND opt_out = 1");
                        while (my ($opt_out_uid) = $opt_out_sth->fetchrow()) {
                            $opt_outs->{$opt_out_uid}++;
                            $self->debug_msg(5, "User $opt_out_uid opted out of channel $cid");
                        }
                        $opt_out_sth->finish();
                    }

                    # now check the opt-outs
                    foreach my $check_uid (@group_uids) {
                        unless (exists $opt_outs->{$check_uid}) {
                            push @uids, $check_uid;
                            $self->debug_msg(5, "User $check_uid was part of the group subscription");
                        }
                    }

                } else {
                    push @uids, @group_uids;
                    $self->debug_msg(5, 'This group subscription was not optional. All uids in the group are added.');
                }
            }

            # store the uid/medium combination
            $self->debug_msg(5, "Will send message to uid's @uids");

            foreach my $uid (@uids) {
                $self->debug_msg(5, "Processing subscription uid $uid");

                # avoid sending duplicate emails if multiple subscriptions apply
                next if ($visited_users{$uid}{$medium}++);

                # We sent the message to each user who have subscribed to this channel
                $self->debug_msg(4, "Processing user uid=$uid - medium=$medium");

                ## do permission checks
                #CKIM: the permission checks were only being performed for newsiid, not iid
                if ($object) {
                    $user = Metadot::UserFactory->get_user($uid);
                    $self->debug_msg(4, "Checking permission for userid: $uid, email: $user->{email}");

                    ## consider user logged in if accepted
                    $user->{approval_needed} ||= '';  #prevent a perl warning
                    $user->{_logged_in} = ($user->{approval_needed} ne 'rejected' || $user->{approval_needed} ne 'required') ? 1 : 0;
                    next unless $object->is_allowed_to_do('show', $user);
                    $self->debug_msg(4, "user is allowed to view this message");
                }

                ##############################
                # send the message to the user
                ##############################
                if ($medium eq "email") {
                    my  $subject;
                    my $mail_content_type;

                    if($isa && ($isa eq Channel->CHANNEL_TYPE_DISCUSSION())) {

                        $mail_content_type = Discussion->get_message_content_type();

                        my $line_break;
                        if ($mail_content_type =~ /html/) {
                            $line_break = "<br />";
                        } else {
                            $line_break = "\n";
                        }



                        if ($frequency eq Subscription->FREQUENCY_DAILY()) {
                            $subject = "[$channelname] - Daily Digest";
                        } elsif ($frequency eq Subscription->FREQUENCY_WEEKLY()) {
                            $subject = "[$channelname] - Weekly Digest";
                        } elsif ($show_discussion_channel_in_subject && $subject !~ m/\[$channelname\]/) {
                            $subject = "[$channelname] $title";
                        } else {
                            $subject = $title;
                        }

                        my($email, $password) = DBAccess->sqlSelect("email, password", "users", "userid=$uid");

                        unless (Utility->is_valid_email($email) || $email == 'mel@localhost') {
                            $self->debug_msg(2, "Found bad email address '$email'");
                            next;
                        }

                        my $msg ="";

                        # CKIM: restored this, because it's needed by users.
                        $msg = <<EOM
From:    $channelname discussion <$sender_email>$line_break
Discussion: $channelname$line_break
EOM
;

                        unless ($frequency ne 'all') {
                            $msg .= <<EOM
Author:  $authorname$line_break
Date:    $date$line_break
Subject: $subject$line_break$line_break
EOM
;
                        }

                        my $discussion_footer_link;
                        my $unsubscribe_footer_link;

                        if( $mail_content_type =~ /html/ ){
                            $discussion_footer_link  = "<a href='$basename/index.pl?iid=$iid'>click here</a>";
                            $unsubscribe_footer_link = "<a href='$basename/userchannel.pl?id=$cid&isa=NewsChannel&op=preview'>click here</a>";
                        } else {
                            $discussion_footer_link  = "go to $basename/index.pl?iid=$iid";
                            $unsubscribe_footer_link = "go to $basename/userchannel.pl?id=$cid&isa=NewsChannel&op=preview";
                        }
                        $msg .= <<EMAIL_FOOTER

$message

$line_break$line_break
$HORIZONTAL_RULE$line_break
$HORIZONTAL_RULE$line_break
To view this discussion
$discussion_footer_link$line_break$line_break

To cancel your subscription or modify your subscription details  
$unsubscribe_footer_link$line_break

$HORIZONTAL_RULE
EMAIL_FOOTER
;
                        my $result;

                        #my $sm = Sendmail->new($email, '', '',$subject, $msg);
                        my $sm = $self->get_sendmail_object(to=>$email, subject=>$subject, message=>$msg, content_type=>$mail_content_type);
                        $sm->set_from("$channelname discussion <$sender_email>");

                        if ($self->fetch_flag()) {
                            my $message_id2 = "+" . $server_name . "+" . $iid . "+" . $mid;
                            #my $addl_params = {from_name => "$channelname", from_sender => "$sender_email"};
                            $sm->set_reply_to("\"$message_id2\" <$sender_email>");
                            $sm->set_message_id($message_id2);
                        }

                        $result = $sm->sendMail();

                        if ($result) {
                            $self->increment_number_processed();
                        } else {
                            # moved: $c->finish();
                            return $self->send_mail_failed();
                        }

                    } else {
                        # email subscription
                        $self->notify_email($uid, $channelname, $message, $frequency, "No", $title, $cid);
                        return $self->number_processed() if ($self->error());

                        $self->debug_msg(4, "Sending email subscription (frequency=$frequency) to uid=$uid");
                        $self->debug_msg(5, $message);
                    }
                } elsif ($medium eq "pager") {
                    #send to pager email
                    $self->notify_pager($uid, $channelname, $message, $frequency, "No", $title);
                    return $self->number_processed() if ($self->error());

                    $self->debug_msg(4, "Sending pager subscription (frequency=$frequency) to uid=$uid");
                    $self->debug_msg(5, $message);
                } elsif ($medium eq "icq") {
                    #send message to icq
                    $self->debug_msg(4, "Sending ICQ subscription (frequency=$frequency) to uid=$uid");
                    $self->notify_icq($uid, $channelname, $message, $frequency, "No", $title);
                    return $self->number_processed() if ($self->error());
                }


            } # each uid

            # update the subscription with the current item's ciid and processdate so it won't
            # repeat older messages
            if (defined $processdate || defined $ciid) {
                $self->debug_msg(4, "Marking subscription $sid as complete as of $processdate");
                DBAccess->sqlUpdate("subscription", "sid=$sid", (last_ciid=>$ciid, processdate => $processdate));
            }
        }  # each sid

        $c->finish();

    }

    return $self->number_processed();
}




sub determine_mail_sender_address {
    my $self = shift;
    my $sender_email;

    # use the 'discussion account' email if available, else just use the default.
    my $sender_login    = $PARAMS->{subscriptionsender} || '';
    my $sender_host     = $PARAMS->{subscriptionhost} || '';

    $self->debug_msg(4, "determine_mail_sender_address starts with sender_login[$sender_login], sender_host[$sender_host]. fetching? ".($self->fetch_flag() ? 'yes' : 'no'));

    # some implementations (SPE) prefer to use the global mailsenderaddress
    if ($sender_login && $sender_host && $self->fetch_flag() && !MetadotConfig->get_param_value('metadotd_always_uses_global_mailsenderaddress')) {
        $sender_email = "$sender_login\@$sender_host";
    } else {
        $sender_email = $PARAMS->{mailsenderaddress};
    }

    $self->debug_msg(4, "determine_mail_sender_address returns $sender_email");
    return $sender_email;
}



=item C<notify_pager>

Sends a subscription notification via pager.

$cp->notify_pager($uid, $channelname, $message, $frequency, $urg_flag, $title);

=cut

sub notify_pager {
  my ($self, $uid, $channelname, $message, $frequency, $urg_flag, $title)=@_;

  if (!$uid || !$message){
    return $self->notify_error("notify_pager: there is a problem: uid=$uid or message is null???!!");
  }

  # get user info.
  my($fullname, $nickname, $pager)=DBAccess->sqlSelect("fullname, nickname, pager", "users", "userid=$uid");

  # check for errors in user info.
  $fullname = $nickname || $fullname;
  if (!$pager){
    return $self->notify_error("notify_pager: there is a problem user: uid=$uid $fullname -$nickname- doesn't appear to have a pager address -$pager-!");
  }

  # build pager message
  my $to = $pager;
  my $cc="";
  my $bcc="";
  my $urgent = "No" if (!$urg_flag);
  my $subject = $channelname;

  $message .= "\n-- MetaDot subscription.";

  if (Utility->is_valid_email($to) || $to == 'mel@localhost' ) {
      #my $sm = Sendmail->new($to, $cc, $bcc,$subject, $message);
      my $sm = $self->get_sendmail_object(to=>$to, cc=>$cc, bcc=>$bcc, subject=>$subject, message=>$message);

      if ($sm->sendMail()) {
          $self->increment_number_processed();
      } else {
          return $self->send_mail_failed();
      }
  } else {
      $self->debug_msg(2, "Found bad email address '$to'");
  }
}






=item C<notify_icq>

Sends a subscription notification via icq.

$cp->notify_pager($uid, $channelname, $message, $frequency, $urg_flag, $title);

=cut

sub notify_icq {
  my ($self, $uid, $channelname, $message, $frequency, $urg_flag, $title)=@_;

  if (!$uid || !$message){
    return $self->notify_error("notify_icq: there is a problem: uid=$uid or message is null???!!");
  }

  # get user info.
  my($fullname, $nickname, $icqnumber)=DBAccess->sqlSelect("fullname, nickname, icqnumber", "users", "userid=$uid");

  # check for errors in user info.
  $fullname = $nickname || $fullname;
  if (!$icqnumber){
    return $self->notify_error("notify_icq: there is a problem user: uid=$uid $fullname -$nickname- doesn't appear to have an ICQ numner  -$icqnumber-!");
  }

  # build icq message



  my $to = "$icqnumber\@pager.icq.com";
# my $subject = $channelname;
  my $subject = $title;
  $self->debug_msg(4, "notify_icq: orig_message=$message");

  ## remove links otherwise ICQ will not send
  $message =~ s/(http:\/\/.*)\s//g;
  $message .= "\n$channelname";
  $message .= "\nVisit your MetaDot site for more details.";
  $message .= "\n-- MetaDot subscription.";
  $self->debug_msg(3, "notify_icq: sent_message=$message\nto:$to subject:$subject");

  if (Utility->is_valid_email($to) || $to == 'mel@localhost' ) {
      #my $sm = Sendmail->new($to, '', '',$subject, $message);
      #$sm->set_debug($self->debug_level());

      my $sm = $self->get_sendmail_object(to=>$to, subject=>$subject, message=>$message);

      if ($sm->sendMail()) {
          $self->increment_number_processed();
      } else {
          return $self->send_mail_failed();
      }
  } else {
      $self->debug_msg(2, "Found bad email address '$to'");
  }

#  $ua->agent("AgentName/0.1 " . $ua->agent);
#  my $url = "http://wwp.icq.com/scripts/WWPMsg.dll";
#  my $parameters = "to=$to&from=$from&fromemail=$from&subject=$subject&body=$message";
#  my $req = new HTTP::Request(POST => $url);
#     $req->content_type('application/x-www-form-urlencoded');
#     $req->content($parameters);
#  my ($result) = $ua->request($req);


}





=item C<notify_email>

Sends a subscription notification via email.

$cp->notify_pager($uid, $channelname, $message, $frequency, $urg_flag, $title, $cid);

=cut

sub notify_email {
    my ($self, $uid, $channelname, $message, $frequency, $urg_flag, $title, $cid)=@_;

    # just in case this needs to be changed later
    my $format_html = 1;

    if (!$uid || !$message){
      return $self->notify_error("notify_email: there is a problem: uid=$uid or message is null???!!");
    }

    #
    # select user information
    #
    my($fullname, $nickname, $email, $password)=DBAccess->sqlSelect("fullname, nickname, email, password", "users", "userid=$uid");

    #
    # check if user is alright...
    #
    $fullname ||= $nickname;
    $nickname ||= $fullname;
    if (!$email){
        return $self->notify_error("notify_email: there is a problem user: uid=$uid $fullname -$nickname- doesn't appear to have an email address -$email-!");
    }

    #
    # build email
    #
    my $to=$email;
    my $cc="";
    my $bcc="";
    my $urgent = "No" if(!$urg_flag);
    my $subject= $channelname;

    $subject .= ":" . $title if ($frequency eq Subscription->FREQUENCY_ALL());
    $subject .= " - Daily Digest" if ($frequency eq Subscription->FREQUENCY_DAILY());
    $subject .= " - Weekly Digest" if ($frequency eq Subscription->FREQUENCY_WEEKLY());

    my $basename = $PARAMS->{basename};
    my $m        = $PARAMS->{subsmessage};

    my $forgot_password_url = "$basename/index.pl?isa=Session&op=forgot_password";
    my $website_url         = "$basename/index.pl";
    my $profile_url         = "$basename/index.pl?isa=User&op=modify_self";
    my $subscription_url    = "$basename/userchannel.pl?id=$cid&isa=NewsChannel&op=preview";

    if ($format_html) {
        $forgot_password_url    = qq~<a href="$forgot_password_url">$forgot_password_url</a>~;
        $website_url            = qq~<a href="$website_url">$website_url</a>~;
        $profile_url            = qq~<a href="$profile_url">$profile_url</a>~;
        $subscription_url       = qq~<a href="$subscription_url">$subscription_url</a>~;
    }

    $m=~s/<<forgot_pwd_url>>/$forgot_password_url/gi;
    $m=~s/<<website_url>>/$website_url/gi;
    $m=~s/<<email>>/$email/gi;
    $m=~s/<<fullname>>/$fullname/gi;
    $m=~s/<<nickname>>/$nickname/gi;
    $m=~s/<<password>>/$password/gi;
    $m=~s/<<profile_url>>/$profile_url/gi;
    $m=~s/<<subscription_url>>/$subscription_url/gi;

    $message .= $m;

    ## mike: NEWSITEMS' EMAILS ARE SENT AS HTML SINCE ABSTRACTS ARE HTML
    if ($format_html) {
        $message =~ s/\n/<br \/>/g;
    }


    if (Utility->is_valid_email($to) || $to == 'mel@localhost' ) {
      #my $sm = Sendmail->new($to, $cc, $bcc, $subject, $message);

      my $sm = $self->get_sendmail_object(to=>$to, cc=>$cc, bcc=>$bcc, subject=>$subject, message=>$message, content_type=>'html');

      if ($sm->sendMail()) {
          $self->increment_number_processed();
      } else {
          return $self->send_mail_failed();
      }
    } else {
      $self->debug_msg(2, "Found bad email address '$to'");
    }

}





=item C<get_sendmail_object>

Gets a sendmail object.

my $sm = $cp->get_sendmail_object(
    to      => $to,
    cc      => $cc,
    bcc     => $bcc,
    subject => $subject,
    message => $message,
);

=cut
sub get_sendmail_object {
    my ($self, %args) = @_;

    my $sm_to       = $args{to};
    my $sm_cc       = $args{cc};
    my $sm_bcc      = $args{bcc};
    my $sm_subject  = $args{subject};
    my $sm_body     = $args{body} || $args{message};
    my $content_type    = $args{content_type};

    my $result;
    my $sm_class = $self->sendmail_class() || 'Sendmail';
    my $package = Utility->package_to_filename($sm_class);
    eval {
        require $package;
        $result = $sm_class->new({
            to              => $sm_to,
            cc              => $sm_cc,
            bcc             => $sm_bcc,
            subject         => $sm_subject,
            body            => $sm_body,
            debug_level     => $self->debug_level(),
            suppress_emails => $self->silent(),
            content_type    => $content_type,
        });
    };
    if ($@) {
        die("Error instantiating sendmail class $sm_class:\n".$@);
    }

    return $result;
}


=item C<send_mail_failed>

Indicate that sendmail has failed.

$cp->send_mail_failed();

=cut
sub send_mail_failed {
    my $self = shift;

    my $abort_msg = "Errors occurred while attempting to send mail. Please check your error logs for additional details.";
    return $self->notify_error($abort_msg);
}



=item C<notify_error>

Indicate that some error has occurred.

$cp->notify_error();

=cut
sub notify_error {
    my $self = shift;
    my $error_msg = shift || 'An error was encountered. No additional information was provided. Please check your error logs for details.';
    $self->debug_msg(0, $error_msg);
    $self->error($error_msg);

    return $self->number_processed();
}




=item C<delete_old_news>

Delete news older than a certain time. Default is one week ago.

$cp->delete_old_news();


=cut

sub delete_old_news {
    my $self        = shift;
    my $threshold   = shift || time - ( 86400 * 7 );

    my $datetime = Utility->db_time_format($threshold);

    $self->debug_msg(2, "Deleting old news where creationdate <= '$datetime'");
    DBAccess->sqlDelete('channelitem',"urgent='oldnews' AND creationdate <= ".DBAccess->quote($datetime));
}


=item C<increment_number_processed>

Increments the counter of items processed. Default increment is 1.

$cp->increment_number_processed();
$cp->increment_number_processed(3);

=cut
sub increment_number_processed {
    my $self        = shift;
    my $increment   = shift;

    $increment = 1 unless (defined $increment);
    $self->debug_msg(5, "increment_number_processed by $increment");

    return $self->number_processed($increment + $self->number_processed());
}



=item C<reset>

Resets status indicators, such as number_processed and error. Also resets
persistent information that should not carry over from one vserver to the next.

$cp->reset();

=cut
sub reset {
    my $self = shift;

    $self->number_processed(0);
    $self->error(undef);
    $self->fetched_urls({});
}



sub debug_msg {
    my ($self, $level, $msg) = @_;

    return if (!defined $level);
    return if (!defined $msg);

    my $eol="\n";
    my $head = "";

    if ($level <= $self->debug_level()) {
        print STDERR " $head debug: $msg$eol";
        #print "************* debug: $msg$eol";
    }
}





use constant HTTP_RETURN_CODES =>
{
    100 => 'Continue',
    101 => 'Switching Protocols',
    200 => 'OK',
    201 => 'Created',
    202 => 'Accepted',
    203 => 'Non-Authoritative Information',
    204 => 'No Content',
    205 => 'Reset Content',
    206 => 'Partial Content',
    300 => 'Multiple Choices',
    301 => 'Moved Permanently',
    302 => 'Moved Temporarily',
    303 => 'See Other',
    304 => 'Not Modified',
    305 => 'Use Proxy',
    400 => 'Bad Request',
    401 => 'Unauthorized',
    402 => 'Payment Required',
    403 => 'Forbidden',
    404 => 'Not Found',
    405 => 'Method Not Allowed',
    406 => 'Not Acceptable',
    407 => 'Proxy Authentication Required',
    408 => 'Request Time-out',
    409 => 'Conflict',
    410 => 'Gone',
    411 => 'Length Required',
    412 => 'Precondition Failed',
    413 => 'Request Entity Too Large',
    414 => 'Request-URI Too Large',
    415 => 'Unsupported Media Type',
    500 => 'Internal Server Error',
    501 => 'Not Implemented',
    502 => 'Bad Gateway',
    503 => 'Service Unavailable',
    504 => 'Gateway Time-out',
    505 => 'HTTP Version not supported'
};












1;




