#!/usr/bin/perl -w
#
# Test file for Metadot::ChannelProcessor.pm
# $Id: ChannelProcessor.001.t,v 1.1.2.3 2005/07/01 22:41:28 nico Exp $
#

use strict;
use Test::More tests=>25;

use SmokeMetadot;
use DBAccess;
use MetadotConfig;
use Metadot::PortalFactory;
use Subscription;
use Gizmo::Category;
use Gizmo::Newsitem;


# set up info
my $TEST_CLASS   = 'Metadot::ChannelProcessor';
my $HOME_PAGE = Metadot::PortalFactory->get_home_object();
my $ADMIN_UID = MetadotConfig->get_reserved_uid('ADMIN');

my @USERS;
my ($CHANNEL_PAGE, $CHANNEL_PAGE_ID, $CHANNEL, $CHANNEL_ID);

my @NEWSITEMS;
my $CP;

################################################################################
# Test: Use the test class
################################################################################
{
    use_ok($TEST_CLASS);
}


################################################################################
# Test: Create a category
################################################################################
{
    my $test_page_name = '_TEST_PAGE_'.$0;

    $CHANNEL_PAGE = Category->new({
        uid => $ADMIN_UID,
        parent_id => $HOME_PAGE->id(),
        name => $test_page_name,
    });
}


################################################################################
# Test: Add the category to the db
################################################################################
{
    $CHANNEL_PAGE_ID = $CHANNEL_PAGE->add();
    ok(Utility->is_valid_integer($CHANNEL_PAGE_ID), "Got a valid iid $CHANNEL_PAGE_ID");
}


################################################################################
# Test: Create a channel for the page
################################################################################
{
    $CHANNEL = $CHANNEL_PAGE->create_channel();
    $CHANNEL_ID = $CHANNEL->get_cid();
    ok(Utility->is_valid_integer($CHANNEL_ID), 'Channel was created');
}


################################################################################
# Test: Subscribe the USERS to the channel
################################################################################
{
    for (my $i = 0; $i<3; $i++) {
        my $user = SmokeMetadot->create_test_user();
        push @USERS, $user;
        my $userid = $user->uid();
        ok(       
            Subscription->add_user_subscription({
                userid      => $userid,
                cid         => $CHANNEL_ID,
                frequency   => Subscription->FREQUENCY_ALL(),
                medium      => Subscription->MEDIUM_EMAIL(),
            }),
            "Subscribed user $userid to channel"
        );
    }
}



################################################################################
# Test: Check that subscriptions records exist
################################################################################
{
    my ($subscription_count) = DBAccess->sqlSelect('count(*)', 'subscription', 'cid='.$CHANNEL_ID);

    my $expected_count = @USERS;
    ok($subscription_count == $expected_count, "Subscription record was created, $expected_count expected");
}


################################################################################
# Test: Add some NEWSITEMS to the channel
################################################################################
{
    for (my $i = 0; $i<5; $i++) {
        my $newsitem_name = 'Test news item '.$i;
        my $newsitem_body = "News item $i body body body";
        my $newsitem = Newsitem->new({
            uid         => $ADMIN_UID,
            parent_id   => $CHANNEL_PAGE_ID,
            name        => $newsitem_name,
            description => $newsitem_body,
        });

        ok((defined $newsitem && $newsitem->add()), "Added a news item");
        push @NEWSITEMS, $newsitem;
    }
}


################################################################################
# Test: process the channel items
################################################################################
{
    my ($result, $item_count);

    ok($CP = &get_channel_processor(), "Instantiated the channel processor");    
    ok($result = $CP->do_channels($CHANNEL_ID), "Processed items for channel $CHANNEL_ID, got result '$result'");
    
    my $expected_count = @NEWSITEMS;
    ok(($item_count = $CP->number_processed()) && ($item_count == $expected_count), "$item_count channelitems were processed, $expected_count expected");
}



################################################################################
# Test: process the user subscriptions
################################################################################
{
    my ($result, $item_count);

    ok($result = $CP->do_subscriptions(channelid=>$CHANNEL_ID), "Processed subscriptions for channel $CHANNEL_ID, got result '$result'");

    my $expected_count = scalar(@NEWSITEMS) * scalar(@USERS);
    ok(($item_count = $CP->number_processed()) && ($item_count == $expected_count), "$item_count subscription notifications were processed, $expected_count expected");
}


################################################################################
# Test: process the user subscriptions redundantly
################################################################################
{
    my ($result, $item_count);

    # get a new channel processor
    $CP = &get_channel_processor();
    
    ok(0 == $CP->do_subscriptions(channelid=>$CHANNEL_ID), "Processed subscriptions for channel $CHANNEL_ID, no redundant messages.");
}


################################################################################
# Test: gracefully skip invalid emails
################################################################################
{
    my ($result, $item_count);

    my $previous_email = $USERS[0]->email();
    $USERS[0]->email($previous_email.'f3!');
    $USERS[0]->save();

    my $newsitem = Newsitem->new({
            uid         => $ADMIN_UID,
            parent_id   => $CHANNEL_PAGE_ID,
            name        => 'test of graceful email handling',
            description => 'no body',
        });

    $newsitem->add();
    push @NEWSITEMS, $newsitem;

    sleep(1); # avoid having the channel's processdate overlap with the newsitem's creation date, to get an accurate count
    
    # get a new channel processor
    $CP = &get_channel_processor();

    ok($result = $CP->do_channels($CHANNEL_ID), "Test invalid email: processed channel $CHANNEL_ID, got result '$result'");
    
    ok($result = $CP->do_subscriptions(channelid=>$CHANNEL_ID), "Test invalid email: processed subscriptions for channel $CHANNEL_ID, got result '$result'");

    my $expected_count = @USERS - 1;
    ok(($item_count = $CP->number_processed()) && ($item_count == $expected_count), "$item_count subscription notifications were processed, $expected_count expected");

    # restore the original email
    $USERS[0]->email($previous_email);
    $USERS[0]->save();

}
























################################################################################
# Test: Remove a user from the db, look for dangling subscriptions
################################################################################
{
    my $user = shift @USERS;
    my $userid = $user->uid();

    $user->delete(1);
    my ($subscription_count) = DBAccess->sqlSelect('count(*)', 'subscription', 'userid='.$userid);
    ok($subscription_count == 0, "User subscriptions records were removed");
}



################################################################################
# Test: Channel record was removed from the db
################################################################################
{
    $CHANNEL->delete();
    my ($channel_count) = DBAccess->sqlSelect('count(*)', 'channel', 'cid='.$CHANNEL_ID);
    ok($channel_count == 0, "Channel records were removed");
}


################################################################################
# Test: Channelitem records was removed from the db
################################################################################
{
    my ($channel_count) = DBAccess->sqlSelect('count(*)', 'channelitem', 'cid='.$CHANNEL_ID);
    ok($channel_count == 0, "Channelitem records were removed");
}


################################################################################
# Test: Check that subscriptions were removed when channel removed
################################################################################
{
    my ($subscription_count) = DBAccess->sqlSelect('count(*)', 'subscription', 'cid='.$CHANNEL_ID);
    ok($subscription_count == 0, "Subscription records were deleted");
}



################################################################################
# Cleanup
################################################################################
{
    #print STDERR "Deleting newsitems...\n";
    foreach my $newsitem (@NEWSITEMS) {
        #print STDERR "Deleting newsitem $newsitem\n";
        $newsitem->delete();
    }
    #print STDERR "Newsitems deleted.\n";

    #print STDERR "Deleting page...\n";
    $CHANNEL_PAGE->delete();
    #print STDERR "Page deleted.\n";

    #print STDERR "Deleting users...\n";
    foreach my $user (@USERS) {
        $user->delete(1);
    }
    #print STDERR "User deleted.\n";
}


#################################################
# That's it!!!!
#################################################








sub get_channel_processor {
    my $CP = $TEST_CLASS->new(silent=>1, no_process_external=>1)
}





