#!/usr/bin/perl -w
#
# Test file for Metadot::ChannelProcessor.pm
# $Id: ChannelProcessor.002.t,v 1.1.2.5 2005/09/02 17:24:35 cory Exp $
#

use strict;

use File::Copy;
use Test::More tests=>32;

use SmokeMetadot;
use DBAccess;
use MetadotConfig;
use Metadot::PortalFactory;
use Subscription;
use Gizmo::Category;
use Gizmo::Newsitem;


# set up info
my $TEST_CLASS   = 'Metadot::ChannelProcessor';
my $ADMIN_UID = MetadotConfig->get_reserved_uid('ADMIN');

my $RSS_FILENAME    = "rss_test.xml";
my $TEST_FILE_DIR   = 'Metadot/ChannelProcessor.test';
my $TARGET_FILE_DIR = MetadotConfig->get_web_resource_dir();
my $TARGET_FILEPATH = $TARGET_FILE_DIR . '/' . $RSS_FILENAME;

my $RSS_ITEMS = 3;

my @USERS;
my ($CHANNEL, $CHANNEL_ID);

my $CP;

################################################################################
# Test: Use the test class
################################################################################
{
    use_ok($TEST_CLASS);
}


################################################################################
# Test: Create an external channel
################################################################################
{
    $CHANNEL = Channel->new();

    $CHANNEL->{is_a} = 'External';
    $CHANNEL->{name} = "Test channel $0";
    $CHANNEL->{description} = "This is a test channel created by $0";
    $CHANNEL->{url} = SmokeMetadot->SMOKE_URI_SCHEMA().'://'.SmokeMetadot->SMOKE_HOSTNAME().MetadotConfig->get_web_resource_location().'/'.$RSS_FILENAME;
    
    $CHANNEL_ID = $CHANNEL->add();
    ok(Utility->is_valid_integer($CHANNEL_ID), 'Channel was created');
}


################################################################################
# Test: Subscribe the USERS to the channel
################################################################################
{
    for (my $i = 0; $i<3; $i++) {
        my $user = SmokeMetadot->create_test_user();
        push @USERS, $user;
        my $userid = $user->uid();
        ok(       
            Subscription->add_user_subscription({
                userid      => $userid,
                cid         => $CHANNEL_ID,
                frequency   => Subscription->FREQUENCY_ALL(),
                medium      => Subscription->MEDIUM_EMAIL(),
            }),
            "Subscribed user $userid to channel"
        );
    }
}



################################################################################
# Test: Check that subscriptions records exist
################################################################################
{
    my ($subscription_count) = DBAccess->sqlSelect('count(*)', 'subscription', 'cid='.$CHANNEL_ID);

    my $expected_count = @USERS;
    ok($subscription_count == $expected_count, "Subscription record was created, $expected_count expected");
}


################################################################################
# Setup: move the test RSS file into the expected location
################################################################################
{
    my $test_file = $TEST_FILE_DIR.'/rss_test_01.xml';
    ok(copy($test_file, $TARGET_FILEPATH), "Successfully copied $test_file TO $TARGET_FILEPATH");
}


################################################################################
# Test: fetch the rss channel
################################################################################
{
    my $result;

    ok($CP = &get_channel_processor(), "Instantiated the channel processor");    
    ok($result = $CP->fetch_channels($CHANNEL_ID), "Fetched RSS for channel $CHANNEL_ID, got $result");
}


################################################################################
# Test: process the channel items
################################################################################
{
    my ($result, $item_count);

    ok($result = $CP->do_channels($CHANNEL_ID), "Processed items for channel $CHANNEL_ID, got result '$result'");
    
    my $expected_count = $RSS_ITEMS;
    ok(($item_count = $CP->number_processed()) && ($item_count == $expected_count), "$item_count channelitems were processed, $expected_count expected");
}



################################################################################
# Test: process the user subscriptions
################################################################################
{
    my ($result, $item_count);

    ok($result = $CP->do_subscriptions(channelid=>$CHANNEL_ID), "Processed subscriptions for channel $CHANNEL_ID, got result '$result'");

    my $expected_count = $RSS_ITEMS * @USERS;
    ok(($item_count = $CP->number_processed()) && ($item_count == $expected_count), "$item_count subscription notifications were processed, $expected_count expected");
}


################################################################################
# Test: process the user subscriptions redundantly
################################################################################
{
    # get a new channel processor
    sleep(1); # guarantee a new timestamp
    $CP = &get_channel_processor();
    
    ok(0 == $CP->do_subscriptions(channelid=>$CHANNEL_ID), "Processed subscriptions for channel $CHANNEL_ID, no redundant messages.");
}

################################################################################
# Test: remove the XML file and process the subscriptions
################################################################################
{
    ok(unlink($TARGET_FILEPATH), "Removed the test file");

    # get a new channel processor
    sleep(1); # guarantee a new timestamp
    $CP = &get_channel_processor();
    
    warn("The following error message is expected:\n");
    ok(1 == $CP->fetch_channels($CHANNEL_ID), "Fetched RSS for channel $CHANNEL_ID");
    ok(0 == $CP->do_channels($CHANNEL_ID), "Processed items for channel $CHANNEL_ID, no results");
    ok(0 == $CP->do_subscriptions(channelid=>$CHANNEL_ID), "Processed subscriptions for channel $CHANNEL_ID, no redundant messages.");

    ok($CHANNEL = Channel->new($CHANNEL_ID), "Reloaded channel $CHANNEL_ID");
    my @items = ($CHANNEL->{html} =~ m/NEWSITEM/g);
    ok(scalar(@items) == $RSS_ITEMS, "I still have my original items: ".scalar(@items)." (was $RSS_ITEMS)");
}


################################################################################
# Test: process an updated, but overlapping, XML file
################################################################################
{
    my ($result, $expected_count);
    my $test_file = $TEST_FILE_DIR.'/rss_test_02.xml';

    ok(copy($test_file, $TARGET_FILEPATH), "Successfully copied $test_file TO $TARGET_FILEPATH");
    ok($CP = &get_channel_processor(), "Instantiated the channel processor");    

    ok(1 == $CP->fetch_channels($CHANNEL_ID), "Fetched RSS for channel $CHANNEL_ID");
    
    ok(unlink($TARGET_FILEPATH), "Removed the test file");
    
    $expected_count = $RSS_ITEMS;
    ok($expected_count == ($result = $CP->do_channels($CHANNEL_ID)), "Processed items for channel $CHANNEL_ID, expected $expected_count, got $result");
    
    $expected_count = @USERS;  # only one should be new, so the users will each get one
    ok($expected_count == ($result = $CP->do_subscriptions(channelid=>$CHANNEL_ID)), "Processed subscriptions for channel $CHANNEL_ID, expected $expected_count, got $result");

    ok($CHANNEL = Channel->new($CHANNEL_ID), "Reloaded channel $CHANNEL_ID");
    my @items = ($CHANNEL->{html} =~ m/NEWSITEM/g);
    ok(scalar(@items) == $RSS_ITEMS, "I still have the correct number of items: ".scalar(@items));
}





################################################################################
# Test: Remove a user from the db, look for dangling subscriptions
################################################################################
{
    my $user = shift @USERS;
    my $userid = $user->uid();

    $user->delete(1);
    my ($subscription_count) = DBAccess->sqlSelect('count(*)', 'subscription', 'userid='.$userid);
    ok($subscription_count == 0, "User subscriptions records were removed");
}



################################################################################
# Test: Channel record was removed from the db
################################################################################
{
    $CHANNEL->delete();
    my ($channel_count) = DBAccess->sqlSelect('count(*)', 'channel', 'cid='.$CHANNEL_ID);
    ok($channel_count == 0, "Channel records were removed");
}


################################################################################
# Test: Channelitem records was removed from the db
################################################################################
{
    my ($channel_count) = DBAccess->sqlSelect('count(*)', 'channelitem', 'cid='.$CHANNEL_ID);
    ok($channel_count == 0, "Channelitem records were removed");
}


################################################################################
# Test: Check that subscriptions were removed when channel removed
################################################################################
{
    my ($subscription_count) = DBAccess->sqlSelect('count(*)', 'subscription', 'cid='.$CHANNEL_ID);
    ok($subscription_count == 0, "Subscription records were deleted");
}



################################################################################
# Cleanup
################################################################################
{
    #print STDERR "Deleting users...\n";
    foreach my $user (@USERS) {
        $user->delete(1);
    }
    #print STDERR "User deleted.\n";
}


#################################################
# That's it!!!!
#################################################








sub get_channel_processor {
    my $CP = $TEST_CLASS->new(silent=>1)
}





