#!/usr/bin/perl -w
#
# Test file for Metadot::ChannelProcessor.pm
# $Id: ChannelProcessor.003.t,v 1.1.2.4 2006/03/30 15:07:09 bob Exp $
#

use strict;
use Test::More tests=>28;

use SmokeMetadot;
use DBAccess;
use MetadotConfig;
use Metadot::PortalFactory;
use Subscription;
use Gizmo::Discussion;
use Message;

# set up info
my $TEST_CLASS   = 'Metadot::ChannelProcessor';
my $HOME_PAGE = Metadot::PortalFactory->get_home_object();
my $ADMIN_UID = MetadotConfig->get_reserved_uid('ADMIN');

my @USERS;
my ($DISCUSSION, $DISCUSSION_ID, $CHANNEL, $CHANNEL_ID);

my @MESSAGES;
my $CP;

################################################################################
# Test: Use the test class
################################################################################
{
    use_ok($TEST_CLASS);
}


################################################################################
# Test: Create a category
################################################################################
{
    my $test_name = '_TEST_DISCUSSION_'.$0;

    $DISCUSSION = Discussion->new({
        uid => $ADMIN_UID,
        parent_id => $HOME_PAGE->id(),
        name => $test_name,
    });
}


################################################################################
# Test: Add the discussion to the db
################################################################################
{
    $DISCUSSION_ID = $DISCUSSION->add();
    ok(Utility->is_valid_integer($DISCUSSION_ID), "Got a valid iid $DISCUSSION_ID");
}


################################################################################
# Test: Create a channel for the page
################################################################################
{
    $CHANNEL = Channel->new();
    $CHANNEL_ID = $CHANNEL->create($DISCUSSION_ID, 'Discussion', $DISCUSSION->name(), $DISCUSSION->description());
    ok(Utility->is_valid_integer($CHANNEL_ID), 'Channel was created');
}


################################################################################
# Test: Subscribe the USERS to the channel
################################################################################
{
    for (my $i = 0; $i<3; $i++) {
        my $user = SmokeMetadot->create_test_user();
        push @USERS, $user;
        my $userid = $user->uid();
        ok(       
            Subscription->add_user_subscription({
                userid      => $userid,
                cid         => $CHANNEL_ID,
                frequency   => Subscription->FREQUENCY_ALL(),
                medium      => Subscription->MEDIUM_EMAIL(),
            }),
            "Subscribed user $userid to channel"
        );
    }
}



################################################################################
# Test: Check that subscriptions records exist
################################################################################
{
    my ($subscription_count) = DBAccess->sqlSelect('count(*)', 'subscription', 'cid='.$CHANNEL_ID);

    my $expected_count = @USERS;
    ok($subscription_count == $expected_count, "Subscription record was created, $expected_count expected");
}


################################################################################
# Test: Add some messages to the channel
################################################################################
{
    for (my $i = 0; $i<5; $i++) {
        my $message_name = 'Test message '.$i;
        my $message_body = "Message $i body body body";

        my $message = Message->add_message(
              userid           => $ADMIN_UID,
              authorname       => 'Test Author',
              authoremail      => 'Author email',
              iid              => $DISCUSSION_ID,
              parentmid        => 0,
              name             => $message_name,
              description      => $message_body,
              status           => 'OK',
               );

        ok((defined $message), "Added a message");
        push @MESSAGES, $message;
    }

    sleep(1); # avoid having the channel's processdate overlap with the newsitem's creation date, to get an accurate count
}


################################################################################
# Test: process the channel items
################################################################################
{
    my ($result, $item_count);

    ok($CP = &get_channel_processor(), "Instantiated the channel processor");    
    ok($result = $CP->do_channels($CHANNEL_ID), "Processed items for channel $CHANNEL_ID, got result '$result'");
    
    my $expected_count = @MESSAGES;
    ok(($item_count = $CP->number_processed()) && ($item_count == $expected_count), "$item_count channelitems were processed, $expected_count expected");
}



################################################################################
# Test: process the user subscriptions
################################################################################
{
    my ($result, $item_count);

    ok($result = $CP->do_subscriptions(channelid=>$CHANNEL_ID), "Processed subscriptions for channel $CHANNEL_ID, got result '$result'");

    my $expected_count = scalar(@MESSAGES) * scalar(@USERS);
    ok(($item_count = $CP->number_processed()) && ($item_count == $expected_count), "$item_count subscription notifications were processed, $expected_count expected");
}


################################################################################
# Test: process the user subscriptions redundantly
################################################################################
{
    my ($result, $item_count);

    # get a new channel processor
    $CP = &get_channel_processor();
    
    ok(0 == $CP->do_subscriptions(channelid=>$CHANNEL_ID), "Processed subscriptions for channel $CHANNEL_ID, no redundant messages.");
}


################################################################################
# Test: gracefully handle failed sendmail
################################################################################
{
    my ($result, $expected);

    # add a new message
    my $message_name = 'Test of failing sendmail';
    my $message_body = "Message  body body body";

    my $message = Message->add_message(
          userid           => $ADMIN_UID,
          authorname       => 'Test Author',
          authoremail      => 'Author email',
          iid              => $DISCUSSION_ID,
          parentmid        => 0,
          name             => $message_name,
          description      => $message_body,
          status           => 'OK',
           );

    ok((defined $message), "Added a message");
    push @MESSAGES, $message;

    sleep(1); # avoid having the channel's processdate overlap with the newsitem's creation date, to get an accurate count


    # get a new channel processor
    my $sendmail_class = 'Metadot::Testing::FailingSendmail';
    $CP = &get_channel_processor(sendmail_class=>$sendmail_class);
   
    ok($sendmail_class = $CP->sendmail_class(), "I am using a failing mailer $sendmail_class");
   
    $expected = 1;
    ok($expected == ($result = $CP->do_channels($CHANNEL_ID)), "Processed channels for channel $CHANNEL_ID, expected $expected, got result '$result'");    

    ok($expected == ($result = $CP->do_subscriptions(channelid=>$CHANNEL_ID)), "Processed subscriptions for channel $CHANNEL_ID, (failure expected) expected $expected, got result '$result'");    
    # send the rest using a valid mailer
    $CP = &get_channel_processor();

#    $expected = 0; # no new messages
    $expected = @MESSAGES - 1; # send the leftover messages
    ok($expected == ($result = $CP->do_channels($CHANNEL_ID)), "Processed channels for channel $CHANNEL_ID, expected $expected, got result '$result'");    
    
    $expected = @USERS - 1; # send the leftover messages from last run

    ok($expected == ($result = $CP->do_subscriptions(channelid=>$CHANNEL_ID)), "Processed subscriptions for channel $CHANNEL_ID, expected $expected, got result '$result'");    
}




################################################################################
# Test: Remove a user from the db, look for dangling subscriptions
################################################################################
{
    my $user = shift @USERS;
    my $userid = $user->uid();

    $user->delete(1);
    my ($subscription_count) = DBAccess->sqlSelect('count(*)', 'subscription', 'userid='.$userid);
    ok($subscription_count == 0, "User subscriptions records were removed");
}



################################################################################
# Test: Channel record was removed from the db
################################################################################
{
    $CHANNEL->delete();
    my ($channel_count) = DBAccess->sqlSelect('count(*)', 'channel', 'cid='.$CHANNEL_ID);
    ok($channel_count == 0, "Channel records were removed");
}


################################################################################
# Test: Channelitem records was removed from the db
################################################################################
{
    my ($channel_count) = DBAccess->sqlSelect('count(*)', 'channelitem', 'cid='.$CHANNEL_ID);
    ok($channel_count == 0, "Channelitem records were removed");
}


################################################################################
# Test: Check that subscriptions were removed when channel removed
################################################################################
{
    my ($subscription_count) = DBAccess->sqlSelect('count(*)', 'subscription', 'cid='.$CHANNEL_ID);
    ok($subscription_count == 0, "Subscription records were deleted");
}



################################################################################
# Cleanup
################################################################################
{
    #print STDERR "Deleting discussion...\n";
    $DISCUSSION->delete();
    #print STDERR "Discussion deleted.\n";

    #print STDERR "Deleting users...\n";
    foreach my $user (@USERS) {
        $user->delete(1);
    }
    #print STDERR "User deleted.\n";
}


#################################################
# That's it!!!!
#################################################








sub get_channel_processor {
    my $CP = $TEST_CLASS->new(silent=>1, no_process_external=>1, @_)
}









1;
