###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


# a class for dynamically adding properties to an instance

package Metadot::DynaProperties::DynaProperty;

use strict;
use vars qw(@ISA);

use MetadotObject;
use Metadot::ObjectAccess::ObjectAccessor;
use Metadot::ObjectAccess::Restorable;
use Metadot::ObjectAccess::Memento;

use Utility;

# these are the property types that a DynaProperty supports.
use constant INTEGER => 1;
use constant NUMBER  => 2;
use constant STRING  => 3;

@ISA = qw(MetadotObject Metadot::ObjectAccess::Restorable);

use constant MO_INSTANCE_PROPERTIES => {
    property_id => undef, # unique identifier for the property instance
    class_name  => undef, # the name of the class that owns this property
    instance_id => undef, # the instance id of the object that owns this property
    category_id => undef, # an arbitrary identifier for categorizing properties
    name        => undef, # the name of the property
    type        => undef, # the property's data type
    sort_order  => undef, # the property's sort order
    _valueSTRING    => undef, # internal storage for string
};


sub new {
    my $proto = shift;
    my $args  = $proto->normalize_constructor_args(@_);
    my $self  = $proto->SUPER::new($args);

    $self->value($args->{value});
    return $self;
}


# sets or gets the value to the appropriate data type
sub value {
    my $self  = shift;
    $self->_valueSTRING(@_);
}





# ObjectAccessor
#======================================

use constant DB_FIELDMAP => {
    property_id     => 'propertyId',
    class_name      => 'className',
    instance_id     => 'instanceId',
    category_id     => 'categoryId',
    name            => 'name',
    type            => 'type',
    sort_order      => 'sortOrder',
    _valueSTRING    => 'valueSTRING',
};


my $OBJECT_ACCESSOR = Metadot::ObjectAccess::ObjectAccessor->new(
   owner        => __PACKAGE__,
   properties   => [ get_property_names() ],
   tables       => [ 'dynaProperties' ],
   key_property => 'property_id',
   is_cached    => 0,
   field_map    => DB_FIELDMAP(),
);

sub get_object_accessor {
    my $class = shift;
    return $OBJECT_ACCESSOR;
}

sub get_property_names {
    return keys(%{DB_FIELDMAP()});
}



# Data Access methods
#=========================================

# adds a new property to the database. returns the property's id.
# if you do not set a name for the property, then it will assign the
# property id value to the name
sub add {
    my $self = shift;
    $self->_validate();
    my $id = $self->get_object_accessor()->create( object=>$self );

    # if there is no name defined, also set that to be the property id
    unless(defined $self->name()) {
        $self->name($id);
        $self->save();
    }

    return $id;
}


# restores a property from the database
sub restore {
    my $proto = shift;
    my $id    = shift;
    my $args  = shift || {};

    $id = undef unless Utility->is_valid_integer($id); # secu_200401
    return undef unless (defined $id);

    my $result = $proto->get_object_accessor()->retrieve(key=>$id);
    return $result;
}


# saves an existing property in the database
sub save {
    my $self = shift;
    my $id = $self->property_id();
    return $self->add() if (!defined($id));

    $self->_validate();
    $self->get_object_accessor()->update(object=>$self);
}


# deletes a property from the database
sub delete {
    my $self = shift;
    my $id   = $self->property_id();

    if(!defined $id){
      $self->debug_msg(3, "Error $self\->delete(id=$id): I don't know what id to delete");
      return 0;
    }

    $self->get_object_accessor()->delete( key=>$id );
}



# implement the code necessary to restore the object's state from a Memento object
sub restore_state {
    my ($self, $state) = @_;
    die unless $state->isa('Metadot::ObjectAccess::Memento');
    foreach my $property (get_property_names()) {
        $self->$property($state->get_state($property));
    }
}

# implement the code necessary to extract the object's state to a Memento object
sub extract_state {
    my $self = shift;
    my $state = Metadot::ObjectAccess::Memento->new();
    foreach my $property (get_property_names()) {
        $state->set_state($property, $self->$property());
    }
    return $state;
}


# validates a property object as valid for saving to the database
sub _validate {
    my $self = shift;
    # check required fields
    die("Must specify class_name") unless defined($self->class_name());
}

1;
