###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::DynaProperties::DynaPropertyOwner;

use strict;
use Metadotclass;
use Metadot::DynaProperties::DynaProperty;

use constant PROPERTIES_LIST => '_dynaPropertiesList';
use constant PROPERTIES_MAP  => '_dynaPropertiesMap';


# stores a dynamic property
sub set_dyna_property {
    my $self = shift;
    my $name        = shift;
    my $value       = shift;
    my $category_id = shift;

    # check if this property exists. if not, create a new one.
    # if so, just update the value.

    Metadotclass->debug_msg(5, "Setting dynaproperty category[$category_id], name[$name], value[$value]");
    my $property_obj;
    if (defined $name) {
        $property_obj = $self->get_dyna_property_object($name, $category_id);
    }
    if (defined $property_obj) {
        Metadotclass->debug_msg(5, "This is an existing property $property_obj");
        $property_obj->value($value);
        $property_obj->save();
    } else {
        Metadotclass->debug_msg(5, "This is a new property");
        $property_obj = $self->add_dyna_property_object($name, $value, $category_id);
   }

   return $property_obj;
}


# retrieves a dynamic property
sub get_dyna_property {
    my $self        = shift;
    my $name        = shift || die("Must specify property name");
    my $category_id = shift;

    my $property = $self->get_dyna_property_object($name, $category_id);
    if (!defined $property) {
        return undef;
    } else {
        return $property->value();
    }
}


# removes a dynamic property
sub remove_dyna_property {
    my $self        = shift;
    my $name        = shift || die("Must specify property name");
    my $category_id = shift;

    my $property = $self->get_dyna_property_object($name, $category_id);
    Metadotclass->debug_msg(5, "I will remove dyna_property $property, with name[$name] and category_id[$category_id]");

    $self->remove_dyna_property_object($property);
}


# gets a dynamic property object, rather than just its value
sub get_dyna_property_object {
    my $self = shift;
    my $name        = shift || die("Must specify property name");
    my $category_id = shift;

    Metadotclass->debug_msg(5, "I will try to get dynaproperty name[$name] category[$category_id]");
    # make sure properties list has been initialized
    $self->_initialize_dyna_properties();

    return $self->{PROPERTIES_MAP()}->{$category_id}->{$name};
}


# adds a new dynamic property object to the system
sub add_dyna_property_object {
    my $self = shift;
    my $name        = shift;
    my $value       = shift;
    my $category_id = shift;

    my $class = ref($self) || $self;

    # make sure properties list has been initialized
    $self->_initialize_dyna_properties();

    my $property_obj = Metadot::DynaProperties::DynaProperty->new(
        name        => $name,
        value       => $value,
        category_id => $category_id,
        class_name  => $class,
        instance_id => $self->get_instance_id(),
    );
    $property_obj->add();
    $self->reload_dyna_properties;

    return $property_obj;
}


# removes a dynamic property object
sub remove_dyna_property_object {
    my $self = shift;
    my $property_to_remove = shift;

    if (defined $property_to_remove) {
        $property_to_remove->delete();
        $self->reload_dyna_properties;
    }
}


# removes all dynamic property object for the instance
sub remove_dyna_property_objects {
    my $self = shift;
    foreach my $property ($self->get_dyna_property_objects()) {
        $property->delete();
    }

    $self->reload_dyna_properties;
}


# gets all the dynaproperty objects for this instance.
# optionally, specify a category id
sub get_dyna_property_objects {
    my $self = shift;
    my $category_id = shift;

    my $class = ref($self) || $self;

    # specify the class name
    my @where = (
        "className = '$class'",
    );

    # specify the instance id
    push @where, 'instanceId = '.$self->get_instance_id();

    # specify the category id if requested
    if (defined $category_id) {
        push @where, "categoryId = $category_id";
    }


    my @results = $self->get_dyna_property_objects_where(\@where);

    use Data::Dumper;
    #print STDERR "I have dyna property objects:\n", Dumper \@results;

    if (wantarray) {
        return @results;
    } else {
        return \@results;
    }
}


# gets dynaproperty objects based on sql parameters
sub get_dyna_property_objects_where {
    my $class = shift;
    my $args  = shift || [];

    my $sort_order = $class->get_dyna_property_sort_order;
    if (defined $sort_order) {
        $sort_order = "ORDER BY $sort_order";
    }

    return Metadot::DynaProperties::DynaProperty->get_object_accessor()->retrieve_where(
        where => $args,
        other => $sort_order,
        );
}


# specifies the db fields to use for sorting
sub get_dyna_property_sort_order {
    return 'categoryId, sortOrder';
}

# gets the class_name associated with this instance's properties
sub get_class_name {
    my $self = shift;
    return ref($self) || $self;
}

# gets the instance id of this instance
sub get_instance_id {
    my $self = shift;
    if ($self->can('get_id')) {
        return $self->get_id();
    } elsif ($self->can('id')) {
        return $self->id();
    } else {
        die("Could not resolve instance id for $self");
    }
}


# stores the dynamic properties in the object
sub _initialize_dyna_properties {
    my $self  = shift;
    my $force = shift;

    return if (!$force && exists $self->{PROPERTIES_LIST()});

    #print STDERR "Initializing dynaproperties...\n";

    $self->{PROPERTIES_LIST()} = $self->get_dyna_property_objects() || [];
    $self->{PROPERTIES_MAP()}  = {};
    foreach my $property (@{$self->{PROPERTIES_LIST()}}) {
        $self->{PROPERTIES_MAP()}->{$property->category_id()}->{$property->name()} = $property;
    }
}


# forces the dynaproperties to reload. this is useful if you edit
# a dynaproperty directly
sub reload_dyna_properties {
    my $self = shift;
    $self->_initialize_dyna_properties(1);
}

1;
