###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Form3000::Fields::FileField;

=head1 NAME

Metadot::Form3000::Fields::FileField

=head1 VERSION

Version $Revision: 1.2.2.2 $

=head1 SYNOPSIS

my $form_field = Metadot::Form3000::Fields::FileField->new();

$form_field->file_info($file_info);

$form_field->process_input($input);

=head1 DESCRIPTION

FileField is a type of Field that handles uploading files to the portal. It
supports uploading new files and deletion of existing files.

=cut


use strict;
use vars qw(@ISA);
use Metadotclass;

use Metadot::Form3000::Fields::BaseFormField;
use Metadot::UploadsManager;

@ISA = qw(Metadot::Form3000::Fields::BaseFormField);

=head1 PROPERTIES

file_info: a file info record as produced by Metadot::UploadsManager::file_info

template:  the template used to render this field

=cut
use constant MO_INSTANCE_PROPERTIES => {
    file_info   => undef,
    template    => 'file_field.tt2',
};

=head1 CONSTANTS

UPLOAD_PUBLIC: the field value for public uploads

UPLOAD_USER_SELECTED: the field value for access-controlled uploads

UPLOAD_DEFAULT: the default upload type

=cut
use constant UPLOAD_PUBLIC        => Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC();
use constant UPLOAD_USER_SELECTED => Metadot::UploadsManager::UPLOAD_TYPE_USER_SELECTED();
use constant UPLOAD_DEFAULT       => UPLOAD_USER_SELECTED;

=head1 METHODS

=head2 process_input($args)

Process form input. The logic is:

=over 4

=item * If a new file is uploaded, the value will be the filename

=item * If a delete was requested, the value will be empty

=item * If a delete was requested but cancelled, the value will be the original
  filename

=item * If a delete was requested and a new file was uploaded, the value will
  be the new filename

=back

=cut
sub process_input {
    my $self = shift;
    my $args = shift || {};

    my $input = $args->{input} || {};
    my $field = $args->{field} || $self;
    my $value;

    Metadotclass->debug_msg(4, "process_input for ".$field->form_key);

    if ($field->is_delete_cancelled($input)) {
        $value = $field->original_filename($input);
        Metadotclass->debug_msg(5, "File delete was cancelled; keeping original");
    } else {
        # check for a deletion
        my $delete_requested = $field->is_delete_requested($input);
        if ($delete_requested) {
            Metadotclass->debug_msg(5, "File delete was requested");
            $value = '';
        }

        # check for a new file
        $value = $self->get_raw_input($input);
        if ($value) {
            Metadotclass->debug_msg(5, "New file was uploaded");
        } elsif (!$delete_requested) {
            $value = $self->original_filename($input);
            Metadotclass->debug_msg(5, "No file uploaded; keeping original");
        }
    }

    Metadotclass->debug_msg(4, "process_input returns ".($value || ''));
    return $value;
}


=head2 upload_type($input)

Determines the upload type based on the current form input.
Expects $input to be some input hash.

=cut
sub upload_type {
    my $self  = shift;
    my $input = shift || die('Must provide input');

    my $result = $self->get_raw_input($input, $self->upload_type_key());
    return $result || UPLOAD_DEFAULT;
}


=head2 is_delete_requested($input)

Determines whether this file was requested for deletion.
Expects $input to be some input hash.

=cut
sub is_delete_requested {
    my $self  = shift;
    my $input = shift || die('Must provide input');

    my $result = $self->get_raw_input($input, $self->delete_key());
    return ($result);
}


=head2 is_delete_cancelled($input)

Determines whether a file was requested for deletion, but canceled.
Expects $input to be some input hash.

=cut
sub is_delete_cancelled {
    my $self  = shift;
    my $input = shift || die('Must provide input');

    my $result = $self->get_raw_input($input, $self->delete_cancel_key());
    return $result;
}


=head2 original_filename($input)

Gets the original filename value.
Expects $input to be some input hash.

=cut
sub original_filename {
    my $self  = shift;
    my $input = shift || die('Must provide input');

    my $result = $self->get_raw_input($input, $self->original_filename_key());
    return $result;
}


=head2 has_new_upload($input)

Determines whether a new upload is requested.
Expects $input to be some input hash.

=cut
sub has_new_upload {
    my $self  = shift;
    my $input = shift || die('Must provide input');

    my $result;
    if ($self->is_delete_cancelled($input)) {
        Metadotclass->debug_msg(5, "has_new_upload? no, delete was cancelled");
        $result = 0;
    } else {
        my $value    = $self->get_raw_input($input) || '';
        my $original = $self->original_filename($input) || '';

        # if I have a value, then there may be a new upload, but I need to check
        # whether the existing file is the same
        if ($value) {
            # the new value is different, so this is a new upload
            if ($value ne $original) {
                $result = 1;
            } else {
                # the values are the same, but the file itself may not exist.
                # in that case, it's a new upload
                my $file_info = $self->file_info();
                if ($file_info && $file_info->file_name()) {
                    $result = 0;
                } else {
                    $result = 1;
                }
            }
        }
    }

    Metadotclass->debug_msg(5, "has_new_upload? '$result'");
    return $result;
}

=head2 upload_type_key()

Produces a form key to determine upload type.

=cut
sub upload_type_key {
    my $self = shift;
    my $key  = $self->form_key();
    return $key.'.upload_type';
}

=head2 delete_key($input)

Produces a form key to indicate deletion of the file is desired.

=cut
sub delete_key {
    my $self = shift;
    my $key  = $self->form_key();
    return $key.'.delete';
}

=head2 delete_cancel_key($input)

Produces a form key to indicate deletion of the file was canceled.

=cut
sub delete_cancel_key {
    my $self = shift;
    my $key  = $self->form_key();
    return $key.'.deleteCancel';
}


=head2 original_filename_key($input)

Produces a form key to indicate the original filename.

=cut
sub original_filename_key {
    my $self = shift;
    my $key = $self->form_key();
    return $key.'.original';
}







1;
