#######################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Form3000::Form;

use strict;

use Metadot qw($DISPLAY %FORM $SESSION %REQUEST_VARS);
use MetadotObject;
use Metadot::Form3000::Fields::FormField;

use vars qw(@ISA);
@ISA = qw(MetadotObject);


use constant MO_INSTANCE_PROPERTIES => {
    name            => 'form3000',
    action          => 'index.pl',
    method          => 'post',
    fields          => undef,
    values          => undef,
    mappings        => undef,
    title           => undef,
    errors          => undef,
    enctype         => undef,
    submit_actions  => undef,
    calling_url     => undef,
    landing_url     => undef,
    header_text     => undef,
    footer_text     => undef,
    field_layout    => undef,

    enable_cancel   => 1,
    enable_reset    => 1,
};

# the key where I store form processing errors
use constant REQUEST_KEY_ERRORS => 'form_errors';


use constant FIELD_LAYOUT_TWO_COLUMN => 'two_column';
use constant FIELD_LAYOUT_STANDARD   => 'standard_layout';

sub initialize {
    my $self = shift;
    my $args = shift;

    $self->SUPER::initialize($args);

    # initialize field and value data structures
    $self->fields([])  unless defined($self->fields());
    $self->values({})  unless defined($self->values());
    $self->mappings([])  unless defined($self->mappings());

    # initialize errors
    $self->errors($REQUEST_VARS{REQUEST_KEY_ERRORS()} || [])
        unless defined($self->errors());

    # initialize calling and landing urls
    $self->calling_url( $FORM{calling_url} || $SESSION->referer())
        unless defined($self->calling_url());

    # initialize calling and landing urls
    $self->landing_url( $FORM{landing_url} || $self->calling_url())
        unless defined($self->landing_url());
}


sub process_input {
    my $self   = shift;
    my $input  = shift || \%FORM;

    # now iterate through my fields and validate input
    $self->clear_errors();

    foreach my $form_field (@{ $self->fields()}) {
        my $error = $form_field->input_validator()->validate_input({
            field => $form_field,
            input => $input,
        });

        if (defined $error) {
            $self->add_error($error);
        } else {
            my $value = $form_field->input_processor()->process_input({
                field => $form_field,
                input => $input,
            });
            $self->set_value($form_field->form_key(), $value);
        }

    }

    return $self;
}


sub clear_errors {
    my $self = shift;
    $self->errors([]);
    $self->_persist_errors_in_request();
}


sub add_error {
    my $self  = shift;
    my $error = shift;

    my $errors = $self->errors() || [];
    push @$errors, $error;

    $self->errors($errors);
    $self->_persist_errors_in_request();
}


sub _persist_errors_in_request {
    my $self = shift;
    $REQUEST_VARS{REQUEST_KEY_ERRORS()} = $self->errors();
}


# returns true if there are any errors in the current form obj.
sub has_errors {
    my $self = shift;
    my $errors = $self->errors();

    return (defined $errors && defined $errors->[0]);
}



# a convenience method to quickly create and add a FormField object to the form
sub generate_form_field {
    my $self = shift;
    my $args = MetadotObject->normalize_constructor_args(@_);

    # get a field instance
    my $field = Metadot::Form3000::Fields::FormFieldFactory->get_instance($args);

    # add the field to my list
    $self->add_field($field);
    $self->set_value($field->form_key(), $args->{value});
}


# add a FormField object to the form
sub add_field {
    my $self  = shift;
    my $field = shift;

    (defined $field && $field->isa('Metadot::Form3000::Fields::FormField'))
        || die("Not a FormField $field");

    push @{ $self->fields() }, $field;
}


# returns the FormField object with a given key
sub get_field {
    my $self = shift;
    my $form_key = shift;

    if (defined $form_key) {
        foreach my $field (@{ $self->fields() }) {
            return $field if ($field->form_key() eq $form_key);
        }
    }
}


# gets or sets a value based on a form object
sub value_for {
    my $self  = shift;
    my $field = shift;
    my $result;

    if (defined $field) {
        my $key = $field->form_key();
        if (defined $key) {
            if (@_) {
                $result = $self->set_value($key);
            } else {
                $result = $self->get_value($key);
            }
        }
    }

    return $result;
}

# return a single value based on a form field name
sub get_value {
    my $self = shift;
    my $key  = shift;

    return $self->values()->{$key};
}


# sets a single value based on a form field name
sub set_value {
    my $self  = shift;
    my $key   = shift;
    my $value = shift;

    if (defined $key) {
        $self->values()->{$key} = $value;
    }
}





# Renders HTML form fields. For backwards compatibility with Model 1
# code.
# Inputs:
# - fields: a list of FormField objects to render
# - values: a hashtable of field->key_name to values
sub render_form_fields_html {
    my $class = shift;
    my $args  = shift || {};

    my $fields = $args->{'fields'} || die('Must specify fields');
    my $values = $args->{'values'} || {};

    my $result;;
    my $tt = Portal->get_portal_template_object();

    my $vars = {
        form_fields => $fields,
        form_values => $values,
        };

    $tt->process('form_3000_fields.tt2', $vars, \$result);

    if ($tt->error()) {
        $DISPLAY->debug_msg(0, $tt->error());
    }

    return $result;
}







1;





