package Metadot::Form3000::PropertySettings::BasePropertySettingsControl;

use CGI qw(escapeHTML);
use MetadotObject;
use Metadot qw($DISPLAY %FORM);
use Metadot::Form3000::Form;

use vars qw(@ISA);
@ISA = qw(MetadotObject);

use constant MO_INSTANCE_PROPERTIES => {
    property_mapping    => undef,
    };

use constant PROPERTY_SETTING_FIELD_BASE => 'property_setting';
use constant PROPERTY_SETTING_FIELD_SEPARATOR => '.';


# render the fields for the user profile management form
sub render_control {
    my $self = shift;

    my $result = '';

    foreach my $field (@{$self->_generate_fields()}) {
        $result .= '<div>'.$field->render_html({
            field_layout  => Metadot::Form3000::Form->FIELD_LAYOUT_STANDARD(),
        })."</div>\n";
    }

    return $result;
}


# process input from the user profile management form. this method
# processes input for all the properties passed to it.
sub process_input {
    my $self = shift;
    my $input = shift || \%FORM;

    my $property_mapping = $self->property_mapping();

    if (defined $input) {
        # iterate through each of the sub-fields and process the input
        my $fields = $self->_generate_fields();
        foreach my $field (@$fields) {
            # get the field's form key
            my $form_key = $field->form_key();

            # get the attribute that this field was generated for
            my $attribute_name = $self->get_attribute_from_form_key($form_key);

            # get the form value
            # Abstract select fields represent their choices as key/value pairs
            my $value;
            if ($attribute_name eq 'select_options' && $field->isa('Metadot::Form3000::Fields::AbstractSelectionField')) {
                $DISPLAY->debug_msg(5, "process_input processing select_options for ".$property_mapping->property_name());
                $value = $field->unflatten_select_options($input->{$form_key}, OPTION_VALUE_SEPARATOR());
            } else {
                $value = $input->{$form_key};
            }

            # apply the value to the property if applicable
            if ($property_mapping->can($attribute_name)) {
                $property_mapping->$attribute_name($value);
            }

            # apply the value to the field definition
            if (defined($property_mapping->field_definition)) {
                $property_mapping->field_definition()->{$attribute_name} = $value;
            }
        }
    }
}



use constant PROPERTY_CONTROL_DEFINITIONS => {
    label => {
        type  => 'simple_text',
        size  => 40,
        label => 'Field name',
        },
    default_value => {
        type  => 'simple_text',
        size  => 40,
        label => 'Default value',
        },
    size => {
        type  => 'simple_text',
        size  => 3,
        label => 'Input size',
        },
    is_mandatory => {
        type     => 'checkbox',
        label    => 'Mandatory?',
        },
    checked_value => {
        type  => 'simple_text',
        size  => 10,
        label => 'Value when checked',
        },
    rows => {
        type  => 'simple_text',
        size  => 3,
        label => 'Rows',
        },
    cols => {
        type  => 'simple_text',
        size  => 3,
        label => 'Columns',
        },
    select_options => {
        type  => 'textarea',
        label => 'Select options',
        rows  => 5,
        cols  => 40,
        inline_help => 'Enter value/label pairs as in:<br /><i>value label<br />value label<br />value label<br /></i>',
        },
    heading_text => {
        type  => 'simple_text',
        size  => 40,
        label => 'Label heading',
        },
    inline_help => {
        type  => 'textarea',
        label => 'Inline help',
        rows  => 5,
        cols  => 40,
        },

    };


use constant OPTION_VALUE_SEPARATOR => ' ';

# generates the fields that allow the admin to modify registration properties
sub _generate_fields {
    my $self = shift;
    my $property_mapping = $self->property_mapping();

    my @results;
    my %visited = ();

    # determine if the property is configurable
    if (defined $property_mapping && $property_mapping->configurable()) {

        # first get the possible fields from the property mapping itself
        foreach my $attribute_name (@{$property_mapping->MO_properties_list()}) {
            $DISPLAY->debug_msg(5, "I have attribute $attribute_name");

            # avoid getting this property later
            unless ($visited{$attribute_name}++) {

                # get the standard definition for the form field
                my $base_definition = PROPERTY_CONTROL_DEFINITIONS->{$attribute_name};
                if ($base_definition) {
                    $DISPLAY->debug_msg(5, "I have base definition $base_definition");

                    # create a more complete picture of the field definition
                    my %field_definition = (
                        %$base_definition,
                        name =>  $self->generate_form_key(
                                    $property_mapping->property_name(),
                                    $attribute_name,
                                    ),
                        default_value => escapeHTML($property_mapping->$attribute_name()),
                    );

                     # create a FormField object
                     my $field = Metadot::Form3000::Fields::FormFieldFactory->get_instance(
                         \%field_definition
                     );

                     $DISPLAY->debug_msg(5, "I have property field $field");
                     if (defined $field) {
                         push @results, $field;
                     }
                 }
            }
        }

        # the following block does the same as the previous block,
        # except it inspects the property mapping's form field object
        my $form_field = $property_mapping->generate_form_field();
        if (defined $form_field) {
            foreach my $attribute_name (@{$form_field->MO_properties_list()}) {
                $DISPLAY->debug_msg(5, "I have attribute $attribute_name");

                # avoid getting this property later
                unless ($visited{$attribute_name}++) {
                    my $base_definition = PROPERTY_CONTROL_DEFINITIONS->{$attribute_name};
                    if ($base_definition) {

                        my $default_value;

                        # Abstract select fields represent their choices as key/value pairs
                        if ($attribute_name eq 'select_options') {
                            $default_value = '';
                            my @options = @{$form_field->select_options()};
                            while (@options) {
                                $default_value .= shift(@options).OPTION_VALUE_SEPARATOR().shift(@options)."\n";
                            }
                        } else {
                            $default_value = $form_field->$attribute_name();
                        }


                        my %field_definition = (
                            %$base_definition,
                            name =>  $self->generate_form_key(
                                        $property_mapping->property_name(),
                                        $attribute_name,
                                        ),
                            default_value => escapeHTML($default_value),
                        );

                         my $field = Metadot::Form3000::Fields::FormFieldFactory->get_instance(
                             \%field_definition
                         );

                         if (defined $field) {
                             push @results, $field;
                         }
                     }
                }
            }
        }
    }

    return \@results;
}



# generates a form key that is unique per mapping_name + attribute_name combination
sub generate_form_key {
    my $class = shift;
    my $mapping_name   = shift;
    my $attribute_name = shift;

    return  PROPERTY_SETTING_FIELD_BASE()
           .PROPERTY_SETTING_FIELD_SEPARATOR()
           .$mapping_name
           .PROPERTY_SETTING_FIELD_SEPARATOR()
           .$attribute_name;
}


# based on a form key, returns the attribute that was used to generate it
sub get_attribute_from_form_key {
    my $class = shift;
    my $form_key = shift;

    my $separator = PROPERTY_SETTING_FIELD_SEPARATOR();
    my ($base, $mapping_name, $attribute) = split(/\Q$separator\E/, $form_key);
    $DISPLAY->debug_msg(5, "Split '$form_key' by '$separator', got '$base', '$mapping_name', '$attribute'");
    return $attribute;
}



1;
