###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::FormBuilder;

#========================================
# This module is the interface for the form building capabilities.
# It uses FormElement, which implements methods for managing and
# displaying specific form elements.
#========================================


use strict;
use Template;
use Data::Dumper;
use CGI;
use XML::Simple;

use MetadotConfig;
use Metadot::FormElement;
use Metadot::FormDataObj;
use Metadot::TableInfoFactory;
use DBAccess;
use ErrorHandling;
use Metadot::JSFormFactory ;
use Metadot::JSForm ;

use constant FIELD_TYPES => { 12 => 'char', 11 => 'datetime', 9 => 'date', -1 => 'text', 4 => 'number'};

# constructor
# takes optional arg $dataobj.  If set, this object will be essentially
# used as a 'database table' object, and the formbuilder methods will map to
# the instance variables in this object rather than to a specified table
# in the db.
sub new {
    my $proto = shift;
    my $dataobj = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    if ($dataobj) {
    $self->{_dataobj} = $dataobj;
    }

    bless ($self, $class);
    return $self;
}

# return a hash of objects (form elements) corresponding to the form
# variable descriptions that have been specified.  Will return undef
# if form var specs have not been processed yet.
sub getFormTemplateData {
    my $self = shift;
    return $self->{_form_vars}; # is not set until after form var specs
                # have been processed.
}

sub initJSFormObj {
    my $self = shift;

    $self->{jsform} = Metadot::JSFormFactory->get_jsform() ;
}

sub getJSFormObj {
    my $self = shift;
    return $self->{jsform};
}

# read an xml form description config file and return
# the parsed nested hash corresponding to it. In addition, set the form name
# and the flag which indicates whether or not to use popup calendars for
# date fields.  The form description can also be augmented (or created
# entirely) via method calls.
sub getFormDescription {
    my ($self, $config_file) = @_;

    my $xs = XML::Simple->new();
    if (!(-e $config_file)) {
#          die "given config file $config_file does not exist."
        ErrorHandling->printError("given config file $config_file does not exist.");
    exit(0);
    }
    my $ref = $xs->XMLin($config_file, forcearray => 1);
#    print STDERR "got description for form name " . $ref->{name} . "\n";
    $self->{_formname} = $ref->{name};
    if (exists $ref->{use_popcals} && $ref->{use_popcals} eq "true") {
    $self->{_use_popcals} = 1;
    }
    else {
    $self->{_use_popcals} = 0;
    }
    if (!exists $ref->{'fvar'}) {
    $ref->{'fvar'} = {};
    }
    $self->{_form_descr} = $ref;
    return $ref;
}


# process a nested hash corresponding to a form description.  First,
# the description of the form elements is processed.  Then, the form variables
# are processed and associated with the form element info where specified.
sub processFormDescription {
    my ($self, $pkeys) = @_;

    # if no primary key values defined, just create an empty hash.
    if (!$pkeys) {
    $pkeys = {};
    }
    my $descr_ref = $self->{_form_descr};
    my $mode = lc($descr_ref->{'mode'});
    my $form_elts_ref = $descr_ref->{'element_descriptor'};
    $self->_processFormElementSpecs2($form_elts_ref, $mode);
    my $form_var_ref = $descr_ref->{'fvar'};
    my $tables = $descr_ref->{'table'};
    if (!defined $tables) {
    $tables = {};
    }
    $self->_processFormVarSpecs2($form_var_ref, $tables, $pkeys, $mode);
}

# should be called only after processFormVarSpecs
# (or processFormDescription) is run.
# Processes the given template against the information
# constructed from the specification of the form element objects.
sub buildTemplateString {
    my ($self, $template_path, $template_name) = @_;

    if (!exists $self->{_form_vars}) {
    ErrorHandling->printError("Error: Form data has not been defined yet");
    exit(0);
    }
    else {
    my $data =  $self->getFormTemplateData();
#   print STDERR "got form template data: ", Dumper($data), "\n";
    # create a new template
    my $template = Template->new({
        INCLUDE_PATH => $template_path,
    }) || die $Template::ERROR, "\n";
    my $htmlstring;
    # generate html from the template and the data structure
    $template->process($template_name, $data, \$htmlstring)
        || die $template->error(), "\n";
    my $jsform = $self->getJSFormObj();
    if ($jsform) {
        my $jsformcontent = $jsform->get_form($htmlstring, $jsform->get_ALL_ERRORS()) ;
        # return the result
        return $jsformcontent;
    }
    else {
        return $htmlstring;
    }
    }
}

# process the element_descriptor info (if any) read in from the xml
# specification file.   A  form element object is created for each description.
# The info from the xml file can be further augmented
# by method calls to create specific descriptors, e.g. addTextElementDescriptor.
sub processFormElementSpecs {
    my ($self) = @_;

    my $descr_ref = $self->{_form_descr};
    my $mode = lc($descr_ref->{'mode'});
    my $form_elts_ref = $descr_ref->{'element_descriptor'};
    $self->_processFormElementSpecs2($form_elts_ref, $mode);
}

# takes a set of form element objects, built from xml form element
# descriptors and/or method calls.  Uses these objects, plus info
# about the db tables or data obj that they are tied to, to
# instantiate their value and type info.
sub processFormVarSpecs {
    my ($self, $pkeys) = @_;

    # if no primary key values defined, just create an empty hash.
    if (!$pkeys) {
    $pkeys = {};
    }
    my $descr_ref = $self->{_form_descr};
    my $mode = lc($descr_ref->{'mode'});
    my $form_var_ref = $descr_ref->{'fvar'};
#      print STDERR "got form vars: ", Dumper($form_var_ref), "\n";
    my $tables = $descr_ref->{'table'};
    if (!defined $tables) {
    $tables = {};
    }
    $self->_processFormVarSpecs2($form_var_ref, $tables, $pkeys, $mode);
}

# build a form element object for each form element description, according
# to its specs.
sub _processFormElementSpecs2 {
    my ($self, $form_elts_ref, $mode) = @_;

#      print STDERR "in processFormElementSpecs, with mode $mode\n";
    my $keyhash; my $form_elt;
    my $status;
    my $form_elt_hash = {};
    if (defined $form_elts_ref) {
    foreach my $key (sort keys %$form_elts_ref) {
        $keyhash = $form_elts_ref->{$key};
        $form_elt = Metadot::FormElement->new($key, $self->{_formname},
                       $self->{jsform}, $self->{_use_popcals}, $keyhash);
        # add to object hash
        $form_elt_hash->{$key} = $form_elt;
    }
    #set instance variable with result
#   print STDERR "processed form elts: ", Dumper($form_elt_hash);
    $self->{_form_elt_hash} = $form_elt_hash;
    }
    else {          # if form_elts_ref not defined
    $self->{_form_elt_hash} = {};
    }
}

# set the value of a form element object 'by hand' (rather than having it
# e.g. tied to the value of a db table record attribute)
sub setFormVarItemValue {
    my ($self, $varname, $value) = @_;

    my $form_vars = $self->{_form_vars};
    if (!$form_vars) {
    #print STDERR "In setFormVarItemValue: Error: form vars hash not created yet\n";
    }
    else {
#      print STDERR "in setFormVarItemValue, with form vars: ", Dumper($form_vars), "\n";
    my $form_elt = $form_vars->{$varname};
#      print STDERR "in setFormVarItemValue, with form elt: ", Dumper($form_elt), "\n";
    if ($form_elt) {
        $form_elt->setValue($value);
    }
    else {
        #print STDERR "In setFormVarItemValue: warning: form variable $varname does not exist.\n";
    }
    }
}

# set the value of a given form element object, where the element is a
# member of a 'form list'.  (The list will have been generated dynamically
# from either a list of table columns or a list of 'data object'
# instance variables.
sub setFormVarListValue {
    my ($self, $listvarname, $elementname, $value) = @_;

    my $form_vars = $self->{_form_vars};
    if (!$form_vars) {
    #print STDERR "In setFormVarItemValue: Error: form vars hash not created yet\n";
    }
    else {
    my $form_list = $form_vars->{$listvarname};
    if ($form_list) {
        # iterate through the list until find the elt with the given elementname
        foreach my $elt (@$form_list) {
        if (($elt->getName()) eq $elementname) {
            #print STDERR "in setFormVarListValue: found match with $elementname: ", Dumper($elt), "\n";
            # set the element's value
            $elt->setValue($value);
            last; # break out of the foreach.
        }
        }
    }
    else {
        #print STDERR "In setFormVarListValue: warning: form list variable $listvarname does not exist.\n";
    }
    }
}


# The form variable mappings tie a form element i.d. to a template
# variable.  These mappings can be specified either in the xml config
# file or via the methods addAtomicFormVar and addListFormVar.  This
# method takes those mappings, creates a form element object for the
# given i.d.  if one does not already exist, and instantiates the form
# element object with value and type info according to its associated
# database or 'data object'.
sub _processFormVarSpecs2 {
    my ($self, $form_var_ref, $tables, $pkeys, $mode) = @_;

#      print STDERR "in processFormVarSpecs2, with tables: " ,
#        Dumper($tables), " pkeys: ", Dumper($pkeys), " and mode $mode\n";

    if (!defined $pkeys) {
    $pkeys = {};
    }
    if ($mode eq 'create') {
    my $field_info = {};
    if ( exists($self->{_dataobj})) {
        # get form field values from object
        my $col_info_hash = Metadot::FormDataObj->getFieldInfo($self->{_dataobj},
                              $self->{_form_elt_hash});
        $field_info->{form_data_obj} = $col_info_hash;
    }
    else {
        foreach my $table (keys %$tables) {
        my $col_info_hash =
          Metadot::TableInfoFactory->getTableInfoObj()->getFieldInfo($table);
        $field_info->{$table} = $col_info_hash;
        }
    }
    # now process the field info results against the form specs
#   print STDERR "processing the field info against the form specs\n";
    my $form_vars = {};
    if (defined $form_var_ref) {
        foreach my $pvar (keys %$form_var_ref) {
        if (exists($form_var_ref->{$pvar}->{type}) &&
            $form_var_ref->{$pvar}->{type} eq 'item') { #  if atomic
            $self->_processAtomicFormVar($field_info, $form_var_ref, $form_vars, $pvar)
            }
        elsif (exists($form_var_ref->{$pvar}->{type}) &&
               $form_var_ref->{$pvar}->{type} eq 'list') { # if list
            $self->_processListFormVar($field_info, $form_var_ref, $form_vars, $pvar);
        }
        else {
            #  'item' is default
            $self->_processAtomicFormVar($field_info, $form_var_ref, $form_vars, $pvar)
        }
        }
#       print STDERR "****got form vars result: " , Dumper($form_vars), "\n";
        $self->{_form_vars} = $form_vars;
    }
    else {          # if form_var_ref not defined
        $self->{_form_vars} = {};
    }
    }

    elsif ($mode eq 'edit') {
    my $query_results = {};
    if ( exists($self->{_dataobj})) {
        # get form field values from object
        my $col_info_hash = Metadot::FormDataObj->getDataInfo($self->{_dataobj},
                              $self->{_form_elt_hash});
        $query_results->{form_data_obj} = $col_info_hash;
#       print STDERR "got value info from object: ", Dumper($col_info_hash), "\n";
    }
    else {          #get form field values from the db
        # first, for each table & its key info, generate a set of field/value/type triplets
        my $prim_key; my $pkval;
        foreach my $table (keys %$tables) {
        $prim_key = $tables->{$table}->{'prim_key'};
#       print STDERR "got primary key for table $table: ", Dumper($prim_key), "\n";
        my $pkey_query_clause = $self->_get_pkey_query_clause($table, $prim_key, $pkeys);
        my $col_info_hash =
            Metadot::TableInfoFactory->getTableInfoObj()->getDataInfo("*" ,
                              $table, $pkey_query_clause);
        $query_results->{$table} = $col_info_hash;
        }
    }
    # now process the query results against the form specs
    my $form_vars = {};
    foreach my $pvar (keys %$form_var_ref) {
        if (exists($form_var_ref->{$pvar}->{type}) &&
        $form_var_ref->{$pvar}->{type} eq 'item') { #  if atomic
        $self->_processAtomicFormVar($query_results, $form_var_ref, $form_vars, $pvar)
        }
        elsif (exists($form_var_ref->{$pvar}->{type}) &&
           $form_var_ref->{$pvar}->{type} eq 'list') { # if list
        $self->_processListFormVar($query_results, $form_var_ref, $form_vars, $pvar);
        }
        else {
        # item is default
        $self->_processAtomicFormVar($query_results, $form_var_ref, $form_vars, $pvar)
        }
    }
    $self->{_form_vars} = $form_vars;

    }               # end if 'edit'
    else {
    #print STDERR "$mode: unknown form mode\n";
    ErrorHandling->printError("$mode: unknown form mode\n");
    exit(0);
    }
    if (exists $self->{_form_vars}) {
    # set mode
    $self->{_form_vars}->{mode} = $mode;
    # set form name
    $self->{_form_vars}->{formname} = $self->{_formname};
    # set config hash
    if (exists $self->{_form_descr}->{config}) {
        $self->{_form_vars}->{config} = $self->{_form_descr}->{config};
    }
    # set info about tables used
    my @tinfo = ();
    foreach my $table (keys %$tables) {
        my $prim_key = $tables->{$table}->{'prim_key'};
        my $pkey_istring = "";
        if (exists $pkeys->{"$table.$prim_key"}) {
        $pkey_istring = "<input type=hidden name=\"$table" . "__" . $prim_key. "\" value=\"" .
            $pkeys->{"$table.$prim_key"} . "\">";
        }
        push (@tinfo, {name => $table,
               prim_key => "$table." . $tables->{$table}->{'prim_key'},
               pkey_istring => $pkey_istring }
          );
    }
    $self->{_form_vars}->{tables} = \@tinfo;
   }
}


sub _get_pkey_query_clause {
    my ($self, $table, $prim_key, $pkeys) = @_;

    # split the $prim_key info if necessary
    my $where_clause = "";
    my @pkey_names = split(',', $prim_key);
    my $first = 1;
    foreach my $pk (@pkey_names) {
    my $val = $pkeys->{"$table.$pk"};
    if (!$first) {
        $where_clause .= " and ";
    }
    else {
        $first = 0;
    }
    $where_clause .= "$pk = '$val'";
    }
    return $where_clause;
}

# process an 'atomic' form var -- that is, one which maps to a single
# form element id.
# (The alternative is a 'list form var', which maps to a
# list of form elements in that it maps to a list of of db table
# columns or a set of instance variable fields in a 'data object').
# If the form var maps to an element id without an associated element
# descriptor object, then one will be created for it.
sub _processAtomicFormVar {
    my ($self, $query_results, $form_var_ref, $form_vars, $pvar) = @_;

    my $element_name;
    if (exists $form_var_ref->{$pvar}->{elt}) {
    $element_name = $form_var_ref->{$pvar}->{elt};
#       print STDERR "element name: $element_name\n";
    }
    else {
    # else, default is that element name is same as form name
    $element_name = $pvar;
    }
    #get the query result for that element
    #split element name to find table and field
    my $qres;
    my ($table, $field) = split('\.', $element_name);
    if ($self->{_dataobj} &&
    exists ($query_results->{form_data_obj}->{$element_name})) {
    $qres = $query_results->{form_data_obj}->{$element_name};
#   print STDERR "extracted qres ", Dumper($qres), " for $element_name\n";
    }
    elsif (exists $query_results->{$table}->{$field}) {
    $qres = $query_results->{$table}->{$field};
    }
    else {
    $qres = {};
    }

    my $form_elt = $self->_getOrCreateFormElt($element_name, $qres);
    $form_vars->{$pvar} = $form_elt;
}

# look for the form element object named by the form var mapping.  If
# it does not exist yet, create it.  In the latter case, since no
# descriptor for that i.d. was specified, just use defaults.  If the
# i.d. maps to a table column, use info about the table col. type (and
# record value when appropriate).
sub _getOrCreateFormElt {
    my ($self, $element_name, $qres) = @_;
#      print STDERR "in _getOrCreateFormElt for ", Dumper($element_name), " with qres: ", Dumper($qres), "\n";
    my $qval="";
    if (exists($qres->{'value'}) && defined($qres->{'value'})) {
    $qval = $qres->{'value'};
    }
    my $qtype = $qres->{'type'};
    my $prec = $qres->{'precision'};

    my $form_elt;
    #look for form element in spec list
    if (exists $self->{_form_elt_hash}->{$element_name}) {
    # set the value for the form elt
#   print STDERR "found existing form elt description for $element_name: ",
#         Dumper($self->{_form_elt_hash}->{$element_name}), "\n";
    $form_elt = $self->{_form_elt_hash}->{$element_name};
    }
    else {
    # create the form elt, then set its value
    my $form_elttype = 'text';
    if ($qtype && ($qtype eq 'text')) {
        $form_elttype = 'textarea';
    }
    my $attrhash = {'label' => $element_name,
            'type' => $form_elttype};
    if ($prec) {
        $attrhash->{'precision'} =  $prec;
    }
    $form_elt = Metadot::FormElement->new($element_name, $self->{_formname},
                          $self->{jsform}, $self->{_use_popcals}, $attrhash);
    }
    $form_elt->setValue($qval, $qtype);
    return $form_elt;
}


# Map a form variable to a form element list by mapping a list of of
# db table columns or a set of instance variable fields in a 'data
# object', to the form variable.  If the form list var maps to an element
# id without an associated element descriptor object, then one will be
# created for it.
sub _processListFormVar {
    my ($self, $query_results, $form_var_ref, $form_vars, $pvar) = @_;

    #process table elements
    my $var_info = $form_var_ref->{$pvar};
#      print STDERR "var info: " , Dumper($var_info), "\n";

    my $tables = $var_info->{'table'};
    my $form_elt_listhash = {};
    #process each table
    foreach my $table (keys %$tables) {
    my $table_info = $tables->{$table};
#   print STDERR "processing table $table for list var $pvar\n";
    my @te = @{$table_info->{'exclude'}};
    #tbd: remove CRs first before splitting
    my @excludelist = split(',', $te[0]);
    if ($excludelist[0] eq 'none') {
        @excludelist = ();
    }
#       print STDERR "got exclude list: ", Dumper(\@excludelist), "\n";
    #use the query results for that table and the exclude list to build a
    #list of form elements.
    my $qres = $query_results->{$table};
    my $form_elt;
    foreach my $qelt (keys %$qres) {
        if (!_member(lc($qelt), @excludelist)) {
#       print STDERR "in processListFormVar\n";
        $form_elt = $self->_getOrCreateFormElt("$table.$qelt", $qres->{$qelt});
        $form_elt_listhash->{"$table.$qelt"} = $form_elt;
        }
    }
    }
    #process the singly specified fields
    my @flist = @{$var_info->{'fields'}};
    my @fieldslist = split(',', $flist[0]); # tbd: remove CRs
    if ($fieldslist[0] eq 'none') {
    @fieldslist = ();
    }
    foreach my $fieldvar (@fieldslist) {
    my ($table, $field) = split('\.', $fieldvar);
    my $qres = $query_results->{$table};
    my $form_elt = $self->_getOrCreateFormElt($fieldvar, $qres->{$field});
    $form_elt_listhash->{$fieldvar} = $form_elt;
    }

    #sort the result as it will be used in the form (alphabetically here).
    $form_vars->{$pvar} = $self->_sortFormEltListhash($form_elt_listhash);

}

# make a list sorted from the keys of a hash
sub _sortFormEltListhash {
    my ($self, $fl) = @_;

    my @form_elt_sorted_list = ();
    foreach my $k (sort keys %$fl) {
    push (@form_elt_sorted_list, $fl->{$k});
    }
    return \@form_elt_sorted_list;
}

# utility fcn -- check for membership in a list via 'eq'
sub _member {
    my ($elt, @lst) = @_;
    foreach my $lstelt (@lst) {
    if ($elt eq $lstelt) {
        return 1;
    }
    }
    return 0;
}


#========================================
# interface methods to add info to a form variable description: this is an alternate
# interface to the xml file.  They can be combined by reading in an XML file and then
# augmenting it with additional info via the method calls.  Method calls are allowed
# to overwrite/redefine info set by the XML file.

# set the name of the form that will be built in the html template.
# Optionally, set use_popcals to 'true' if
# you want the form elements to have a link to a javascript popup calendar placed
# automatically. [if not set, the popup calendar can still be added via variable
# access in the template].
# set the processing mode for the form.  Currently, 'edit' and 'create'
# are supported.
sub setFormName {       # will create form hash obj if it does not exist.
    my ($self, $formname, $use_popcals) = @_;

    $self->{_form_descr}->{'name'} = $formname;
    $self->{_formname} = $formname;

    if ($use_popcals eq 'true') {
    $self->{_use_popcals} = 1;
    }
    else {
    $self->{_use_popcals} = 0;
    }
    $self->{_form_descr}->{'use_popcals'} = $use_popcals; #  'true' or 'false'

    # create element_descriptor and fvar sub hashes if they do not exist.
    if (!exists $self->{_form_descr}->{'element_descriptor'}) {
    $self->{_form_descr}->{'element_descriptor'} = {};
    }
    if (!exists $self->{_form_descr}->{'fvar'}) {
    $self->{_form_descr}->{'fvar'} = {};
    }
}

# set the 'mode' of the db table handling.  Currently 'edit' and
# 'create' are supported.  'create' mode supports adding a new entry.
# 'edit' mode requires a primary key to be given for each table being
# accessed (passed as an arg to processFormVarSpecs), and retrieves
# the data object that matches the primary key.
# Is required to be called either after the xml definition file is
# loaded and/or after the setFormName method has been called to
# create the form description structure.
sub setMode {
    my ($self, $mode) = @_;
    if (exists $self->{_form_descr}) {
    $self->{_form_descr}->{'mode'} = $mode;
    }
    else {
#   die "can't set mode: form description does not exist yet.";
    ErrorHandling->printError("can't set mode: form description does not exist yet.");
    }
}

# add an attribute/value pair to the 'config' template variable.  This
# is a catch-all variable that can be used for stashing arbitrary info
# about the form that you want to generate, that is not associated
# with a specific form element.
sub addConfigAttrValue {
    my ($self, $attr, $value) = @_;

    if (!exists ($self->{_form_descr})) {
    ErrorHandling->printError("can't set attribute/value: form description does not exist yet.");
    exit(0);
    }
    my $descr_ref = $self->{_form_descr};
    my $config = $descr_ref->{'config'};
    if ($config) {
    $config->{$attr}->{'value'} = $value;
    }
    else {
    $config = {$attr => { value => $value}};
    $self->{_form_descr}->{'config'} = $config;
    }
}

# add info to the form description hash indicating that the given table
# will be used in building the form info.  $prim_key is
# the name of the primary key field for the table.  It is required.
# e.g., tablename = 'message', prim_key = 'mid'.
sub addTableInfo {
    my ($self, $tablename, $prim_key) = @_;

    if (!defined($tablename) || !defined($prim_key)) {
#   die "error: did not provide tablename or primary key information";
    ErrorHandling->printError( "error: did not provide tablename or primary key information");
    exit(0);
    }
    my $descr_ref = $self->{_form_descr};
    $descr_ref->{'table'}->{$tablename} = {prim_key => $prim_key};
}

# ==============================
# the following four methods provide an interface for adding form
# 'element descriptors' to the form description hash object.
# These describe the form element to use for a given named descriptor.
# Then, the descriptor may be mapped to a form variable referenced in the
# template, either via xml <fvar> instructions or via the addAtomicFormVar
# and addListFormVar methods.

# If the descriptor is to map to a field in a db table, the 'name' arg
# in the methods below must take the form "<tablename>.<fieldname>".
# If the descriptor is not tied to a db table field, the name can be
# any string appropriate for use as an html form element name.

# The following method is used for both input and textarea element types.
# 'type' should be one of the following form input types:
#    text, password, submit, reset, hidden, file, textarea
# 'label': the label to be used for the input field in the form
# 'attrs': any other form element attributes, aside from 'name', that
# should be added when building the element's html.  E.g. 'maxlength'.
sub addTextElementDescriptor {
    my ($self, $name, $type, $label, $attrs) = @_;
    # tbd: check that all necess. variables instantiated
    #create
    my $form_elth = {type => $type, label =>$label};
    # copy the attributes
    foreach my $attr (keys %$attrs) {
    $form_elth->{$attr} = $attrs->{$attr};
    }
    # add to _form_elt_hash (will overwrite/redefine elt if already exists)
    my $form_elt = Metadot::FormElement->new($name, $self->{_formname}, $self->{jsform},
                         $self->{_use_popcals}, $form_elth);
    $self->{_form_elt_hash}->{$name} = $form_elt;
#      print STDERR "after add, form elt hash is: ", Dumper($self->{_form_elt_hash}), "\n";
}

# this method is used to create a radio or checkbox form descriptor.
# 'type' should be either 'radio' or 'checkbox'.
# 'label': the label for the set of buttons/checkboxes (set to empty string if you want)
# 'buttonlist': this is a list of hash refs, with each hash containing two attributes:
# 'label', and 'value'. This info will be used to construct the individual form
# elements with the given name.  e.g.
#my $buttonlist = [{label => 'good quote', value => 'this is a good quote'},
#         {label => 'bad quote', value => 'this is a bad quote'}];
# Note that you often want only one checkbox element per name, but it is possible to
# use more, in which case the result is a multi-value.
# 'attrs': any other form elements, aside from name and value, that should be added
# when building the element's html.
sub addRadioCheckboxElementDescriptor {
    my ($self, $name, $type, $label, $buttonlist, $attrs) = @_;
    # tbd: check that all necess. variables instantiated etc.
    #create
    my $form_elth = {type => $type, label =>$label};
    # copy the attributes
    foreach my $attr (keys %$attrs) {
    $form_elth->{$attr} = $attrs->{$attr};
    }
    $form_elth->{'button'} = $buttonlist;
    # add to _form_elt_hash (will overwrite/redefine elt if already exists)
    my $form_elt = Metadot::FormElement->new($name, $self->{_formname}, $self->{jsform},
                         $self->{_use_popcals}, $form_elth);
    $self->{_form_elt_hash}->{$name} = $form_elt;
}

# the following two methods allow 'select' element descriptors to be defined.
# The first builds the select list from a static array; the second populates
# it from a database query.

# define select form element using a static content list.
# if 'multiple' evaluates to true, multiple selections will be allowed.
# 'valuelist' is a list of arrayrefs, each containing: name, value, [selected]
# 'attrs': any other form elements, aside from name, value, and multiple,
# that should be added when building the element's html.
sub addSelectElementDescriptorStaticValue {
    my ($self, $name, $label, $multiple, $valuelist, $attrs) = @_;
    # tbd: check that all necess. variables instantiated
    #create
    my $form_elth = {type => 'select', label =>$label, valuetype => 'static'};
    if ($multiple) {
    $form_elth->{'multiple'} = 't';
    }
    $form_elth->{'value'} = $valuelist;
    # copy the attributes
    foreach my $attr (keys %$attrs) {
    $form_elth->{$attr} = $attrs->{$attr};
    }
    my $form_elt = Metadot::FormElement->new($name, $self->{_formname}, $self->{jsform},
                         $self->{_use_popcals}, $form_elth);
#      print STDERR "in addSelectElementDescriptorStaticValue: got elt: ", Dumper($form_elt), "\n";
    $self->{_form_elt_hash}->{$name} = $form_elt;
}


# define select form element using a database query to populate the list
# contents.
# if 'multiple' evaluates to true, multiple selections will be allowed.
# 'query': the database query to run to populate the select list.  Each row
# of the query results should return a pair in this order: value, label.
# 'attrs': any other form elements, aside from name, value, and multiple,
# that should be added when building the element's html.
sub addSelectElementDescriptorDynamicValue {
    my ($self, $name, $label, $multiple, $null_first_item, $query, $attrs) = @_;
    # tbd: check that all necess. variables instantiated
    #create
    my $form_elth = {type => 'select', label =>$label, valuetype => 'dynamic'};
    $form_elth->{'query'} = $query;
    if ($multiple) {
    $form_elth->{'multiple'} = 't';
    }
    if ($null_first_item) {
    $form_elth->{'null_first_item'} = $null_first_item;
    }
    # copy the attributes
    foreach my $attr (keys %$attrs) {
    $form_elth->{$attr} = $attrs->{$attr};
    }
    my $form_elt = Metadot::FormElement->new($name, $self->{_formname}, $self->{jsform},
                         $self->{_use_popcals}, $form_elth);
    $self->{_form_elt_hash}->{$name} = $form_elt;
}

# ==============================
# the following two methods allow a form var specifier to be added to
# the form description hash object.

# add to the form description info: map a form variable name to a
# form element descriptor.
sub addAtomicFormVar {
    my ($self, $varname, $elt_uid) = @_;
    my $descr_ref = $self->{_form_descr};
    my $form_var_ref = $descr_ref->{'fvar'};
    # if elt_uid not provided, use varname
    if (!defined( $elt_uid)) {
    $elt_uid = $varname;
    }
    $form_var_ref->{$varname} = {type => 'item', elt => $elt_uid};
}

# add form var info which will expand into a list of form descriptor elements.
# 'tableinfo' is a ref to a hash. The keys are table names and the key values are
# 'excludelist' strings.
# 'fieldstring' and 'excludelist' strings must be set to the string 'none' if empty.
# tbd: does the split work okay if done on an empty string also?
sub addListFormVar {
    my ($self, $varname, $tableinfo, $fields_string) = @_;
    # tbd: check for proper var instantiation
    my $descr_ref = $self->{_form_descr};
    my $form_var_ref = $descr_ref->{'fvar'};
    $form_var_ref->{$varname}->{'type'} = 'list';
    $form_var_ref->{$varname}->{'fields'} = [ $fields_string ];
    foreach my $table (keys %$tableinfo) {
    my $excludelist = $tableinfo->{$table};
    $form_var_ref->{$varname}->{'table'}->{$table}->{'exclude'} = [ $excludelist ];
    }
#      print STDERR "after adding list var, got: ", Dumper($form_var_ref), "\n";
}


1;

__END__

=head1 NAME

Metadot::FormBuilder - A library class which provides an interface to a set of
form building capabilities.
It uses L<Metadot::FormElement>, which implements methods for managing and
displaying specific form elements.


=head1 SYNOPSIS

    use Metadot::FormBuilder
    $fb = Metadot::FormBuilder->new([$dataobj]);

    $hashref = $fb->getFormTemplateData();
    $hashref = $fb->getFormDescription ($config_file);
    $fb->processFormDescription([$pkeys]);
    $htmlstring = $fb->buildTemplateString($template_path, $template_name);
    $fb->processFormElementSpecs();
    $fb->processFormVarSpecs([$pkeys]);
    $fb->setFormVarItemValue($varname, $value);
    $fb->setFormVarListValue($listvarname, $elementname, $value);
    $fb->setFormName($formname, $use_popcals);
    $fb->setMode($mode);
    $fb->addConfigAttrValue($attr, $value);
    $fb->addTableInfo($tablename, $prim_key);
    $fb->addTextElementDescriptor($name, $type, $label, $attrs);
    $fb->addRadioCheckboxElementDescriptor($name, $type, $label, $buttonlist, $attrs);
    $fb->addSelectElementDescriptorStaticValue($name, $label, $multiple, $valuelist, $attrs);
    $fb->addSelectElementDescriptorDynamicValue($name, $label, $multiple, $null_first_item, $query, $attrs);
    $fb->addAtomicFormVar($varname, $elt_uid);
    $fb->addListFormVar($varname, $tableinfo, $fields_string);

=head1 DESCRIPTION

The FormBuilder library class provides an interface to a set of form
building capabilities.  It uses L<Metadot::FormElement>, which
implements methods for managing and displaying specific form elements.


=head2 Methods

Constructor;  instantiates a new FormBuilder object:

    $fb = Metadot::FormBuilder->new([$dataobj]);

getFormTemplateData: Returns a hash of objects (form elements) corresponding to the form
variable descriptions that have been specified.  Will return undef
if form var specs have not been processed yet.

    $hashref = $fb->getFormTemplateData();

getFormDescription: Reads an xml form description config file and returns
the parsed nested hash corresponding to it. In addition, set the form name
and the flag which indicates whether or not to use popup calendars for
date fields.  The form description can also be augmented (or created
entirely) via method calls.

    $hashref = $fb->getFormDescription ($config_file);

processFormDescription: Process a nested hash corresponding to a form
description.  First, the description of the form elements is
processed.  Then, the form variables are processed and associated with
the form element info where specified.

    $fb->processFormDescription([$pkeys]);

buildTemplateString: should be called only after processFormVarSpecs
(or processFormDescription) is run.  Processes the given template
against the information constructed from the specification of the form
element objects.

    $htmlstring = $fb->buildTemplateString($template_path, $template_name);

processFormElementSpecs: process the element_descriptor info (if any)
read in from the xml specification file.  A form element object is
created for each description.  The info from the xml file can be
further augmented by method calls to create specific descriptors,
e.g. addTextElementDescriptor.

    $fb->processFormElementSpecs();

processFormVarSpecs: takes a set of form element objects, built from
xml form element descriptors and/or method calls.  Uses these objects,
plus info about the db tables or data obj that they are tied to, to
instantiate their value and type info.

    $fb->processFormVarSpecs([$pkeys]);

setFormVarItemValue: set the value of a form element object 'by hand'
(rather than having it e.g. tied to the value of a db table record
attribute)

    $fb->setFormVarItemValue($varname, $value);

setFormVarListValue: set the value of a given form element object,
where the element is a member of a 'form list'.  (The list will have
been generated dynamically from either a list of table columns or a
list of 'data object' instance variables.

    $fb->setFormVarListValue($listvarname, $elementname, $value);

setFormName: set the name of the form that will be built in the html
template.  Optionally, set use_popcals to 'true' if you want the form
elements to have a link to a javascript popup calendar placed
automatically. [if not set, the popup calendar can still be added via
variable access in the template].  set the processing mode for the
form.  Currently, 'edit' and 'create' are supported.

    $fb->setFormName($formname, $use_popcals);

setMode: set the 'mode' of the db table handling.  Currently 'edit' and
'create' are supported.  'create' mode supports adding a new entry.
'edit' mode requires a primary key to be given for each table being
accessed (passed as an arg to processFormVarSpecs), and retrieves
the data object that matches the primary key.
Is required to be called either after the xml definition file is
loaded and/or after the setFormName method has been called to
create the form description structure.

    $fb->setMode($mode);

addConfigAttrValue: add an attribute/value pair to the 'config'
template variable.  This is a catch-all variable that can be used for
stashing arbitrary info about the form that you want to generate, that
is not associated with a specific form element.

    $fb->addConfigAttrValue($attr, $value);

addTableInfo: add info to the form description hash indicating that
the given table will be used in building the form info.  $prim_key is
the name of the primary key field for the table.  It is required.
e.g., tablename = 'message', prim_key = 'mid'.

    $fb->addTableInfo($tablename, $prim_key);

The following four methods provide an interface for adding form
'element descriptors' to the form description hash object.
These describe the form element to use for a given named descriptor.
Then, the descriptor may be mapped to a form variable referenced in the
template, either via xml <fvar> instructions or via the addAtomicFormVar
and addListFormVar methods.

If the descriptor is to map to a field in a db table, the 'name' arg
in the methods below must take the form "<tablename>.<fieldname>".
If the descriptor is not tied to a db table field, the name can be
any string appropriate for use as an html form element name.

addTextElementDescriptor: The following method is used for both input
and textarea element types.
'type' should be one of the following form input types:

   text, password, submit, reset, hidden, file, textarea

'label': the label to be used for the input field in the form
'attrs': any other form element attributes, aside from 'name', that
should be added when building the element's html.  E.g. 'maxlength'.

    $fb->addTextElementDescriptor($name, $type, $label, $attrs);

this method is used to create a radio or checkbox form descriptor.
'type' should be either 'radio' or 'checkbox'.
'label': the label for the set of buttons/checkboxes (set to empty string if you want)
'buttonlist': this is a list of hash refs, with each hash containing two attributes:
'label', and 'value'. This info will be used to construct the individual form
elements with the given name.  e.g.

  my $buttonlist = [{label => 'good quote', value => 'this is a good quote'},
            {label => 'bad quote', value => 'this is a bad quote'}];

Note that you often want only one checkbox element per name, but it is possible to
use more, in which case the result is a multi-value.
'attrs': any other form elements, aside from name and value, that should be added
when building the element's html.

   $fb->addRadioCheckboxElementDescriptor($name, $type, $label, $buttonlist, $attrs);

The following two methods allow 'select' element descriptors to be defined.
The first builds the select list from a static array; the second populates
it from a database query.

addSelectElementDescriptorStaticValue: define select form element
using a static content list.  if 'multiple' evaluates to true,
multiple selections will be allowed.  'valuelist' is a list of
arrayrefs, each containing: name, value, [selected] 'attrs': any other
form elements, aside from name, value, and multiple, that should be
added when building the element's html.

    $fb->addSelectElementDescriptorStaticValue($name, $label, $multiple, $valuelist, $attrs);

addSelectElementDescriptorDynamicValue: define select form element
using a database query to populate the list contents.  if 'multiple'
evaluates to true, multiple selections will be allowed.  'query': the
database query to run to populate the select list.  Each row of the
query results should return a pair in this order: value, label.
'attrs': any other form elements, aside from name, value, and
multiple, that should be added when building the element's html.

    $fb->addSelectElementDescriptorDynamicValue($name, $label, $multiple, $null_first_item, $query, $attrs);

The following two methods allow a form var specifier to be added to
the form description hash object.

addAtomicFormVar: add to the form description info: map a form
variable name to a form element descriptor.

    $fb->addAtomicFormVar($varname, $elt_uid);

addListFormVar: add form var info which will expand into a list of
form descriptor elements.  'tableinfo' is a ref to a hash. The keys
are table names and the key values are 'excludelist' strings.
'fieldstring' and 'excludelist' strings must be set to the string
'none' if empty.

    $fb->addListFormVar($varname, $tableinfo, $fields_string);

=cut


