###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::FormElement;

#========================================
# FormElement provides methods for managing and displaying specific form
# element descriptors.  Used by the FormBuilder module.
#========================================

use strict;
use Template;
use Data::Dumper;
use CGI;
use XML::Simple;

use MetadotConfig;
use DBAccess;
use Metadot::PopupMgr;
use ErrorHandling;


use constant SEPARATOR => "__";
use constant DEFAULT_TEXTFIELD_SIZE => 25;
use constant DEFAULT_TEXTAREA_SIZE => 35;

# this info is used in generating the element's display strings
my @SPECIAL_TEXT_ATTRS = ('type', 'label', 'name', 'form_id', 'datatype', 'precision',
              'readonly', 'disabled', 'check_type');
my @SPECIAL_TEXTAREA_ATTRS = ('type', 'label', 'name', 'form_id', 'value', 'datatype', 'precision', 'check_type');
my @SPECIAL_SELECT_ATTRS = ('type', 'label', 'name', 'form_id', 'value', 'datatype', 'precision', 'check_type',
                'valuetype', 'multiple', 'query', 'null_first_item');

#constructor
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $name = shift;
    my $formname = shift;
    my $jsform = shift;
    my $use_popcals = shift;
    my $attr_hash = shift;
    my $self = {};
    if (!$name || !$formname) {
#   die "name and formname must be provided";
    ErrorHandling->printError( "name and formname must be provided");
    exit(0);
    }
    else {
    $self->{_name} = $name;
    $self->{_formname} = $formname;
    }
    $self->{_use_popcals} = $use_popcals; # whether or not to automatically use javascript
                      # popup calendars for text input fields for vars of
                      # type date*
    bless ($self, $class);
    if ($attr_hash) {
    $self->{_attr_hash} = $attr_hash;
    # if the new element is of type 'select' and requires a dynamic query to
    # populate it select list, do the query.
    if ($attr_hash->{'type'} eq 'select' && $attr_hash->{'valuetype'} eq 'dynamic') {
        $self->_doDynamicSelectQuery();
    }
    }
    if ($jsform) {
    $self->{jsform} = $jsform;
    }
    $self->setJSInfoSimple();
    return $self;
}


#accessor -- get element's label
sub getLabel {
    my $self = shift;
    return $self->{_attr_hash}->{'label'};
}

# get element's name
sub getName {
    my $self = shift;
    return $self->{_name};
}


# accessor -- get element's value info.
# Can be called from the template interface if the designer wants to get just
# the object's value, instead of rendering it as a form element.
# Should only be called for
# string values, as it will not return useful info for object values.
# That is, this method does not currently check as to whether the
# value is a string; if the value is an object, the print string for
# it will not be useful.
sub getSingleValue {
    my $self = shift;
    if (exists $self->{_attr_hash}->{'value'}) {
    return $self->{_attr_hash}->{'value'};
    }
    else {
    return "";
    }
}

# set info in the jsform object, corresponding to the element info & provided
# check_type.
sub setJSInfoSimple {
    my $self  = shift;

#      print STDERR "in setJSInfoSimple for " . $self->getInputName() . "\n";
    my $ahash = $self->{_attr_hash};
#      print STDERR "attr hash is: ", Dumper($ahash) , "\n";
    if ($self->{jsform}) {
    if ($ahash->{check_type}) {
        $self->{jsform}->add_check({
        name => $self->getInputName(),
        label_text => $ahash->{label},
        check_type => $ahash->{check_type}
        }) ;
    }
#   print STDERR "jsform obj is: ", Dumper($self->{jsform}) , "\n";
    }
}

# set the element descriptor's attribute hash.  This contains both special-purpose
# display info as well as generic html attributes for the form element.
sub setAttrHash {
    my ($self, $attr_hash) = @_;
    if ($attr_hash) {
    $self->{_attr_hash} = $attr_hash;
    # if the new element is of type 'select' and requires a dynamic query to
    # populate it select list, do the query.
    if ($attr_hash->{'type'} eq 'select' && $attr_hash->{'valuetype'} eq 'dynamic') {
        $self->_doDynamicSelectQuery();
    }
    }
}

sub getCalendarPopup {
    my $self = shift;
    my $name = $self->getInputName();
    return Metadot::PopupMgr->getCalPopupObj->getCalendarPopup($name, $self->{_formname});
}

sub getCalendarPopupT {
    my $self = shift;
    my $name = $self->getInputName();
    return Metadot::PopupMgr->getCalPopupObj->getCalendarPopupT($name, $self->{_formname});
}

# set the value of the form element descriptor.  The specifics of this
# depend upon the element descriptor type.
sub setValue {
    my ($self, $value, $dtype) = @_;

    if ($dtype) {
    $self->{_attr_hash}->{'datatype'} = $dtype;
    }
    my $ftype = lc ($self->{_attr_hash}->{'type'});
    if ($value ne "" && $ftype eq 'select') {
    # find the value, mark it selected
    my @valuelist = @{$self->{_attr_hash}->{'value'}};
    my $found = 0;
    foreach my $item (@valuelist) {
        if ($item->{'value'} eq $value) {
        $item->{'selected'} = 't';
        $found = 1;
        }
    }
    if (!$found) {
#       print STDERR Dumper($self->{_attr_hash}), "\n";
#       die "error: object's current value $value not in value options for $ftype object.";
        ErrorHandling->printError( "error: object's current value $value not in value options for $ftype object.");
        exit(0);
    }
    }
    elsif ($value ne "" && ($ftype eq 'radio' || $ftype eq 'checkbox')) {
    # find the value, mark it checked.
    my @blist = @{$self->{_attr_hash}->{'button'}};
    my $found = 0;
    foreach my $button (@blist) {
        if ($button->{'value'} eq $value) {
        $button->{'checked'} = 't';
        $found = 1;
        }
    }
    if (!$found) {
#       die "error: object's current value $value not in value options for $ftype object.";
        ErrorHandling->printError( "error: object's current value $value not in value options for $ftype object.");
        exit(0);
    }
    }
    else {
    if ($value ne "") { # if value, override default
        $self->{_attr_hash}->{'value'} = $value;
    }
    else {
        # else, only set to empty string if default doesn't exist.
        if (!exists($self->{_attr_hash}->{'value'})) {
        $self->{_attr_hash}->{'value'} = "";
        }
    }
    }
}

# generate an html form element string for the object.  Calls various
# submethods to actually generate the html depending upon the element
# type.
sub buildFormInputString {
    my $self = shift;
    if (!exists $self->{_attr_hash}) {
#   die "error: attribute hash not set";
    ErrorHandling->printError( "error: attribute hash not set");
    exit(0);
    }
    else {
    my $ahash = $self->{_attr_hash};
#   print STDERR "in buildFormInputString for: ", $self->{name}, ", ",  Dumper($ahash);
    my $type = lc($ahash->{type});
    my $label = lc($ahash->{label});
    if (!$label) {
        $label = $self->{_name}; # tbd: just use end of path expr
    }
    if ($type eq 'text' || $type eq 'password' ||
        $type eq 'submit' || $type eq 'reset' ||
        $type eq 'hidden' || $type eq 'file') {
        return $self->_buildInputTextString($type, $ahash);
    }
    elsif ($type eq 'textarea') {
        return $self->_buildInputTextAreaString($ahash);
    }
    elsif ($type eq 'select') {
        return $self->_buildInputSelectString($ahash);
    }
    elsif ($type eq 'checkbox' || $type eq 'radio') {
        return $self->_buildInputChkOrRadioString($type, $ahash);
    }
    else {
        #print STDERR "error: unknown form input type $type\n";
        ErrorHandling->printError("error: unknown form input type $type\n");
    }
    }
}

# get the 'name' attr of the form element, in terms of the name string
# that will be used in building the html.  This string replaces the
# '.' in the actual name's path expression (e.g. 'instance.iid') with
# '__'.  This is because it can cause problems in using javascript if
# element names have '.'s in them.  In FormProcess, the real path
# expression is reconstructed and used to support its db save/modify
# ops.
sub getInputName {
    my $self = shift;

    my $iname;
    # if a specific form id was specified, use it.
    if (exists $self->{_attr_hash}->{'form_id'}) {
    $iname = $self->{_attr_hash}->{'form_id'};
    }
    # otherwise, use the element's name.  If it is a path expression,
    # replace the '.' in the path with the given separator.
    else {
    $iname = $self->{_name};
    my $sep = SEPARATOR;
    $iname =~ s/\./$sep/g;
    }
    return $iname;
}

#get a list of the radio buttons for the element -- this method can be used
#from the form template
sub getRadioList {
    my $self = shift;
    my $ahash = $self->{_attr_hash};
    return $self->_buildInputChkOrRadioList('radio', $ahash);
}

#get a list of the checkbox buttons for the element -- this method can be used
#from the form template
sub getCheckboxList {
    my $self = shift;
    my $ahash = $self->{_attr_hash};
    return $self->_buildInputChkOrRadioList('checkbox', $ahash);
}

# ======= 'private' methods ================

# build a form input html string for a 'select' element
sub _buildInputSelectString {
    my ($self, $ahash) = @_;

    my $formstring = "<select ";
    my $iname = $self->getInputName();
    $formstring .= " name=\"$iname\" ";
    if ($ahash->{'multiple'}) {
    $formstring .= " multiple ";
    }
    my $lckey;
    foreach my $key (keys %$ahash) {
    $lckey = lc $key;
    if (!_member($lckey, @SPECIAL_SELECT_ATTRS)) {
        # then add to input attrs
        if (!defined ($ahash->{$key})) {
        $formstring .= "$key=\"\" ";
        }
        else {
        $formstring .= "$key=\"" . $ahash->{$key} . "\" ";
        }
    }
    }
    $formstring .= ">\n";
    #get select menu contents from value attr of attr hash
#      if (lc ($ahash->{'listtype'}) eq 'static') {
    my @valuelist = @{$ahash->{'value'}};
    foreach my $item (@valuelist) {
    my $selected = "";
    if ($item->{'selected'}) {
        $selected = "selected";
    }
    if (exists $item->{'value'}) {
        $formstring .= "<option $selected value=\"" . $item->{'value'} .
        "\">" . $item->{'label'} . "</option>\n";
    }
    else {
        $formstring .= "<option $selected>" . $item->{'label'}  . "</option>\n";
    }
    }
    $formstring .= "</select>";
    $self->{_formstring} = $formstring;
    return $formstring;
}

# build a list of hash objects that can be used within the form
# template to display a group of radio or checkbox objects.
# (form descriptor elements can contain multiple 'buttons' of the same name).
sub _buildInputChkOrRadioList {
    my ($self, $type, $ahash) = @_;

    my @blist = @{$ahash->{'button'}};
    my $iname = $self->getInputName();
    my @inputlist = ();
    foreach my $button (@blist) {
    my $formstring = "<input type=\"$type\" ";
    $formstring .= " name=\"$iname\" ";
    if ($button->{'checked'}) {
        $formstring .= " checked ";
    }
    my $value = $button->{'value'};
    $formstring .= " value=\"$value\">";
    my $b = {label => $button->{'label'},
         inputstring => $formstring};
    push (@inputlist, $b);
    }
    $self->{_inputlist} = \@inputlist;
    return \@inputlist;
}

# build a form input html string for a radio or checkbox element
# (these elements can contain multiple 'buttons' of the same name).
# This method (or rather its parent method) is an alternative to
# _buildInputChkOrRadioList above.  The above method allows more
# control within the template of the html placement; this returns the
# html for all the buttons all in one string.
sub _buildInputChkOrRadioString {
    my ($self, $type, $ahash) = @_;

    my @blist = @{$ahash->{'button'}};
    my $formstring = ""; my $value = "";
    my $iname = $self->getInputName();
    foreach my $button (@blist) {
    $formstring .= $button->{'label'} . ": <input type=\"$type\" ";
    $formstring .= " name=\"$iname\" ";
    if ($button->{'checked'}) {
        $formstring .= " checked ";
    }
    $value = $button->{'value'};
    $formstring .= " value=\"$value\"><br />\n ";
    }
    $self->{_formstring} = $formstring;
    return $formstring;
}

# build a form input html string for an 'input' element
sub _buildInputTextString {
    my ($self, $type, $ahash) = @_;

    my $formstring = "<input type=\"$type\" ";
    my $iname = $self->getInputName();
#      print STDERR "in buildInputTextString for $iname\n";
    $formstring .= " name=\"$iname\" ";
    if ($ahash->{'disabled'}) {
    $formstring .= " disabled ";
    }
    if ($ahash->{'readonly'}) {
    $formstring .= " readonly ";
    }
    if (!exists($ahash->{'size'})) {
    if (($ahash->{datatype} eq 'date') ||
        ($ahash->{datatype} eq 'datetime')) {
        $ahash->{'size'} = $ahash->{'precision'} + 1;
    }
    else {
        $ahash->{'size'} = DEFAULT_TEXTFIELD_SIZE;
    }
    }
    if (!exists($ahash->{'maxlength'})) {
    $ahash->{'maxlength'} = $ahash->{'precision'};
    }
    my $lckey;
    foreach my $key (keys %$ahash) {
    $lckey = lc $key;
    if (!_member($lckey, @SPECIAL_TEXT_ATTRS)) {
        # then add to input attrs
        if (!defined ($ahash->{$key})) {
        $formstring .= "$key=\"\" ";
        }
        else {
        $formstring .= "$key=\"" . $ahash->{$key} . "\" ";
        }
    }
    }
    $formstring .= ">";
    if ($self->{_use_popcals}  && $ahash->{datatype}) {
    if ($ahash->{datatype} eq 'date') {
        $formstring .= " " . $self->getCalendarPopup();
    }
    elsif ($ahash->{datatype} eq 'datetime') {
        $formstring .= " " . $self->getCalendarPopupT();
    }
    }
#      print STDERR "got form input string: $formstring\n";
    $self->{_formstring} = $formstring;
    return $formstring;
}

# build a form input html string for a 'textarea' element
sub _buildInputTextAreaString {
    my ($self, $ahash) = @_;

    my $formstring = "<textarea ";
    my $iname = $self->getInputName();
    $formstring .= " name=\"$iname\" ";
    my $lckey;
    if (!exists $ahash->{'rows'}) {
    $ahash->{'rows'} = 5;
    }
    if (!exists $ahash->{'cols'}) {
    $ahash->{'cols'} = DEFAULT_TEXTAREA_SIZE;
    }
    foreach my $key (keys %$ahash) {
    $lckey = lc $key;
    if (!_member($lckey, @SPECIAL_TEXTAREA_ATTRS)) {
        # then add to input attrs
        $formstring .= "$key=\"" . $ahash->{$key} . "\" ";
    }
    }
    my $value = $ahash->{'value'};
    $formstring .= ">$value</textarea>";
#      print STDERR "got form input string: $formstring\n";
    $self->{_formstring} = $formstring;
    return $formstring;
}


# this method is called in the context of building a form element
# descriptor of type 'select'.  'select' objects can be configured to
# have a dynamically populated select list, populated via a db query
# This method runs the query and constructs the select list from the
# result.  The query is expected to return result rows that consist of pairs:
# (value, label).
sub _doDynamicSelectQuery {
    my ($self) = @_;

    my $sth;
    eval {
#   print STDERR "dynamic query is: ", $self->{_attr_hash}->{'query'}, "\n";
    $sth = DBAccess->get_db_object->sqlSimpleSelect($self->{_attr_hash}->{'query'});
    };
    if ($@) {
    #print STDERR "error: $@ \n";
    ErrorHandling->printError("error: $@ \n");
    exit(0);
    }
    my @valuelist = ();
    if (exists ($self->{_attr_hash}->{'null_first_item'})) {
    my $nlabel = $self->{_attr_hash}->{'null_first_item'};
    my $item = {label => $nlabel, value => "", selected => 't'};
    push @valuelist, $item;
    }
    while (my ($value, $label) = $sth->fetchrow)  {
    my $item = {label => $label, value => $value};
    push @valuelist, $item;
    }
    $self->{_attr_hash}->{'value'} = \@valuelist;
}


# check if a given element is a member of a list.  does an 'eq' only; won't
# work for complex objects.
sub _member {
    my ($elt, @lst) = @_;
    foreach my $lstelt (@lst) {
    if ($elt eq $lstelt) {
        return 1;
    }
    }
    return 0;
}

1;
