#######################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Gizmo3000::ClassBuilder;

use strict;
no strict 'refs';  # allows dynamic code generation

use Metadot::Gizmo3000;
use Metadot::Gizmo3000::ClassRegistry;
use Metadot::Form3000::PropertyMapping;
use Metadot::Gizmo3000::Container;
use Metadot::Gizmo3000::DedicatedChild;
use Operations;

use constant INITIALIZATION_FLAG   => 'G3K_CLASS_INITIALIZED';
use constant TARGET_BASE_PACKAGE   => 'Metadot::Gizmo3000';
use constant CONTAINER_CLASS       => 'Metadot::Gizmo3000::Container';
use constant DEDICATED_CHILD_CLASS => 'Metadot::Gizmo3000::DedicatedChild';



# static method to initialize a Gizmo3000 class
# descriptors and run-time properties.
# returns a ClassDescriptor object that describes the class that was built.
sub initialize_class {
    my $class  = shift;
    my $target = shift;
    my $visited = shift || { $target => 1 };  # prevents loops

    (defined($target) && $target->isa(TARGET_BASE_PACKAGE))
        || die("$target is not a ".TARGET_BASE_PACKAGE);

    return if($class->is_built($target));


    # make sure the ancestors have been initialized
    # get my ancestor hierarchy
    foreach my $ancestor ($target->class_hierarchy()) {
        $class->initialize_class($ancestor, $visited);
    }


    #print STDERR "$class\->initialize_class($target)\n";


    # this needs to happen before we create the fields, because we
    # use introspection to make sure that to 'stored_at_column' value
    # is valid
    $target->MO_make_methods();
    #print STDERR "$class\->initialize_class($target) made methods\n";

    # build a class descriptor
    my $class_descriptor = $class->build_descriptor($target);
    #print STDERR "$class\->initialize_class($target) build descriptor $class_descriptor\n";


    # generate and run the code to set up the class' properties
    Metadot::Gizmo3000::ClassBuilder->build_runtime_properties($target, $class_descriptor);
    #print STDERR "$class\->initialize_class($target) build runtime properties($target, $class_descriptor)\n";


    # generate the default permissions object
    my $perms = Operations->new();
    my %perms_visited = ();
    foreach my $ancestor ($target->class_hierarchy({inclusive => 1, base_class=>'UNIVERSAL'})) {
        unless($perms_visited{$ancestor}++) {
            if ($ancestor->can('customize_permissions')) {
                # print STDERR "Customizing $target\'s permissions object $perms with ancestor $ancestor\n";
                $ancestor->customize_permissions($perms);
            }
        }
    }

    # then add the permissions to the descriptor
    $class_descriptor->set_default_permissions($perms);

    #print STDERR Dumper $class_descriptor;
    #print STDERR "$class initialization($target) complete.\n\n\n\n";

    return $class_descriptor;
}



# static method to build the ClassDescriptor
sub build_descriptor {
    my $class           = shift;
    my $target          = shift;
    my $target_class    = ref($target) || $target;

    # verify the inputs
    (defined($target) && $target->isa('Metadot::Gizmo3000'))
        || die('Must provide a reference to a Gizmo3000 class');


    #print STDERR "$class\->build_descriptor($target), class=$target_class\n";

    # Build a ClassDescriptor object based on the target's GIZMO_INFO
    my $class_descriptor = Metadot::Gizmo3000::ClassDescriptor->new(
        $target->GIZMO_CONFIG()->{$target->GIZMO_INFO()}
    );

    $class_descriptor->class_name($target_class);
    #print STDERR "$class\->build_descriptor($target)has descriptor $class_descriptor\n";


    # iterate through the GIZMO_FIELD hash tables and generate
    # a Form3000::FormField object for each.

    # make sure that no name or column is re-used
    my %used_names   = ();
    my %used_columns = ();

    foreach my $field_definition (@{$target->GIZMO_CONFIG()->{$target->GIZMO_FIELDS()}}) {
        my $field_mapping    = Metadot::Form3000::PropertyMapping->new(
            # TODO: remove these redundancies (field_definition has the same values)
            property_name       => $field_definition->{name},
            mapped_property     => $field_definition->{store_at_column},
            display_order       => $field_definition->{display_order},
            label               => $field_definition->{label},
            field_definition    => $field_definition,
        );

        # make sure this field descriptor is valid
        my $field_name    = $field_mapping->property_name();
        my $display_order = $field_mapping->display_order();
        my $field_column  = $field_mapping->mapped_property();

        defined($field_name) || die("Error in $class: fields must all have names");
        defined($field_column) || die("Error in $class, dynamic columns are not yet supported");

        !($used_names{$field_name}++) || die("Error in $class: field name '$field_name' used multiple times!");
        !($used_columns{$field_column}++) || die("Error in $class: field column '$field_column' used multiple times!");
        $target->can($field_column) || die("Error in $class: field column '$field_column' is not defined!");

        $class_descriptor->add_property_mapping($field_mapping);
        #print STDERR "$class\->build_descriptor($target) added property mapping $field_mapping\n";
    }

    # allow the class (and ancestors) to make some tweaks to the descriptor
    my %descriptor_visited = ();
    foreach my $ancestor ($target->class_hierarchy({inclusive => 1})) {
        unless ($descriptor_visited{$ancestor}++) {
            $class_descriptor = $ancestor->customize_descriptor($class_descriptor);
        }
    }

    return $class_descriptor;
}



# static method to build the runtime properties of a Gizmo3000 class
sub build_runtime_properties {
    my $class  = shift;
    my $target = shift;
    my $descriptor = shift;

    # verify the inputs
    (defined($target) && $target->isa('Metadot::Gizmo3000'))
        || die('Must provide a reference to a Gizmo3000 class');

    (defined($descriptor) && $descriptor->isa('Metadot::Gizmo3000::ClassDescriptor'))
        || die('Must provide a Gizmo3000 ClassDescriptor object');


    my $flag_name = INITIALIZATION_FLAG();
    my $code = <<EOM
package $target;

use constant $flag_name => 1;

EOM
;

    # check if this class is a container, and if so, make sure that
    # is set in the class hierarchy
    if (defined $descriptor->child_types()) {
        $code .= $class->_generate_container_code($target, $descriptor);
    }


    # check if this class is a dedicated child, and if so, set that
    # in the class
    if ($descriptor->is_dedicated_child()) {
        $code .= $class->_generate_dedicated_child_code($target, $descriptor);
    }


    # create all the accessors/mutators
    my $field_mappings = $descriptor->get_property_mappings();
    foreach my $field_mapping (@$field_mappings) {
        $code .= $class->_generate_method_code($target, $field_mapping);
    }

    #print STDERR "$class\->_generate_method_code generated:\n\n$code\n\n\n\n";
    eval $code;
    if ($@) {
        my $error_message = "Error making methods for $class\n".$@;
        $error_message .= "\n\nAttempted to eval code:\n";

        my @lines = split("\n", $code);
        for (my $i=0; $i<@lines; $i++) {
            $error_message .= ($i+1).' '.$lines[$i]."\n";
        }

        die($error_message);
    } else {
        #print STDERR "Successfully made methods for $class\n";
    }
}



# returns true if a class's runtime properties have been built
sub is_built {
    my $class  = shift;
    my $target = shift;

    my $result;
    if (defined $target) {
        my $flag_ref = $target.'::'.INITIALIZATION_FLAG();
        $result = (defined(&$flag_ref));
        #print STDERR "Checking if $target is built, using $flag_ref, got '$result'\n";
    }

    return $result;
}



# generates the accessors/mutators for each field
sub _generate_method_code {
    my $class   = shift;
    my $target  = shift;
    my $mapping = shift;

    (defined($target) && $target->isa('Metadot::Gizmo3000'))
        || die ('Not a Gizmo3000 object');
    (defined($mapping) && $mapping->isa('Metadot::Form3000::PropertyMapping'))
        || die ('Not a Metadot::Form3000::PropertyMapping');

    my $key = $mapping->property_name();
    my $mapped_property = $mapping->mapped_property();

    my $result;

    if (defined($key) && defined($mapped_property)) {

        ($target->can($mapped_property)) || die("$target does not have a method for '$mapped_property'");

        my $method_name = $key;
        if ($target->can($method_name)) {
            $method_name = "_$key";
        }

        # generate the perl-style combo accessor/mutator
        unless ($target->can($method_name)) {
            $result .= <<EOM;

    sub $method_name {
        my \$self = shift;
        return \$self->$mapped_property(\@_);
    }
EOM
;
            #print STDERR "Generated method code for $target\->$method_name()\n";
        }

        # auto-generate the accessor/mutator methods ONLY if this key is not private
        unless ($key =~ /^_/) {

            # generate the getter
            my $method_name = "get_$key";
            unless ($target->can($method_name)) {
                $result .= <<EOM

    sub $method_name {
        my \$self = shift;
        return \$self->$key();
    }
EOM
;
                #print STDERR "Generated method code for $target\->$method_name()\n";
            }


            # generate the setter
            $method_name = "set_$key";
            unless ($target->can($method_name)) {
                $result .= <<EOM

    sub $method_name {
        my \$self = shift;
        return \$self->$key(\@_);
    }
EOM
;
                #print STDERR "Generated method code for $target\->$method_name()\n";
            }
        }


    } else {
        #print STDERR "$target\->_generate_method_code did not have either key: $key or mapped_property: $mapped_property\n";
    }

    #print STDERR "Generated method code for $class, {$key}\n";
    return $result;
}




# a static method that turns a Gizmo3000 into a container
sub _generate_container_code {
    my $class  = shift;
    my $target = shift;
    my $descriptor = shift;

    # verify the inputs
    (defined($target) && $target->isa('Metadot::Gizmo3000'))
        || die('Must provide a reference to a Gizmo3000 class');

    (defined($descriptor) && $descriptor->isa('Metadot::Gizmo3000::ClassDescriptor'))
        || die('Must provide a Gizmo3000 ClassDescriptor object');

    my $code = '';


    # update the ISA
    if ($target->isa(CONTAINER_CLASS())) {
        #print STDERR "$target is already a ".CONTAINER_CLASS()."\n";
    } else {
        my $isa_ref = "${target}\::ISA";
        unshift @{$isa_ref}, CONTAINER_CLASS();
        #print STDERR "Adding ",CONTAINER_CLASS()," to $target's ISA; $isa_ref=",join(', ', @{$isa_ref}),"\n";
    }


    # add the allowed_child_types method
    my $method_name = 'allowed_child_types';
    unless (defined(&{"${target}\::$method_name"}) || !defined($descriptor->child_types())) {
        my $child_type_list = 'qw('.join(' ',@{$descriptor->child_types()}).')';
        $code .= <<EOM

    sub $method_name {
        my \$self = shift;
        my \$child_types = \$self->SUPER::$method_name() || \[];
        push \@{\$child_types}, $child_type_list;

        return \$child_types;
    }


EOM
;
    }



    # add the prohibited_child_types method

    $method_name = 'prohibited_child_types';
    unless (defined(&{"${target}\::$method_name"}) || !defined($descriptor->prohibited_child_types())) {
        my $child_type_list = 'qw('.join(' ',@{$descriptor->prohibited_child_types()}).')';
        $code .= <<EOM

    sub $method_name {
        my \$self = shift;
        my \$child_types = \$self->SUPER::$method_name() || \[];
        push \@{\$child_types}, $child_type_list;

        return \$child_types;
    }


EOM
;
    }

    #print STDERR "Generated container code:\n$code\n\n";
    return $code;
}





# a static method that turns a Gizmo3000 into a container
sub _generate_dedicated_child_code {
    my $class  = shift;
    my $target = shift;
    my $descriptor = shift;

    # verify the inputs
    (defined($target) && $target->isa('Metadot::Gizmo3000'))
        || die('Must provide a reference to a Gizmo3000 class');

    (defined($descriptor) && $descriptor->isa('Metadot::Gizmo3000::ClassDescriptor'))
        || die('Must provide a Gizmo3000 ClassDescriptor object');

    my $code = '';

    # update the ISA
    if ($target->isa(DEDICATED_CHILD_CLASS())) {
        #print STDERR "$target is already a ".DEDICATED_CHILD_CLASS()."\n";
    } else {
        my $isa_ref = "${target}\::ISA";
        unshift @{$isa_ref}, DEDICATED_CHILD_CLASS();
        #print STDERR "Adding ",DEDICATED_CHILD_CLASS()," to $target\'s ISA; $isa_ref=",join(', ', @{$isa_ref}),"\n";
    }

    return $code;
}


1;
