package Metadot::Gizmo3000::ClassRegistry;

=head1 NAME

Metadot::Gizmo3000::ClassRegistry

=head1 VERSION

Version $Revision: 1.9.2.2 $

=head1 SYNOPSIS

my $builder = Metadot::Gizmo3000::ClassBuilder->get_instance();
my $registered_modules = $builder->get_registered_modules();
my $searchable_modules = $builder->get_searchable_modules();

=head1 DESCRIPTION

Metadot::Gizmo3000::ClassRegistry keeps a record of all Gizmo3000 classes
available on a portal.

=cut

use strict;

use File::Path;
use File::Copy;

use Utility;
use Metadot::Gizmo3000;
use Metadot::Gizmo3000::ClassBuilder;
use Metadot::Gizmo3000::ClassDescriptor;

# the directory where Gizmo3000 classes are expected to be found
use constant LIBRARY_DIR => 'Gizmo3000';
use constant BASE_CLASS  => 'Metadot::Gizmo3000';




# this is a singleton for now, but this can change if needed
my $_SINGLETON;

# force this to initialize. later, we will put this in the portal
# initialization code
{
    #print STDERR "ClassRegistry initializes. . . .\n";
    __PACKAGE__->get_instance();
    #print STDERR "ClassRegistry initialization complete.\n";
}



=head1 METHODS

=cut

=head2 get_instance()

Gets an instance of the ClassRegistry for a given portal.

=cut
sub get_instance {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self;
    if (defined $_SINGLETON) {
        $self = $_SINGLETON;
    } else {
        $self  = {
            # stores the modules that were found in the system
            _registered_modules => {},
        };

        bless($self, $class);
        $_SINGLETON = $self;
        $self->initialize_modules();
    }

    return $self;
}







=head2 get_registered_modules()

returns a hashtable of all registered modules

=cut
sub get_registered_modules {
    my $self = shift;
    my $args = shift || {};

    die ('Instance method called by '.caller) unless (ref $self);

    my %result;

    my $isa = $args->{isa};

    if ($isa) {
        while (my ($class_name, $value) = each (%{$self->{_registered_modules}})) {
            if (UNIVERSAL::isa($class_name, $isa)) {
                $result{$class_name} = $value;
            }
        }
    } else {
        %result = %{$self->{_registered_modules}};
    }

    return \%result;
}



=head2 searchable_modules()

returns a hashtable of all registered modules which are searchable

=cut
sub searchable_modules {
    my $self       = shift;
    my $candidates = $self->{_registered_modules};
    my %results = ();

    while (my ($key, $candidate) = each %$candidates) {
        if ($candidate->is_searchable()) {
            $results{$key} = $candidate;
        }
    }

    return \%results;
}


=head2 register_class()

registers a Gizmo3000 class

=cut
sub register_class {
    my $self        = shift;
    my $target      = shift;
    my $descriptor  = shift;

    (defined $target && $target->isa(BASE_CLASS))
        || die($target." is not a ".BASE_CLASS);

    #print STDERR "Registering $target with $descriptor\n";
    #print STDERR Dumper $descriptor;
    $self->{_registered_modules}{$target} = $descriptor;
}



=head2 get_descriptor_for()

gets the registered class descriptor for a given class

=cut
sub get_descriptor_for {
    my $self   = shift;
    my $target = shift;

    return $self->{_registered_modules}{$target};
}



=head2 initialize_modules()

initializes all packages in the Gizmo3000 name space

=cut
sub initialize_modules {
    my $self = shift;

    #print STDERR "Registering Gizmo3000 modules. . .\n";
    $self->read_packages(LIBRARY_DIR);
}


=head2 read_packages()

reads all the subdirs of the library. All gizmo3000's are expected to be
in subdirs

=cut
sub read_packages {
    my $self                = shift;
    my $package_hierarchy   = shift || [];

    $package_hierarchy = [$package_hierarchy] unless (ref $package_hierarchy);
    my %results = ();

    my $dir_separator     = '/';
    my $package_separator = '::';

    my $dir_name = join($dir_separator, @$package_hierarchy);

    if (-e $dir_name && -d $dir_name) {
        #print STDERR "Checking package space $dir_name\n";
        # get the contents
        opendir SCRATCH_DIR, $dir_name;
        my @contents = grep !/^\.\.?$/, readdir SCRATCH_DIR;
        closedir SCRATCH_DIR;

        # process the contents
        foreach my $content (@contents) {
            my $file_name = "$dir_name$dir_separator$content";
            #print STDERR "Checking file $file_name\n";
            if (-d $file_name) {
                # this is a dir. try processing this as part of the package space
                #print STDERR "$content is a directory. reading package space.\n";
                $self->read_packages([ @$package_hierarchy, $content ]);
            } elsif ($content =~ /\.pm$/) {
                # this is a package. try evaluating it.
                eval {
                    require $file_name;
                };

                if ($@) {
                    print STDERR "Error evaluating $file_name\n",$@;
                } else {
                    # add the package to the list
                    (my $package_name = $content) =~ s/\.pm$//;
                    $package_name = join($package_separator, @$package_hierarchy, $package_name);
                    if ($package_name->isa(BASE_CLASS)) {
                        # this is a Gizmo3000. We always run initialize_class to initialize
                        # the runtime properties of the class
                        #print STDERR "read_packages() initializing $package_name\n";

                        my $class_descriptor;
                        eval {
                            # try to initialize the package
                            $class_descriptor = Metadot::Gizmo3000::ClassBuilder->initialize_class($package_name);
                        };
                        if ($@) {
                            warn("Error initializing $package_name\n".$@);
                        } else {
                            # register the class
                            $self->register_class($package_name, $class_descriptor);
                            #print STDERR "Added $package_name to registry\n";
                        }
                    } else {
                        #print STDERR "$package_name is NOT a ",BASE_CLASS,"\n";
                    }
                }
            } else {
                #print STDERR "Ignored file $file_name\n";
            }
        }
    } else {
        #print STDERR "Skipped $dir_name\n";
    }
}


=head2 deploy_resources()

deploys all Gizmo3000 resources into the resources directory to enable
web access

=cut
sub deploy_resources {
    my $class      = shift;
    my $source_dir = shift || LIBRARY_DIR;
    my $dest_dir   = shift || Metadot::Gizmo3000->get_resource_directory();

    print "Deploying resources from $source_dir to $dest_dir...\n";

    # blow away the old directory
    if (-e $dest_dir) {
        rmtree($dest_dir);
    }

    # create a fresh new directory
    mkpath($dest_dir, 0, 0755) || die ("Error: can't create $dest_dir");

    # put a readme file in that directory
    open (README, ">$dest_dir/README") || die("Error: can't write to $dest_dir");
    my $date = localtime(time);

    print README <<EOM
Process ID=$$;  Date: $date

WARNING: this directory is automatically generated by the upgrade script.
Any files you add here are subject to deletion!


EOM
;
    close README;

    # open the directory
    opendir(DIR, $source_dir) || die ("Error: can't open $dest_dir");

    # get all entries that are not . or ..
    my @contents = grep !/^\.\.?$/, readdir DIR;
    closedir DIR;

    # filter out .pm or .pl
    @contents = grep !/\.p[m|l]$/, @contents;

    # filter out .t
    @contents = grep !/\.t$/, @contents;

    # filter out .test
    @contents = grep !/\.test$/, @contents;

    # filter out CVS
    @contents = grep !/^CVS$/, @contents;

    foreach my $file (@contents) {
        my $source_file = "$source_dir/$file";
        my $dest_file   = "$dest_dir/$file";

        print "found: $source_file\n";
        if (-d $source_file) {
            $class->deploy_resources($source_file, $dest_file);
        } else {
            copy($source_file, $dest_file);
        }
    }
}
















1;
