###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Gizmo3000::Container;

use strict;

use Portal;
use Metadot qw($DISPLAY %FORM %REQUEST_VARS $USER $SESSION);
use Metadot::BaseGizmoContainer;
use Metadot::Gizmo3000;
use Metadot::GizmoContainer;

use vars qw(@ISA);
@ISA = qw(Metadot::Gizmo3000 Metadot::BaseGizmoContainer);

use constant BASIC_CHILD_OPS => [
    {
      op            => 'move_child_up_down',
      icon          => 'uparrow.gif',
      description   => 'Move me up',
      arguments     => {
          delta => -1,
          }
    },
    {
      op            => 'move_child_up_down',
      icon          => 'downarrow.gif',
      description   => 'Move me down',
      arguments     => {
          delta => 1,
          }
    },
];


sub customize_permissions {
   my $class = shift;
   my $perms = shift || Operations->new();

   #print STDERR "__PACKAGE__ $class\->customize_permissions($perms)\n";
   $perms->add_op('MOD', 'add_child');
   $perms->add_op('MOD', 'create_child');
   $perms->add_op('MOD', 'move_child_up_down');
}



# Must override the delete method so that the Container provides some handling of
# its children.
sub delete {
    my $self = shift;

    # first delete children
    $self->delete_children();

    # then delete self
    $self->SUPER::delete();
}


# this accepts a list of objects and adds them to the container.
sub add_children {
    my $self        = shift;
    my $child_list  = shift;
    my $args        = shift;

    die ('Abstract method');

}


# this operation generates the form which allows uses to add a child
# for now, a Gizmo3000::Container can only contain Gizmo3000 children
sub do_add_child {
    my $self = shift;

    my $parentid = Portal->defined_or_exit($self->id,"parent id is missing.");

    my $child = $self->build_child_object({
        parent_id   => $parentid,
        child_type  => $FORM{child_type},
        });

    # check for Gizmo3000
    if ($child->isa('Metadot::Gizmo3000')) {
        return $self->output_add_child_form($child);
    } else {
        $DISPLAY->print_error("Error attempting to add a child $child");
    }
}



# this operation accepts form input and actually creates the child object
sub do_create_child {
    my $self = shift;

    my $parentid = Portal->defined_or_exit($self->id,"parent id is missing.");

    my $child = $self->build_child_object({
        parent_id   => $parentid,
        child_type  => $FORM{child_type},
        });

    if (defined $child && $child->isa('Metadot::Gizmo3000')) {
        my $form = $child->process_edit_form();
        if ($form->has_errors()) {
            return $self->output_add_child_form($child);
        } else {
            $child->instance_order($self->next_vertical_position());
            $child->add();
            return $self->show_create_child_success($child);
        }
    } else {
        $DISPLAY->print_error("Error attempting to add a child $child");
    }
}


sub do_move_child_up_down {
    my $self = shift;

    my $delta = $FORM{delta};
    my $child_id = $FORM{target_iid};

    $self->move_child_up_down(Gizmo->restore($child_id), $delta);
    $DISPLAY->md_redirect($SESSION->referer());
}




# sub validate child type
sub _validate_child_type {
    my $self       = shift;
    my $child_type = shift || die('No child type specified');

    unless ($self->is_child_type_allowed($child_type)) {
        $DISPLAY->print_error($self->get_gizmo_name()." does not support ".$child_type->get_gizmo_name());
        exit;
        }
}




# builds the child object
sub build_child_object {
    my $self = shift;
    my $args = shift || {};

    my $parent_id = $args->{parent_id};
    $parent_id    = $self->id() unless (defined $parent_id);

    my $child_type = $args->{child_type} || die("Must specify the child_type");

    $self->_validate_child_type($child_type);

    my $child_constructor_args = {
        parent_id   => $parent_id,
        uid         => $args->{uid} || $USER->uid(),
    };

    my $child = $child_type->new($child_constructor_args);

    return $child;
}



# implement the internal add_child action for Gizmo3000. This will
# eventually replace www_add_child
sub output_add_child_form {
    my $self   = shift;
    my $child  = shift;

    # validate the data
    (defined $child && $child->isa('Metadot::Gizmo3000'))
        || die("Child $child must be a Metadot::Gizmo3000");


    $DISPLAY->debug_msg(4, "$self\->output_add_child_form with self $self; child $child");

    my ($fields, $values) = $child->define_form_fields();

    $DISPLAY->debug_msg(5, "$self\->output_add_child_form with child $child has fields $fields");

    # build the form object
    my $form = Metadot::Form3000::Form->new(
        'action'  => 'index.pl',
        'method'  => 'post',
        'enctype' => 'multipart/form-data',
        'name'    => 'fm',
        'title'   => 'Adding '.$child->get_gizmo_name(),
        'fields'  => $fields,
        'values'  => $values,
        'submit_actions' => [
            {
                name  => 'submit_form',
                value => 'Create',
            }
        ],
    );

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'isa',
        value => $self->is_a());

    $DISPLAY->debug_msg(5, "hidden field iid='".$self->iid()."'");
    $form->generate_form_field(
        type  => 'hidden',
        name  => 'iid',
        value => $self->iid());

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'op',
        value => 'create_child');

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'child_type',
        value => $child->is_a());

    $REQUEST_VARS{form_object} = $form;


    # Send output through the Metadot controller
    my $controller = Metadot::PortalFactory->get_controller();
    my $context = Metadot::PortalFactory->build_context({
        subject => $self,
    });

    my $template_name = 'form_3000.tt2';
    $DISPLAY->debug_msg(3, "$self\->will process template '$template_name'");

    $controller->output({
        context  => $context,
        template => $template_name,
        });
}







1;
