###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

# The common root between Gizmo and Gizmo3000

package Metadot::GizmoBase;

use strict;

use Carp qw(&carp &croak &confess);

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $ACCESS_BROKER %REQUEST_VARS $CGI $DISPLAY);
use MetadotObject;
use Metadot::PortalFactory;
use Metadot::ObjectAccess::ObjectAccessor;
use Metadot::ObjectAccess::Restorable;
use Auditable;

use vars qw(@ISA);
@ISA = qw(MetadotObject Auditable Metadot::ObjectAccess::Restorable);




# hierarchy-related methods
#######################################################################
#######################################################################

# returns the container of an instance
sub get_parent {
  my $self = shift;

  my $container = undef;
  my $parent_id = $self->parent_id();

  my $my_id = $self->id();


  if (defined $parent_id) {
    unless (defined($my_id) && $parent_id == $my_id) {
      $DISPLAY->debug_msg(5, "$self\->get_parent uses parent_id $parent_id. My id=". ($self->id() || '') );
      $container = Metadot::GizmoBase->restore($parent_id);
      defined($container) && $DISPLAY->debug_msg(5, "$self\->get_parent got parent $container, whose id=".$container->id());
    }
  }
  return $container;
}

# Determines if a gizmo is a descendant of another gizmo.
# By definition, any gizmo is a descendant of itself
sub is_descendant_of {
  my $self = shift;
  my $subject = shift || die("Must supply the subject object to evaluate");
  my $result = 0;

  my @path = $self->get_path_to_root();
  my $subject_id = $subject->id();
  foreach my $cursor (@path) {
    return 1 if ($subject_id == $cursor->id());
  }

  return 0;
}


# returns the closest ancestor that is navigable. May return self, unless
# the arg 'exclude_self' is provided
sub get_navigation_parent {
  my $self = shift;
  my $args = shift || {};

  my $parent = undef;

  if ($args->{exclude_self}) {
    $parent = $self->get_parent() || Metadot::PortalFactory->get_home_object();
  } else {
    $parent = $self;
  }

  while ($parent && !$parent->isa('Metadot::Navigable')) {
    if ($parent->get_iid() < 0) {
      $parent = Metadot::PortalFactory->get_home_object();
    } else {
      $parent = $parent->get_parent();
    }
  }

  $DISPLAY->debug_msg(5, "get_navigation_parent returns $parent; defined? ".defined($parent));
  return $parent;
}


# returns the furthest ancestor that is navigable. May return self
sub get_navigation_root {
  my $self = shift;
  my $parent = undef;

  $parent = $self->get_navigation_parent();
  my %seen_objects = ();    # prevent infinite loop
  #print STDERR "get_navigation_root starts with parent: ".$parent->get_id()."\n";
  my $next_parent;
  while ( ($next_parent = $parent->get_parent() )
      && !defined ($seen_objects{$next_parent->get_iid()})
      && $next_parent->isa('Metadot::Navigable')
      && $next_parent->get_iid() >= 0) {
    $parent = $next_parent;
    $seen_objects{$next_parent->get_iid()} = 1;
    #print STDERR "get_navigation_root looks up to parent: ".$next_parent->get_id()."\n";
  }

  return $parent;
}


# returns the path from the site home to the current Gizmo
sub get_path_to_root {
  my $self = shift;
  my $cursor = shift || $self;

  my @results = ($cursor);
  my $visited;

  # move the cursor up the chain
  $cursor = $cursor->get_parent();
  while (defined($cursor) && defined($cursor->id()) && !$visited->{$cursor->id()}++) {
    push @results, $cursor;
    $cursor = $cursor->get_parent();
  }

  if (wantarray) {
    return @results;
  } else {
    return \@results;
  }
}









# ACCESS CONTROL METHODS (Auditable Interface Methods)
#######################################################################
#######################################################################

# should always be overridden in derived classes
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  croak "get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  confess "Thou shalt never instantiate Gizmos, for they are abstract.";
}

# called by Auditable->is_allowed_to_do
sub get_owner_uid {
  my $self = shift;

  return $self->uid();
}

# let my object accessor know that one of my members has changed
sub save_permissions {
  my $self = shift;

  $self->SUPER::save_permissions(@_);
  $self->get_object_accessor()->update(object=>$self);
}


# Gizmos first check their parent's permissions
sub is_allowed_to_do {
    my $self = shift;
    my $op   = shift || confess "must provide operation name";
    my $user = shift || $USER;
    my $result;

    # check the request cache
    my $op_bundle = $self->get_bundle_of_op($op);
    $result = Metadot::RequestCache::AuditableRequestCache->check_cache($self, $op_bundle, $user);

    unless (defined $result)
    {
        my $id = $self->id();
        $DISPLAY->debug_msg(4, "is_allowed_to_do (".($id||'').") op=".($op||''));

        my $is_authority;
        if (defined $id) {
            # check if I am one of the special authoritative instances
            my @authorities = (
                MetadotConfig->get_reserved_iid('ROOT_CATEGORY'),
                MetadotConfig->get_reserved_iid('HIDDEN_CATEGORY'),
                MetadotConfig->get_reserved_iid('PARENT_OF_ROOT'),
                MetadotConfig->get_reserved_iid('PORTAL_PARENT'),
                MetadotConfig->get_reserved_iid('PARENT_BASIC_TOOL'),
                MetadotConfig->get_reserved_iid('PARENT_CLIPBOARD'),
            );

            foreach my $authority (@authorities) {
                if ($authority && ($id == $authority)) {
                    $is_authority = 1;
                    last;
                }
            }

        }
        # authorities do not have to check their parents, but all others do.
        if ($is_authority) {
            $result = $self->SUPER::is_allowed_to_do($op, $user, @_);
            $DISPLAY->debug_msg(5, "I, $id, am an authority, result '$result'");
        } else {
            my $parent = $self->get_parent();
            # in case the database has an invalid entry, check that the parent is
            # valid
            if (defined $parent) {
                $DISPLAY->debug_msg(5, $id.' '.($self->name()||'unnamed')." checking permission from parent $parent (".$parent->id().")");

                #my ($activate_logged_in_cb, $ignore_logged_in_status) = @_; # disable logged_in_cb for this check
                #my $parent_allows = $parent->is_allowed_to_do('show', $user, 0, $ignore_logged_in_status);

                my $parent_allows = $parent->is_allowed_to_do('show', $user, @_);


                $result = ($parent_allows && $self->SUPER::is_allowed_to_do($op, $user, @_));
                $DISPLAY->debug_msg(5, "$id checked permission from parent $parent (".$parent->id()."), got '$parent_allows', combined result '".($result||'[false]')."'");
            } else {
                $result = $self->SUPER::is_allowed_to_do($op, $user, @_);
                $DISPLAY->debug_msg(5, "instance id=".(defined $id ? $id : 'undef')." doesn't have a parent, result '$result'");
            }
        }

    }

    return $result;
}






# MetadotObject-related methods
#######################################################################
#######################################################################

use constant MO_INSTANCE_PROPERTIES => {
                    id                  => undef,
                    uid                 => undef,
                    parent_id           => undef,
                    is_a                => undef,
                    name                => undef,
                    short_name          => undef,
                    description         => undef,
                    long_description    => undef,
                    cool                => undef,
                    show_childs         => undef,
                    display_look        => undef,
                    url                 => undef,
                    keywords            => undef,
                    instance_order      => undef,
                    show_from           => undef,
                    show_end            => undef,
                    creation_date       => undef,
                    update_date         => undef,
                    nb_items            => 0,
                    nb_reads            => 0,
                    updated_by          => undef,
                    hidden_comment      => undef,
                    last_post           => undef,
                    file1               => undef,
                    file1_size          => undef,
                    created_by          => undef,
                    t1                  => undef,
                    t2                  => undef,
                    t3                  => undef,
                    t4                  => undef,
                    t5                  => undef,
                    t6                  => undef,
                    t7                  => undef,
                    t8                  => undef,
                    t9                  => undef,
                    t10                 => undef,
                    c1                  => undef,
                    c2                  => undef,
                    c3                  => undef,
                    c4                  => undef,
                    c5                  => undef,
                    d1                  => undef,
                    d2                  => undef,
                    d3                  => undef,
                    d4                  => undef,
                    d5                  => undef,
                    i1                  => undef,
                    i2                  => undef,
                    i3                  => undef,
                    i4                  => undef,
                    i5                  => undef,
                       };










# Object accessor methods
#######################################################################
#######################################################################

use constant OBJECT_ACCESSOR => Metadot::ObjectAccess::ObjectAccessor->new(
                                       owner        => __PACKAGE__,
                                       properties   => [keys(%{&MO_INSTANCE_PROPERTIES})],
                                       tables       => [ 'instance' ],
                                       key_property => 'id',
                                       is_cached    => 1,
                                       field_map    => {
                                                id               => 'iid',
                                                uid              => 'userid',
                                                parent_id        => 'parentiid',
                                                is_a             => 'isa',
                                                long_description => 'longdescription',
                                                show_childs      => 'showchilds',
                                                display_look     => 'displaylook',
                                                instance_order   => 'instanceorder',
                                                show_from        => 'showfrom',
                                                show_end         => 'showend',
                                                update_date      => 'updatedate',
                                                nb_items         => 'nbitems',
                                                nb_reads         => 'nbreads',
                                                updated_by       => 'updatedby',
                                                hidden_comment   => 'hiddencomment',
                                                last_post        => 'lastpost',
                                                created_by       => 'createdby',
                                                creation_date    => 'creationdate',
                                               },
                                      );

sub get_object_accessor {
  my $class = shift;
  return &OBJECT_ACCESSOR;
}


# helps ObjectAccessor determine what class
# to instantiate, in case it has no idea
sub determine_class_from_memento {
  my $class = shift;
  my $memento = shift;
  ($memento && ref($memento) && $memento->isa('Metadot::ObjectAccess::Memento')) || die("Must provide a memento");
  my $result = $memento->get_state('is_a');
  return $result;
}









# Restorable methods
#######################################################################
#######################################################################

sub extract_state {
  my ($self) = shift;
  my $memento = Metadot::ObjectAccess::Memento->new();
  foreach my $key (keys %{&MO_INSTANCE_PROPERTIES}) {
    $memento->set_state($key, $self->{$key});
  }
  return $memento;
}

sub restore_state {
  my ($self, $memento) = @_;

  $DISPLAY->debug_msg(5, "$self\->restore_state using $memento");
  foreach my $key (keys %{&MO_INSTANCE_PROPERTIES}) {
    $self->{$key} = $memento->get_state($key);
  }
  return $self;
}









#####################################################
######                                        #######
######               CONSTRUCTOR              #######
######                                        #######
#####################################################

# GizmoBase expects the arguments to be a hashref
sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;

  my $args  = $class->normalize_constructor_args(@_);
  my $self;
  my $id;

  $id = $args->{id};
  $DISPLAY->debug_msg(4, "$class\->new(".($id || '').")");

  #print STDERR "BEGIN <$proto>".__PACKAGE__."->new() id=[$id]\n";

  if (defined $id && $id ne '') {
    # I'm retrieving an instance from the db. Gizmo->restore() takes
    # care of all instantiation duties by calling new() if needed.
    # Therefore, we can just return the result.
    return $class->restore($id);
  } else {
    # this is a truly new instance
    $DISPLAY->debug_msg(5, "$class\->new(undef) creates a new instance");
    $self  = $class->SUPER::new($args);

    # set the is_a to the class name
    $self->is_a($class);
  }

  $DISPLAY->debug_msg(4, "$class\->new() id=".($id||'')." returns ".($self || ''));
  #print STDERR "END <$proto>".__PACKAGE__."->new() id=[$id], returns $self\n";
  return $self;
}


# Persistence methods
#######################################################################
#######################################################################

sub save {
  my $self = shift;
  my $id = $self->id();

  return unless (defined($id));

  # drop the list request cache, because obviously something was changed.
  Metadot::RequestCache::GizmoListRequestCache->drop_cache();

  # Auditable requirement
  $self->save_permissions();

  # filter out reserved sql characters from text fields
  foreach my $field (qw(name description long_description t1 t2 t3 t4 t5 t6 t7 t8 t9 t10 c1 c2 c3 c4 c5)) {
    if ($self->{$field}) {
        $self->{$field} =~ s/[\x93\x94]/\"/g;
    }
  }

  my $db_now = Utility->db_now();
  $self->update_date($db_now);

  $self->get_object_accessor()->update(object=>$self);

    # EXPERIMENTAL: WARP
    #  $self->flush_page_cache();
}


# adds a new instance to the database
sub add {
  my $self = shift;
  $DISPLAY->debug_msg(3, "add (".ref($self)." object)");

  my $db_now = Utility->db_now();

  $self->creation_date(Utility->db_now());
  $self->update_date(Utility->db_now());

  # Auditable requirement: make sure that the permissions are initialized.
  $self->_get_permissions();

  my $id = $self->get_object_accessor()->create( object=>$self );

  my $caller = caller;
  die "caller:  $caller  no id" unless $id;


  # consistency check
  croak "must supply isA (".$self->is_a().")" unless($self->is_a());

  # Auditable requirement
  $self->save_permissions();


  # EXPERIMENTAL: WARP
  #  $self->flush_page_cache();

  return $id;
}


# restores an instance from the database
sub restore {
  my $proto = shift;
  my $id    = shift;

  $DISPLAY->debug_msg(4, "$proto\->restore($id)");
  $id = undef unless Utility->is_valid_integer($id); # sec_form_initializeu_200401
  return undef unless (defined $id);

  # this method is often called directly from Gizmo, or from the wrong
  # class. This means that I have to determine the true class to instantiate.
  my $class = ref($proto) || $proto;
  my $self  = undef;

  $DISPLAY->debug_msg(5, "$proto\->restore($id) will call get_object_accessor()->retrieve");

  # This retrieves the instance from the persistence layer. You should not
  # ever need to call this directly. If you do, make sure you're calling it
  # from the right Class, or else it might not be the right type.
  $self = $class->get_object_accessor()->retrieve(class_field=>'is_a', class=>$class, key=>$id);

  if (defined $self) {
      $DISPLAY->debug_msg(5, "$proto\->restore instantiated $self, isa="
                  . ($self->is_a()||'')
                  . ", id="
                  . ($self->id()||'')
                 );
  }

  return $self;
}


sub delete {
  my $self = shift;

  my $id = $self->id();
  $DISPLAY->debug_msg(3, __PACKAGE__.": $self delete; id=$id");

  return unless (defined $id);

  # remove any attachments
  $self->delete_attachments();

  # Auditable requirement
  $self->delete_permissions();

  # delete all shortcuts for this gizmo
  GizmoShortcut->delete_shortcuts_for($id);

  $DISPLAY->debug_msg(5, "Will ask object accessor ".$self->get_object_accessor()." to delete key=>$id");
  $self->get_object_accessor()->delete( key=>$id );

  $DISPLAY->debug_msg(4, "$self deleted (id=$id)");

    # EXPERIMENTAL: WARP
    # Flush the page cache
  #$self->flush_page_cache();

  return 1;
}


sub delete_attachments {
    die 'Abstract method';
}


# Makes a copy of a gizmo. Supports the following arguments:
# - save: saves the copy to the database
# - deep: performs a deep copy. Note: deep copies automatically include -save
# - properties: properties to overwrite in the copy or copies.
# - visited: hash of iid's already copied in this request, used to prevent looping
#
# returns the copy
sub copy {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);

    my $deep = $args->{deep};
    my $save = $deep || $args->{save};
    my $copy_properties   = $args->{properties} || {};

    my $iid = $self->id();

    $DISPLAY->debug_msg(3, "$self\->copy ($iid)");

    # record my id in the visited hash to prevent looping
    my $visited = $args->{visited};
    unless (defined $visited) {
        $args->{visited} = $visited = {};
    }
    $visited->{$iid}++;

    # extract all my properties and combine them with properties provided in
    # args
    my $self_properties   = $self->MO_get_all_properties();
    my $target_properties = Utility->merge_hashes($self_properties, $copy_properties);

    # remove the id arg, because we want to be unique
    delete $target_properties->{id};

    # create a new instance
    my $class  = ref($self) || die("Instance method");
    my $result = $class->new();
    $result->MO_set_properties($target_properties, 1);

    # save the instance if needed
    if ($save) {
        $result->add();
        $DISPLAY->debug_msg(3, "Saved $result (".$result->id().")");
    }

    # copy the uploads
    $self->copy_uploads($result);

    # recur if needed
    if ($deep && $self->isa('Metadot::BaseGizmoContainer')) {

        my $children = $self->build_child_list();
        foreach my $child (@$children) {
            my %properties = (%$copy_properties);
            # set the parent_id of child copies
            $properties{parent_id} = $result->id();
            $args->{properties} = \%properties;
            $child->copy($args);
        }
    }

    return $result;
}


# copy all uploads from self to the target
sub copy_uploads {
    my ($self, $target) = @_;

    # this method should be overridden in subclasses
}


# Flush the page cache
sub flush_page_cache {
    my $class = shift;

    $DISPLAY->debug_msg(4, "Updated $class, will flush cache");
    $CACHE->set_as_dirty('PAGE_CACHE');
}








# Content scheduling methods
# TODO: this needs to be refactored into a new interface Schedulable
#######################################################################
#######################################################################

# this method isolates the code necessary to determine whether it is
# time to show a gizmo, based on its 'show_from' value
sub is_show_time {
  my $self = shift;
  my %args = @_;

  $DISPLAY->debug_msg(3, "$self\->is_show_time()") ;

  my $show_from = $args{show_from} || $self->show_from();
  my $show_end  = $args{show_end}  || $self->show_end();

  ## This flag when turned on will return -1, 0 , or 1
  ##  -1: has not reached show date
  ##   1: is between show date and end date
  ##   0: is past end date
  ## When this flag is off, will only return 1 or 0
  ##   1: is between show date and end date
  ##   0: is not between show date and end date
  my $detailed_return = $args{detailed_return} || 0;

  my $time = localtime(time);
  my $now = Utility->db_now();

  $show_end ||= $now;

  my $cmp1 = $show_from cmp $now;
  if ($detailed_return && $cmp1 == 1) {
    return -1;
  } else {
    return ( ($cmp1 != 1) && (($now cmp $show_end) != 1) );
  }
}


# called by get_buttons, which is in itself deprecated!
sub show_scheduling_buttons {
  my $self   = shift;

  return '' unless($SESSION->admin_mode() && $self->is_allowed_to_do('modify', $USER));

  my $image = '';
  my $imageslocation = $PARAMS->{imageslocation};

  if ($self->show_from() || $self->show_end()) {
    my $show_time_value = $self->is_show_time(detailed_return => 1);
    if ($show_time_value == 1) {
      $image  = $DISPLAY->get_content_live_image();
    } elsif ($show_time_value == -1) {
      $image  = $DISPLAY->get_content_live_soon_image();
    } else {
      $image  = $DISPLAY->get_content_expired_image();
    }
    $image .= "\n";
  }

  return $image;
}
















# Search methods
#######################################################################
#######################################################################


# return a record that tells the gizmo what data to search against.
# here is an example record:
# {
#   searchable_fields   => [ qw( name description keywords) ],
#   displayed_fields    => [ qw( description) ],
#   searchable_children => [
#       {
#           class              => 'SomeChildClass',
#           searchable_fields  => [ qw( name description t1 ) ],
#           displayed_fields   => [ qw( description t1 ) ],
#       },
#   ],
# }
#
# alternately, you can use this (deprecated)
# {
#   searchable_fields => [ qw( name description keywords) ],
#   displayed_fields  => [ qw( description) ],
#   child_isa => 'SomeChildClass',
#   child_searchable_fields => [ qw( name description t1 ) ],
#   child_displayed_fields => [ qw( description t1 ) ],
# }


sub get_search_params {
    my $self = shift;
    die 'Abstract method';
}


# fetches a single result set
sub _fetch_search_result_set {
    my $class               = shift;
    my ($search_terms, $searchable_fields, $parent_class) = @_;

    my $sth;
    if (defined $searchable_fields && ref($searchable_fields) && @$searchable_fields) {
        # define the fields
        my $fields = 'i.iid, i.isa, i.parentiid';

        # define the tables
        my $tables = 'instance i';

        # filter by class
        my $where = 'i.isa = '.DBAccess->quote($class);

        # if a parent_class was specified, we do a join and filter on the
        # parent class
        if (defined $parent_class) {
            $tables .= ', instance j';
            $where .= ' AND i.parentiid = j.parentiid AND j.isa = '.DBAccess->quote($parent_class);
        }


        # filter by parentiid
        my $portal_parent_iid = MetadotConfig->get_reserved_iid('PORTAL_PARENT');
        $where .= " AND (i.parentiid >=0 OR i.parentiid = $portal_parent_iid)";

        # filter by search terms
        $where .= ' AND ('.join(' OR ', map("UPPER($_) LIKE $search_terms", @$searchable_fields)) .')';

        # get the result set
        $DISPLAY->debug_msg(5, "Searching $class by SELECT $fields FROM $tables WHERE $where");
        $sth = DBAccess->sqlSelectMany($fields, $tables, $where);
    }

    return $sth;
}

# returns one or more resultsets containing the matching records
sub get_search_results {
    my $class        = shift;
    my $search_terms = shift;

    my @results = ();
    my $search_struct = $class->get_search_params();

    if (defined $search_struct) {
        # clean quote and other chars
        $search_terms = DBAccess->quote('%'.uc($search_terms).'%');

        # Search for instances of my own class that match
        if (defined($search_struct->{searchable_fields})) {
            my $search_class = $class;
            my $searchable_fields = $search_struct->{searchable_fields};
            # only dispatch this search if there really is something to search for
            if (ref($searchable_fields) && @$searchable_fields) {
                my $displayed_fields  = $search_struct->{displayed_fields}
                                        || $searchable_fields;

                my $sth =  $search_class->_fetch_search_result_set(
                    $search_terms,
                    $searchable_fields,
                );

                push @results, {
                    sth              => $sth,
                    displayed_fields => $displayed_fields,
                };
            }
        }

        # Search for instances of my children that match
        if (defined $search_struct->{searchable_children}) {
            foreach my $child_search_struct (@{$search_struct->{searchable_children}}) {
                if (ref $child_search_struct) {
                    my $search_class      = $child_search_struct->{class};
                    my $searchable_fields = $child_search_struct->{searchable_fields};
                    my $displayed_fields  = $child_search_struct->{displayed_fields}
                                            || $searchable_fields;

                    my $sth =  $search_class->_fetch_search_result_set(
                        $search_terms,
                        $searchable_fields,
                    );

                    push @results, {
                        sth              => $sth,
                        displayed_fields => $displayed_fields,
                    };
                }
            }
        }

        # Search for instances of my children using the deprecated definition
        if (defined $search_struct->{child_isa}) {
            my $search_class      = $search_struct->{child_isa};
            my $searchable_fields = $search_struct->{child_searchable_fields};
            my $displayed_fields  = $search_struct->{child_displayed_fields}
                                    || $searchable_fields;

            my $sth =  $search_class->_fetch_search_result_set(
                $search_terms,
                $searchable_fields,
            );

            push @results, {
                sth              => $sth,
                displayed_fields => $displayed_fields,
            };
        }
    }

    return \@results;
}


sub show_search_results {
  my $class         = shift;
  my $search_terms  = shift;
  my $start_from    = shift || 0;
  my $max_items     = shift || 25;

  $DISPLAY->debug_msg(3, "in show_search_results($search_terms)");

  # result string
  my $s='';

  # First get the result sets
  my @results = @{$class->get_search_results($search_terms)};

  # the number of matches found
  my $total_rows = 0;

  # the number of matches displayed
  my $num_rows   = 0;

  # Iterate through the result sets
  foreach my $record (@results) {
      my $sth = $record->{sth};
      if (defined $sth) {
          my $displayed_fields = $record->{displayed_fields};
          while (my ($iid, $isa, $parent_id) = $sth->fetchrow()) {
              my $instance = $isa->restore($iid);
              if ($instance && $instance->is_allowed_to_do('show')) {
                  if ( $num_rows < $max_items && $total_rows >= $start_from) {
                      my $one_result = $isa->format_search_result($instance, $displayed_fields);
                      $s .= "<li>$one_result</li><br />\n\n";
                      $num_rows++;
                  }
                  $total_rows++;
              }
          }
          $sth->finish();
      }
  }

  $s = $class->show_search_result_group({
      gizmo_name    => $class,
      total_rows    => $total_rows,
      max_rows      => $max_items,
      start_from    => $start_from,
      num_rows      => $num_rows,
      filter        => $search_terms,
      results       => $s,
  });

  $DISPLAY->debug_msg(4, "show_search_results finishes for $class");

  return $s;
}


# defines the name to be displayed in search results
sub get_search_display_name {
    my $self = shift;
    return $self->get_name();
}



# displays one group of search results. Results are grouped by gizmo type
sub show_search_result_group {
    my $self  = shift;
    my $args  = shift;

    return '' unless ($args->{results});

    my $s = '';

    # save the args to REQUEST_VARS
    foreach my $key (%{$args}) {
        $REQUEST_VARS{$key} = $args->{$key};
    }

    # template mode uses template-driven output
    if ($PARAMS->{use_controller}) {
        # create a context for this template
        my $context = Metadot::PortalFactory->build_context({
            subject         => Portal->new(),
            op              => $FORM{op} || 'search',
            user            => $USER,
            form            => \%FORM,
            session         => $SESSION,
        });

        $s = Metadot::MetadotController->view({
            context  => $context,
            template => 'search_result_group.tt2',
        });
    } else {
        # legacy mode

        my $totalrows   = $args->{total_rows} || $args->{totalrows};
        my $maxitem     = $args->{max_rows}   || $args->{maxitem} || 50;
        my $startfrom   = $args->{start_from} || $args->{startfrom} || 0;
        my $filter      = $args->{filter};
        my $gizmo_name  = $args->{gizmo_name};
        my $i           = $args->{num_rows};

        $s = $args->{results};

        my $gizmo_print_name;
        eval {
            $gizmo_print_name = $gizmo_name->get_gizmo_name();
        };
        if ($@) {
            $gizmo_print_name = $gizmo_name;
        }
        if ($totalrows > $maxitem) {
            my $url = "index.pl?isa=Portal&amp;op=search&amp;filter=$filter&amp;type=$gizmo_name";
            my ($nf)= Components->get_previous_next_button($maxitem, $i, $totalrows, $startfrom, $url); #PREVIOUS NEXT MAGIC
            $s = "<i><b><font size=+2>&nbsp; Results in " . $gizmo_print_name . "<\/font><\/b><\/i><br />$nf<br />\n\n$s\n\n\n\n<br /><br /><p />";
        }
        elsif ($totalrows >= 1 && $totalrows <= $maxitem) {
            my $entries;
          if ($totalrows == 1) {
            $entries = "entry";
          } else {
            $entries = "entries";
          }
          $s= "<i><b><font size=+2>&nbsp; Results in " . $gizmo_print_name . "<\/font><\/b><\/i><br /><br /><i><b>&nbsp; Found $totalrows $entries<\/b><\/i><p /><br />$s<br /><br /><p />";
        }
    }
    return ($s);
}


#######################
sub format_search_result {
  my $class = shift;
  my ($instance, $displayed_fields) = @_;

  my $s = '';

  if (defined($instance)){
      # CHECK IF THIS WAS CALLED THE NEW OR OLD WAY
      if (UNIVERSAL::isa($instance, __PACKAGE__)) {
          if (defined($displayed_fields) && ref($displayed_fields)) {
              my $iid = $instance->id();
              my $isa = $instance->is_a();

              # if the search result is of my child class, generate a link to the
              # parent
              my $parent_link;
              if ($isa ne $class) {
                  my $parent = $instance->get_parent();
                  my $parent_name = Utility->trim($parent->get_search_display_name());
                  if ($parent_name) {
                      my $parent_id   = $parent->id();
                      my $parent_isa  = $parent->is_a();
                      $parent_link = '<a href="'
                            .$DISPLAY->make_url("index.pl?iid=$parent_id&amp;isa=$parent_isa")
                            ."\">$parent_name</a>\n";
                  }
              }

              #include a link to the instance
              my $instance_name = Utility->trim($instance->get_search_display_name()) || 'No title';
              my $instance_link;
              if ($instance_name) {
                  $instance_link = '<a href="'
                       .$DISPLAY->make_url("index.pl?iid=$iid&amp;isa=$isa")
                       ."\">$instance_name</a>\n";
              }

              # combine the two links
              $s .= $parent_link
                   . (($parent_link && $instance_link) ? ' : ' : '')
                   . $instance_link;


              # include each of my displayed fields (except for name, which I've already
              # used for the link
              foreach my $field (@$displayed_fields) {
                  unless (defined($field) && $field eq 'name') {
                      my $text = $instance->$field();
                      if ($text) {
                          my $nh = Verify->removeH($text);
                          my $shortened = substr( $nh, 0, 150 ) . "...";
                          $s .= " - $shortened";
                      }
                  }
              }
          }
      } else {
          # this method was called the old way, so we remap
          my $name = $displayed_fields;
          $s .= $class->format_search_result_OLD(@_);
      }
  }

  return $s;
}




sub format_search_result_OLD {
  my $self = shift;
  my ( $iid, $isa, $name, $fields_text )=@_;

  if (!$iid || !$isa ){
    return "";
  }

  $name = "No title" if (!$name || $name eq 'null');

  my $s="";

  $s = "<a href=" . $self->make_url("index.pl?iid=$iid&amp;isa=$isa&amp;op=show") . "><b>$name</b></a>";

  foreach my $text ( @$fields_text ) {

      if ($text) {
      my $nh = Verify->removeH($text);
      my $shortened = substr( $nh, 0, 150 ) . "...";
#     print STDERR "in format_search_result, shortened string is: $shortened\n";

      #aju -- why was this done?? Try using the shortened version instead..
#       $s .= " - $text";
      $s .= " - $shortened";
      }
  }

  return $s;
}




# Utility methods
#######################################################################
#######################################################################

sub DESTROY{
  # EMPTY destroy method to please mod_perl when
  # it complains about a missing DESTROY function.
}


# adopt the MetadotObject's constructor arg options
sub normalize_constructor_args {
  my $class = shift;

  if (@_ && !ref($_[0])) {
    #old calling method $id is only param
    return { id => $_[0] };
  } else {
    return $class->SUPER::normalize_constructor_args(@_);
  }
}

# this is now an alias for Display->debug_msg. There are too many
# invocations of $self->debug_msg to change this now.
sub debug_msg {
  my $class = shift;
  return $DISPLAY->debug_msg(@_);
}


# check to see if parent is a hidden item or in clipboard.
# returns true if there is NO hidden parent
# TODO: This method was transferred from Gizmo.pm.
#       Replace this method with something that works better.
sub parentCheck {
  my $self = shift;
  my $args = shift || {};

  $args->{visited} = {} unless (defined $args->{visited});

  my $id = (ref $self) ? $self->id() : undef;

  $DISPLAY->debug_msg(6, "parentCheck on $id ($self)");

  # if I've been here already, it must be okay
  if ($args->{visited}->{$id}++) {
    $DISPLAY->debug_msg(6, "parentCheck checked visited, passed");
    return 1;
  }

  # portal parent page is always ok
  if ($id == MetadotConfig->get_reserved_iid('PORTAL_PARENT')) {
    $DISPLAY->debug_msg(6, "parentCheck checked if portal parent page, passed");
    return 1;
  }

  # home page is always ok
  if ($id == MetadotConfig->get_reserved_iid('ROOT_CATEGORY')) {
    $DISPLAY->debug_msg(6, "parentCheck checked if portal parent page, passed");
    return 1;
  }

  # negative ids are hidden
  if ($id < 0) {
    $DISPLAY->debug_msg(6, "parentCheck checked negative iid, failed");
    return 0;
  }


  my $parent = $self->get_parent();
  if (defined $parent) {
    $DISPLAY->debug_msg(6, "parentCheck checking parent ($parent)");
    return $parent->parentCheck($args);
  } else {
    # no parent, therefore it's okay
    $DISPLAY->debug_msg(6, "parentCheck ran out of conditions to check. passed.");
    return 1;
  }
}


# Gets all Gizmo classes which should appear in the navigation.
sub list_navigation_gizmos {
    my $class = shift;

    my @out;

    # get the Gizmo types
    my @gl = $class->get_gizmo_list();
    foreach my $g (@gl) {
        if ($g->is_navigation_gizmo) {
            push @out, $g;
        }
    }

    # get the Gizmo3000 types
    push @out, keys(%{Metadot::Gizmo3000::ClassRegistry->get_instance()->get_registered_modules({isa=>'Metadot::Navigable'})});

    return @out;
}





# Aliases
#######################################################################
#######################################################################

# alias for id
sub iid {
  my $self = shift;
  return $self->id(@_);
}

sub get_iid {
  my $self = shift;
  return $self->iid();
}

sub set_iid {
  my $self = shift;
  return $self->iid(@_);
}


# alias for parent_id
sub parent_iid {
  my $self = shift;
  return $self->parent_id(@_);
}

sub get_parent_iid {
  my $self = shift;
  return $self->parent_id();
}

sub set_parent_iid {
  my $self = shift;
  return $self->parent_id(@_);
}

# alias for longdescription
sub longdescription {
    my $self = shift;
    return $self->long_description(@_);
}

# in modes before template mode, specifies the decorator for gizmos when displayed
# within a list
sub gizmo_list_decorator {
  return 'HTML::Decorator::NullDecorator';
}



# gizmo short_names
use constant SHORT_NAME_FORM_KEY => '_gizmo_short_name';
sub get_short_name_control {
    my $self = shift;

    my $short_name = $self->short_name();
    my $form_key   = SHORT_NAME_FORM_KEY();
    my $out  = <<EOM
    <h1>Short name</h1>
    <p>The Gizmo's short name is used for anchor tags. If blank, one will
    be automatically generated based on the internal id.</p>
    <b>This gizmo's short name:</b><br />
    <input type="text" value="$short_name" name="$form_key" size="80" maxlength="255"/>

EOM
;
    return $out;
}


sub process_short_name_control {
    my $self  = shift;
    my $input = shift || \%FORM;

    my $value = $input->{SHORT_NAME_FORM_KEY()};
    $self->short_name($value);
}


# gets the display name for a gizmo
sub get_gizmo_name {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    die('Abstract method');
}


# makes a standard url for a given gizmo
sub make_gizmo_url {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);

    my $result;

    my $iid = $self->id();
    my $isa = $self->is_a();
    my $op  = $args->{op};
    my $app = $args->{app} || $FORM{app};

    if (defined $iid) {
        $result = "index.pl?iid=$iid";
        if (defined $isa) {
            $result .= "&isa=$isa";
        }
        if (defined $op) {
            $result .= "&op=$op";
        }
        if (defined $app) {
            $result .= "&app=$app";
        }
    } else {
        #print STDERR "Could not make_gizmo_url for a gizmo with no id\n";
    }

    return $result;
}




1;
