###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::JSForm ;
use strict;

use Metadot qw ($PARAMS) ;

use Metadot::JSForm::FilterFactory ;
use Metadot::JSForm::Filter ;
use Metadotclass;

#########################################################
##  Class Variables
#########################################################
my $SHOW_ONE_ERROR = 1 ;
my $SHOW_ALL_ERRORS = 2 ;

my $JSF_OK_CHARS = '-a-zA-Z0-9_' ;  # A restrictive list
my $JSF_CONFIG = {
    # http://, https:// or imageslocation-relative path
    libfile_location => "js/ClientParameters.js",
} ;

#########################################################
##  Class Methods
#########################################################
sub get_ONE_ERROR {
    my $class = shift ;
    return $SHOW_ONE_ERROR ;
}

sub get_ALL_ERRORS {
    my $class = shift ;
    return $SHOW_ALL_ERRORS ;
}

#########################################################
##  Constructor
#########################################################
sub new {
    my $proto = shift ;
    my $p = shift ;
    my $class = __PACKAGE__ ;
    my $self = {};

    bless ($self, $class) ;

    # getting parameters of the constructor
    $self->{p} = $p if defined($p) && ref($p) ;

    # setting internal parameters
    $self->{is_enabled} = $PARAMS->{is_jsform_enabled} ;
    # XXX [vadim] remove the following line for production
    $self->{is_enabled} = 1 ;

    $self->{jsform_filter} = $self->{p}->{jsform_filter} || "Default" ;
    $self->{form_index} = -1 ;
    $self->{checks} = [()] ;

    return $self ;
}

#########################################################
##  Public Methods
#########################################################
# add client side check (simple syntax)
sub add_check_simple {
    my $self = shift ;
    my $name = shift || return ;
    my $label_text = shift ;
    my $check_type = shift ;
    my $label_name = shift ;

    $self->add_check(
        {
            name        => $name,
            label_text  => $label_text,
            check_type  => $check_type,
            label_name  => $label_name,
        }
    ) ;
}

# add client side check
sub add_check {
    my $self = shift ;
    my $check = shift || return ;
    return unless ref($check) ;

    my $name       = $check->{name} || return ;
    my $label_text = $check->{label_text} || $name ;
    my $check_type = $check->{check_type} || "notempty" ;
    my $label_name = $check->{label_name} || "js_null" ;

    # escape values to avoid errors in script
    # and funny text in diagnostics
    $name   =~ s/[^$JSF_OK_CHARS]//go ;
    $label_text  =~ s/<.+?>//g ;
    $label_text  =~ s/['|"]//g ;
    $check_type  =~ s/[^$JSF_OK_CHARS]//go ;
    $label_name     =~ s/[^$JSF_OK_CHARS]//go ;

    push (@{$self->{checks}},
        {
            name        => $name,
            label_text  => $label_text,
            check_type  => $check_type,
            label_name  => $label_name,
        } ) ;
}

# remove client side check
sub remove_check {
    my $self = shift ;
    my $name = shift ;
    my $checks = $self->{checks} ;

    return unless (defined($name) && $name !~ /^\s*$/) ;
    return unless (defined($checks) && ref($checks) eq "ARRAY") ;

    my @new_checks = () ;
    foreach my $check (@{$checks}) {
        push (@new_checks, $check) unless ($check->{name} eq $name) ;
    }

    $self->{checks} = \@new_checks ;
}

# add an array of client side checks
sub add_check_array {
    my $self = shift ;
    my $array_ref = shift || return ;

    return unless ref($array_ref) eq "ARRAY" ;

    # we should iterate and add them individually to enforce correct values
    foreach my $check (@{$array_ref}) {
        $self->add_check($check) ;
    }
}

# return array_ref of the client side checks
sub get_check_array {
    my $self = shift ;
    return $self->{checks} ;
}

# remove an array of client side checks
sub remove_check_array {
    my $self = shift ;
    my $array_ref = shift || return ;

    return unless ref($array_ref) eq "ARRAY" ;

    foreach my $check (@$array_ref) {
        $self->remove_check($check) ;
    }
}

# remove all client side checks
sub remove_all_checks {
    my $self = shift ;
    $self->{checks} = [()] ;
}

# resets the jsform, right now only remove_all_checks is called
sub reset_form {
    my $self = shift ;
    $self->remove_all_checks() ;
}

# get client side checked form
sub get_form {
    my $self = shift ;
    my $form_html = shift || "" ;
    my $mode = shift || $self->get_ONE_ERROR() ;
    return $form_html unless $self->{is_enabled} ;

    Metadotclass->debug_msg(5, "In $self\->get_form()");

    my $result = $self->{_form_html};
    unless ($result) {
        # now that Gizmos are cached, form objects are also cached.
        # the following code causes problems if called repeatedly
        # enforce the unique id for the checked form on the page
        # to avoid multiple functions definition - but we need to know
        # if this is the first form we generate
        $self->{form_index} += 1 ;

        my $data = {} ;
        # enforce the unique name for the function that processes our form
        srand ;
        $data->{random_name}    = ('a'..'z')[26*rand].('a'..'z')[26*rand].('a'..'z')[26*rand] ;
        $data->{library_js}     = $self->_jsf_get_libfiles_js() ;
        $data->{initialize_js}  = "" ;
        $data->{checks}         = $self->{checks} ;
        $data->{final_js}       = "" ;

        $data->{mode}           = $mode ;
        $data->{form_index}     = $self->{form_index} ;
        $data->{form_html}      = $form_html ;

        $result = $self->_handle_data($data);
        $self->{_form_html} = $result;

        #use Data::Dumper;
        #print STDERR "$self\->get_form() build data ",Dumper($data),"\n\n";
    }


    return $result;
}

#########################################################
##  Private Methods
#########################################################
sub _jsf_get_libfiles_js {
    my $self = shift ;
    my $imageslocation = $PARAMS->{imageslocation} ;
    my $libfile_location = $JSF_CONFIG->{libfile_location} ;

    if ($libfile_location !~ /^http:\/\//i && $libfile_location !~ /^https:\/\//i) {
        $libfile_location = $imageslocation."/".$libfile_location ;
    }

    my $output = "" ;

    $output .= "\n<script language=\"JavaScript\" src=\"" ;
    $output .= $libfile_location ;
    $output .= "\"></script>" ;

    return $output ;
}

sub _handle_data {
    my $self = shift ;
    my $data = shift || return "" ;
    my $jsform_filter = $self->{jsform_filter} ;
    return "" unless ref($data) ;

    $data = $self->_apply_filter($data, $jsform_filter) ;
    my $html = $self->_construct_html($data) ;

    return $html ;
}

#### abstract methods, must be overridden
sub _apply_filter {
    my $self = shift ;
    my $data = shift || return undef ;
    my $jsform_filter = shift || "Default" ;
    return undef unless ref($data) ;

    die "abstract, must be overridden" ;
    return $data ;
}

sub _construct_html {
    my $self = shift ;
    my $data = shift || return undef ;
    return "" unless ref($data) ;
    my $html = $data->{form_html} ;

    die "abstract, must be overridden" ;
    return $html ;
}

#####################################################
###                   THE END                     ###
#####################################################

1;

=head1 NAME

JSForm -

=head1 SYNOPSIS

    package YourClass ;
    use Form::JavaScriptForm ;
    $gizmo= Gizmo->new ($args);

    $gizmo->lists_gizmos ;
    $gizmo->filtered_gizmos ;
    $gizmo->get_is_public ;
    $gizmo->get_gizmo_name ;
    $gizmo->get_AC;
    $gizmo->get_parent ;
    $gizmo->get_owner_uid ;
    $gizmo->not_logged_in_callback ;
    $gizmo->restore ($id);
    $gizmo->www_show ;
    $gizmo->www_change_owner ;
    $gizmo->www_edit_permissions ;
    &go_back_to_parent_after_editing_permissions ;
    $gizmo->process_edit_form;  [interface]
    $gizmo->www_modify ([$errormsg]);
    $gizmo->www_save ;
    $gizmo->www_delete ;
    $gizmo->www_delete_ok ;
    $gizmo->www_up ;
    $gizmo->www_down ;
    $gizmo->www_cut ;
    $gizmo->www_paste ;
    $gizmo->www_download ;
    &show_search_results ($gizmo_name, $thingtosearch,
              [$startfrom=0, $maxitem=50]);
    $gizmo->last_position ;
    $gizmo->delete ;
    $gizmo->reorder_parent;
    $gizmo->save ;
    $gizmo->cid ;
    $gizmo->down;
    $gizmo->up ;
    $gizmo->print_attributes;
    $gizmo->add ;
    $gizmo->DESTROY;
    $gizmo->AUTOLOAD ;
    $gizmo->url ([$url]);
    $gizmo->list_ascendants ([$id, $nbcalls=0]);
    $gizmo->show ;
    $gizmo->show_edit_form ($title);
    $gizmo->show_title ([$title]);
    $gizmo->show_path;
    $gizmo->show_panel ([$id, $isa]);
    $gizmo->show_up_down_buttons ([$id, $isa]);
    $gizmo->show_edit_button ([$id, $isa]);
    $gizmo->show_del_button ([$id, $isa]);
    $gizmo->show_cut_button ([$id, $isa]);
    $gizmo->show_ac_button ([$id, $isa]);
    $gizmo->parentCheck ([$id]);
    $boolean = $gizmo->delete_attachment();


=head1 DESCRIPTION

Gizmo is an abstract class from which concrete gizmos (eg. Item and Category) are derived. This class provides a default implementation for many methods that concrete gizmos will seldom need to override. Methods in this class are essentially infrastructure that makes life easier for Gizmo developers.

=head2 Methods

$self = ConcreteGizmo->SUPER::new([ HASHREF | SCALAR ]);

Object constructor for this class. The main function of this method is to load concrete gizmo's instance data from database storage. Must always be called from concrete gizmos own constructors, so infrastructure initializations (such data loading and Permissions initialization) can take place on instance creation.
Two forms of paramters can be passed to it: a) Named parameters can in the form of a hashref to override default values on object creation (eg. if we want to specify a parent id when creating gizmos programatically). b) A single numeric scalar to signify the instance id of an already existing gizmo.This form is mostly used when rendering gizmos from within the Category gizmo.


  @gizmo_list = Gizmo->lists_gizmos()

Classwide method that returns a list containing the names of gizmos installed in the system. It is used in Category to determine what gizmos it should be rendering. Should be of use to implement generic container gizmos.


  $is_public = $gizmo->get_is_public()

Determines whether the current gizmo instance is searchable by the built in search infrastructure. Currently it is a classwide method, so it allows to specify whether a whole set of concrete gizmos is searchable. Gizmos that globally are not meant to be searched should override this method to return 0.


  @gizmo_list = Gizmo->filtered_gizmos();

This method returns a list of gizmos that are meant to be searchable. It's used within Portal.pm to implement search infrastructure.


  Gizmo->get_gizmo_name;

This is a classwide method that returns the name of the gizmo.

  $gizmo->get_AC;

  $gizmo->get_parent;

  $gizmo->get_owner_uid;

  $gizmo->not_logged_in_callback;

These methods are required by the Auditable interface. See documentation in Auditable.pm.


  Gizmo->restore($id);

Classwide method. Will restore a gizmo object back from the database. The gizmo will be blessed to the class of the concrete Gizmo it represents, even if called from the Gizmo parent class.


  $gizmo->www_show ;

Will render the content of the concrete gizmo. Will be invoked by the system as users browswe the gizmo tree. Most concrete gizmos will need to override this method.


  $gizmo->www_change_owner ;

Will be invoked by the system when a user wants to change the owner of a gizmo. Concrete gizmos will not ususally need to override this method.


  $gizmo->www_edit_permissions ;

Will be invoked by the system when a user wants to change permissions of a gizmo. Concrete gizmos will not ususally need to override this method.


  $gizmo->go_back_to_parent_after_editing_permissions ;

Determines if the system should jump to the parent object after editing permissions of an object. The default implementation returns true, which is the behavior most gizmos need.


  $gizmo->www_modify ([$errormsg]);

This method is invoked by the system when a user wants to modify the content of a gizmo (usually by clicking in the "Edit" icon). This method will generally not be overridden in derived classes, instead the show_edit_form method should be. If an error message is supplied this will appear before the html from show_edit_form.


  $gizmo->show_edit_form ($title);

Must be overridden in concrete classes (ie. the ones derived from Gizmo) to generate the HTML specific to the data fields required in the concrete class. Will be invoked by the system from the www_modify method and from Category::www_add_child.


  $gizmo->process_edit_form;  [interface]

This method will read gizmo specific data from the posted form when an gizmo is being created.It is invoked from www_create_child in Category.pm. Most gizmos will need to override this to process their specific data.

  $gizmo->www_save;

This method will be invoked by the system after www_modify to commit field changes to the database. Some gizmos will not use this method and instead save directly to the database (eg. Poll.pm.). Some gizmos override this method (eg. Table).

  $gizmo->save;

This method will commit new gizmo data to the instance table. It's called from www_save and from other save mathods in concrete gizmos.

  $gizmo->www_delete ;

Will be called by the system when users request a gizmo to be deleted.

  $gizmo->www_delete_ok ;

Will be called by the system when users confirm that they want a gizmo to be deleted.

  $gizmo->www_up ;
  $gizmo->www_down ;

Will be called by the system when users request that a gizmo is moved up or down in the rendering order.

  $gizmo->www_cut ;
  $gizmo->www_paste ;

Will be called by the system when users request to cut or paste a gizmo.

  $gizmo->www_download ;

Will be called by the system when users request to download a file contained in a gizmo.(eg. from an Item gizmo).

  &show_search_results ($gizmo_name, $thingtosearch,
               [$startfrom=0, $maxitem=50]);

Will perform a search for matching words in gizmos of the specified type.

  $gizmo->last_position ;

Utility method that returns the next position number for a gizmo class. The position number is used to determine the order in which gizmos are rendered.

  $gizmo->delete ;

Will delete a gizmo instance from the instance table.

  $gizmo->reorder_parent;

Will update the instance order numbers of a gizmo type in the table. It is called when gizmos are added or removed or cut and when their order is changed.

  $gizmo->save;

Will commit a gizmo to the database.

  $gizmo->cid;

Returns the channel id associated with a gizmo.

  $gizmo->down;

  $gizmo->up ;

Will change the position in which a gizmo is rendered with respect to gizmos of the same kind under the same parent.

  $gizmo->print_attributes;

Will print the values in the gizmo data fields. Not used anywhere. Could be used for debugging.

  $gizmo->add;

Will insert a newly created gizmo into the database.

  $gizmo->DESTROY;

Not used but to comply with a mod_perl requirement.

  $gizmo->AUTOLOAD ;

Will generate accessor and mutator functions for all gizmo datafields. These functions allow to accedd and to set gizmo contents, eg. $gizmo->name(); will get the name field of a gizmo, and $gizmo->name('new_name'); will set it.

  $gizmo->url ([$url]);

Accessor method to the url field. This method is not taken care of by AUTOLOAD because the CGI.pm module exports a method with the same name.

  $gizmo->list_ascendants ([$id, $nbcalls=0]);

Will return a list of the ids of all Categories above a gizmo.  This is use to generate links to parent categories when rendering a gizmo.

  $gizmo->show ;

Will generate HTML to render the content of the gizmo. Must be overidden in concrete classes.

  $gizmo->show_title ([$title]);

Will render a title when rendering the gizmo. For the case of Categories and Tables it will also call show_panel(). It will be called by the rendering infrastructure and will seldom need to be invoked by gizmo writers.

  $gizmo->show_path;

Will return the HTML for the clickable path that allows to go to go to back to the parent categories under which a gizmo is found. It's used by rendering infrastructure.

  $gizmo->show_panel ([$id, $isa]);

Will return the html for the edit, delete, cut, shortcut up, down and edit permissions icons when the user is in edit mode. It's used by the rendering infrastructure.

  $gizmo->show_edit_button ([$id, $isa]);

  $gizmo->show_del_button ([$id, $isa]);

  $gizmo->show_cut_button ([$id, $isa]);

  $gizmo->show_shortcut_button ([$id, $isa]);

  $gizmo->show_up_down_buttons ([$id, $isa]);

  $gizmo->show_ac_button ([$id, $isa]);

Will generate html to display icons for edit, delete, cut, up, down, and edit permissions. These methods are used by show_panel() and would rarely need to be used or overridden in derived classes.

  $gizmo->parentCheck ([$id]);

Utility function used by the search infrastructure to determine if an item is hiden or in a clipboard, in which case it should nto be displayed as a search result.

Back end method for deleting a file, called by www_delfileok. Will attempt to delete a file attachment whose name is stored in the file1 field of the gizmo.

    $boolean = $gizmobuilder->delete_attachment();


=cut

# [vadim] little easter egg, please don't remove :)
# if you have any question feel free to contact:
# vadim@metadot.com, jelezniakov@hotmail.com

__DATA__
<HTML><HEAD>
<TITLE>The Metadot Portal Server 4.5.1 Product Team !</TITLE>
<SCRIPT LANGUAGE="JavaScript1.2" TYPE="text/javascript">
NS4 = (document.layers);
IE4 = (document.all);

appVer = navigator.appVersion;
IEmac = (IE4 && appVer.indexOf("Mac") != -1);
IE4mac = (IEmac && appVer.indexOf("MSIE 4") != -1);
IE40mac = (IE4mac && appVer.indexOf("4.0;") != -1);
IE45mac = (IE4mac && appVer.indexOf("4.5;") != -1);
NSpre401 = (NS4 && (parseFloat(appVer) <= 4.01));
NSpre403 = (NS4 && (parseFloat(appVer) <= 4.03));

FDRboxWid = 400;
FDRboxHgt = 400;
FDRbackCol = "#003366";
FDRborWid = 4;
FDRborSty = "double";
FDRborCol = "#000000";
FDRboxPad = 10;

FDRtxtAln = "center";
FDRlinHgt = "25pt";
FDRfntFam = "helvetica";
FDRfntSiz = "20pt";
FDRfntWgh = "bold";
FDRfntSty = "normal";
FDRfntCol = "#F0FFFF";
FDRlnkDec = "none";
FDRlnkCol = "#F0F8FF";
FDRhovCol = "#00FF7F";

FDRblendInt = 3;
FDRblendDur = 1;
FDRmaxLoops = 1;
FDRendWithFirst = true;
FDRreplayOnClick = true;
FDRjustFlip = false;
FDRhdlineCount = 1;

FDRgifSrc = "";
FDRgifInt = 60

prefix = "";

arNews = [
"<br /><br /><br /><br />Metadot Corporation<br />a Schlumberger Company","http://www.metadot.com",
"<br /><br /><br /><br />Metadot Portal Server 4.5.1<br />Product Team","",
"<br /><br /><br /><br /><br />Mission Control","",
"<br /><br /><br /><br />Daniel Guermeur<br />President/CEO<br />(France)","",
"<br /><br /><br />Philippe Herve<br />Director of Business Development<br />(France)","",
"<br /><br /><br />Serge Dacic<br />Director of Professional Services<br />(France)","",
"<br /><br /><br /><br /><br />Management Team","",
"<br /><br /><br /><br />Roger Aamador<br />Business Development Manager<br />(USA)","",
"<br /><br /><br /><br />Keith Burns<br />Regional Sales Manager<br />(USA)","",
"<br /><br /><br />Lewis Etheridge<br />Network Operations Manager<br />(USA)","",
"<br /><br /><br />Senior Architect<br /><br />Amy Unruh<br />(USA)","",
"<br /><br /><br /><br /><br />Team Lead","",
"<br /><br /><br /><br />Ravi Kumar<br />Team Leader<br />(USA)","",
"<br /><br /><br /><br />Mirko Scavazzin<br />Team Leader<br />(France)","",
"<br /><br /><br /><br />Amy Unruh<br />Team Leader<br />(USA)","",
"<br /><br /><br /><br />Stan von Miller<br />Team Leader<br />(USA)","",
"<br /><br /><br /><br /><br />Development Team","",
"<br /><br /><br /><br />Claudio Garcia<br />Developer<br />(Mexico)","",
"<br /><br /><br /><br />Vadim Jelezniakov<br />Developer<br />(Russia)","",
"<br /><br /><br /><br />Cory Kim<br />Developer<br />(USA)","",
"<br /><br /><br /><br />John Kropka<br />Developer<br />(USA)","",
"<br /><br /><br /><br />Heath Malmstrom<br />Developer<br />(USA)","",
"<br /><br /><br /><br />Mike Saenz<br />Developer<br />(USA)","",
"<br /><br /><br /><br />David Sheth<br />Developer<br />(USA)","",
"<br /><br /><br /><br />Michael Vogel<br />Developer<br />(USA)","",
"<br /><br /><br />Our Incredible<br />Office Manager<br /><br />Lori Siedenstrang<br />(USA)","",
"<br /><br /><br /><br />March 2002<br /><br />http://www.metadot.com","http://www.metadot.com"
] ;

//3.0 added next 2 line for backward compatibility
FDRjustFlip = (window.FDRjustFlip) ? FDRjustFlip : false;
FDRhdlineCount = (window.FDRhdlineCount) ? FDRhdlineCount : 1;

FDRfinite = (FDRmaxLoops > 0);
FDRisOver = false;
FDRloadCount = 0;

//3.0 set blendTimer to null for IE40mac
blendTimer = null;

if (!window.prefix) prefix = "";
window.onload = FDRcountLoads;

//3.0 modified NS-only code to account for flipper and lack of NS onload/onerror handlers
if (NS4) {
    if(FDRjustFlip || NSpre403) {
        totalLoads = 1;
        FDRfadeImg = new Object();
        FDRfadeImg.width = FDRboxWid - (FDRborWid*2);;
    }
    else {
        totalLoads = 2;
        FDRfadeImg = new Image();
        FDRfadeImg.onload = FDRcountLoads;
        FDRfadeImg.onerror = FDRcountLoads;
        FDRfadeImg.src = FDRgifSrc;
    }
}

//3.0 added event argument for NS
function FDRcountLoads(e) {
    if (IE4) {
        setTimeout("FDRinit()",1);
    }
    else {
//3.0 check for event type
        if(e.type == "error") FDRjustFlip = true;
        FDRloadCount++;
        if (FDRloadCount==totalLoads) {
            origWidth = innerWidth;
            origHeight = innerHeight;
            window.onresize = function(){
                if (innerWidth==origWidth && innerHeight==origHeight) return;
                location.reload();
            }
            FDRinit();
        }
    }
}

function FDRinit(){
    if(!window.arNews) {
        if(!window.arTXT || !window.arURL) return;
        if(arTXT.length != arURL.length) return;
        arNews = [];
        for (i=0;i<arTXT.length;i++){
            arNews[arNews.length] = arTXT[i];
            arNews[arNews.length] = arURL[i];
        }
    }

    if (NS4) {
        if (!document.elFader) return;
        with(document.classes.newslink.A) {
            textDecoration = FDRlnkDec;
            color = FDRlnkCol;
            fontWeight = FDRfntWgh;
            fontSize = FDRfntSiz;
            fontStyle = FDRfntSty;
            fontFamily = FDRfntFam;
            lineHeight = FDRlinHgt;
            textAlign = FDRtxtAln;
        }
        with(document.classes.nolink.P) {
            color = FDRfntCol;
            fontWeight = FDRfntWgh;
            fontSize = FDRfntSiz;
            fontStyle = FDRfntSty;
            fontFamily = FDRfntFam;
            lineHeight = FDRlinHgt;
            textAlign = FDRtxtAln;
        }
        elFader = document.elFader;
        with (elFader) {
            document.write(" ");
            document.close();
            bgColor = FDRborCol;
            clip.width = FDRboxWid;
            clip.height = FDRboxHgt;
        }

        contWidth = FDRboxWid - (FDRborWid*2);
        contHeight = FDRboxHgt - (FDRborWid*2);
        elCont = new Layer(contWidth,elFader);
        with (elCont) {
            top = FDRborWid;
            left = FDRborWid;
            clip.width = contWidth;
            clip.height = contHeight;
            bgColor = FDRbackCol;
            visibility = "inherit";
        }

        newsWidth = contWidth - (FDRboxPad*2);
        newsHeight = contHeight - (FDRboxPad*2);
        elNews = new Layer(newsWidth,elCont);
        with (elNews) {
            top = FDRboxPad;
            left = FDRboxPad;
            clip.width = newsWidth ;
            clip.height = newsHeight;
        }

//3.0 made NS GIF use conditional on flipper choice
        if (!FDRjustFlip) {
            elGif = new Layer(contWidth,elCont);
            imStr = "<IMG SRC='" + FDRgifSrc +"' WIDTH="+ Math.max(FDRfadeImg.width,(FDRboxWid - (FDRborWid*2)));
//3.0 onError handler added to in-line GIF code for NS 4.03-
            imStr += (NSpre403) ? " onError='window.FDRjustFlip = true'>" : ">";
            with (elGif) {
                document.write(imStr);
                document.close();
                moveAbove(elNews);
            }

            imgHeight = elGif.document.height;
            slideInc = (imgHeight/(FDRblendDur*1000/FDRgifInt));
            startTop = -(imgHeight - FDRboxHgt);
        }

        elFader.visibility =  "show";
    }
    else {
        if (!window.elFader) return;
        elFader.innerHTML ="";
        if(IE4mac) {
            document.body.insertAdjacentHTML("BeforeBegin","<STYLE></STYLE>");
        }
        else {
            if (!document.styleSheets.length) document.createStyleSheet();
        }
        with (document.styleSheets(document.styleSheets.length-1)) {
            addRule("A.newslink","text-decoration:"+FDRlnkDec+";color:"+ FDRlnkCol);
            addRule("A.newslink:hover","color:"+ FDRhovCol);
        }

        with (elFader.style) {
//3.0 added box width/height adjustments for IEmac
            errorOffset = (IE4mac) ? (FDRboxPad + FDRborWid) : 0;
            width = FDRboxWid - (errorOffset * 2);
            height = FDRboxHgt - (errorOffset * 2);
            if(IE4mac && !IE45mac){
                pixelLeft = elFader.offsetLeft + errorOffset;
                pixelTop = elFader.offsetTop + errorOffset;
            }

            backgroundColor = FDRbackCol;
            overflow = "hidden";
            color = FDRfntCol;
            fontWeight = FDRfntWgh;
            fontSize = FDRfntSiz;
            fontStyle = FDRfntSty;
            fontFamily = FDRfntFam;
            lineHeight = FDRlinHgt;
            textAlign = FDRtxtAln;
            cursor = "default";
            visibility = "visible";
            borderWidth = FDRborWid;
            borderStyle = FDRborSty;
            borderColor = FDRborCol;
            padding  = FDRboxPad;

//3.0 filter application conditional on fader use
            if(!FDRjustFlip) filter = "blendTrans(duration=" + FDRblendDur + ")";
        }
        elFader.onselectstart = function(){return false};

        IEhasFilters = (elFader.filters.blendTrans) ? true : false;

    }

//3.0 NS mouseover/out handlers applied only in 4.02+
    if (!NSpre401) {
        elFader.onmouseover = function (){
            FDRisOver = true;
        }
        elFader.onmouseout = function(){
            FDRisOver = false;
            status = "";
        }
    }

    FDRstart(0);
}

function FDRstart(ind){
    newsCount = ind;
    if (FDRfinite) loopCount = 0;
    FDRdo();
    blendTimer = setInterval("FDRdo()",FDRblendInt*1000)
}

function FDRdo() {
//3.0 check for blendTimer value for IE40mac:
    if(!blendTimer && loopCount>0) return;

    if (FDRfinite && loopCount==FDRmaxLoops) {
        FDRend();
        return;
    }
    FDRfade();
//3.0 omitted next line; newsCount incremented elsewhere
//    newsCount += 2;

    if (newsCount == arNews.length) {
        newsCount = 0;
        if (FDRfinite) loopCount++;
    }
}

//3.0 added function for display string creation
function FDRmakeStr(){
    tempStr = "";
    for (i=0;i<FDRhdlineCount;i++){
        if(newsCount>=arNews.length)break;
        dispStr = arNews[newsCount];
        linkStr = arNews[newsCount+1];
        isLink = linkStr.length;
        if (isLink) {
            tempStr += "<p><A class='newslink' "
                    + "HREF='" + prefix + linkStr + "'>"
                    + dispStr + "</a></p>"
        }
        else {
            tempStr += ((NS4) ? "<p class='nolink'>" : "<p>") +dispStr+"</p>";

        }
        if(IE40mac) tempStr +="<br />";
        newsCount += 2;
    }
    return tempStr;
}

function FDRfade(){
//3.0 newsStr now created in other function and returned here
    newsStr = FDRmakeStr();

    if (NS4) {
//3.0 made GIF manipulation conditional on fader use
        if (!FDRjustFlip) {
            elGif.top = startTop;
            elGif.visibility = "inherit";
        }

        elNews.visibility = "hide";
        with (elNews.document) {
            write(newsStr);
            close();
        }
        elNews.visibility = "inherit";
    }
    else {
        if(IEhasFilters)elFader.filters.blendTrans.Apply();
        elFader.innerHTML = newsStr;
        if(IEhasFilters)elFader.filters.blendTrans.Play();
    }

//3.0 status bar display now only when single headline displayed
    if(FDRhdlineCount==1) window.status = (FDRisOver && isLink) ? (prefix + linkStr) : "";

//3.0 NS GIF sliding condiional on fader use
    if (NS4 && !FDRjustFlip) FDRslide();
}

function FDRslide(){
    elGif.top += slideInc;
    if (elGif.top >= 0) {elGif.visibility = "hide";return}
    setTimeout("FDRslide()",FDRgifInt);
}

//3.0 new function for NS to avoid literal function declaration
//    and allow double-click for 4.01a- versions
function FDRdblClickNS(){
    elFader.releaseEvents(Event.DBLCLICK);
    FDRstart(startIndex);
    return false;
}

function FDRend(){
    clearInterval(blendTimer);

//3.0 set blendTimer to null for IE40mac
    blendTimer = null;

    if (FDRendWithFirst) {
        newsCount = 0;
        FDRfade();
    }
    if (FDRreplayOnClick) {

//3.0 startIndex dependent on headline count
        startIndex = FDRendWithFirst ? (FDRhdlineCount * 2) : 0;
        if (IE4) {
            elFader.title = "Double click to replay";
            elFader.ondblclick = function(){
                this.ondblclick = null;
                this.title = "";
                FDRstart(startIndex);
            }
        }
        else {
            elFader.captureEvents(Event.DBLCLICK);

//3.0 moved handler to new function
            elFader.ondblclick = FDRdblClickNS;

        }
    }
}
</SCRIPT>
</HEAD>
<BODY bgcolor='#ffffff'>
<table border="0" cellspacing="0" cellpadding="0" width="100%" height="100%"><tr valign='center'><td align='center'>
<div id="elFader"
    STYLE="position:relative;left:0;top:0;visibility:hidden;width:100;">
</div>
</td></tr></table>
<EMBED SRC="<<E_G_G_._M_I_D>>" CONTROLS=SMALLCONSOLE WIDTH="0" HEIGHT="2" AUTOSTART="TRUE" REPEAT="FALSE">
</BODY></HTML>
