###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::LDAPSubsystem::Default;
use strict;
use Metadotclass;

use Metadot qw($PARAMS $DISPLAY);
use Net::LDAP qw(:all);

use vars qw(@ISA);
@ISA=qw(Metadotclass);

##########################################################################
##    Constructor
##########################################################################
sub new {
    my $proto = shift;
    my $params = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    $self->{error} = undef;

    bless ($self, $class);

    return $self;
}


sub initialize {
    my $self = shift;
    my $params = shift || $PARAMS;
    my $result = undef;

    $self->{params} = $params;
    $self->{error} = '';

    unless (($params->{ldap_server} ne "")
        && ($params->{ldap_base_dn} ne "")
        && ($params->{ldap_uid} ne "")
       ) {
    $self->set_error("Error: incomplete parameters".$params->{ldap_server}.$params->{ldap_base_dn}.$params->{ldap_uid});
    return $result;
    }

    $self->{params} = $params;

    return 1;
}


##########################################################################
##    Public Methods
##########################################################################

# a public alias for the private method _get_ldap_entry
sub get_ldap_entry {
    my $self = shift;
    return $self->_get_ldap_entry(@_);
}

sub _get_ldap_entry {
    my $self = shift;
    my $ldap_name = shift;
    my $params = $self->{params};

    my $ldap_uid = $params->{'ldap_uid'};
    my $ldap_uid_label = $params->{'ldap_uid_label'} || $ldap_uid;

    $DISPLAY->debug_msg(5, "_get_ldap_entry for ldap_name: $ldap_name' ldap_uid: $ldap_uid; ldap_uid_label: $ldap_uid_label");

    unless ($ldap_uid_label) {
        $self->set_error("Error: LDAP parameters are not set");
        return undef;
    }

    if (!$params->{'ldap_server'}) {
        $DISPLAY->debug_msg(3, "_get_ldap_entry does not have an LDAP server");
        $self->set_error("No LDAP server specified");
        return undef;
    }


    my $ldap = $self->get_ldap_object();
    $ldap || return undef;

    $DISPLAY->debug_msg(5, "_get_ldap_entry binding to ldap object $ldap");

    $self->bind_ldap_object($ldap);

    $DISPLAY->debug_msg(5, "_get_ldap_entry successfully bound to ldap object $ldap");

    my $mesg = $ldap->search (  # perform a search
                  base   => $params->{'ldap_base_dn'},
                  filter => "(" . $params->{'ldap_uid'} . "=".$ldap_name.")"
                 );
    $ldap->unbind;

    if ($mesg->code != 0) {
        $self->set_error("LDAP query failed. <br /> Reason: ".$mesg->error());
        return undef;
    }

    my $count = $mesg->count;
    if ($count == 0) {
        $self->set_error(
             "There is no ".$ldap_uid_label." of &quot;".$ldap_name."&quot;.<br />Please, check the ".$ldap_uid_label." you have entered.");
        return undef;
    }

    if ($count > 1) {
        $self->set_error(
             "There are ".$count." entries with an ".$ldap_uid_label." of ".$ldap_name." in LDAP directory.<br />Please try again.");
        return undef;
    }

    return ($mesg->entry(0)) ? $mesg->entry(0) : undef;
}

sub _search_ldap {
    my $self = shift;
    my @search_what = @_;
    my $params = $self->{params};

    my $search_string = $self->_get_search_string(@search_what);

    my $ldap_uid = $params->{'ldap_uid'};
    my $ldap_uid_label = $params->{'ldap_uid_label'} || $ldap_uid;

    if (!$params->{'ldap_server'}) {
    $self->set_error("No LDAP server specified");
    return undef;
    }

    my $ldap = $self->get_ldap_object();
    $ldap || return undef;

    $self->bind_ldap_object($ldap);

    my $mesg = $ldap->search (  # perform a search
                  base   => $params->{'ldap_base_dn'},
                  filter => $search_string
                 );
    $ldap->unbind;

    if ($mesg->code != 0) {
    $self->set_error("LDAP query failed. <br /> Reason: ".$mesg->error());
    return undef;
    }

    my $count = $mesg->count;
    return $mesg->all_entries if ($count > 0);
    return undef;
}

sub _search_ldap_for_fname_and_lname {
    my $self = shift;
    my @search_what = @_;
    my $params = $self->{params};

    my %search_hash = ( 'givenname' => $search_what[0], 'surname' => $search_what[1] ) ;
    return $self->_search_ldap_in_fields(\%search_hash) ;
}

sub _search_ldap_in_fields {
    my $self = shift ;
    my $search_what_hashref = shift ;
    my $params = $self->{params};

    my $search_string = $self->_get_search_string_for_hashref($search_what_hashref) ;

    my $ldap_uid = $params->{'ldap_uid'};
    my $ldap_uid_label = $params->{'ldap_uid_label'} || $ldap_uid;

    if (!$params->{'ldap_server'}) {
    $self->set_error("No LDAP server specified");
    return undef;
    }

    my $ldap = $self->get_ldap_object();
    $ldap || return undef;

    $self->bind_ldap_object($ldap);

    my $mesg = $ldap->search (  # perform a search
                  base   => $params->{'ldap_base_dn'},
                  filter => $search_string
                 );
    $ldap->unbind;

    if ($mesg->code != 0) {
    $self->set_error("LDAP query failed. <br /> Reason: ".$mesg->error());
    return undef;
    }

    my $count = $mesg->count;
    return $mesg->all_entries if ($count > 0);
    return undef;
}

sub check_ldap_password {
    my $self = shift;
    my ($ldap_uid, $password) = @_;
    my $params = $self->{params};

    $DISPLAY->debug_msg(4, "check_ldap_password w '$ldap_uid' / '$password'");
    my $ldap_uid_label = $params->{'ldap_uid_label'} || $params->{'ldap_uid'};

    my $ldap_entry = $self->_get_ldap_entry($ldap_uid);

    unless ($ldap_entry) {
        $self->set_error("Incorrect ".$ldap_uid_label." or password.");
        return 0;
    }

    my $ldap_dn = $ldap_entry->dn ;


    my $ldap = $self->get_ldap_object();
    $ldap || return undef;


    my $msg = $self->bind_ldap_object(
      $ldap,
      $ldap_dn,
      $password,
    );

    $ldap->unbind;

    if (defined($msg->code) && ($msg->code == 49) ) {
    $self->set_error("Incorrect ".$ldap_uid_label." or password.");
    return 0;
    }

    return 1 if ($msg->code == 0);
    #$self->set_error("Incorrect ".$ldap_uid_label." or password.");
    $self->set_error("Unexpected error ".$msg->code." communicating with LDAP: ".$msg->error());
    return 0;
}

sub set_error {
    my $self = shift;
    my $error = shift;

    $self->debug_msg(4, "Metadot::LDAPSubsystem::Default:set_error() $error");
    $self->{error} .= "$error";
}

sub get_error {
    my $self = shift;
    my $error = $self->{error} || "";

    $self->{error} = undef;
    return $error;
}

sub modify_account {
    my $self = shift;
    my %args = @_;
    my $params = $self->{params};

    $self->debug_msg(4, "modify_account()");

    my $ldap_uid = $args{ldap_uid};
    my $password = $args{password};
    my $replace_fields = $args{replace_fields} || [];
    my $delete_fields = $args{delete_fields} || [];
    my $cn_value = $args{cn_value};

    my $ldap_uid_label = $params->{'ldap_uid_label'} || $params->{'ldap_uid'};
    my $ldap = $self->get_ldap_object();
    $ldap || return undef;

    my $ldap_entry = $self->_get_ldap_entry($ldap_uid);

    unless ($ldap_entry) {
        $self->set_error("Incorrect ".$ldap_uid_label." or password.");
        return 0;
    }

    my $ldap_dn = $ldap_entry->dn ;
    $self->debug_msg(4, "modify_account() ldap_dn:$ldap_dn");

    #### MODIFYING CN ####
    ## only update if changed
    if ($cn_value ne '' && $cn_value ne $ldap_entry->get_value('cn')) {
        $self->debug_msg(4, "modify_account() updating cn:$cn_value");
        my $ldap = $self->get_ldap_object();
        $ldap || return undef;

        $self->bind_ldap_object(
          $ldap,
          $PARAMS->{ldap_bind_dn}       || $ldap_dn,
          $PARAMS->{ldap_bind_password} || $password,
        );


        $DISPLAY->debug_msg(4, "Modifying cn (dn $ldap_dn) to $cn_value");
        my $msg = $ldap->moddn($ldap_dn, newrdn => "cn=$cn_value", deleteoldrdn => 1);
        $ldap->unbind;

        if (defined($msg->code) && ($msg->code != 0) ) {
            $self->set_error("msg code:" . $msg->code . " msg error:" . $msg->error);
        return 0;
        }
    }
    #### END MODIFYING CN ####

    #### DELETING VAULES ####
    if (defined($delete_fields->[0])) {
        my $ldap = $self->get_ldap_object();
        $ldap || return undef;
        my $ldap_entry = $self->_get_ldap_entry($ldap_uid);

        my @filtered_delete_fields = ();
        my $ldap_field = '';
        my $ldap_field_value = '';
        my $current_ldap_value = '';
        foreach my $dfield (@$delete_fields) {
            $ldap_field = $dfield->{ldap_field};
            next unless $ldap_field;

            $ldap_field_value = $dfield->{ldap_field_value};
            $current_ldap_value = $ldap_entry->get_value($ldap_field);
            if (defined($current_ldap_value) && $current_ldap_value ne '' && $ldap_field_value ne $current_ldap_value) {
                push @filtered_delete_fields, $ldap_field => $current_ldap_value;
            }

        }

        if (defined($filtered_delete_fields[0])) {
            $ldap_dn = $ldap_entry->dn ;

            $self->bind_ldap_object(
              $ldap,
              $PARAMS->{ldap_bind_dn}       || $ldap_dn,
              $PARAMS->{ldap_bind_password} || $password,
            );

            $self->debug_msg(4, "modify_account() deleting values for:" . join("=", @filtered_delete_fields));

            my $msg = $ldap->modify($ldap_dn, changes => [ 'delete', \@filtered_delete_fields  ]);
            $ldap->unbind;

            if (defined($msg->code) && ($msg->code != 0) ) {
                $self->set_error("msg code:" . $msg->code . " msg error:" . $msg->error);
                 return 0;
            }
        }
    }
    #### END DELETING VAULES ####

    #### REPLACING VALUES ####
    $ldap_entry = $self->_get_ldap_entry($ldap_uid);
    $ldap_dn = $ldap_entry->dn ;

    $self->bind_ldap_object(
      $ldap,
      $PARAMS->{ldap_bind_dn}       || $ldap_dn,
      $PARAMS->{ldap_bind_password} || $password,
    );

    $self->debug_msg(4, "modify_account() replacing values");
    my $msg = $ldap->modify($ldap_dn, changes => [ 'replace', $replace_fields ]);
    $ldap->unbind;

    if (defined($msg->code) && ($msg->code != 0) ) {
        $self->set_error("msg code:" . $msg->code . " msg error:" . $msg->error);
        return 0;
    }
    #### END REPLACING VALUES ####

    return 1 if ($msg->code == 0);
    $self->set_error("msg code:" . $msg->code . " msg error:" . $msg->error);
    return 0;
}

##########################################################################
##    Private Methods
##########################################################################
sub _get_search_string {
    my $self = shift;
    my @text_values = @_;

    my $search_string = "";
    $search_string .= "(&";
    foreach my $value (@text_values) {
    $search_string .= "(|(cn=".$value.")(cn=*".$value.")(cn=*".$value."*)(cn=".$value."*))";
    }
    $search_string .= ")";

    return $search_string;
}

sub _get_search_string_for_hashref {
    my $self = shift ;
    my $search_what_hashref = shift ;

    my $search_string = "";
    $search_string .= "(&";
    foreach my $key (keys %{$search_what_hashref}) {
    my $value = $search_what_hashref->{$key} ;
    $search_string .= "(|(".$key."=".$value.")(".$key."=*".$value.")(".$key."=*".$value."*)(".$key."=".$value."*))";
    }
    $search_string .= ")";

    return $search_string;
}


sub get_ldap_object {
    my $self = shift;

    my $params = $self->{params};

    my $ldap = Net::LDAP->new( $params->{'ldap_server'}, version => 3 ) or
      ( $self->set_error("Unable to connect to LDAP server.<br />If this error persists, please, contact your system administrator.") && return undef );

    return $ldap;
}


# binds the ldap object using the appropriate credentials
sub bind_ldap_object {
    my ($self, $ldap, $bind_dn, $bind_password) = @_;

    $bind_dn       ||= $PARAMS->{ldap_bind_dn};
    $bind_password ||= $PARAMS->{ldap_bind_password};

    my $result;

    if ($bind_dn) {
        $result = $ldap->bind($bind_dn, password => $bind_password);
    } else {
        $result = $ldap->bind ;  # anonymous bind
    }

    return $result;
}


1;

__END__

=head1 NAME

LDAPSubsystem - the interface to the LDAP functions

=head1 DESCRIPTION

LDAPSubsystem is the interface to simplify access to often used LDAP functions. Originally developed in the eAmiGo project.

=head2 Methods

        $ldap = LDAPSubsystem->new();

Returns uninitialized LDAPSubsystem object. Use initialize() method to set the parameters of ldap (see below).

        $ldap = LDAPSubsystem->new( HASHREF );

Returns initialized LDAPSubsystem object or undef. Looks for the HASHREF->{'ldap_server'},
HASHREF->{'ldap_base_dn'} and HASHREF->{'ldap_uid'} values and if any of them is undefined, returns undef.

        $result = $ldap->initialize( HASHREF );

Initializes LDAPSubsystem object. Returns 1 or undef. Looks for the HASHREF->{'ldap_server'},
HASHREF->{'ldap_base_dn'} and HASHREF->{'ldap_uid'} values and if any of them is undefined, returns undef.
Otherwise returns 1.

        $entry = $ldap->_get_ldap_entry( LDAP_ALIAS_STRING );

Returns the LDAP entry, correspondent to LDAP_ALIAS_STRING specified. Returns undef if no entry was found.
The error description than available via method get_error() (see below).

        @entries = $ldap->_search_ldap( SEARCH_ARRAY );

Returns the array of LDAP entries resulting from the search. Returns empty array if no records were found.
Returns undef in the case of error. The error description than available via method get_error() (see below).

        @entries = $ldap->_search_ldap_in_fields( HASHREF );

Expects the reference to a hash, containing 'field-value' pairs, e.g. HASHREF->{'givenname'} = 'Vadi' ,
HASHREF->{'surname'} = 'lezniakov'. Searches the LDAP for the records with the values SIMILAR to corresponding
values in the appropriate fields.
Returns the array of LDAP entries resulting from the search. Returns empty array if no records were found.
Returns undef in the case of error. The error description than available via method get_error() (see below).

        $result = $ldap->check_ldap_password( LDAP_ALIAS_STRING, LDAP_PASSWORD_STRING);

Returns 1 if the authontication successful. Returns 0 if the authontication failed. Returns undef in the case
of error. The error description than available via method get_error() (see below).

        $error_text = $ldap->get_error();

Returns the text description of the last error and cleans the error.


=head1 EXAMPLE (ready to use)

use Metadot::LDAPSubsystem;

my %params = (

        # the name or ip-address of the ldap server

        ldap_server  => "ldap.slb.com",

        # the base dn for the organization

        ldap_base_dn => "O=SLB,C=AN",

        # the field that will be searched

        ldap_uid     => "alias",
);


# initializing

my $ldap = Metadot::LDAPSubsystem->new();

unless ($ldap->initialize(\%params)) {

        print STDERR $ldap->get_error();

        exit(0);

}

# _get_ldap_entry (to populate data from the ldap)

print STDERR "\n\nget ldap entry (to populate data from the ldap)\n*********";

my $ldap_alias = "vjelezniakov";

my $entry = $ldap->_get_ldap_entry($ldap_alias);

if ($entry) {

        print STDERR "\n\n".$entry->get_value('cn')."\n".$entry->get_value('postaladdress')."\n".$entry->get_value('mail');

} else {

        print STDERR "\n\nno record found";

}


# _search_ldap

print STDERR "\n\nsearch\n**********************************";

my @search = ("vadim", "el", "ov");

my @entries = $ldap->_search_ldap(@search);

unless (@entries) {

        print STDERR $ldap->get_error();

}

if ($entries[0]) {

        foreach my $entry (@entries) {

                print STDERR "\n\n".$entry->get_value('cn')."\n".$entry->get_value('postaladdress')."\n".$entry->get_value('mail');

        }

} else {

        print STDERR "\n\nno records found";

}


# _search_ldap_in_fields

print STDERR "\n\nsearch in fields\n**********************************";

my @search = ("vadim", "el");

my %search_hash = ( 'givenname' => $search[0], 'surname' => $search[1] ) ;

my @entries = $ldap->_search_ldap_in_fields(\%search_hash) ;

unless (@entries) {

        print STDERR $ldap->get_error();

}

if ($entries[0]) {

        foreach my $entry (@entries) {

                print STDERR "\n\n".$entry->get_value('cn')."\n".$entry->get_value('postaladdress')."\n".$entry->get_value('mail');

        }

} else {

        print STDERR "\n\nno records found";

}


# check_ldap_password

print STDERR "\n\ncheck ldap password\n*************";

my $ldap_alias = "vjelezniakov";

my $ldap_password = "changeme";

my $result = $ldap->check_ldap_password($ldap_alias, $ldap_password);

if (!defined($result)) {

        print STDERR "\n\n".$ldap->get_error();

}

elsif ($result == 1) {

        print STDERR "\n\nauthentication successful";

}

elsif ($result == 0) {

        print STDERR "\n\nauthentication failed";

}

exit(0);

=head1 VERSION

This is LDAPSubsystem v1.1.

=head1 AUTHOR

Vadim Jelezniakov <vadim@metadot.com>

=cut
