package Metadot::MIME::Types;

$VERSION = '0.15';

use strict;
use Metadot::MIME::Type;

=head1 NAME

 Metadot::MIME::Types - Definition of MIME types

=head1 SYNOPSIS

 use Metadot::MIME::Types;
 my $mimetypes = Metadot::MIME::Types->new;
 my Metadot::MIME::Type $plaintext = $mimetype->type('text/plain');
 my Metadot::MIME::Type $imagegif  = $mimetype->extension('gif');

=head1 DESCRIPTION

MIME types are used in MIME entities, for instance as part of e-mail
and HTTP traffic.  Sometimes real knowledge about a mime-type is need.
This module will supply it.

=cut

#-------------------------------------------

=head1 METHODS

=over 4

=cut

#-------------------------------------------

=item new OPTIONS

Create a new C<Metadot::MIME::Types> object which manages the data.  In the current
implementation, it does not matter whether you create this object often
within your program, but in the future this may change.

 OPTIONS                   DEFAULT
 only_complete             0

=over 4

=item * only_complete =E<gt> BOOLEAN

Only include complete mime-type definitions: requires extension and
encoding.  This will reduce the number of entries --and with that
the amount of memory consumed-- enormously.

In your program you have to decide: the first time that you call
the creator (C<new>) determines whether you get the full or the partial
information.

=back

=cut

my %list;
sub new(@) { (bless {}, shift)->init( {@_} ) }

sub init($)
{   my ($self, $args) = @_;

    unless(keys %list)
    {   local $_;

        # CKIM: PersistentPerl has a problem using the __DATA__ file handle for some reason,
        #       so I'm splitting this out to a file.
        open DATA_FILE, "<./Metadot/MIME/type_data.txt";
        while(<DATA_FILE>)
        {   
            #print STDERR "MIME DATA: $_\n";
            s/\#.*//;
            next if m/^$/;

            my $os = s/^(\w+)\:// ? qr/$1/i : undef;

            my ($type, $extensions, $encoding) = split;
            my $extent = $extensions ? [ split /\,/, $extensions ] : undef;

            next if $args->{only_complete}
                 && ! ($extensions && $encoding);
                    
            my $simplified = Metadot::MIME::Type->simplified($type);
            push @{$list{$simplified}}, Metadot::MIME::Type->new
              ( type       => $type
              , extensions => $extent
              , encoding   => $encoding
              , system     => $os
              );
        }
    }

    close DATA_FILE ;
    $self;
}

my %type_index;
sub create_type_index()
{   my $self = shift;

    my @os_specific;
    while(my ($simple, $definitions) = each %list)
    {   foreach my $def (@$definitions)
        {   if(defined $def->system && $^O =~ $def->system)
            {   # OS specific definitions will overrule the
                # unspecific definitions, so must be postponed till
                # the end.
                push @os_specific, $def;
            }
            else
            {   $type_index{$_} = $def foreach $def->extensions;
            }
        }
    }

    foreach my $def (@os_specific)
    {   $type_index{$_} = $def foreach $def->extensions;
    }

    $self;
}

#-------------------------------------------

=item type STRING

Return the C<Metadot::MIME::Type> which describes the type related to STRING.  One
type may be described more than once.  Different extensions is use for
this type, and different operating systems may cause more than one
C<Metadot::MIME::Type> object to be defined.  In scalar context, only the first
is returned.

=cut

sub type($)
{  my $mime  = Metadot::MIME::Type->simplified($_[1]);
   return () unless exists $list{$mime};
   wantarray ? @{$list{$mime}} : $list{$mime}[0];
}

#-------------------------------------------

=item mimeTypeOf FILENAME

Returns the C<Metadot::MIME::Type> object which belongs to the FILENAME (or simply
its filename extension).  The extension is used, and considered
case-insensitive.

Examples:

 my Metadot::MIME::Types $types = Metadot::MIME::Types->new;
 my Metadot::MIME::Type  $mime = $types->mimeTypeOf('gif');

 my Metadot::MIME::Type  $mime = $types->mimeTypeOf('jpg');
 print $mime->isBinary;

=cut

sub mimeTypeOf($)
{   my ($self, $name) = @_;
    $self->create_type_index unless keys %type_index;
    $name =~ s/.*\.//;
    $type_index{lc $name};
}

#-------------------------------------------

=item addType TYPE, ...

Add one or more TYPEs to the set of known types.  Each TYPE is a
C<Metadot::MIME::Type> which must be experimental: either the main-type or
the sub-type must start with C<x->.

Please inform the maintainer of this module when registered types
are missing.

=cut

sub addType(@)
{   my $self = shift;
    foreach my $type (@_)
    {
        if($type->isRegistered)
        {   use Carp;
            carp "Please report the registered type $type to the module author."
        }

        my $simplified = $type->simplified;
        push @{$list{$simplified}}, $type;
    }

    %type_index = ();
    $self;
}

#-------------------------------------------

=back

=head1 EXPORT

The next methods are provided for backward compatibility with Metadot::MIME::Types
versions 0.06 and below.  This code originates from Jeff Okamoto
<F<okamoto@corp.hp.com>> and others.

=over 4

=cut

#-------------------------------------------

require Exporter;
use vars qw/@ISA @EXPORT_OK/;
@ISA       = qw(Exporter);
@EXPORT_OK = qw(by_suffix by_mediatype import_mime_types);

#-------------------------------------------

=item by_suffix FILENAME|SUFFIX

Like C<mimeTypeOf> but does not return an C<Metadot::MIME::Type> object.

Example:

 use Metadot::MIME::Types 'by_suffix';
 my ($mediatype, $encoding) = by_suffix 'image.gif';

 my $refdata =  by_suffix 'image.gif';
 my ($mediatype, $encoding) = @$refdata;

=cut

my $mime_types;

sub by_suffix($)
{   my $filename = shift;
    $mime_types ||= Metadot::MIME::Types->new;
    my $mime     = $mime_types->mimeTypeOf($filename);

    my @data     = defined $mime ? ($mime->type, $mime->encoding) : ('','');
    wantarray ? @data : \@data;
}

#-------------------------------------------

=item by_mediatype TYPE

This function takes a media type and returns a list or anonymous array of
anonymous three-element arrays whose values are the file name suffix used to
identify it, the media type, and a content encoding.

TYPE can be a full type name (contains '/', and will be matched in full),
a partial type (which is used as regular expression) or a real regular
expression.

=cut

sub by_mediatype($)
{   my $type = shift;
    my @found;

    if($type =~ m!/!)
    {   my $simplified = Metadot::MIME::Type->simplified($type);
        my $mime = $list{$simplified};
        push @found, @$mime if defined $mime;
    }
    else
    {   my $mime = ref $type ? $type : qr/$type/i;
        @found = map {@{$list{$_}}}
                    grep {$_ =~ $mime}
                        keys %list;
    }

    my @data;
    foreach my $mime (@found)
    {   push @data, map { [$_, $mime->type, $mime->encoding] }
                       $mime->extensions;
    }

    wantarray ? @data : \@data;
}

#-------------------------------------------

=item import_mime_types

This method has been removed: mime-types are only useful if understood
by many parties.  Therefore, the IANA assigns names which can be used.
In the table kept by this C<Metadot::MIME::Types> module all these names, plus
the most often used termporary names are kept.  When names seem to be
missing, please contact the maintainer for inclussion.

=cut

sub import_mime_types($)
{   my $filename = shift;
    use Carp;
    croak <<'CROAK';
import_mime_types is not supported anymore: if you have types to add
please send them to the author.
CROAK
}

#-------------------------------------------

=back

=head1 SEE ALSO

L<Metadot::MIME::Type>

=head1 AUTHOR

Original module and data collection by Jeff Okamoto and the Apache team.
Mark Overmeer (F<mimetypes@overmeer.net>).
All rights reserved.  This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

=head1 VERSION

This code is beta version 0.15.

Copyright (c) 2001 by Jeff Okamoto and Mark Overmeer. All rights reserved.
This program is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;


