###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

# The DistributedCache shares update information across server instances.
# This cache manages dirtiness at the element level

package Metadot::ObjectAccess::DistributedCache;
use strict;
use vars qw(@ISA);

use Metadot qw(%FORM $SESSION $PARAMS %REQUEST_VARS);
use MetadotObject;
use Metadot::SystemMessage;
use Metadot::SystemMessageManager;
use Data::Dumper;
use Metadotclass;

@ISA = qw(MetadotObject);

use constant DEFAULT_MAX_ELEMENT_AGE => 3600;
use constant DEFAULT_MAX_SIZE => 500;

sub initialize {
    my $self = shift;
    my $args = shift;
    $self->SUPER::initialize($args);

    $self->_element_hashtable({});
    $self->_creation_time(time);

    unless ($args->{max_element_age}) {
        $self->max_element_age($PARAMS->{accelerator_max_element_age} || DEFAULT_MAX_ELEMENT_AGE);
    }

    unless ($args->{max_size}) {
        $self->max_size($PARAMS->{accelerator_max_size} || DEFAULT_MAX_SIZE);
    }
}


use constant MO_INSTANCE_PROPERTIES => {
    owner           => undef,    # the class that owns the cache object
    max_size        => undef,    # the maximum number of elements in the cache
    max_element_age => undef,    # maximum age before an element is flushed from cache

    _creation_time      => undef,
    _element_hashtable  => undef,
    _first_node         => undef,
    _last_node          => undef,
    _size               => 0,
    _hits               => 0,
    _misses             => 0,
    _last_notification_check   => undef,
};


###############################################################

# puts an element into the cache
sub put {
    my ($self, $key, $value) = @_;
    defined($key) || die("Must specify the key for placing an element in the cache");

    # before I process the cache, see if I have any new notifications
    $self->initialize_request();

    # check if this key is in use
    my $hashtable = $self->_element_hashtable();
    my $existing  = $hashtable->{$key};

    # if the key is in use, update the element
    if ($existing) {
        Metadotclass->debug_msg(4, "DistributedCache::put() Updating existing value, key='$key', value=$value");
        $self->_debug_element($existing);
        $existing->{value} = $value;
        $self->_set_MRU($existing);
    } else {
        my $element = { value=>$value, timestamp=>time, key=>$key };
        Metadotclass->debug_msg(4, "DistributedCache::put() Adding new value, key='$key', value=$value");
        $self->_debug_element($element);
        $hashtable->{$key} = $element;
        $self->_add_to_list($element);
    }
}

# retrieves an element from the cache
sub get {
    my ($self, $key) = @_;
    defined($key) || die("Must specify the key for placing an element in the cache");
    my $result;

    # before I process the cache, see if I have any new notifications
    $self->initialize_request();

    my $hashtable = $self->_element_hashtable();
    my $element = $hashtable->{$key};
    if ($element) {
        # first check that element hasn't expired
        my $age = time - $element->{timestamp};
        my $max_element_age = $self->max_element_age();
        if ($max_element_age && ($age > $max_element_age)) {
            Metadotclass->debug_msg(4, "DistributedCache::get() Key '$key' has expired");
            $self->delete($key, 1);
            $self->_misses($self->misses() + 1);
        } else {
            $self->_set_MRU($element);
            $result = $element->{value};
            $self->_hits($self->hits() + 1);
            Metadotclass->debug_msg(5, "DistributedCache::get() Successfully retrieved element, key='$key' ($element)");
        }
    } else {
        $self->_misses($self->misses() + 1);
        Metadotclass->debug_msg(4, "DistributedCache::get() Cache miss on key '$key'");
    }
    return $result;
}

# deletes an element from the cache
sub delete {
    my ($self, $key, $no_message) = @_;
    defined($key) || die("Must specify the key for deleting an element in the cache");

    # before I process the cache, see if I have any new notifications
    $self->initialize_request();

    Metadotclass->debug_msg(4, "DistributedCache::delete() Deleting an element with key '$key'");
    my $hashtable = $self->_element_hashtable();
    my $element = $hashtable->{$key};

    if ($element) {
        $self->_delete_from_list($element);
        delete $hashtable->{$key};
        unless ($no_message) {
            $self->send_notification("DROP $key");
        }
    }
}

# measures and returns the number of elements in the cache
sub size {
    my $self = shift;

    # before I process the cache, see if I have any new notifications
    $self->initialize_request();

    return $self->_size() || 0;
}

# returns the number of cache hits
sub hits {
    my $self = shift;

    # before I process the cache, see if I have any new notifications
    $self->initialize_request();

    return $self->_hits() || 0;
}

# returns the number of cache misses
sub misses {
    my $self = shift;

    # before I process the cache, see if I have any new notifications
    $self->initialize_request();

    return $self->_misses() || 0;
}


# completely empties the cache
sub flush {
    my ($self, $no_message) = @_;

    # before I process the cache, see if I have any new notifications
    $self->initialize_request();

    $self->_size(0);
    $self->_element_hashtable({});
    $self->_first_node(undef);
    $self->_last_node(undef);
    $self->_hits(0);
    $self->_misses(0);

    unless ($no_message) {
        $self->send_notification("FLUSH");
    }

    Metadotclass->debug_msg(3, "DistributedCache::flush() Cache was flushed");
}



sub check_max_size {
    my $self = shift;
    # if the maximum size is reached, delete something first
    my $max_size = $self->max_size();
    while ($max_size && ($self->size() > $max_size)) {
        my $lru = $self->_get_LRU();
        $lru || die("Trying to trim cache, but LRU could not be found");
        Metadotclass->debug_msg(4, "DistributedCache::_add_to_list() Maximum size of $max_size has been exceeded. Trimming LRU, key='$lru->{element}->{key}'");
        $self->delete($lru->{key}, 1);
    }
}


# some actions should be performed prior to every request
sub initialize_request {
    my $self = shift;

    # set a property in the request to avoid repeating this action
    my $request_key = "__$self"."__request";
    unless ($REQUEST_VARS{$request_key}++) {
        $self->check_notifications();
        $self->check_max_size();
    }
}


# send system notifications to other caches
sub send_notification {
    my $self = shift;

    defined (my $command = shift) || die("Must specify the notification message");
    # commands supported: FLUSH
    #                     DROP [key]

    my $message = Metadot::SystemMessage->new(
        {
            sender          => $self->get_signature(),
            sender_class    => ref($self) || $self,
            content         => $command,
        },
    );

    Metadot::SystemMessageManager->send_message($message);
}


# check for cache notifications
sub check_notifications {
    my $self = shift;
    # note, I calculate the time since last update instead of directly
    # sending any time values to the message manager. This is because
    # the two may be on systems with differing times.
    my $last_update_time = $self->_last_notification_check();
    $self->_last_notification_check(time);

    #Metadotclass->debug_msg(5, "DistributedCache::check_notifications; last_update_time was $last_update_time; current time=".time);
    Metadotclass->debug_msg(5, "DistributedCache::check_notifications; ($self)<br />\nlast update: ".(defined $last_update_time ? localtime($last_update_time) : 'undef')."<br />\ncurrent time: ".localtime(time));

    if (defined($last_update_time)) {
        my $time_since_last_update = (time - $last_update_time);
        $time_since_last_update += 10;  # add a few seconds for safe measure

        Metadotclass->debug_msg(4, "DistributedCache::check_notifications within last $time_since_last_update seconds");

        # check for updates
        my @messages = Metadot::SystemMessageManager->check_messages(
            sender  => $self->get_signature(),
            max_age => ($time_since_last_update),
            );

        foreach my $message (@messages) {
            my $content = $message->get_content();
            Metadotclass->debug_msg(4, "DistributedCache::check_notifications() Got message $message, content='$content'");
            my ($command, @args) = split(/\s+/, $content);
            $command = uc($command);
            Metadotclass->debug_msg(4, "DistributedCache::check_notifications() Parsed command='$command' args=@args");
            if ($command) {
                if ($command eq 'FLUSH') {
                    $self->flush(1);
                    Metadotclass->debug_msg(4, "check_notifications() got FLUSH command");
                } elsif ($command eq 'DROP') {
                    # default command is a key value to delete
                    my $key = $args[0];
                    $self->delete($key, 1);
                    Metadotclass->debug_msg(4, "check_notifications() got 'DROP $key'");
                }
            }
        }
    }
}


sub get_creation_time {
    my $self = shift;
    return $self->_creation_time();
}


# an alias for owner's get_signature
sub get_signature
{
    my $self = shift;
    return $self->get_owner()->get_signature();
}


###############################################################
# private operations

# adds an element to the element list
sub _add_to_list {
    my ($self, $element) = @_;
    defined($element) || die("Element not defined");

    # each element has a list node, and these are doubly linked.
    my $list_node = $element->{list_node};
    # if the element already has a list node, this is taken care of
    unless ($list_node) {
        $list_node = { element=> $element };
        $element->{list_node} = $list_node;
    }

    # get the first element in the list
    my $first = $self->_first_node();

    # if one was found, give it a link to the new first element
    if ($first) {
        $first->{previous_node} = $list_node;
        $list_node->{next_node} = $first;
    }

    # designate the new element as the first element
    $self->_first_node($list_node);

    # if there was no last element, then set this one as last
    unless ($self->_last_node()) {
        $self->_last_node($list_node);
    }

    # adjust the size
    $self->_size($self->size() + 1);
}

# deletes an element from the element list
sub _delete_from_list {
    my ($self, $element) = @_;
    defined($element) || die("Element not defined");

    my $list_node = $element->{list_node} || die("Could not resolve list node from element");

    # get the previous and next
    my $previous = $list_node->{previous_node};
    my $next     = $list_node->{next_node};

    # unlink this node
    $list_node->{previous_node} = undef;
    $list_node->{next_node} = undef;

    # stitch the links between previous and next
    if ($previous) {
        # link the previous to the next
        if ($next) {
            $previous->{next_node} = $next;
        # unless there is not one, in which case this is now the last one
        } else {
            $self->_last_node($previous);
        }
    } else {
        # this is the first node. reset the list's first_node
        $self->_first_node($next);
    }

    if ($next) {
        # link the next to the previous
        if ($previous) {
            $next->{previous_node} = $previous;
        # unless there is no previous, in which case this is now the first one
        } else {
            $self->_first_node($next);
        }
    } else {
        # this is the last node. reset the list's last_node
        $self->_last_node($previous);
    }

    # adjust the size
    $self->_size( $self->size() - 1);
    $self->_show_list_info();
}


# sets the MRU element
sub _set_MRU {
    my $self = shift;
    my $element = shift || die("Specify an element to set_MRU");
    Metadotclass->debug_msg(5, "DistributedCache::_set_MRU key='$element->{key}'");
    $self->_delete_from_list($element);
    $self->_add_to_list($element);
    $self->_show_list_info();
}


# gets the LRU element
sub _get_LRU {
    my $self = shift;
    return $self->_last_node()->{element};
}


# provides an abbreviated dump of an element
sub _debug_element {
    my $self = shift;

    return unless ($SESSION->{debug_level} >= 5);

    my $element = shift;
    if ($element) {
        if (ref($element) eq 'HASH') {
            Metadotclass->debug_msg(5, "DistributedCache Dumping element $element");
            foreach my $key (keys %$element) {
                Metadotclass->debug_msg(5, "$key = $element->{$key}");
            }
            Metadotclass->debug_msg(5, ('-'x20));
        } else {
            Metadotclass->debug_msg(5, "Element is not a hash");
        }
    } else {
        Metadotclass->debug_msg(5, "Not an element");
    }
}


sub _show_list_info {
    my $self = shift;

    return unless ($SESSION->{debug_level} >= 6);

    my $first = $self->_first_node();
    my $last = $self->_last_node();
    my $size = $self->size();

    Metadotclass->debug_msg(6, "DistributedCache showing list info:");
    Metadotclass->debug_msg(6, "First: $first->{element}->{key}");
    Metadotclass->debug_msg(6, "Last : $last->{element}->{key}");
    Metadotclass->debug_msg(6, "Size : $size");
    Metadotclass->debug_msg(6, ('-'x20));
    my $node = $first;
    my $count = 1;
    while ($node) {
        Metadotclass->debug_msg(6, "List index #$count: $node->{element}->{key}");
        $node = $node->{next_node};
    }
    Metadotclass->debug_msg(6, ('-'x20));
}


# sets the first element in the list
sub _first_node {
    my $self = shift;

    if (@_) {
        my $node = shift;
        if (defined($node)) {
            $node->{previous_node} = undef;
        }
        return $self->__first_node($node);
    } else {
        return $self->__first_node();
    }
}


# sets the last element in the list
sub _last_node {
    my $self = shift;

    if (@_) {
        my $node = shift;
        if (defined($node)) {
            $node->{next_node} = undef;
        }
        return $self->__last_node($node);
    } else {
        return $self->__last_node();
    }
}


1;
