###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Permissions;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $DISPLAY);

use Metadot::Serialization::SerializerFactory;
use DBAccess;
use Carp qw(croak);
use HTML::Components;
use HTML::TableFrame;


###
### Constructor
###
### May take a reference to a
### frozen Permissions object (a string reference)
###
sub new {
    my $proto = shift;
    my $bundles_href = shift || undef;
    my $acl_href     = shift || undef;
    my $recursive    = shift || undef;

    my $class = ref($proto) || $proto;

    my $self = {
        BUNDLES => {},
        ACL => {},
        RECURSIVE => $recursive,
    };

    if(defined $bundles_href) {
      (ref($bundles_href) eq 'HASH') || croak "named params should be passed as hash ref";
      $self->{BUNDLES} = $bundles_href;
    }

    if(defined $acl_href) {
      (ref($acl_href) eq 'HASH') || croak "named params should be passed as hash ref";
      $self->{ACL} = $acl_href;
    }


    bless ($self, $class);

    return $self;
}


### creates a Permissions object from an Operations object
sub generate_permissions_object
{
    my $class = shift;
    my $operations = shift || die('Must specify an Operations object');

    my $result = $class->new(
          # bundles
          {map {$_, $operations->{BUNDLES}{$_}{DEFAULT_LEVEL}} (keys %{$operations->{BUNDLES}})},

          # acl
          $operations->{ACL},

          # recursive
          $operations->{RECURSIVE},
      );

    return $result;
}





=head2

serialize(permissions)

Similar to frozen, but uses a different method of serializing/unserializing.

=cut

sub serialize
{
    my $proto  = shift;
    my $object = shift;

    # allow this to function as an instance method
    if (!defined $object && ref($proto))
    {
        $object = $proto;
    }

    my $serializer = Metadot::Serialization::SerializerFactory->get_instance();
    if (defined $serializer) {
        return $serializer->serialize($object);
    }

}


=head2

deserialize(serialized_permissions)

Deserializes a serialized permissions object

=cut

sub deserialize
{
    my $class  = shift;
    my $data   = shift || croak('Must provide serialized permissions');
    my $deserialized;

    my $serializer = Metadot::Serialization::SerializerFactory->get_instance($data);

    $DISPLAY->debug_msg(5, "I have serializer $serializer");

    if (defined $serializer) {
        $deserialized = $serializer->deserialize($data);
    }

    # some older permissions objects were blessed as 'Permissions', not
    # 'Metadot::Permissions'
    if (UNIVERSAL::isa($deserialized, 'Permissions')) {
        bless ($deserialized, $class);
    }

    return $deserialized;
}



# returns a boolean based on whether a bundle exists
sub has_bundle {
    my $self = shift;
    my $bundle_id = shift || croak "must always provide an bundle_id: $!";

    return exists ($self->{BUNDLES}{$bundle_id});
}


sub get_bundle_level {
  my $self = shift;
  my $bundle_id = shift || croak "must always provide an bundle_id: $!";

  my $result = $self->{BUNDLES}{$bundle_id};

  return $result;
}

sub set_bundle_level {
  my $self = shift;
  my $bundle_id = shift || croak "must always provide bundle_id: $!";
  my $level = shift;

  croak "must always provide a level" unless defined $level;

  $self->{BUNDLES}{$bundle_id} = $level;

}

sub set_recursive {
    my $self = shift;
    my $input = shift;

    $self->{RECURSIVE} = $input;
}

sub is_recursive {
    my $self = shift;
    my $result = ($self->{RECURSIVE});
    return $result;
}


sub is_user_in_acl {
  my $self = shift;
  my $bundle_id = shift || croak "must provide bundle_id";
  my $user = shift || croak "must provide reference to user object";

  my $uid = $user->uid();
  #$DISPLAY->debug_msg(5, "Checking if user $user ($uid, $self->{ACL}{FOR_USERS}{$uid}) is in acl for $bundle_id");

  return 0 if (!defined $uid);
  return 1 if ($uid && $self->{ACL}{FOR_USERS}{$uid}{$bundle_id});

  #aju -- handle overrides
  if (!($self->{ACL}{FOR_USERS}{$uid}{user_overrides})) {
      foreach my $grp_id (keys %{$self->{ACL}{FOR_GROUPS}}) {
          if($self->{ACL}{FOR_GROUPS}{$grp_id}{$bundle_id}) {
              if( $USER->belongs_to_grp_id($grp_id, $user->{uid})) {
                  return 1;
              } else {
                  # nothing
              }
          }
      }
  }
  else {
      #print STDERR "in is_user_in_acl -- user settings override, don't look @ group\n";
  }
  return undef;
}

#  sub is_user_in_acl {
#    my $self = shift;
#    my $bundle_id = shift || croak "must provide bundle_id";
#    my $user = shift || croak "must provide reference to user object";

#    my $uid = $user->uid();

#    return 1 if ($uid && $self->{ACL}{FOR_USERS}{$uid}{$bundle_id});

#    foreach my $grp_id (keys %{$self->{ACL}{FOR_GROUPS}}) {
#      if($self->{ACL}{FOR_GROUPS}{$grp_id}{$bundle_id}) {
#        return 1 if( $USER->belongs_to_grp_id($grp_id, $user->{uid}) );
#      }
#    }
#    return undef;
#  }

sub add_group_acl {
  my $self = shift;
  my $group = shift || croak "must provide group id";

  unless(defined $self->{ACL}{FOR_GROUPS}{$group}) {
    $self->{ACL}{FOR_GROUPS}{$group} = {};
  }
}

sub add_group_acl_pair {
  my $self = shift;
  my $group = shift || croak "must provide group id";
  my $bundle_id = shift || croak "must always provide bundle_id";

  $self->{ACL}{FOR_GROUPS}{$group}{$bundle_id} = 1;

}


sub is_group_acl_pair_there {
  my $self = shift;
  my $group = shift || croak "must provide group";
  my $bundle_id = shift || croak "must provide bundle_id";

  return $self->{ACL}{FOR_GROUPS}{$group}{$bundle_id};
}

sub is_user_acl_pair_there {
  my $self = shift;
  my $uid = shift;

  defined $uid || croak "must provide uid";

  my $bundle_id = shift || croak "must provide bundle_id";

  return $self->{ACL}{FOR_USERS}{$uid}{$bundle_id};
}


sub get_acl_groups {
  my $self = shift;

  my @result = sort (keys %{$self->{ACL}{FOR_GROUPS}});

  return \@result;

}

sub get_acl_users {
  my $self = shift;
  my @result = sort (keys %{$self->{ACL}{FOR_USERS}});
  return \@result;
}


sub add_user_acl_pair {
  my $self = shift;
  my $uid = shift;

  defined $uid || croak "must provide uid";

  my $bundle_id = shift || croak "must always provide bundle_id";

  $self->{ACL}{FOR_USERS}{$uid}{$bundle_id} = 1;

}

#aju
sub add_user_acl_overrides {
    my ($self, $uid) = @_;

    defined $uid || croak "must provide uid";
    #print STDERR "in add_user_acl_overrides\n";
    $self->{ACL}{FOR_USERS}{$uid}{user_overrides} = 1;
}

sub add_user_acl {
  my $self = shift;
  my $uid = shift;

  defined $uid || croak "must provide uid";

  unless(defined $self->{ACL}{FOR_USERS}{$uid}) {
    $self->{ACL}{FOR_USERS}{$uid} = {};
  }
}


sub remove_empty_acls {
  my $self = shift;

  foreach my $uid (keys %{$self->{ACL}{FOR_USERS}}) {
    unless ((scalar (keys %{$self->{ACL}{FOR_USERS}{$uid}})) > 0) {
      delete $self->{ACL}{FOR_USERS}{$uid};
    }
  }

  foreach my $group (keys %{$self->{ACL}{FOR_GROUPS}}) {
    unless ((scalar (keys %{$self->{ACL}{FOR_GROUPS}{$group}})) > 0) {
      delete $self->{ACL}{FOR_GROUPS}{$group};
    }
  }

}

sub remove_all_acls {
  my $self = shift;

  $self->{ACL}{FOR_USERS} = {};
  $self->{ACL}{FOR_GROUPS} = {};

}


sub copy_bundle_acl {
  my $self = shift;
  my $bundle_id = shift;
  my $source_perms = shift;


  foreach my $group (@{$source_perms->get_acl_groups()}) {

    if($source_perms->is_group_acl_pair_there($group, $bundle_id)) {

      $self->add_group_acl_pair($group, $bundle_id);

    }

  }

  foreach my $uid (@{$source_perms->get_acl_users()}) {

    if($source_perms->is_user_acl_pair_there($uid, $bundle_id)) {

      $self->add_user_acl_pair($uid, $bundle_id);

    }
  }
}


### XXX [claudio] needed for recursive perms code

sub remove_user_acl_pair {
  my $self = shift;
  my $uid = shift || croak "must provide uid";
  my $bundle_id = shift || croak "must provide bundle_id";

  delete $self->{ACL}{FOR_USERS}{$uid}{$bundle_id};
}

sub remove_group_acl_pair {
  my $self = shift;
  my $group = shift || croak "must provide group id";
  my $bundle_id = shift || croak "must provide bundle_id";

  delete $self->{ACL}{FOR_GROUPS}{$group}{$bundle_id};
}

###


1;


__END__

=head1 NAME

Permissions -

=head1 SYNOPSIS

    use Permissions;
    $permissions = Permissions->new ([$bundles_href]);

    Permissions->deserialize ($frozen_perms);
    $bundle_level = $permisisons->get_bundle_level ($bundle_id);
    $permisisons->set_bundle_level ($bundle_id, $level);
    $frozen_perms = $permisisons->serialize() ;
    $boolean = $permisisons->is_user_in_acl ($bundle_id, $user);
    $permisisons->add_group_acl ($group);
    $permisisons->add_group_acl_pair ($group, $bundle_id);
    $bundle_id = $permisisons->is_group_acl_pair_there ($group, $bundle_id);
    $bundle_id = $permisisons->is_user_acl_pair_there ($uid, $bundle_id);
    $group_list_ref = $permisisons->get_acl_groups ;
    $user_list_ref = $permisisons->get_acl_users ;
    $permisisons->add_user_acl_pair ($uid, $bundle_id);
    $permisisons->add_user_acl ($uid);
    $permisisons->remove_empty_acls ;
    $permisisons->remove_all_acls ;
    $permisisons->copy_bundle_acl ($bundle_id, $source_perms);


=head1 DESCRIPTION

Permissions are handled through the methods of the Auditable abstract class.  Therefore, to work with permissions objects,
see the documentation for the Auditable abstract class.

=cut
