###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

# processes system messages. used for IPC between server instances
package Metadot::SystemMessageManager;

use strict;
use vars qw(@ISA);

use MetadotObject;
use Metadot::SystemMessage;

@ISA = qw(MetadotObject);

use constant _TABLE_ => 'system_messages';

sub send_message {
    my $class = shift;
    my $message = shift;

    (defined $message && $message->isa('Metadot::SystemMessage')) || die("Must supply a Metadot::SystemMessage object");
    defined ($message->get_sender()) || die("Must supply the sender's signature");
    defined ($message->get_sender_class()) || die("Must supply the sender's class");

    #print STDERR "Sending message\n",Dumper $message;

    DBAccess->sqlInsert(
        _TABLE_,
        sender          => $message->get_sender(),
        recipient       => $message->get_recipient(),
        sender_class    => $message->get_sender_class(),
        recipient_class => $message->get_recipient_class(),
        content         => $message->get_content()
    );
}

sub check_messages {
    my $class = shift;
    my %args = @_;

    #my $sender          = $args{sender};
    #my $recipient       = $args{recipient};
    #my $since           = $args{since};
    #my $max_age         = $args{max_age};
    #my $content         = $args{content};

    my @results = ();

    my $where = $class->_compute_where_clause(%args);

    Metadotclass->debug_msg(4, "SystemMessageManager::check_messages using WHERE='$where'");

    my $sth = DBAccess->sqlSelectMany( 'timestamp, sender, recipient, sender_class, recipient_class, content', _TABLE_, $where);
    while ( my ($timestamp, $sender, $recipient, $sender_class, $recipient_class, $content) = $sth->fetchrow()) {
        Metadotclass->debug_msg(4, "SystemMessageManager::check_messages got message '$content'");
        push @results, Metadot::SystemMessage->new(
            {
                timestamp       => $timestamp,
                sender          => $sender,
                sender_class    => $sender_class,
                recipient_class => $recipient_class,
                recipient       => $recipient,
                content         => $content,
            }
        )
    }
    $sth->finish();

    if (wantarray) {
        return @results;
    } else {
        return \@results;
    }
}


sub clean_messages {
    my $class   = shift;
    my %args    = @_;

    my $where = $class->_compute_where_clause(%args);
    #print STDERR "cleaning system messages WHERE '$where'\n";
    DBAccess->sqlDelete( _TABLE_, $where );
}


sub _compute_where_clause {
    my $class = shift;
    my %args  = @_;

    my $sender          = $args{sender};
    my $recipient       = $args{recipient};
    my $sender_class    = $args{sender_class};
    my $recipient_class = $args{recipient_class};
    my $since           = $args{since};
    my $max_age         = $args{max_age};   # records that are no older than x
    my $min_age         = $args{min_age};   # records that ARE older than x
    my $content         = $args{content};
    my $now             = DBAccess->db_now_function();

    ($sender || $recipient || $sender_class || $recipient_class || $since || $max_age || $min_age || $content ) || die("Must specify at least one search criterion");

    my @conditions = ();
    if (defined $sender) {
        push @conditions, 'sender = '.DBAccess->quote($sender);
    }
    if (defined $recipient) {
        push @conditions, 'recipient ='.DBAccess->quote($recipient);
    }
    if (defined $sender_class) {
        push @conditions, 'sender_class ='.DBAccess->quote($sender_class);
    }
    if (defined $recipient_class) {
        push @conditions, 'recipient_class ='.DBAccess->quote($recipient_class);
    }
    if (defined $since) {
        push @conditions, 'timestamp >= '.$since;
    }
    if (defined $max_age) {
        push @conditions, DBAccess->db_timediff_function($now, 'timestamp')." <= ".$max_age;
    }
    if (defined $min_age) {
        push @conditions, DBAccess->db_timediff_function($now, 'timestamp')." >= ".$min_age;
    }
    if (defined $content) {
        push @conditions, 'content like '.DBAccess->quote($content);
    }

    my $where = join (' AND ', @conditions);
    return $where;
}

1;

