###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

# extends WWW::Mechanize to provide some Metadot-specific automation

package Metadot::Testing::Mechanizer;

use strict;
use vars qw(@ISA);

use CGI qw(escape unescape);
use WWW::Mechanize;
use SmokeMetadot;

@ISA = qw(WWW::Mechanize);

sub new {
    my $proto = shift;
    my %args  = @_;
    my $class = ref($proto) || $proto;

    my $self = $class->SUPER::new();
    bless ($self, $class);

    # set the http stuff
    $self->hostname($args{hostname} || SmokeMetadot->SMOKE_HOSTNAME());
    $self->uri_schema($args{uri_schema} || SmokeMetadot->SMOKE_URI_SCHEMA());
    $self->request_path($args{request_path} || SmokeMetadot->SMOKE_REQUEST_PATH());
    $self->script_name($args{script_name} || SmokeMetadot->SMOKE_SCRIPT_NAME());

    return $self;
}


# visits a Metadot page with a iid, op, isa, etc...
# home page)
sub get_page {
    my $self = shift;
    my %args = @_;

    $self->get(
        $self->generate_url(
            url   => $self->base_url(),
            query => \%args,
        )
    );

    while ($self->is_redirect) {
        $self->follow_redirect();
    }

    #print STDERR "get_page returns ",$self->content(),"\n";
    return $self->response();
}


# logs in with the given account_name and password
sub login {
    my $self = shift;
    my %args = @_;

    my $account_name  = $args{account_name} || die('No account_name');
    my $password      = $args{password}     || die('No password');

    # get the login page
    $self->get_page(
        isa => 'Session',
        op  => 'login',
    );

#print STDERR "Successfully retrieved login page. Will submit $account_name/$password\n";

    # submit the login form
    $self->submit_form(
        form_name => 'login_form',
        fields    => { email => $account_name, password => $password },
        button    => 'submit',
    );
}


# logs out of the current session
sub logout {
    my $self = shift;
    $self->get_page(
        isa => 'Session',
        op  => 'logout',
    );
}


# follows a Metadot redirect
sub follow_redirect {
    my $self = shift;

    # see if this was a redirect
    my $status = $self->status();
    if (defined($status) && ($status == 301 || $status == 302) ) {
        $self->redirect_ok($self, $self->response());
    } else {
        my $content = $self->content();
        if (defined $content) {
            $content =~ m/http-equiv=\"refresh\" +content="\d*;?url=(.*?)"/s;
            my $url = $1;
#print STDERR "Found via meta-tag the URL '$url'\n";
            $self->get($url);

        }
    }

#print STDERR "Status was: ",$self->status(),"\n";
}


# determine whether the current page is a redirect
sub is_redirect {
    my $self = shift;

    # check if the HTTP status is a redirect
    my $status = $self->status();
    my $result = (defined($status) && ($status == 301 || $status == 302) );

    # check if the content had a meta-tag
    unless ($result) {
        my $content = $self->content();
        if (defined $content) {
            $content =~ m/http-equiv=\"refresh\" +content="\d*;?url=(.*?)"/s;
            my $url = $1;

            $result = (defined $url);
        }
    }
}


# generates a valid URL
sub generate_url {
    my $self = shift;
    my %args = @_;

    my $url   = $args{url}   || $self->base_url() || die('No URL defined');
    my $query = $args{query} || {};

    # cycle through the
    my @vars = ();
    while (my ($key, $value) = each %$query) {
        push @vars, escape($key).'='.escape($value);
    }
    my $query_string = join('&', @vars);
    my $result = $url;
    if (defined $query_string) {
        $result .= '?'.$query_string;
    }

#print STDERR "generate_url returns $result\n";
    return $result;
}


# returns the current session id, as found in the current request
sub get_session_id {
    my $self = shift;

    my $cookie_jar = $self->cookie_jar();
    my $cookie_name = 'sessionid';
    my $cookie_value = undef;
    my $hostname = $self->hostname();

    # scan for the session cookie
    $cookie_jar->scan(sub {
        my ($version, $key, $value, $path, $domain) = @_;
        if ($key eq $cookie_name && $domain =~ m/\Q$hostname\E/) {
            $cookie_value = $value;
#print STDERR "I have key $key; value=$value; domain=$domain\n";
        }
    });

    return $cookie_value;
}


# computes the base url for any request
sub base_url {
    my $self = shift;
    my $result = $self->uri_schema()
                 . '://' .$self->hostname()
                 . $self->request_path()
                 . '/' . $self->script_name();

    return $result;
}


# checks if an instance is retrievable
sub instance_retrievable {
    my $self = shift;
    my %args = @_;

    defined(my $iid  = $args{iid})  || die('No instance id');
    my $expr = $args{expr} || die('No expr to check');

    $self->get_page( iid=>$iid );
    my $result = ($self->content() =~ m/$expr/s);

#print STDERR "instance $iid visible via \"$expr\", result='$result'\n";
#print STDERR '='x72, $self->content()."\n\n";
    return $result;
}



# Accessors / Mutators
#######################################################################
sub hostname {
    my $self = shift;
    my $property = 'hostname';
    return (@_) ? $self->{$property} = shift : $self->{$property};
}

sub uri_schema {
    my $self = shift;
    my $property = 'uri_schema';
    return (@_) ? $self->{$property} = shift : $self->{$property};
}

sub request_path {
    my $self = shift;
    my $property = 'request_path';
    return (@_) ? $self->{$property} = shift : $self->{$property};
}

sub script_name {
    my $self = shift;
    my $property = 'script_name';
    return (@_) ? $self->{$property} = shift : $self->{$property};
}

sub userid {
    my $self = shift;
    my $property = 'userid';
    return (@_) ? $self->{$property} = shift : $self->{$property};
}







1;
