###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


package Metadot::UpdateNotification::UpdateNotifier;


=head1 NAME

Metadot::UpdateNotification::UpdateNotifier

=head1 VERSION

Version $Revision: 1.1.2.11 $

=head1 SYNOPSIS

Instance methods, used by class descendants
-------------------------------------------

use Metadot::UpdateNotification::UpdateNotifier;
@ISA = qw( Gizmo Metadot::UpdateNotification::UpdateNotifier );

..

if ($self->is_update_notification_enabled())
{
    my $cid = $self->get_cid();
}


Class methods, used by any Gizmo
--------------------------------

if (Metadot::UpdateNotification::UpdateNotifier->child_has_notifying_parent())
{
    Metadot::UpdateNotification::UpdateNotifier->show_notification_properties_for_child($self);
}





=head1 DESCRIPTION

UpdateNotifier is a class that allows instances to send news updates via a Channel.
This is typically used to allow end-users to notify their readers about changes to
the site.

=head1 METHODS

=cut




use strict;

use Channel;
use ErrorHandling;
use Metadot qw($DISPLAY $PARAMS %FORM $SESSION %REQUEST_VARS);
use Metadot::MetadotController;
use Metadot::PortalFactory;
use Metadot::ChannelProcessor;
use Gizmo::GizmoShortcut;


=item C<customize_permissions>

Creates new ops that are used for notifications. Gizmo3000 instances call this
method automatically. Other gizmos should call this method in class context in
order to get the correct ops.

=cut
sub customize_permissions {
    my ($self, $ac) = @_;

    # send notifications
    $ac->add_op('MOD','create_update_notification');

    return $ac;
}


=item C<exec_create_update_notification>

The handler for the 'create_update_notification' op

$self->exec_create_update_notification($portal_context);

=cut
sub exec_create_update_notification {
    my $self = shift;
    my $context = shift;

    $DISPLAY->debug_msg(3, "In create_update_notification");

    my $return_url  = $FORM{return_url} || $SESSION->referer();
    my $landing_url = $FORM{landing_url} || $return_url;

    if ($FORM{send_notification}) {

        $self->create_notification ({
            heading => $FORM{notification_heading},
            body    => $FORM{notification_body},
        });

        # show confirmation
        $DISPLAY->print_choices(
            "Your notification has been submitted.",
            "confirm",
            "Continue", $landing_url
        );
        exit();

    } elsif ($FORM{skip_notification}) {

        $DISPLAY->md_redirect($return_url);

    } else {

        return $self->show_update_notification_form({
            landing_url => $landing_url,
            return_url  => $return_url,
        });

    }

}


=item C<create_update_notification>

Performs the actual work of creating a notification

$self->create_notification({
    heading     => 'the message heading',
    body        => 'the message body',
    channel_processor => $channel_processor_object, #(optional)
});

=cut
sub create_notification {
    my $self = shift;
    my $args = shift || {};

    my $cp = $args->{channel_processor} || Metadot::PortalFactory->get_channel_processor();
    my $iid = $self->id();
    my $basename    = $PARAMS->{basename};
    my $url         = $basename.'/index.pl?iid='.$iid;

    $cp->add_channel_item(
        cid         => $self->get_cid(),
        name        => $args->{heading},
        description => $args->{body},
        url         => $url,
        iid         => $iid,
    );
}

=item C<get_cid>

Gets the channel item for this instance. Creates a channel if none exists.

my $cid = $self->get_cid();

=cut
sub get_cid {
    my $self = shift;
    my $iid = $self->iid();

    my $cid;

    if (defined $iid) {
        # look for an existing channel
        $cid = Channel->get_cid_for_iid($iid);

        unless (defined $cid) {
            # create a new channel
            my $newchan = Channel->new();
            $cid = $newchan->create($iid, 'Internal', $self->name(), $self->description());
        }
    }

    return $cid;
}


=item C<get_cid>

Flag to indicate whether an instance has update notification enabled. By default
this is disabled. Implementing classes should override this method.

if ($self->is_update_notification_enabled()) {
  # do something
}

=cut
sub is_update_notification_enabled {
    return 0;
}



=item C<get_notifying_parents_for_child>

Returns all structural parents that implement UpdateNotifier and have notification
enabled. Typically called in class context by a Gizmo instance.

my @notifying_parents = Metadot::UpdateNotification::UpdateNotifier
  ->get_notifying_parents_for_child(
    $self,
    short_circuit    => [ flag to determine whether to stop after finding one parent; defaults to false ],
    follow_shortcuts => [ flag to determine whether to follow gizmo shortcuts; defaults to true ],
    visited          => { visited instances; prevents looping recursion },
    );

=cut
sub get_notifying_parents_for_child {
    my $class = shift;
    my $child = shift || die("Must specify the child");
    my %args  = @_;

    my $short_circuit = $args{short_circuit};
    my $follow_shortcuts = $args{follow_shortcuts};
    unless (defined $follow_shortcuts) {
        $follow_shortcuts = 1;
    }

    my $visited = $args{visited} ||= {};

    # avoid circular recursion
    return undef if ($visited->{$child}++);

    my @results = ();

    $DISPLAY->debug_msg(5, "get_notifying_parents_for_child $child");

    # check the parent. If it is an UpdateNotifier, return the parent if it
    # has notification enabled. If not enabled, stop the search at that point.
    my $parent = $child->get_parent();
    if ($parent && $parent->isa(__PACKAGE__)) {
        if ($parent->is_update_notification_enabled()) {
            if ($short_circuit) {
                return $parent;
            } else {
                push @results, $parent;
            }
        }
    } else {
        # if the parent is not an UpdateNotifier, then check its parent
        my @recursive_results = $class->get_notifying_parents_for_child($parent, %args);
        if (@recursive_results) {
            if ($short_circuit) {
                return @recursive_results;
            } else {
                @results = (@results, @recursive_results);
            }
        }
    }

    # follow gizmo shortcuts if necessary
    if ($follow_shortcuts) {
        my @shortcuts = GizmoShortcut->get_all_shortcuts_for($child->id());
        foreach my $shortcut (@shortcuts) {
            my @shortcut_results = $class->get_notifying_parents_for_child($shortcut, %args);
            if (@shortcut_results) {
                if ($short_circuit) {
                    return @shortcut_results;
                } else {
                    @results = (@results, @shortcut_results);
                }
            }
        }
    }

    return @results;
}



=item C<child_has_notifying_parent>

Determines whether a Gizmo child has a parent that implements UpdateNotifier and
has notification enabled. This method will check direct ancestry and will also
follow GizmoShortcuts.

Typically called in class context by a Gizmo instance.

if (Metadot::UpdateNotification::UpdateNotifier->child_has_notifying_parent($self)) {
  # do something
}

=cut
sub child_has_notifying_parent {
    my $class = shift;
    my $child = shift || die("Must specify the child");
    my %args  = @_;

    my $parent = $class->get_notifying_parents_for_child(
        $child,
        short_circuit => 1,
    );

    return ($parent);
}



=item C<show_update_notification_form>

Generates an update notification form to be filled out by an end user

$self->show_update_notification_form({
    landing_url => 'url to land after successful completion',
    return_url  => 'url to return in case of abort',
});
=cut
sub show_update_notification_form {
    my $self = shift;
    my $args = shift || {};

    $DISPLAY->debug_msg(3, "In show_update_notification_form");


    if ($PARAMS->{use_controller}) {

        # create a context for this template
        my $context = Metadot::PortalFactory->build_context({
            subject         => $self,
        });

        return Metadot::MetadotController->output({
            context  => $context,
            template => 'update_notification_form.tt2',
            data     => {
                landing_url => $args->{landing_url},
                return_url  => $args->{return_url},
            },
        });

    } else {

        ErrorHandling->printError("Non-skins mode is deprecated. Update notification form cannot be displayed.");

    }
}



=item C<show_notification_properties_for_child>

Typically called within show_edit_properties_form. This allows a child to
present a notification form that will then be routed to its parent

Typically called in class context by a Gizmo instance.

my $html = Metadot::UpdateNotification::UpdateNotifier->show_notification_properties_for_child($self);
=cut
sub show_notification_properties_for_child {
    my $class = shift;
    my $child = shift || die('Must specify child');

    $DISPLAY->debug_msg(4, "show_notification_properties_for_child $child");

    my $display_name;
    if ($child->isa('Metadot::GizmoBase')) {
        $display_name = $child->get_gizmo_name();
    } else {
        $display_name = ref($child) || $child;
    }

    my $child_id    = $child->id();
    my $op          = 'notification_for_child';
    my $href        = $DISPLAY->escapeHREF(
        "index.pl?isa=$class&op=$op&child_id=$child_id"
    );

    my $out = qq~
<div>
<b>Click this button to send subscription-based notifications to users about this $display_name:
<a class="text_button"
   href=$href
   target="notify_popup"
   >Notify by subscription</a>
<img src="$PARAMS->{imageslocation}/new_window.gif" />
</div>
~;

    return $out;
}


sub exec_notification_for_child {
    my $class   = shift;
    my $context = shift;

    $DISPLAY->debug_msg(3, "exec_notification_for_child");

    defined (my $child_id = $FORM{child_id}) || die('No child_id specified');
    my $child = Metadot::PortalFactory->get_gizmo_object($child_id);

    $DISPLAY->debug_msg(4, "exec_notification_for_child has child $child ($child_id)");

    $REQUEST_VARS{notification_child} = $child;

    if ($FORM{send_notification}) {

        my @notifiers = $class->get_notifying_parents_for_child($child);

        # create the notification here
        my $cp = Metadot::PortalFactory->get_channel_processor();

        foreach my $notifier (@notifiers) {
            $notifier->create_notification ({
                channel_processor   => $cp,
                heading             => $FORM{notification_heading},
                body                => $FORM{notification_body},
            });
        }

        $DISPLAY->print_choices(
            "Your notification has been submitted.",
            "confirm",
            "Close this window", "javascript:window.close()",
        );
        exit();
    } else {
        return 'update_notification_form_for_child.tt2';
    }
}









1;
