package Metadot::User::User3000::PropertyManager;

use strict;

use Metadot::Serialization::SerializerFactory;
use Metadot::User::User3000;

use Metadot qw($PARAMS $CACHE $DISPLAY $USER);

use constant CACHE_NAME => 'USER3000_PROPERTIES';
use constant PARAMS_KEY => '_user3000_custom_properties';

sub get_custom_properties {
    my $class = shift;

    my $properties;

    my $packed = $PARAMS->{PARAMS_KEY()};
    if (defined $packed) {
        # try to get the unpacked version from cache
        $class->validate_or_refresh_cache();
        $properties = $CACHE->get_value(CACHE_NAME());

        # otherwise unpack them from params
        unless (defined $properties) {
            # see if we use the old or new way
            if ($class->packed_data_is_current($packed)) {
                $properties = $class->_unpack_data($packed);
            } else {
                $properties = $class->_unpack_data_OLD($packed);
            }

            $CACHE->set_value(CACHE_NAME(), $properties);
        }
    }

    return $properties;
}


sub set_custom_properties {
    my $class = shift;
    my $properties = shift;

    if (defined $properties) {
        my $packed = $class->_pack_data($properties);

        $PARAMS->set_property(PARAMS_KEY(), $packed);
        $CACHE->set_as_dirty(CACHE_NAME());
    } else {
        $PARAMS->set_property(PARAMS_KEY(), undef);
    }
}


sub packed_data_is_current {
    my $class  = shift;
    my $packed = shift;

    my $pack_property_separator = $class->PACK_PROPERTY_SEPARATOR();
    return ($packed =~ m/\Q$pack_property_separator\E$/ );
}


use constant PACK_ATTRIBUTE_SEPARATOR => '~_';
use constant PACK_PROPERTY_SEPARATOR => '~!';

sub _unpack_data {
    my $class  = shift;
    my $packed = shift;

    $DISPLAY->debug_msg(5, "unpack_data");
    my @results = ();

    if (defined $packed) {
        $DISPLAY->debug_msg(5, "unpack_data has something to unpack\n\n$packed\n\n");

        my $pack_property_separator = PACK_PROPERTY_SEPARATOR();
        my $pack_attribute_separator = PACK_ATTRIBUTE_SEPARATOR();

        foreach my $packed_property (split(/\Q$pack_property_separator\E/, $packed)) {
            my $property = undef;
            foreach my $packed_line (split(/\Q$pack_attribute_separator\E/, $packed_property)) {
                my ($key, $value) = split('=', $packed_line, 2);
                if (defined $key) {
                    $property ||= {};
                    $property->{$key} = $value;
                }
            }
            if (defined $property) {
                my $mapping = $USER->generate_user_property_mapping($property);
                push @results, $mapping;
#use Data::Dumper;
#print STDERR "_unpack_data created property\n",Dumper($mapping);
            }
        }
    }

    return \@results;
}


sub _pack_data {
    my $class      = shift;
    my $properties = shift;
    my $packed = '';

    if (defined $properties) {
        # Store the properties in a text file. The text file consists of
        # a series of entries, one for each property.
        # Each property entry in turn consists of a series of lines,
        # one for each key/value pair of the property
        foreach my $property (@$properties) {
            my $config_hash = $USER->generate_field_config_hash($property);
            my $packed_field = '';
            while (my ($key, $value) = each %$config_hash) {
                $packed_field .= $key.'='.($value || '').PACK_ATTRIBUTE_SEPARATOR();
            }
            $packed .= $packed_field . PACK_PROPERTY_SEPARATOR();
        }
    }

    return $packed;
}



sub _unpack_data_OLD {
    my $class  = shift;
    my $packed = shift;

    $DISPLAY->debug_msg(5, "unpack_data");
    my $result;

    if (defined $packed) {
        $DISPLAY->debug_msg(5, "unpack_data has something to unpack");
        my $serializer = Metadot::Serialization::SerializerFactory->get_instance($packed);
        if (defined $serializer) {
            $result = $serializer->deserialize($packed);
        }
    }

    return $result;
}


sub _pack_data_OLD {
    my $class      = shift;
    my $properties = shift;

    if (defined $properties) {
        my $serializer = Metadot::Serialization::SerializerFactory->get_instance();
        if (defined $serializer) {
            return $serializer->serialize($properties);
        }
    }
}



sub validate_or_refresh_cache {
    my $class = shift;
    my $db_timestamp = $CACHE->is_dirty(CACHE_NAME());

    my $result;

    #return if the cache is up to date
    if ($db_timestamp) {
        $DISPLAY->debug_msg(5, "Cache was flushed");
        $CACHE->set_value(CACHE_NAME(), undef);
        $CACHE->set_timestamp(CACHE_NAME(), $db_timestamp);
    } else {
        $DISPLAY->debug_msg(5, "Cache is valid");
        $result = $CACHE->get_value(CACHE_NAME());
    }

    return $result;
}





1;
