###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


# this is a handy base class
package MetadotObject;

use strict;
no strict 'refs';  # allows dynamic code generation

use Data::Dumper;

use Utility;
use Metadotclass;
use Metadot::ClassUtilities;

# this is the method that will always be called at the end of new()
use constant MO_INITIALIZER_METHOD => 'initialize';

# a hashtable that defines all local properties for an instance and
# their default values
use constant MO_INSTANCE_PROPERTIES => {};



# send an argument "properties" with a hash ref of desired properties
# of the object.
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    bless $self, $class;

    #print STDERR "Beginning <$proto> ".__PACKAGE__."\->new()\n";

    # make sure the args are always in the expected key=>value form
    my $args = __PACKAGE__->normalize_constructor_args(@_);

    # make sure the methods exist in that class
    $class->MO_make_methods();

    # initalize all of the properties of this instance.
    #print STDERR "MetadotObject initializes properties for $self\n";
    Metadotclass->debug_msg(5, "initializes properties for $self");
    $self->MO_initialize_properties($args);
    Metadotclass->debug_msg(5, "finished initializing properties for $self");


    # See if I need to call an initializer method on this object.
    # Note: this always take place AFTER MO_initialize_properties, so
    #       many properties may already have been set.
    my $initializer_method = $class->MO_INITIALIZER_METHOD();

    if ($self->can($initializer_method)) {
        #print STDERR "MetadotObject calls initializer method $self\->$initializer_method()\n";
        Metadotclass->debug_msg(5, "calls initializer method $self\->$initializer_method()");
        $self->$initializer_method($args);
        Metadotclass->debug_msg(5, "FINISHED initializer method $self\->$initializer_method()");
    }

    #print STDERR "Finished <$proto> ".__PACKAGE__."\->new(), returns $self\n";
    #print STDERR Dumper $self;

    return $self;
}


# this is just a stub to prevent errors
sub initialize {
    my $self = shift;
    my $args = shift || {};

    return $self;
}


# Returns a new object with the same properties as the current object.
# If any property is a reference, the same reference is used. Be careful of that!
sub clone {
    my $self  = shift;
    my $class = ref($self) || die("Instance method");

    my $clone = $class->new();
    foreach my $key (keys(%{$self})) {
        $clone->{$key} = $self->{$key};
    }
    return $clone;
}

# Base accessor
sub access_property {
    my $self        = shift;
    defined(my $property = shift) || die("No property to get");
    my $default     = shift;
    my $result      = $self->{$property};
    unless (defined($result)) {
        $result = $default;
    }
    return $result;
}

# Base mutator
sub mutate_property {
    my $self        = shift;
    defined(my $property = shift) || die("No property to get");
    my $value       = shift;
    $self->{$property} = $value;
}


# Deprecated: user MO_set_properties
sub set_properties {
    my $self = shift;
    return $self->MO_set_properties(@_);
}




# sets all the properties at once
sub MO_set_properties {
    my ($self, $properties, $overwrite) = @_;

    # overwrite defaults to true
    unless (defined $overwrite) {
        $overwrite = 1;
    }

    # initialize each property, even if no arg was sent.
    foreach my $property ( keys %$properties ) {
        #print STDERR "Checking property $property\n";
        # check that the property is valid
        if ($self->can($property)
            # avoid overwriting
            && ($overwrite || !(defined $self->$property()))
            )
        {
            my $value = $properties->{$property};
            #print STDERR "Setting $self\->$property($value)\n";
            $self->$property($value);
        }
    }
}


# Deprecated: user MO_exists_property
sub exists_property {
    my $self = shift;
    return $self->MO_exists_property(@_);
}

# determines if a property has been set
sub MO_exists_property {
    my $self = shift;
    my $property    = shift || die("No property to get");
    my $result = exists($self->{$property});
    return $result;
}


# deprecated: user MO_get_all_properties
sub get_all_properties {
    my $self = shift;
    return $self->MO_get_all_properties(@_);
}

# returns all the instance properties in a hashtable
sub MO_get_all_properties {
    my $self = shift;
    my %result = ();
    my $properties = $self->MO_instance_properties(1);
    foreach my $key (keys %$properties) {
        $result{$key} = $self->$key();
    }

    return \%result;
}






















# returns all the properties associated with instances of this class.
sub MO_instance_properties {
    my $proto     = shift;
    my $recursive = shift;

    my $class = ref($proto) || $proto;
    my %properties = ();

    if ($recursive) {
        #print STDERR "recursively checking instance properties for $class\n";

        # get my ancestor hierarchy
        my @ancestor_list = $class->MO_class_hierarchy();

        # get my ancestor's properties
        foreach my $ancestor (@ancestor_list) {
            my $ancestor_properties = $ancestor->MO_instance_properties($recursive);
            Utility->merge_hashes(\%properties, $ancestor_properties);
        }
    }

    # now get my local properties

    my $ref_name = "${class}::MO_INSTANCE_PROPERTIES";
    my $class_properties;

    # try to get the class properties from the defined constant (the new way)
    #print STDERR "$class is looking for instance properties in $ref_name\n";
    if (defined &$ref_name) {
        #print STDERR "$class found ".&$ref_name."\n";
        $class_properties = &$ref_name;
    } else {
        # look the old way, in get_constructor_properties
        $ref_name = "${class}::get_constructor_properties";
        #print STDERR "$class is looking for old-style properties in $ref_name\n";
        if (defined &$ref_name) {
            my $constructor_properties = &$ref_name || {};
            #print STDERR "$class found constructor properties\n";
            #print STDERR Dumper $constructor_properties;

            $class_properties = {};
            foreach my $key (keys %$constructor_properties) {
                $class_properties->{$key} = $constructor_properties->{$key}->{value};
            }
        }
    }

    # now merge the class properties with the result
    if (defined $class_properties) {
        Utility->merge_hashes(\%properties, $class_properties);
    }

    #print STDERR "Instance properties for $class:\n", Dumper \%properties;
    return \%properties;
}


sub MO_make_methods {
    my $class = shift;

    my $flag_name = 'MO_METHODS_DEFINED';

    # Don't run this more than once per class. I set a flag constant
    # to check
    return if (defined &{"${class}::$flag_name"});


    # first make sure my ancestors have their methods
    foreach my $ancestor ($class->MO_class_hierarchy()) {
        $ancestor->MO_make_methods();
    }

    #print STDERR "IN $class\->MO_make_methods()\n";

    my $instance_properties = $class->MO_instance_properties();
    #print STDERR "Instance properties:\n",Dumper($instance_properties);

    my $code = <<EOM
package $class;

use constant $flag_name => 1;

EOM
;

    foreach my $key (keys %$instance_properties) {
        $code .= $class->_MO_generate_method_code($key);
        #print STDERR "$class\->MO_make_methods generated code for key '$key'\n";
    }

    #print STDERR "$class\->MO_make_methods generated:\n\n$code\n\n\n\n";
    eval $code;
    if ($@) {
        die("Error making methods for $class");
    } else {
        #print STDERR "Successfully made methods for $class\n";
    }
}



# Generates the method code for accessors and mutators. For any given key 'key',
# this will generate the code for the following methods:
#
# key()      the perl-style combination accessor/mutator
# get_key()  an accessor
# set_key()  a mutator
#
# if key() is already defined, it will try to create
# a method _key() instead, unless that too is defined.
#

sub _MO_generate_method_code {
    my $class = shift;
    my $key   = shift || return();

    my $result = '';


    my $method_name = $key;
    my $method_ref  = "${class}::${method_name}";
    if (defined &$method_ref) {
        $method_name = "_$key";
        #print STDERR "$class already had method '$key' in $method_ref, using '$method_name' instead\n";
    }

    # generate the perl-style combo accessor/mutator
    $method_ref  = "${class}::${method_name}";
    if (defined &$method_ref) {
        #print STDERR "$class already had method '$key' in $method_ref. Giving up.\n";
    } else {
        $result .= <<EOM;

sub $method_name {
    my \$self = shift;
    \@_ ? \$self\->{$key} = shift
       : \$self\->{$key};
}
EOM
;
        #print STDERR "Generated method code for $class\->$method_name()\n";
    }

    # auto-generate the distinct accessor and mutator methods ONLY if this key is not private
    unless ($key =~ /^_/) {

        # generate the getter
        my $method_name = "get_$key";
        unless ($class->can($method_name)) {
            $result .= <<EOM

    sub $method_name {
        my \$self = shift;
        return \$self->$key();
    }
EOM
;
            #print STDERR "Generated method code for $class\->$method_name()\n";
        }


        # generate the setter
        $method_name = "set_$key";
        unless ($class->can($method_name)) {
            $result .= <<EOM

    sub $method_name {
        my \$self = shift;
        return \$self->$key(\@_);
    }
EOM
;
            #print STDERR "Generated method code for $class\->$method_name()\n";
        }
    }

    #print STDERR "Generated method code for $class, {$key}\n";
    return $result;
}


# This is a class method. Initializes the properties of an object based on
# that object's class definitions
sub MO_initialize_properties {
    my $self = shift;
    my $args = shift;

    die('instance method!') unless (ref($self));

    #print STDERR "IN $self\->MO_initialize_properties()\n";
    #print STDERR "with args\n",Dumper $args;

    my %properties = %{$self->MO_instance_properties('recursive')};
    Metadotclass->debug_msg(5, "MO_initialize_properties got instance properties");
    foreach my $key (keys %properties) {
        # check if I was passed a value.
        my $value = $args->{$key};

        # otherwise, look for a default value
        defined($value) || ($value = $properties{$key});

        # set the value in the object
        if (defined $value) {
            $self->$key($value);
            #print STDERR "Initialized $self\->$key($value) confirmed as ".$self->$key(),"\n";
            Metadotclass->debug_msg(5, "Initialized $self\->$key($value)");
        }
    }
}




# DEPRECATED!!
# returns a hashtable of properties that are accepted in the
# constuctor. these hashtable should contain the following structure:
#   $property => { value => $value }
# where the $property is the name of the property, $value is the default value
sub get_constructor_properties {
    return {};
}


# allows one to instantiate a MetadotObject using any of the following:
# - a list of key=>value pairs
# - a single hashref of key=>value pairs
sub normalize_constructor_args {
    my $class = shift;
    my $result = undef;

    if (@_) {
        # a hashref was passed to us
        if (ref($_[0]) && ref($_[0]) eq 'HASH') {
            $result = $_[0];
        } elsif (scalar(@_) % 2) {
            # An uneven number of args was passed to us. This is invalid
            my ($package, $file, $line) = caller;
            die("Invalid constructor arguments called by $package:$line (".join(',', @_).")".Dumper(@_));
        } else {
            # a list was passed to us, which we assume is a hashtable
            $result = {@_};
        }
    } else {
        # nothing was passed to us
       $result = {};
    }

    return $result;
}


# gets the ancestors of this class which are also subclasses of MetadotObject
sub MO_class_hierarchy {
    my $class = shift;
    my $args  = shift || {};

    my $inclusive   = $args->{inclusive};
    my $base_class  = $args->{base_class} || __PACKAGE__;

    return Metadot::ClassUtilities->ancestor_hierarchy(
        'class'      => $class,
        'base_class' => $base_class,
        'inclusive'  => $inclusive,
    );
}


# returns a list of all properties for a MetadotObject class
sub MO_properties_list {
    my $class = shift;
    my $properties = $class->MO_instance_properties('recursive');
    return [keys %$properties];
}


# returns a hashtable of all properties and values of the MetadotObject instance
sub MO_properties_hash {
    my $self = shift;
    my %result;

    if (ref $self) {
        # this is easy because it's a hash internally, but that's not
        # guaranteed to be true in the future.
        %result = %{$self};
    }

    return \%result;
}



1;
